@@ -21,6 +21,8 @@
 
 package org.apache.derby.client.am;
 
+import java.sql.ResultSet;
+import java.sql.Types;
 import org.apache.derby.shared.common.error.ExceptionUtil;
 import org.apache.derby.shared.common.reference.SQLState;
 
@@ -41,7 +43,7 @@ import org.apache.derby.shared.common.reference.SQLState;
  * that does not fit in a VARCHAR (FOR BIT DATA), are split into
  * several calls each operating on a fragment of the LOB.
  *
- * @see Connection#locatorProcedureCall for an example of how to use
+ * @see ClientConnection#locatorProcedureCall for an example of how to use
  * this class.
  */
 class CallableLocatorProcedures 
@@ -49,31 +51,31 @@ class CallableLocatorProcedures
     //caches the information from a Stored Procedure
     //call as to whether locator support is available in
     //the server or not.
-    boolean isLocatorSupportAvailable = true;
+    private boolean isLocatorSupportAvailable = true;
     
     // One member variable for each stored procedure that can be called.
     // Used to be able to only prepare each procedure call once per connection.
-    private CallableStatement blobCreateLocatorCall;
-    private CallableStatement blobReleaseLocatorCall;
-    private CallableStatement blobGetPositionFromLocatorCall;
-    private CallableStatement blobGetPositionFromBytesCall;
-    private CallableStatement blobGetLengthCall;
-    private CallableStatement blobGetBytesCall;
-    private CallableStatement blobSetBytesCall;
-    private CallableStatement blobTruncateCall;
-    private CallableStatement clobCreateLocatorCall;
-    private CallableStatement clobReleaseLocatorCall;
-    private CallableStatement clobGetPositionFromStringCall;
-    private CallableStatement clobGetPositionFromLocatorCall;
-    private CallableStatement clobGetLengthCall;
-    private CallableStatement clobGetSubStringCall;
-    private CallableStatement clobSetStringCall;
-    private CallableStatement clobTruncateCall;
+    private ClientCallableStatement blobCreateLocatorCall;
+    private ClientCallableStatement blobReleaseLocatorCall;
+    private ClientCallableStatement blobGetPositionFromLocatorCall;
+    private ClientCallableStatement blobGetPositionFromBytesCall;
+    private ClientCallableStatement blobGetLengthCall;
+    private ClientCallableStatement blobGetBytesCall;
+    private ClientCallableStatement blobSetBytesCall;
+    private ClientCallableStatement blobTruncateCall;
+    private ClientCallableStatement clobCreateLocatorCall;
+    private ClientCallableStatement clobReleaseLocatorCall;
+    private ClientCallableStatement clobGetPositionFromStringCall;
+    private ClientCallableStatement clobGetPositionFromLocatorCall;
+    private ClientCallableStatement clobGetLengthCall;
+    private ClientCallableStatement clobGetSubStringCall;
+    private ClientCallableStatement clobSetStringCall;
+    private ClientCallableStatement clobTruncateCall;
 
     /**
      * The connection to be used when calling the stored procedures.
      */
-    private final Connection connection; 
+    private final ClientConnection connection;
 
     /**
      * Max size of byte[] and String parameters to procedures
@@ -89,7 +91,7 @@ class CallableLocatorProcedures
      *
      * @param conn the connection to be used to prepare calls.
      */
-    CallableLocatorProcedures(Connection conn) 
+    CallableLocatorProcedures(ClientConnection conn)
     {
         this.connection = conn;
     }
@@ -99,7 +101,6 @@ class CallableLocatorProcedures
      * subsequent operations on this BLOB value will be stored in temporary
      * space on the server.
      *
-     * @throws org.apache.derby.client.am.SqlException
      * @return locator that identifies the created BLOB.
      */
     int blobCreateLocator() throws SqlException
@@ -117,11 +118,11 @@ class CallableLocatorProcedures
                     !blobCreateLocatorCall.openOnClient_) {
                 blobCreateLocatorCall = connection.prepareCallX
                         ("? = CALL SYSIBM.BLOBCREATELOCATOR()",
-                        java.sql.ResultSet.TYPE_FORWARD_ONLY,
-                        java.sql.ResultSet.CONCUR_READ_ONLY,
+                        ResultSet.TYPE_FORWARD_ONLY,
+                        ResultSet.CONCUR_READ_ONLY,
                         connection.holdability());
                 blobCreateLocatorCall
-                        .registerOutParameterX(1, java.sql.Types.INTEGER);
+                        .registerOutParameterX(1, Types.INTEGER);
                 // Make sure this statement does not commit user transaction
                 blobCreateLocatorCall.isAutoCommittableStatement_ = false;
             }
@@ -161,7 +162,6 @@ class CallableLocatorProcedures
      * This method frees the BLOB and releases the resources that it
      * holds. (E.g., temporary space used to store this BLOB on the server.)
      * @param locator locator that designates the BLOB to be released.
-     * @throws org.apache.derby.client.am.SqlException 
      */
     void blobReleaseLocator(int locator) throws SqlException
     {
@@ -169,9 +169,9 @@ class CallableLocatorProcedures
                 !blobReleaseLocatorCall.openOnClient_) {
             blobReleaseLocatorCall = connection.prepareCallX
                 ("CALL SYSIBM.BLOBRELEASELOCATOR(?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
             // Make sure this statement does not commit user transaction
             blobReleaseLocatorCall.isAutoCommittableStatement_ = false;
         }
@@ -195,7 +195,6 @@ class CallableLocatorProcedures
      *        search
      * @param fromPosition the position in the BLOB value
      *        at which to begin searching; the first position is 1
-     * @throws org.apache.derby.client.am.SqlException 
      * @return the position at which the pattern begins, else -1
      */
     long blobGetPositionFromLocator(int locator, 
@@ -206,11 +205,11 @@ class CallableLocatorProcedures
                 !blobGetPositionFromLocatorCall.openOnClient_) {
             blobGetPositionFromLocatorCall = connection.prepareCallX
                 ("? = CALL SYSIBM.BLOBGETPOSITIONFROMLOCATOR(?, ?, ?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
             blobGetPositionFromLocatorCall
-                .registerOutParameterX(1, java.sql.Types.BIGINT);
+                .registerOutParameterX(1, Types.BIGINT);
             // Make sure this statement does not commit user transaction
             blobGetPositionFromLocatorCall.isAutoCommittableStatement_ = false;
         }
@@ -242,7 +241,6 @@ class CallableLocatorProcedures
      * @param searchLiteral the byte array for which to search
      * @param fromPosition the position at which to begin searching; the
      *        first position is 1
-     * @throws org.apache.derby.client.am.SqlException 
      * @return the position at which the pattern appears, else -1
      */
     long blobGetPositionFromBytes(int locator, 
@@ -321,7 +319,6 @@ class CallableLocatorProcedures
      *        for. It is assumed that this length is smaller than the maximum
      *        size of a VARCHAR FOR BIT DATA column.  Otherwise, an exception
      *        will be thrown.
-     * @throws org.apache.derby.client.am.SqlException 
      * @return the position at which the pattern appears, else -1
      */
     private long blobGetPositionFromBytes(int locator,
@@ -334,11 +331,11 @@ class CallableLocatorProcedures
                 !blobGetPositionFromBytesCall.openOnClient_) {
             blobGetPositionFromBytesCall = connection.prepareCallX
                 ("? = CALL SYSIBM.BLOBGETPOSITIONFROMBYTES(?, ?, ?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
             blobGetPositionFromBytesCall
-                .registerOutParameterX(1, java.sql.Types.BIGINT);
+                .registerOutParameterX(1, Types.BIGINT);
             // Make sure this statement does not commit user transaction
             blobGetPositionFromBytesCall.isAutoCommittableStatement_ = false;
         }
@@ -370,7 +367,6 @@ class CallableLocatorProcedures
      * designated by this <code>sourceLocator</code>.
      * 
      * @param sourceLocator locator that identifies the BLOB
-     * @throws org.apache.derby.client.am.SqlException 
      * @return length of the <code>BLOB</code> in bytes 
      */
     long blobGetLength(int sourceLocator) throws SqlException
@@ -378,10 +374,10 @@ class CallableLocatorProcedures
         if (blobGetLengthCall == null || !blobGetLengthCall.openOnClient_) {
             blobGetLengthCall = connection.prepareCallX
                 ("? = CALL SYSIBM.BLOBGETLENGTH(?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
-            blobGetLengthCall.registerOutParameterX(1, java.sql.Types.BIGINT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
+            blobGetLengthCall.registerOutParameterX(1, Types.BIGINT);
             // Make sure this statement does not commit user transaction
             blobGetLengthCall.isAutoCommittableStatement_ = false;
         }
@@ -413,8 +409,7 @@ class CallableLocatorProcedures
      * @param forLength the number of consecutive bytes to be copied; the value
      *        for length must be 0 or greater.  Specifying a length that goes
      *        beyond the end of the BLOB (i.e., <code>fromPosition + forLength
-     *        > blob.length()</code>), will result in an error.
-     * @throws org.apache.derby.client.am.SqlException 
+     *        &gt; blob.length()</code>), will result in an error.
      * @return a byte array containing up to <code>forLength</code> consecutive
      *         bytes from the <code>BLOB</code> value designated by
      *         <code>sourceLocator</code>, starting with the byte at position
@@ -428,10 +423,10 @@ class CallableLocatorProcedures
         if (blobGetBytesCall == null || !blobGetBytesCall.openOnClient_) {
             blobGetBytesCall = connection.prepareCallX
                 ("? = CALL SYSIBM.BLOBGETBYTES(?, ?, ?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
-            blobGetBytesCall.registerOutParameterX(1, java.sql.Types.VARBINARY);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
+            blobGetBytesCall.registerOutParameterX(1, Types.VARBINARY);
             // Make sure this statement does not commit user transaction
             blobGetBytesCall.isAutoCommittableStatement_ = false;
         }
@@ -489,10 +484,9 @@ class CallableLocatorProcedures
      * @param forLength the number of bytes to be written to the
      *        <code>BLOB</code> value from the array of bytes
      *        <code>bytes</code>.  Specifying a length that goes beyond the end
-     *        of the BLOB (i.e., <code>fromPosition + forLength >
+     *        of the BLOB (i.e., <code>fromPosition + forLength &gt;
      *        blob.length()</code>, will result in an error.
      * @param bytes the array of bytes to be written
-     * @throws org.apache.derby.client.am.SqlException 
      */
     void blobSetBytes(int sourceLocator, 
                       long fromPosition, 
@@ -502,9 +496,9 @@ class CallableLocatorProcedures
         if (blobSetBytesCall == null || !blobSetBytesCall.openOnClient_) {
             blobSetBytesCall = connection.prepareCallX
                 ("CALL SYSIBM.BLOBSETBYTES(?, ?, ?, ?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
             // Make sure this statement does not commit user transaction
             blobSetBytesCall.isAutoCommittableStatement_ = false;
         }
@@ -551,16 +545,15 @@ class CallableLocatorProcedures
      * @param sourceLocator locator identifying the Blob to be truncated
      * @param length the length, in bytes, to which the <code>BLOB</code> value
      *        should be truncated
-     * @throws org.apache.derby.client.am.SqlException 
      */
     void blobTruncate(int sourceLocator, long length) throws SqlException
     {
         if (blobTruncateCall == null || !blobTruncateCall.openOnClient_) {
             blobTruncateCall = connection.prepareCallX
                 ("CALL SYSIBM.BLOBTRUNCATE(?, ?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
             // Make sure this statement does not commit user transaction
             blobTruncateCall.isAutoCommittableStatement_ = false;
         }
@@ -580,7 +573,6 @@ class CallableLocatorProcedures
      * subsequent operations on this CLOB value will be stored in temporary
      * space on the server.
      * 
-     * @throws org.apache.derby.client.am.SqlException 
      * @return locator that identifies the created CLOB.
      */
     int clobCreateLocator() throws SqlException
@@ -598,11 +590,11 @@ class CallableLocatorProcedures
                     !clobCreateLocatorCall.openOnClient_) {
                 clobCreateLocatorCall = connection.prepareCallX
                         ("? = CALL SYSIBM.CLOBCREATELOCATOR()",
-                        java.sql.ResultSet.TYPE_FORWARD_ONLY,
-                        java.sql.ResultSet.CONCUR_READ_ONLY,
-                        java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                        ResultSet.TYPE_FORWARD_ONLY,
+                        ResultSet.CONCUR_READ_ONLY,
+                        ResultSet.CLOSE_CURSORS_AT_COMMIT);
                 clobCreateLocatorCall
-                        .registerOutParameterX(1, java.sql.Types.INTEGER);
+                        .registerOutParameterX(1, Types.INTEGER);
                 // Make sure this statement does not commit user transaction
                 clobCreateLocatorCall.isAutoCommittableStatement_ = false;
             }
@@ -642,7 +634,6 @@ class CallableLocatorProcedures
      * This method frees the CLOB and releases the resources that it
      * holds. (E.g., temporary space used to store this CLOB on the server.)
      * @param locator locator that designates the CLOB to be released.
-     * @throws org.apache.derby.client.am.SqlException 
      */
     void clobReleaseLocator(int locator) throws SqlException
     {
@@ -650,9 +641,9 @@ class CallableLocatorProcedures
                 !clobReleaseLocatorCall.openOnClient_) {
             clobReleaseLocatorCall = connection.prepareCallX
                 ("CALL SYSIBM.CLOBRELEASELOCATOR(?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
             // Make sure this statement does not commit user transaction
             clobReleaseLocatorCall.isAutoCommittableStatement_ = false;
         }
@@ -681,7 +672,6 @@ class CallableLocatorProcedures
      * @param searchLiteral the substring for which to search
      * @param fromPosition the position at which to begin searching; the
      *        first position is 1
-     * @throws org.apache.derby.client.am.SqlException 
      * @return the position at which the pattern appears, else -1
      */
     long clobGetPositionFromString(int locator, 
@@ -757,7 +747,6 @@ class CallableLocatorProcedures
      *        for. It is assumed that this length is smaller than the maximum
      *        size of a VARCHAR column.  Otherwise, an exception will be
      *        thrown.
-     * @throws org.apache.derby.client.am.SqlException 
      * @return the position at which the pattern appears, else -1
      */
     private long clobGetPositionFromString(int locator, 
@@ -770,11 +759,11 @@ class CallableLocatorProcedures
                 !clobGetPositionFromStringCall.openOnClient_) {
             clobGetPositionFromStringCall = connection.prepareCallX
                 ("? = CALL SYSIBM.CLOBGETPOSITIONFROMSTRING(?, ?, ?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
             clobGetPositionFromStringCall
-                .registerOutParameterX(1, java.sql.Types.BIGINT);
+                .registerOutParameterX(1, Types.BIGINT);
             // Make sure this statement does not commit user transaction
             clobGetPositionFromStringCall.isAutoCommittableStatement_ = false;
         }
@@ -809,7 +798,6 @@ class CallableLocatorProcedures
      *        search
      * @param fromPosition the position in the CLOB value
      *        at which to begin searching; the first position is 1
-     * @throws org.apache.derby.client.am.SqlException 
      * @return the position at which the pattern begins, else -1
      */
     long clobGetPositionFromLocator(int locator, 
@@ -820,11 +808,11 @@ class CallableLocatorProcedures
                 !clobGetPositionFromLocatorCall.openOnClient_) {
             clobGetPositionFromLocatorCall = connection.prepareCallX
                 ("? = CALL SYSIBM.CLOBGETPOSITIONFROMLOCATOR(?, ?, ?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
             clobGetPositionFromLocatorCall
-                .registerOutParameterX(1, java.sql.Types.BIGINT);
+                .registerOutParameterX(1, Types.BIGINT);
             // Make sure this statement does not commit user transaction
             clobGetPositionFromLocatorCall.isAutoCommittableStatement_ = false;
         }
@@ -846,7 +834,6 @@ class CallableLocatorProcedures
      * designated by this <code>sourceLocator</code>.
      * 
      * @param sourceLocator locator that identifies the CLOB
-     * @throws org.apache.derby.client.am.SqlException 
      * @return length of the <code>CLOB</code> in characters 
      */
     long clobGetLength(int sourceLocator) throws SqlException
@@ -854,10 +841,10 @@ class CallableLocatorProcedures
         if (clobGetLengthCall == null || !clobGetLengthCall.openOnClient_) {
             clobGetLengthCall = connection.prepareCallX
                 ("? = CALL SYSIBM.CLOBGETLENGTH(?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
-            clobGetLengthCall.registerOutParameterX(1, java.sql.Types.BIGINT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
+            clobGetLengthCall.registerOutParameterX(1, Types.BIGINT);
             // Make sure this statement does not commit user transaction
             clobGetLengthCall.isAutoCommittableStatement_ = false;
         }
@@ -889,8 +876,7 @@ class CallableLocatorProcedures
      * @param forLength the number of consecutive characters to be copied; the
      *        value for length must be 0 or greater.  Specifying a length that
      *        goes beyond the end of the CLOB (i.e., <code>fromPosition +
-     *        forLength > clob.length()</code>, will result in an error.
-     * @throws org.apache.derby.client.am.SqlException 
+     *        forLength &gt; clob.length()</code>, will result in an error.
      * @return a string containing up to <code>forLength</code> consecutive
      *         characters from the <code>CLOB</code> value designated by
      *         <code>sourceLocator</code>, starting with the character at
@@ -905,11 +891,11 @@ class CallableLocatorProcedures
                 !clobGetSubStringCall.openOnClient_) {
             clobGetSubStringCall = connection.prepareCallX
                 ("? = CALL SYSIBM.CLOBGETSUBSTRING(?, ?, ?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
             clobGetSubStringCall
-                .registerOutParameterX(1, java.sql.Types.VARCHAR);
+                .registerOutParameterX(1, Types.VARCHAR);
             // Make sure this statement does not commit user transaction
             clobGetSubStringCall.isAutoCommittableStatement_ = false;
         }
@@ -953,25 +939,24 @@ class CallableLocatorProcedures
      * <code>CLOB</code> value designated by <code>sourceLocator</code>.
      * Writing starts at position <code>fromPosition</code> in the
      * <code>CLOB</code> value; <code>forLength</code> characters from the
-     * given string are written. If the end of the <code>Clob</code> value is
+     * given string are written. If the end of the <code>CLOB</code> value is
      * reached while writing the string, then the length of the
-     * <code>Clob</code> value will be increased to accomodate the extra
+     * <code>CLOB</code> value will be increased to accomodate the extra
      * characters.
      * <p> 
      * If <code>forLength</code> is larger than the maximum length of a
      * VARCHAR, the writing to the CLOB value will be split into repeated
      * procedure calls.
      *
-     * @param sourceLocator locator that identifies the Clob to operated on
+     * @param sourceLocator locator that identifies the CLOB to operated on
      * @param fromPosition the position in the <code>CLOB</code> value at which
      *        to start writing; the first position is 1
      * @param forLength the number of characters to be written to the
      *        <code>CLOB</code> value from the string <code>string</code>.
      *        Specifying a length that goes beyond the end of the CLOB (i.e.,
-     *        <code>fromPosition + forLength > clob.length()</code>, will
+     *        <code>fromPosition + forLength &gt; clob.length()</code>, will
      *        result in an error.
      * @param string the string to be written
-     * @throws org.apache.derby.client.am.SqlException 
      */
     void clobSetString(int sourceLocator, 
                        long fromPosition, 
@@ -981,9 +966,9 @@ class CallableLocatorProcedures
         if (clobSetStringCall == null || !clobSetStringCall.openOnClient_) {
             clobSetStringCall = connection.prepareCallX
                 ("CALL SYSIBM.CLOBSETSTRING(?, ?, ?, ?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
             // Make sure this statement does not commit user transaction
             clobSetStringCall.isAutoCommittableStatement_ = false;
         }
@@ -1023,19 +1008,18 @@ class CallableLocatorProcedures
      * than the length+1 of the <code>CLOB</code> value then an
      * <code>SqlException</code> will be thrown.
      * 
-     * @param sourceLocator locator identifying the Clob to be truncated
+     * @param sourceLocator locator identifying the CLOB to be truncated
      * @param length the length, in characters, to which the <code>CLOB</code>
      *        value should be truncated
-     * @throws org.apache.derby.client.am.SqlException 
      */
     void clobTruncate(int sourceLocator, long length) throws SqlException
     {
         if (clobTruncateCall == null || !clobTruncateCall.openOnClient_) {
             clobTruncateCall = connection.prepareCallX
                 ("CALL SYSIBM.CLOBTRUNCATE(?, ?)",
-                 java.sql.ResultSet.TYPE_FORWARD_ONLY, 
-                 java.sql.ResultSet.CONCUR_READ_ONLY, 
-                 java.sql.ResultSet.CLOSE_CURSORS_AT_COMMIT);
+                 ResultSet.TYPE_FORWARD_ONLY,
+                 ResultSet.CONCUR_READ_ONLY,
+                 ResultSet.CLOSE_CURSORS_AT_COMMIT);
             // Make sure this statement does not commit user transaction
             clobTruncateCall.isAutoCommittableStatement_ = false;
         }
