@@ -21,8 +21,25 @@
 
 package org.apache.derby.client.am;
 
+import java.sql.Array;
+import java.sql.Blob;
+import java.sql.CallableStatement;
+import java.sql.Clob;
+import java.sql.Connection;
+import java.sql.DatabaseMetaData;
+import java.sql.NClob;
+import java.sql.PreparedStatement;
+import java.sql.SQLClientInfoException;
 import java.sql.SQLException;
+import java.sql.SQLWarning;
+import java.sql.SQLXML;
+import java.sql.Savepoint;
+import java.sql.Statement;
+import java.sql.Struct;
 import java.util.Map;
+import java.util.Properties;
+import java.util.concurrent.Executor;
+import org.apache.derby.client.ClientPooledConnection;
 import org.apache.derby.shared.common.reference.SQLState;
 
 /**
@@ -38,14 +55,14 @@ import org.apache.derby.shared.common.reference.SQLState;
  * nulled out, only the {@code PooledConnection} instance will maintain a
  * handle to the physical connection.
  */
-public class LogicalConnection implements java.sql.Connection {
+public class LogicalConnection implements Connection {
     /**
      * Underlying physical connection for this logical connection.
      * <p>
      * Set to {@code null} when this logical connection is closed.
      */
-    Connection physicalConnection_;
-    private org.apache.derby.client.ClientPooledConnection pooledConnection_ = null;
+    ClientConnection physicalConnection_;
+    private ClientPooledConnection pooledConnection_ = null;
     /**
      * Logical database metadata object created on demand and then cached.
      * The lifetime of the metadata object is the same as this logical
@@ -54,8 +71,10 @@ public class LogicalConnection implements java.sql.Connection {
      */
     private LogicalDatabaseMetaData logicalDatabaseMetaData = null;
 
-    public LogicalConnection(Connection physicalConnection,
-                             org.apache.derby.client.ClientPooledConnection pooledConnection) throws SqlException {
+    public LogicalConnection(
+            ClientConnection physicalConnection,
+            ClientPooledConnection pooledConnection) throws SqlException {
+
         physicalConnection_ = physicalConnection;
         pooledConnection_ = pooledConnection;
         try {
@@ -65,7 +84,12 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    protected void finalize() throws java.lang.Throwable {
+    //
+    // This method in java.lang.Object was deprecated as of build 167
+    // of JDK 9. See DERBY-6932.
+    //
+    @SuppressWarnings("deprecation")
+    protected void finalize() throws Throwable {
         close();
     }
 
@@ -85,33 +109,33 @@ public class LogicalConnection implements java.sql.Connection {
         synchronized (pooledConnection_) {
             synchronized (this) {
                 try {
-            // we also need to loop thru all the logicalStatements and close them
-            if (physicalConnection_ == null) {
-                return;
-            }
-            if (physicalConnection_.agent_.loggingEnabled()) {
-                physicalConnection_.agent_.logWriter_.traceEntry(this, "close");
-            }
+                    // we also need to loop thru all the logicalStatements and close them
+                    if (physicalConnection_ == null) {
+                        return;
+                    }
+                    if (physicalConnection_.agent_.loggingEnabled()) {
+                        physicalConnection_.agent_.logWriter_.traceEntry(this, "close");
+                    }
 
-            if (physicalConnection_.isClosed()) // connection is closed or has become stale
-            {
+                    if (physicalConnection_.isClosed()) // connection is closed or has become stale
+                    {
                         pooledConnection_.informListeners(new SqlException(null,
-                    new ClientMessageId(
-                        SQLState.PHYSICAL_CONNECTION_ALREADY_CLOSED)));
-            } else {
-                physicalConnection_.checkForTransactionInProgress();
-                physicalConnection_.closeForReuse(
-                        pooledConnection_.isStatementPoolingEnabled());
-                if (!physicalConnection_.isGlobalPending_()) {
-                    pooledConnection_.recycleConnection();
+                                new ClientMessageId(
+                                SQLState.PHYSICAL_CONNECTION_ALREADY_CLOSED)));
+                    } else {
+                        physicalConnection_.checkForTransactionInProgress();
+                        physicalConnection_.closeForReuse(
+                                pooledConnection_.isStatementPoolingEnabled());
+                        if (!physicalConnection_.isGlobalPending_()) {
+                            pooledConnection_.recycleConnection();
+                        }
+                    }
+                    physicalConnection_ = null;
+                    pooledConnection_.nullLogicalConnection();
+                } catch (SqlException se) {
+                    throw se.getSQLException();
                 }
             }
-            physicalConnection_ = null;
-            pooledConnection_.nullLogicalConnection();
-                } catch (SqlException se) {
-            throw se.getSQLException();
-        }
-    }
         }
     }
 
@@ -180,7 +204,7 @@ public class LogicalConnection implements java.sql.Connection {
     // ---------------------- wrapped public entry points ------------------------
     // All methods are forwarded to the physical connection in a standard way
 
-    synchronized public java.sql.Statement createStatement() throws SQLException {
+    synchronized public Statement createStatement() throws SQLException {
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.createStatement();
@@ -190,7 +214,8 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.PreparedStatement prepareStatement(String sql) throws SQLException {
+    synchronized public PreparedStatement prepareStatement(String sql)
+            throws SQLException {
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.prepareStatement(sql);
@@ -200,16 +225,8 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public PreparedStatement preparePositionedUpdateStatement(String sql, Section querySection) throws SqlException {
-        try {
-            checkForNullPhysicalConnection();
-        } catch ( SQLException se ) {
-            throw new SqlException(se);
-        }
-        return physicalConnection_.preparePositionedUpdateStatement(sql, querySection);
-    }
-
-    synchronized public java.sql.CallableStatement prepareCall(String sql) throws SQLException {
+    synchronized public CallableStatement prepareCall(String sql)
+            throws SQLException {
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.prepareCall(sql);
@@ -289,7 +306,7 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.SQLWarning getWarnings() throws SQLException {
+    synchronized public SQLWarning getWarnings() throws SQLException {
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.getWarnings();
@@ -324,7 +341,7 @@ public class LogicalConnection implements java.sql.Connection {
      * @return A database metadata object.
      * @throws SQLException if an error occurs
      */
-    public synchronized java.sql.DatabaseMetaData getMetaData()
+    public synchronized DatabaseMetaData getMetaData()
             throws SQLException {
         try {
             checkForNullPhysicalConnection();
@@ -359,7 +376,7 @@ public class LogicalConnection implements java.sql.Connection {
      * @return The metadata object from the underlying physical connection.
      * @throws SQLException if the logical connection has been closed
      */
-    final synchronized java.sql.DatabaseMetaData getRealMetaDataObject()
+    final synchronized DatabaseMetaData getRealMetaDataObject()
             throws SQLException {
         // Check if the logical connection has been closed.
         // isClosed also checks if physicalConnection_ is null.
@@ -415,7 +432,7 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.Statement createStatement(int resultSetType,
+    synchronized public Statement createStatement(int resultSetType,
                                                            int resultSetConcurrency) throws SQLException {
         try {
             checkForNullPhysicalConnection();
@@ -426,7 +443,7 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.PreparedStatement prepareStatement(String sql,
+    synchronized public PreparedStatement prepareStatement(String sql,
                                                                     int resultSetType,
                                                                     int resultSetConcurrency) throws SQLException {
         try {
@@ -438,7 +455,7 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.CallableStatement prepareCall(String sql,
+    synchronized public CallableStatement prepareCall(String sql,
                                                                int resultSetType,
                                                                int resultSetConcurrency) throws SQLException {
         try {
@@ -470,8 +487,11 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.Statement createStatement(int resultSetType, int resultSetConcurrency,
-                                              int resultSetHoldability) throws SQLException {
+    synchronized public Statement createStatement(
+            int resultSetType,
+            int resultSetConcurrency,
+            int resultSetHoldability) throws SQLException {
+
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability);
@@ -481,9 +501,12 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.CallableStatement prepareCall(String sql, int resultSetType,
-                                                  int resultSetConcurrency,
-                                                  int resultSetHoldability) throws SQLException {
+    synchronized public CallableStatement prepareCall(
+            String sql,
+            int resultSetType,
+            int resultSetConcurrency,
+            int resultSetHoldability) throws SQLException {
+
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
@@ -493,9 +516,12 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.PreparedStatement prepareStatement(String sql, int resultSetType,
-                                                       int resultSetConcurrency, int resultSetHoldability)
-            throws SQLException {
+    synchronized public PreparedStatement prepareStatement(
+            String sql,
+            int resultSetType,
+            int resultSetConcurrency,
+            int resultSetHoldability) throws SQLException {
+
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.prepareStatement(sql, resultSetType, resultSetConcurrency,
@@ -506,8 +532,10 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.PreparedStatement prepareStatement(String sql, int autoGeneratedKeys)
-            throws SQLException {
+    synchronized public PreparedStatement prepareStatement(
+            String sql,
+            int autoGeneratedKeys) throws SQLException {
+
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.prepareStatement(sql, autoGeneratedKeys);
@@ -517,8 +545,10 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.PreparedStatement prepareStatement(String sql, int columnIndexes[])
-            throws SQLException {
+    synchronized public PreparedStatement prepareStatement(
+            String sql,
+            int columnIndexes[]) throws SQLException {
+
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.prepareStatement(sql, columnIndexes);
@@ -528,8 +558,10 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.PreparedStatement prepareStatement(String sql, String columnNames[])
-            throws SQLException {
+    synchronized public PreparedStatement prepareStatement(
+            String sql,
+            String columnNames[]) throws SQLException {
+
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.prepareStatement(sql, columnNames);
@@ -559,7 +591,7 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.Savepoint setSavepoint() throws SQLException {
+    synchronized public Savepoint setSavepoint() throws SQLException {
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.setSavepoint();
@@ -569,7 +601,8 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public java.sql.Savepoint setSavepoint(String name) throws SQLException {
+    synchronized public Savepoint setSavepoint(String name)
+            throws SQLException {
         try {
             checkForNullPhysicalConnection();
             return physicalConnection_.setSavepoint(name);
@@ -579,7 +612,7 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public void rollback(java.sql.Savepoint savepoint) throws SQLException {
+    synchronized public void rollback(Savepoint savepoint) throws SQLException {
         try {
             checkForNullPhysicalConnection();
             physicalConnection_.rollback(savepoint);
@@ -589,7 +622,8 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
-    synchronized public void releaseSavepoint(java.sql.Savepoint savepoint) throws SQLException {
+    synchronized public void releaseSavepoint(Savepoint savepoint)
+            throws SQLException {
         try {
             checkForNullPhysicalConnection();
             physicalConnection_.releaseSavepoint(savepoint);
@@ -623,6 +657,241 @@ public class LogicalConnection implements java.sql.Connection {
         }
     }
 
+    ////////////////////////////////////////////////////////////////////
+    //
+    // INTRODUCED BY JDBC 4.0 IN JAVA 6
+    //
+    ////////////////////////////////////////////////////////////////////
+    public Array createArrayOf(String typeName, Object[] elements)
+        throws SQLException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            return physicalConnection_.createArrayOf( typeName, elements );
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+    public Blob createBlob()
+        throws SQLException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            return physicalConnection_.createBlob();
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+    public Clob createClob()
+        throws SQLException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            return physicalConnection_.createClob();
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+    public NClob createNClob()
+        throws SQLException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            return physicalConnection_.createNClob();
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+    public SQLXML createSQLXML()
+        throws SQLException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            return physicalConnection_.createSQLXML();
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+    public Struct createStruct(String typeName, Object[] attributes)
+        throws SQLException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            return physicalConnection_.createStruct( typeName, attributes );
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+    /**
+     * <code>getClientInfo</code> forwards to
+     * <code>physicalConnection_</code>.
+     * <code>getClientInfo</code> always returns an empty
+     * <code>Properties</code> object since Derby doesn't support
+     * ClientInfoProperties.
+     *
+     * @return an empty <code>Properties</code> object
+     * @exception SQLException if an error occurs
+     */
+    public Properties getClientInfo()
+        throws SQLException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            return physicalConnection_.getClientInfo();
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+    /**
+     * <code>getClientInfo</code> forwards to
+     * <code>physicalConnection_</code>. Always returns a <code>null
+     * String</code> since Derby does not support
+     * ClientInfoProperties.
+     *
+     * @param name a property key to get <code>String</code>
+     * @return a property value <code>String</code>
+     * @exception SQLException if an error occurs
+     */
+    public String getClientInfo(String name)
+        throws SQLException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            return physicalConnection_.getClientInfo(name);
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+    /**
+     * Checks if the connection has not been closed and is still valid.
+     * The validity is checked by running a simple query against the
+     * database.
+     *
+     * @param timeout The time in seconds to wait for the database
+     * operation used to validate the connection to complete. If the
+     * timeout period expires before the operation completes, this
+     * method returns false. A value of 0 indicates a timeout is not
+     * applied to the database operation.
+     * @return true if the connection is valid, false otherwise
+     * @throws SQLException if the call on the physical connection throws an
+     * exception.
+     */
+    synchronized public boolean isValid(int timeout) throws SQLException {
+        try
+        {
+            // Check if we have a underlying physical connection
+            if (physicalConnection_ == null) {
+                return false;
+            }
+            return physicalConnection_.isValid(timeout);
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+
+    public boolean isWrapperFor(Class<?> interfaces)
+        throws SQLException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            return interfaces.isInstance(this);
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+    /**
+     * <code>setClientInfo</code> forwards to
+     * <code>physicalConnection_</code>.
+     *
+     * @param properties a <code>Properties</code> object with the
+     * properties to set
+     * @exception SQLClientInfoException if an error occurs
+     */
+    public void setClientInfo(Properties properties)
+        throws SQLClientInfoException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            physicalConnection_.setClientInfo(properties);
+        } catch (SQLClientInfoException cie) {
+            notifyException(cie);
+            throw cie;
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw new SQLClientInfoException
+            (sqle.getMessage(), sqle.getSQLState(),
+                    sqle.getErrorCode(),
+                    (new FailedProperties40(properties)).getProperties());
+        }
+    }
+
+    /**
+     * <code>setClientInfo</code> forwards to
+     * <code>physicalConnection_</code>.
+     *
+     * @param name a property key <code>String</code>
+     * @param value a property value <code>String</code>
+     * @exception SQLClientInfoException if an error occurs
+     */
+    public void setClientInfo(String name, String value)
+        throws SQLClientInfoException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            physicalConnection_.setClientInfo(name, value);
+        } catch (SQLClientInfoException cie) {
+            notifyException(cie);
+            throw cie;
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw new SQLClientInfoException
+            (sqle.getMessage(), sqle.getSQLState(),
+                    sqle.getErrorCode(),
+             new FailedProperties40
+             (FailedProperties40.makeProperties
+              (name,value)).getProperties());
+        }
+    }
+
+    public <T>T unwrap(Class<T> interfaces)
+        throws SQLException {
+        try
+        {
+            checkForNullPhysicalConnection();
+            // Derby does not implement non-standard methods on JDBC objects
+            try {
+                return interfaces.cast(this);
+            } catch (ClassCastException cce) {
+                throw new SqlException(null,
+                                       new ClientMessageId(SQLState.UNABLE_TO_UNWRAP),
+                                       interfaces).getSQLException();
+            }
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
     ////////////////////////////////////////////////////////////////////
     //
     // INTRODUCED BY JDBC 4.1 IN JAVA 7
@@ -656,5 +925,43 @@ public class LogicalConnection implements java.sql.Connection {
             throw sqle;
         }
     }
-    
+
+    public  void    abort( Executor executor )  throws SQLException
+    {
+        try
+        {
+            if ( physicalConnection_ != null )
+            {
+                physicalConnection_.abort( executor );
+            }
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+    public int getNetworkTimeout() throws SQLException
+    {
+        try
+        {
+            checkForNullPhysicalConnection();
+            return physicalConnection_.getNetworkTimeout();
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
+    public void setNetworkTimeout( Executor executor, int milliseconds ) throws SQLException
+    {
+        try
+        {
+            checkForNullPhysicalConnection();
+            physicalConnection_.setNetworkTimeout( executor, milliseconds );
+        } catch (SQLException sqle) {
+            notifyException(sqle);
+            throw sqle;
+        }
+    }
+
 }
