@@ -21,33 +21,38 @@
 
 package org.apache.derby.client.net;
 
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
+import java.net.Socket;
 import java.net.SocketException;
+import java.security.AccessController;
+import java.security.PrivilegedActionException;
 
 import org.apache.derby.client.am.Agent;
 import org.apache.derby.client.am.DisconnectException;
 import org.apache.derby.client.am.SqlException;
 import org.apache.derby.client.am.ClientMessageId;
+import org.apache.derby.client.am.ClientStatement;
+import org.apache.derby.client.am.LogWriter;
 import org.apache.derby.client.am.Utils;
 import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.shared.common.reference.SQLState;
-import org.apache.derby.shared.common.reference.MessageId;
-import org.apache.derby.shared.common.i18n.MessageUtil;
 
 public class NetAgent extends Agent {
     //---------------------navigational members-----------------------------------
 
     // All these request objects point to the same physical request object.
-    public ConnectionRequestInterface connectionRequest_;
-    public NetConnectionRequest packageRequest_;
-    public StatementRequestInterface statementRequest_;
-    public ResultSetRequestInterface resultSetRequest_;
+    ConnectionRequestInterface connectionRequest_;
+    StatementRequestInterface statementRequest_;
+    ResultSetRequestInterface resultSetRequest_;
 
     // All these reply objects point to the same physical reply object.
-    public ConnectionReply connectionReply_;
-    public ConnectionReply packageReply_;
-    public StatementReply statementReply_;
-    public ResultSetReply resultSetReply_;
+    ConnectionReply connectionReply_;
+    private ConnectionReply packageReply_;
+    StatementReply statementReply_;
+    ResultSetReply resultSetReply_;
 
     //---------------------navigational cheat-links-------------------------------
     // Cheat-links are for convenience only, and are not part of the conceptual model.
@@ -60,29 +65,29 @@ public class NetAgent extends Agent {
 
     // Alias for (Request) super.*Request, all in one
     // In the case of the NET implementation, these all point to the same physical request object.
-    protected Request request_;
-    public NetConnectionRequest netConnectionRequest_;
-    public NetPackageRequest netPackageRequest_;
-    public NetStatementRequest netStatementRequest_;
-    public NetResultSetRequest netResultSetRequest_;
+    private Request request_;
+    NetConnectionRequest netConnectionRequest_;
+    private NetPackageRequest netPackageRequest_;
+    private NetStatementRequest netStatementRequest_;
+    private NetResultSetRequest netResultSetRequest_;
 
     // Alias for (Reply) super.*Reply, all in one.
     // In the case of the NET implementation, these all point to the same physical reply object.
-    protected Reply reply_;
-    public NetConnectionReply netConnectionReply_;
-    public NetPackageReply netPackageReply_;
-    public NetStatementReply netStatementReply_;
-    public NetResultSetReply netResultSetReply_;
+    private Reply reply_;
+    NetConnectionReply netConnectionReply_;
+    private NetPackageReply netPackageReply_;
+    private NetStatementReply netStatementReply_;
+    private NetResultSetReply netResultSetReply_;
 
     //-----------------------------state------------------------------------------
 
-    java.net.Socket socket_;
-    java.io.InputStream rawSocketInputStream_;
-    java.io.OutputStream rawSocketOutputStream_;
+    Socket socket_;
+    private InputStream rawSocketInputStream_;
+    private OutputStream rawSocketOutputStream_;
 
     String server_;
     int port_;
-    int clientSSLMode_;
+    private int clientSSLMode_;
 
     private EbcdicCcsidManager ebcdicCcsidManager_;
     private Utf8CcsidManager utf8CcsidManager_;
@@ -90,27 +95,36 @@ public class NetAgent extends Agent {
     
     // TODO: Remove target? Keep just one CcsidManager?
     //public CcsidManager targetCcsidManager_;
-    public Typdef typdef_;
-    public Typdef targetTypdef_;
-    public Typdef originalTargetTypdef_; // added to support typdef overrides
+    Typdef typdef_;
+    Typdef targetTypdef_;
+    Typdef originalTargetTypdef_; // added to support typdef overrides
 
-    protected int svrcod_;
+    private int svrcod_;
 
-    public int orignalTargetSqlam_ = NetConfiguration.MGRLVL_7;
-    public int targetSqlam_ = orignalTargetSqlam_;
+    int orignalTargetSqlam_ = NetConfiguration.MGRLVL_7;
+    int targetSqlam_ = orignalTargetSqlam_;
 
-    public SqlException exceptionOpeningSocket_ = null;
-    public SqlException exceptionConvertingRdbnam = null;
+    SqlException exceptionOpeningSocket_ = null;
+    SqlException exceptionConvertingRdbnam = null;
     
+    /**
+     * Flag which indicates that a writeChain has been started and data sent to
+     * the server.
+     * If true, starting a new write chain will throw a DisconnectException. 
+     * It is cleared when the write chain is ended.
+     */
+    private boolean writeChainIsDirty_ = false;
     //---------------------constructors/finalizer---------------------------------
+
+    // Only used for testing
     public NetAgent(NetConnection netConnection,
-                    org.apache.derby.client.am.LogWriter logWriter) throws SqlException {
+                    LogWriter logWriter) throws SqlException {
         super(netConnection, logWriter);
         this.netConnection_ = netConnection;
     }
 
     NetAgent(NetConnection netConnection,
-             org.apache.derby.client.am.LogWriter netLogWriter,
+             LogWriter netLogWriter,
              int loginTimeout,
              String server,
              int port,
@@ -129,13 +143,14 @@ public class NetAgent extends Agent {
         }
 
         try {
-            socket_ = (java.net.Socket) java.security.AccessController.doPrivileged(new OpenSocketAction(server, port, clientSSLMode_));
-        } catch (java.security.PrivilegedActionException e) {
+            socket_ = (Socket)AccessController.doPrivileged(
+                new OpenSocketAction(server, port, clientSSLMode_));
+        } catch (PrivilegedActionException e) {
             throw new DisconnectException(this,
                 new ClientMessageId(SQLState.CONNECT_UNABLE_TO_CONNECT_TO_SERVER),
-                new Object[] { e.getException().getClass().getName(), server, 
-                    Integer.toString(port), e.getException().getMessage() },
-                e.getException());
+                e.getException(),
+                e.getException().getClass().getName(), server, port,
+                e.getException().getMessage());
         }
 
         // Set TCP/IP Socket Properties
@@ -145,14 +160,14 @@ public class NetAgent extends Agent {
                 socket_.setKeepAlive(true); // PROTOCOL Manual: TCP/IP connection allocation rule #2
                 socket_.setSoTimeout(loginTimeout * 1000);
             }
-        } catch (java.net.SocketException e) {
+        } catch (SocketException e) {
             try {
                 socket_.close();
-            } catch (java.io.IOException doNothing) {
+            } catch (IOException doNothing) {
             }
             exceptionOpeningSocket_ = new DisconnectException(this,
                 new ClientMessageId(SQLState.CONNECT_SOCKET_EXCEPTION),
-                e.getMessage(), e);
+                e, e.getMessage());
         }
 
         try {
@@ -160,14 +175,14 @@ public class NetAgent extends Agent {
                 rawSocketOutputStream_ = socket_.getOutputStream();
                 rawSocketInputStream_ = socket_.getInputStream();
             }
-        } catch (java.io.IOException e) {
+        } catch (IOException e) {
             try {
                 socket_.close();
-            } catch (java.io.IOException doNothing) {
+            } catch (IOException doNothing) {
             }
             exceptionOpeningSocket_ = new DisconnectException(this, 
                 new ClientMessageId(SQLState.CONNECT_UNABLE_TO_OPEN_SOCKET_STREAM),
-                e.getMessage(), e);
+                e, e.getMessage());
         }
 
         ebcdicCcsidManager_ = new EbcdicCcsidManager();
@@ -199,7 +214,6 @@ public class NetAgent extends Agent {
 
             resultSetRequest_ = (ResultSetRequestInterface) netResultSetRequest_;
             statementRequest_ = (StatementRequestInterface) netStatementRequest_;
-            packageRequest_ = (NetConnectionRequest) netPackageRequest_;
             connectionRequest_ = (ConnectionRequestInterface) netConnectionRequest_;
         } else {
             netResultSetReply_ = new NetResultSetReply(this, netConnection_.commBufferSize_);
@@ -223,12 +237,11 @@ public class NetAgent extends Agent {
 
             resultSetRequest_ = (ResultSetRequestInterface) netResultSetRequest_;
             statementRequest_ = (StatementRequestInterface) netStatementRequest_;
-            packageRequest_ = (NetConnectionRequest) netPackageRequest_;
             connectionRequest_ = (ConnectionRequestInterface) netConnectionRequest_;
         }
     }
 
-    protected void resetAgent_(org.apache.derby.client.am.LogWriter netLogWriter,
+    protected void resetAgent_(LogWriter netLogWriter,
                                //CcsidManager sourceCcsidManager,
                                //CcsidManager targetCcsidManager,
                                int loginTimeout,
@@ -243,14 +256,14 @@ public class NetAgent extends Agent {
         // Set TCP/IP Socket Properties
         try {
             socket_.setSoTimeout(loginTimeout * 1000);
-        } catch (java.net.SocketException e) {
+        } catch (SocketException e) {
             try {
                 socket_.close();
-            } catch (java.io.IOException doNothing) {
+            } catch (IOException doNothing) {
             }
             throw new SqlException(logWriter_, 
                 new ClientMessageId(SQLState.SOCKET_EXCEPTION),
-                e.getMessage(), e);
+                e, e.getMessage());
         }
     }
 
@@ -265,7 +278,7 @@ public class NetAgent extends Agent {
         svrcod_ = CodePoint.SVRCOD_INFO;
     }
 
-    int getSvrcod() {
+    private int getSvrcod() {
         return svrcod_;
     }
 
@@ -281,14 +294,14 @@ public class NetAgent extends Agent {
         if (rawSocketInputStream_ != null) {
             try {
                 rawSocketInputStream_.close();
-            } catch (java.io.IOException e) {
+            } catch (IOException e) {
                 // note when {6} = 0 it indicates the socket was closed.
                 // this should be ok since we are going to go an close the socket
                 // immediately following this call.
                 // changing {4} to e.getMessage() may require pub changes
                 accumulatedExceptions = new SqlException(logWriter_,
                     new ClientMessageId(SQLState.COMMUNICATION_ERROR),
-                    e.getMessage(), e);
+                    e, e.getMessage());
             } finally {
                 rawSocketInputStream_ = null;
             }
@@ -297,14 +310,14 @@ public class NetAgent extends Agent {
         if (rawSocketOutputStream_ != null) {
             try {
                 rawSocketOutputStream_.close();
-            } catch (java.io.IOException e) {
+            } catch (IOException e) {
                 // note when {6} = 0 it indicates the socket was closed.
                 // this should be ok since we are going to go an close the socket
                 // immediately following this call.
                 // changing {4} to e.getMessage() may require pub changes
                 SqlException latestException = new SqlException(logWriter_,
                     new ClientMessageId(SQLState.COMMUNICATION_ERROR),
-                    e.getMessage(), e);
+                    e, e.getMessage());
                 accumulatedExceptions = Utils.accumulateSQLException(latestException, accumulatedExceptions);
             } finally {
                 rawSocketOutputStream_ = null;
@@ -314,14 +327,14 @@ public class NetAgent extends Agent {
         if (socket_ != null) {
             try {
                 socket_.close();
-            } catch (java.io.IOException e) {
+            } catch (IOException e) {
                 // again {6} = 0, indicates the socket was closed.
                 // maybe set {4} to e.getMessage().
                 // do this for now and but may need to modify or
                 // add this to the message pubs.
                 SqlException latestException = new SqlException(logWriter_,
                     new ClientMessageId(SQLState.COMMUNICATION_ERROR),
-                        e.getMessage(), e);
+                        e, e.getMessage());
                 accumulatedExceptions = Utils.accumulateSQLException(latestException, accumulatedExceptions);
             } finally {
                 socket_ = null;
@@ -388,15 +401,15 @@ public class NetAgent extends Agent {
         return timeout;
     }
 
-    protected void sendRequest() throws DisconnectException {
+    private void sendRequest() throws DisconnectException {
         try {
             request_.flush(rawSocketOutputStream_);
-        } catch (java.io.IOException e) {
+        } catch (IOException e) {
             throwCommunicationsFailure(e);
         }
     }
 
-    public java.io.InputStream getInputStream() {
+    public InputStream getInputStream() {
         return rawSocketInputStream_;
     }
 
@@ -404,37 +417,32 @@ public class NetAgent extends Agent {
         return currentCcsidManager_;
     }
     
-    public java.io.OutputStream getOutputStream() {
+    public OutputStream getOutputStream() {
         return rawSocketOutputStream_;
     }
 
-    void setInputStream(java.io.InputStream inputStream) {
+    void setInputStream(InputStream inputStream) {
         rawSocketInputStream_ = inputStream;
     }
 
-    void setOutputStream(java.io.OutputStream outputStream) {
+    void setOutputStream(OutputStream outputStream) {
         rawSocketOutputStream_ = outputStream;
     }
 
-    public void throwCommunicationsFailure(Throwable cause) 
-        throws org.apache.derby.client.am.DisconnectException {
-        //org.apache.derby.client.am.DisconnectException
+    void throwCommunicationsFailure(Throwable cause)
+        throws DisconnectException {
+        //DisconnectException
         //accumulateReadExceptionAndDisconnect
         // note when {6} = 0 it indicates the socket was closed.
         // need to still validate any token values against message publications.
         accumulateChainBreakingReadExceptionAndThrow(
-            new org.apache.derby.client.am.DisconnectException(this,
+            new DisconnectException(this,
                 new ClientMessageId(SQLState.COMMUNICATION_ERROR),
-                cause.getMessage(), cause));
+                cause, cause.getMessage()));
     }
         
     // ----------------------- call-down methods ---------------------------------
 
-    public org.apache.derby.client.am.LogWriter newLogWriter_(java.io.PrintWriter printWriter,
-                                                              int traceLevel) {
-        return new NetLogWriter(printWriter, traceLevel);
-    }
-
     protected void markChainBreakingException_() {
         setSvrcod(CodePoint.SVRCOD_ERROR);
     }
@@ -461,23 +469,41 @@ public class NetAgent extends Agent {
             throw de;
         }
     }
-
+    /**
+     * Marks the agent's write chain as dirty. A write chain is dirty when data
+     * from it has been sent to the server. A dirty write chain cannot be reset 
+     * and reused for another request until the remaining data has been sent to
+     * the server and the write chain properly ended. 
+     * 
+     * Resetting a dirty chain will cause the new request to be appended to the 
+     * unfinished request already at the server, which will likely lead to 
+     * cryptic syntax errors.
+     */
+    void markWriteChainAsDirty() {    
+        writeChainIsDirty_ = true;
+    }
+    
+    private void verifyWriteChainIsClean() throws DisconnectException {
+        if (writeChainIsDirty_) { 
+            throw new DisconnectException(this, 
+                new ClientMessageId(SQLState.NET_WRITE_CHAIN_IS_DIRTY));
+        }
+    }
     public void beginWriteChainOutsideUOW() throws SqlException {
+        verifyWriteChainIsClean();
         request_.initialize();
         writeDeferredResetConnection();
-        super.beginWriteChainOutsideUOW();
     }
 
-    public void beginWriteChain(org.apache.derby.client.am.Statement statement) throws SqlException {
+    public void beginWriteChain(ClientStatement statement) throws SqlException {
+        verifyWriteChainIsClean();
         request_.initialize();
         writeDeferredResetConnection();
         super.beginWriteChain(statement);
     }
 
-    protected void endWriteChain() {
-        super.endWriteChain();
-    }
-
+    protected void endWriteChain() {}
+    
     private void readDeferredResetConnection() throws SqlException {
         if (!netConnection_.resetConnectionAtFirstSql_) {
             return;
@@ -493,51 +519,34 @@ public class NetAgent extends Agent {
         }
     }
 
-    protected void beginReadChain(org.apache.derby.client.am.Statement statement) throws SqlException {
+    protected void beginReadChain(ClientStatement statement)
+            throws SqlException {
+        // Clear here as endWriteChain may not always be called
+        writeChainIsDirty_ = false;
         readDeferredResetConnection();
         super.beginReadChain(statement);
     }
 
     protected void beginReadChainOutsideUOW() throws SqlException {
+        // Clear here as endWriteChain may not always be called
+        writeChainIsDirty_ = false;
         readDeferredResetConnection();
         super.beginReadChainOutsideUOW();
     }
 
-    public void endReadChain() throws SqlException {
-        super.endReadChain();
-    }
-
     /**
      * Switches the current CCSID manager to UTF-8
      */
-    public void switchToUtf8CcsidMgr() {
+    void switchToUtf8CcsidMgr() {
         currentCcsidManager_ = utf8CcsidManager_;
     }
     
     /**
      * Switches the current CCSID manager to EBCDIC
      */
-    public void switchToEbcdicMgr() {
+    void switchToEbcdicMgr() {
         currentCcsidManager_ = ebcdicCcsidManager_;
     }
-
-    public String convertToStringTcpIpAddress(int tcpIpAddress) {
-        StringBuffer ipAddrBytes = new StringBuffer();
-        ipAddrBytes.append((tcpIpAddress >> 24) & 0xff);
-        ipAddrBytes.append(".");
-        ipAddrBytes.append((tcpIpAddress >> 16) & 0xff);
-        ipAddrBytes.append(".");
-        ipAddrBytes.append((tcpIpAddress >> 8) & 0xff);
-        ipAddrBytes.append(".");
-        ipAddrBytes.append((tcpIpAddress) & 0xff);
-
-        return ipAddrBytes.toString();
-    }
-
-    protected int getPort() {
-        return port_;
-    }
-
 }
 
 
