@@ -28,7 +28,9 @@ import org.apache.derby.client.am.SqlException;
 import org.apache.derby.client.am.ClientMessageId;
 import org.apache.derby.shared.common.reference.SQLState;
 
-public class NetConnectionRequest extends Request implements ConnectionRequestInterface {
+class NetConnectionRequest extends Request
+    implements ConnectionRequestInterface {
+
     NetConnectionRequest(NetAgent netAgent, int bufferSize) {
         super(netAgent, bufferSize);
     }
@@ -62,12 +64,6 @@ public class NetConnectionRequest extends Request implements ConnectionRequestIn
 
     }
 
-    void writeDummyExchangeServerAttributes() throws SqlException {
-        // send the exchange server attributes command to the server,
-        // without any parameters
-        buildDummyEXCSAT();
-    }
-
     void writeAccessSecurity(int securityMechanism,
                              String databaseName,
                              byte[] publicKey) throws SqlException {
@@ -180,7 +176,7 @@ public class NetConnectionRequest extends Request implements ConnectionRequestIn
     // The Relational Database Name (RDBNAM) is an optional parameter
     // which will not be sent by this command to reduce size, building,
     // and parsing.
-    void buildRDBCMM() throws SqlException {
+    private void buildRDBCMM() throws SqlException {
         createCommand();
         writeLengthCodePoint(0x04, CodePoint.RDBCMM);
     }
@@ -191,7 +187,7 @@ public class NetConnectionRequest extends Request implements ConnectionRequestIn
     // The Relational Database Name (RDBNAM) is an optional parameter
     // which will not be sent by this command to reduce size, building,
     // and parsing.
-    void buildRDBRLLBCK() throws SqlException {
+    private void buildRDBRLLBCK() throws SqlException {
         createCommand();
         writeLengthCodePoint(0x04, CodePoint.RDBRLLBCK);
     }
@@ -203,7 +199,7 @@ public class NetConnectionRequest extends Request implements ConnectionRequestIn
     // - this driver's product release level
     // - this driver's external name
     // - this driver's server name
-    void buildEXCSAT(String externalName,
+    private void buildEXCSAT(String externalName,
                      int targetAgent,
                      int targetSqlam,
                      int targetRdb,
@@ -264,7 +260,7 @@ public class NetConnectionRequest extends Request implements ConnectionRequestIn
         updateLengthBytes();
     }
 
-    void buildDummyEXCSAT() throws SqlException {
+    private void buildDummyEXCSAT() throws SqlException {
         createCommand();
 
         // begin excsat collection by placing the 4 byte llcp in the buffer.
@@ -278,7 +274,7 @@ public class NetConnectionRequest extends Request implements ConnectionRequestIn
         updateLengthBytes();
     }
 
-    void buildACCSEC(int secmec,
+    private void buildACCSEC(int secmec,
                      String rdbnam,
                      byte[] sectkn) throws SqlException {
         createCommand();
@@ -307,7 +303,7 @@ public class NetConnectionRequest extends Request implements ConnectionRequestIn
         updateLengthBytes();
     }
 
-    void buildSECCHK(int secmec,
+    private void buildSECCHK(int secmec,
                      String rdbnam,
                      String user,
                      String password,
@@ -345,7 +341,7 @@ public class NetConnectionRequest extends Request implements ConnectionRequestIn
     // manager.  The access RDB command then binds the created instance to the target
     // agent and to the RDB. The RDB remains available (accessed) until
     // the communications conversation is terminate.
-    void buildACCRDB(String rdbnam,
+    private void buildACCRDB(String rdbnam,
                      boolean readOnly,
                      byte[] crrtkn,
                      byte[] prddta,
@@ -473,10 +469,10 @@ public class NetConnectionRequest extends Request implements ConnectionRequestIn
      * 
      * Relational Database Name specifies the name of a relational database
      * of the server.
-     * if length of RDB name <= 18 characters, there is not change to the format
+     * if length of RDB name &lt;= 18 characters, there is not change to the format
      * of the RDB name.  The length of the RDBNAM remains fixed at 18 which includes
      * any right bland padding if necessary.
-     * if length of the RDB name is > 18 characters, the length of the RDB name is
+     * if length of the RDB name is &gt; 18 characters, the length of the RDB name is
      * identical to the length of the RDB name.  No right blank padding is required.
      * @param rdbnam  name of the database.
      * @param dontSendOnConversionError omit sending the RDBNAM if there is an
@@ -497,9 +493,22 @@ public class NetConnectionRequest extends Request implements ConnectionRequestIn
             
         }
         
+        //DERBY-4805(Increase the length of the RDBNAM field in the 
+        // DRDA implementation)
+        //The new RDBNAM length in 10.11 is 1024bytes(it used to be 254 bytes).
+        //But if a 10.11 or higher client talks to a 10.10 or under server with
+        // a RDBNAM > 254 bytes, it will result in a protocol exception
+        // because those servers do not support RDBNAM greater than 254 bytes.
+        // This behavior will logged in the jira.
+        //One way to fix this would have been to check the server version
+        // before hand but we do not have that information when the client is
+        // first trying to establish connection to the server by sending the
+        // connect request along with the RDBNAM.
+        int maxRDBlength =
+                 NetConfiguration.RDBNAM_MAX_LEN;
         writeScalarString(CodePoint.RDBNAM, rdbnam,
                 NetConfiguration.PKG_IDENTIFIER_FIXED_LEN, //minimum RDBNAM length in bytes
-                NetConfiguration.PKG_IDENTIFIER_MAX_LEN,   //maximum RDBNAM length in bytes
+                maxRDBlength,  
                 SQLState.NET_DBNAME_TOO_LONG);
                 
     }
@@ -555,7 +564,7 @@ public class NetConnectionRequest extends Request implements ConnectionRequestIn
         writeScalarString(CodePoint.TYPDEFNAM, typdefnam);
     }
 
-    void buildTYPDEFOVR(boolean sendCcsidSbc,
+    private void buildTYPDEFOVR(boolean sendCcsidSbc,
                         int ccsidSbc,
                         boolean sendCcsidDbc,
                         int ccsidDbc,
