@@ -41,84 +41,59 @@ package org.apache.derby.client.net;
 
 import java.net.InetAddress;
 import java.net.UnknownHostException;
-import java.sql.SQLException;
-import java.util.Collections;
-import java.util.Enumeration;
-import java.util.LinkedList;
-import java.util.List;
 import javax.sql.XAConnection;
 import javax.transaction.xa.XAException;
 import javax.transaction.xa.XAResource;
 import javax.transaction.xa.Xid;
 
 import org.apache.derby.client.ClientXid;
-import org.apache.derby.client.am.Connection;
+import org.apache.derby.client.am.ClientConnection;
 import org.apache.derby.client.am.SqlException;
 import org.apache.derby.client.am.ClientMessageId;
+import org.apache.derby.client.am.SqlCode;
+import org.apache.derby.client.am.Utils;
+import org.apache.derby.client.am.XaException;
 import org.apache.derby.shared.common.reference.SQLState;
 
 public class NetXAResource implements XAResource {
-    public static final int TMTIMEOUT = 0x00000100;
-    public static final int ACTIVE_ONLY = -1;
-    public static final int XA_NULL_XID = -1; // null Xid has Format Id of -1
-    public static final int INITIAL_CALLINFO_ELEMENTS = 1;
-    public static final int RECOVER_XID_ARRAY_LENGTH = 10;
-    public static final ClientXid nullXid = new ClientXid();
+    private static final int INITIAL_CALLINFO_ELEMENTS = 1;
+    static final ClientXid nullXid = new ClientXid();
 
     // xaFunction defines, shows which queued XA function is being performed
-    public static final int XAFUNC_NONE = 0;
-    public static final int XAFUNC_COMMIT = 1;
-    public static final int XAFUNC_END = 2;
-    public static final int XAFUNC_FORGET = 3;
-    public static final int XAFUNC_PREPARE = 4;
-    public static final int XAFUNC_RECOVER = 5;
-    public static final int XAFUNC_ROLLBACK = 6;
-    public static final int XAFUNC_START = 7;
-    public static final String XAFUNCSTR_NONE = "No XA Function";
-    public static final String XAFUNCSTR_COMMIT = "XAResource.commit()";
-    public static final String XAFUNCSTR_END = "XAResource.end()";
-    public static final String XAFUNCSTR_FORGET = "XAResource.forget()";
-    public static final String XAFUNCSTR_PREPARE = "XAResource.prepare()";
-    public static final String XAFUNCSTR_RECOVER = "XAResource.recover()";
-    public static final String XAFUNCSTR_ROLLBACK = "XAResource.rollback()";
-    public static final String XAFUNCSTR_START = "XAResource.start()";
+    static final int XAFUNC_COMMIT = 1;
+    private static final int XAFUNC_END = 2;
+    private static final int XAFUNC_FORGET = 3;
+    private static final int XAFUNC_PREPARE = 4;
+    private static final int XAFUNC_RECOVER = 5;
+    static final int XAFUNC_ROLLBACK = 6;
+    private static final int XAFUNC_START = 7;
 
-    public org.apache.derby.client.am.SqlException exceptionsOnXA = null;
+    private static final String XAFUNCSTR_NONE = "No XA Function";
+    private static final String XAFUNCSTR_COMMIT = "XAResource.commit()";
+    private static final String XAFUNCSTR_END = "XAResource.end()";
+    private static final String XAFUNCSTR_FORGET = "XAResource.forget()";
+    private static final String XAFUNCSTR_PREPARE = "XAResource.prepare()";
+    private static final String XAFUNCSTR_RECOVER = "XAResource.recover()";
+    private static final String XAFUNCSTR_ROLLBACK = "XAResource.rollback()";
+    private static final String XAFUNCSTR_START = "XAResource.start()";
 
-    XAConnection xaconn_;
-    org.apache.derby.client.net.NetXAConnection netXAConn_;
-    org.apache.derby.client.net.NetConnection conn_;
+    SqlException exceptionsOnXA = null;
+
+    NetXAConnection netXAConn_;
+    NetConnection conn_;
     private boolean keepIsolationLevel;
-    int rmId_; // unique RmId generated by XAConnection
+
     // TODO: change to a single callInfo field (not an array)
     NetXACallInfo callInfoArray_[] =
             new NetXACallInfo[INITIAL_CALLINFO_ELEMENTS];
-    int numXACallInfo_ = INITIAL_CALLINFO_ELEMENTS;
-    int connectionCount_ = 1;
-    int activeXATransCount_ = 0;
-    String rmIdx_; // userid in case we need to create a secondary connection
-    String rmIdy_; // password in case we need to create a secondary connection
-    // TODO: remove port and ipaddr_
-    int port_;     // port needed to make secondary connection for recover in DS mode.
-    String ipaddr_;  // ip address needed to make secondary connection for recover in DS mode.
-
-    private final List<String> specialRegisters_ =
-            Collections.synchronizedList(new LinkedList<String>());
 
     /** The value of the transaction timeout in seconds. */
     private int timeoutSeconds = 0;
 
-    public NetXAResource(XAConnection xaconn, int rmId,
-                         String userId, String password,
-                         org.apache.derby.client.net.NetXAConnection conn) {
-        xaconn_ = xaconn;
-        rmId_ = rmId;
+    public NetXAResource(XAConnection xaconn,
+                         NetXAConnection conn) {
         conn_ = conn.getNetConnection();
         netXAConn_ = conn;
-        rmIdx_ = userId;
-        rmIdy_ = password;
-        port_ = conn_.netAgent_.getPort();
-        ipaddr_ = conn_.netAgent_.socket_.getLocalAddress().getHostAddress();
         conn.setNetXAResource(this);
 
         // link the primary connection to the first XACallInfo element
@@ -126,15 +101,13 @@ public class NetXAResource implements XAResource {
 
         // construct the NetXACallInfo object for the array.
         for (int i = 0; i < INITIAL_CALLINFO_ELEMENTS; ++i) {
-            callInfoArray_[i] = new NetXACallInfo(null, XAResource.TMNOFLAGS, this,
+            callInfoArray_[i] = new NetXACallInfo(null, XAResource.TMNOFLAGS,
                     null);
         }
 
         // initialize the first XACallInfo element with the information from the
         //  primary connection
         callInfoArray_[0].actualConn_ = conn;
-        callInfoArray_[0].currConnection_ = true;
-        callInfoArray_[0].freeEntry_ = false;
         // ~~~ save conn_ connection variables in callInfoArray_[0]
         callInfoArray_[0].saveConnectionVariables();
     }
@@ -156,7 +129,6 @@ public class NetXAResource implements XAResource {
         callInfo.xaFlags_ = (onePhase ? XAResource.TMONEPHASE :
                 XAResource.TMNOFLAGS);
         callInfo.xid_ = xid;
-        callInfo.xaResource_ = this;
         callInfo.xaRetVal_ = XAResource.XA_OK; // initialize XARETVAL
         try {
             netAgent.beginWriteChainOutsideUOW();
@@ -171,13 +143,10 @@ public class NetXAResource implements XAResource {
             netAgent.endReadChain();
         } catch (SqlException sqle) {
             rc = getSqlExceptionXAErrorCode(sqle);
-            exceptionsOnXA = org.apache.derby.client.am.Utils.accumulateSQLException
-                    (sqle, exceptionsOnXA);
-        } finally {
-            conn_.pendingEndXACallinfoOffset_ = -1; // indicate no pending callinfo
+            exceptionsOnXA = Utils.accumulateSQLException(sqle, exceptionsOnXA);
         }
         if (rc != XAResource.XA_OK) {
-            throwXAException(rc, false);
+            throwXAException(rc);
         }
     }
 
@@ -232,17 +201,15 @@ public class NetXAResource implements XAResource {
         }
 
         NetXACallInfo callInfo = callInfoArray_[conn_.currXACallInfoOffset_];
-        callInfo.setReadOnlyTransactionFlag(conn_.readOnlyTransaction_);
         callInfo.xaFlags_ = flags;
         callInfo.xid_ = xid;
-        callInfo.xaResource_ = this;
         callInfo.xaRetVal_ = XAResource.XA_OK; // initialize XARETVAL
         try {
             netAgent.beginWriteChainOutsideUOW();
             netAgent.netConnectionRequest_.writeXaEndUnitOfWork(conn_);
             netAgent.flowOutsideUOW();
             rc = netAgent.netConnectionReply_.readXaEndUnitOfWork(conn_);
-            conn_.pendingEndXACallinfoOffset_ = -1; // indicate no pending end
+
             if (callInfo.xaRetVal_ != XAResource.XA_OK) { // xaRetVal has possible error, format it
                 callInfo.xaFunction_ = XAFUNC_END;
                 rc = xaRetValErrorAccumSQL(callInfo, rc);
@@ -251,19 +218,17 @@ public class NetXAResource implements XAResource {
             netAgent.endReadChain();
         } catch (SqlException sqle) {
             rc = getSqlExceptionXAErrorCode(sqle);
-            exceptionsOnXA = org.apache.derby.client.am.Utils.accumulateSQLException
-                    (sqle, exceptionsOnXA);
-        } finally {
-            conn_.pendingEndXACallinfoOffset_ = -1; // indicate no pending callinfo
+            exceptionsOnXA = Utils.accumulateSQLException(sqle, exceptionsOnXA);
         }
+
         if (rc != XAResource.XA_OK) {
             // The corresponding XA connection association state
             // is changed by setXaStateForXAException inside the call
             // to throwXAException according the error code of the XAException
             // to be thrown.
-            throwXAException(rc, false);
+            throwXAException(rc);
         }else {
-            conn_.setXAState(Connection.XA_T0_NOT_ASSOCIATED);
+            conn_.setXAState(ClientConnection.XA_T0_NOT_ASSOCIATED);
         } 
     }
 
@@ -289,7 +254,6 @@ public class NetXAResource implements XAResource {
         }
         NetXACallInfo callInfo = callInfoArray_[conn_.currXACallInfoOffset_];
         callInfo.xid_ = xid;
-        callInfo.xaResource_ = this;
         callInfo.xaRetVal_ = XAResource.XA_OK; // initialize XARETVAL
         try {
             // flow the required PROTOCOL to the server
@@ -310,14 +274,12 @@ public class NetXAResource implements XAResource {
                 callInfo.xaRetVal_ = XAResource.XA_OK; // re-initialize XARETVAL
             }
         } catch (SqlException sqle) {
-            exceptionsOnXA = org.apache.derby.client.am.Utils.accumulateSQLException
-                    (sqle, exceptionsOnXA);
+            exceptionsOnXA = Utils.accumulateSQLException(sqle, exceptionsOnXA);
             throwXAException(getSqlExceptionXAErrorCode(sqle));
-        } finally {
-            conn_.pendingEndXACallinfoOffset_ = -1; // indicate no pending callinfo
         }
+
         if (rc != XAResource.XA_OK) {
-            throwXAException(rc, false);
+            throwXAException(rc);
         }
 
     }
@@ -374,7 +336,6 @@ public class NetXAResource implements XAResource {
         int rc = XAResource.XA_OK;
         NetXACallInfo callInfo = callInfoArray_[conn_.currXACallInfoOffset_];
         callInfo.xid_ = xid;
-        callInfo.xaResource_ = this;
         callInfo.xaRetVal_ = XAResource.XA_OK; // initialize XARETVAL
         try {
             netAgent.beginWriteChainOutsideUOW();
@@ -394,13 +355,11 @@ public class NetXAResource implements XAResource {
             netAgent.endReadChain();
         } catch (SqlException sqle) {
             rc = getSqlExceptionXAErrorCode(sqle);
-            exceptionsOnXA = org.apache.derby.client.am.Utils.accumulateSQLException
-                    (sqle, exceptionsOnXA);
-        } finally {
-            conn_.pendingEndXACallinfoOffset_ = -1; // indicate no pending callinfo
+            exceptionsOnXA = Utils.accumulateSQLException(sqle, exceptionsOnXA);
         }
+
         if ((rc != XAResource.XA_OK ) && (rc != XAResource.XA_RDONLY)) {
-            throwXAException(rc, false);
+            throwXAException(rc);
         }
         if (conn_.agent_.loggingEnabled()) {
             conn_.agent_.logWriter_.traceExit(this, "prepare", rc);
@@ -436,11 +395,9 @@ public class NetXAResource implements XAResource {
         }
 
         Xid[] xidList = null;
-        int numXid = 0;
 
         NetXACallInfo callInfo = callInfoArray_[conn_.currXACallInfoOffset_];
         callInfo.xaFlags_ = flag;
-        callInfo.xaResource_ = this;
         callInfo.xaRetVal_ = XAResource.XA_OK; // initialize XARETVAL
         try {
             netAgent.beginWriteChainOutsideUOW();
@@ -454,24 +411,15 @@ public class NetXAResource implements XAResource {
                 callInfo.xaRetVal_ = XAResource.XA_OK; // re-initialize XARETVAL
             }
             netAgent.endReadChain();
-            if (conn_.indoubtTransactions_ != null) {
-                numXid = conn_.indoubtTransactions_.size();
-                xidList = new Xid[numXid];
-                int i = 0;
-                for (Enumeration e = conn_.indoubtTransactions_.keys();
-                     e.hasMoreElements(); i++) {
-                    xidList[i] = (Xid) e.nextElement();
-                }
-            }
+            xidList = conn_.getIndoubtTransactionIds();
+
         } catch (SqlException sqle) {
             rc = getSqlExceptionXAErrorCode(sqle);
-            exceptionsOnXA = org.apache.derby.client.am.Utils.accumulateSQLException
-                    (sqle, exceptionsOnXA);
-        } finally {
-            conn_.pendingEndXACallinfoOffset_ = -1; // indicate no pending callinfo
+            exceptionsOnXA = Utils.accumulateSQLException(sqle, exceptionsOnXA);
         }
+
         if (rc != XAResource.XA_OK) {
-            throwXAException(rc, false);
+            throwXAException(rc);
         }
 
         if (conn_.agent_.loggingEnabled()) {
@@ -502,7 +450,6 @@ public class NetXAResource implements XAResource {
         // update the XACallInfo
         NetXACallInfo callInfo = callInfoArray_[conn_.currXACallInfoOffset_];
         callInfo.xid_ = xid;
-        callInfo.xaResource_ = this;
         callInfo.xaRetVal_ = XAResource.XA_OK; // initialize XARETVAL
         try {
             netAgent.beginWriteChainOutsideUOW();
@@ -518,13 +465,11 @@ public class NetXAResource implements XAResource {
             }
         } catch (SqlException sqle) {
             rc = getSqlExceptionXAErrorCode(sqle);
-            exceptionsOnXA = org.apache.derby.client.am.Utils.accumulateSQLException
-                    (sqle, exceptionsOnXA);
-        } finally {
-            conn_.pendingEndXACallinfoOffset_ = -1; // indicate no pending callinfo
+            exceptionsOnXA = Utils.accumulateSQLException(sqle, exceptionsOnXA);
         }
+
         if (rc != XAResource.XA_OK) {
-            throwXAException(rc, false);
+            throwXAException(rc);
         }
  
     }
@@ -600,16 +545,13 @@ public class NetXAResource implements XAResource {
                 conn_.flowAutoCommit();
         } catch (SqlException sqle) {
             rc = getSqlExceptionXAErrorCode(sqle);
-            exceptionsOnXA = org.apache.derby.client.am.Utils.accumulateSQLException
-                    (sqle, exceptionsOnXA);
+            exceptionsOnXA = Utils.accumulateSQLException(sqle, exceptionsOnXA);
         } 
 
         // update the XACallInfo
         NetXACallInfo callInfo = callInfoArray_[conn_.currXACallInfoOffset_];
         callInfo.xaFlags_ = flags;
-        callInfo.xaInProgress_ = true;
         callInfo.xid_ = xid;
-        callInfo.xaResource_ = this;
         callInfo.xaRetVal_ = XAResource.XA_OK; // initialize XARETVAL
 
         // check and setup the transaction timeout settings
@@ -644,96 +586,89 @@ public class NetXAResource implements XAResource {
             // Setting this is currently required to avoid client from sending
             // commit for autocommit.
             if (rc == XAResource.XA_OK) {
-                conn_.setXAState(Connection.XA_T1_ASSOCIATED);
+                conn_.setXAState(ClientConnection.XA_T1_ASSOCIATED);
             }
 
         } catch (SqlException sqle) {
             rc = getSqlExceptionXAErrorCode(sqle);
-            exceptionsOnXA = org.apache.derby.client.am.Utils.accumulateSQLException
-                    (sqle, exceptionsOnXA);
-        } finally {
-            conn_.pendingEndXACallinfoOffset_ = -1; // indicate no pending callinfo
+            exceptionsOnXA = Utils.accumulateSQLException(sqle, exceptionsOnXA);
         }
+
         if (rc != XAResource.XA_OK) {
-            throwXAException(rc, false);
+            throwXAException(rc);
         }
     }
 
-
-    protected void throwXAException(int rc) throws XAException {
-        throwXAException(rc, rc != XAException.XAER_NOTA);
-    }
-
     private String getXAExceptionText(int rc) {
         String xaExceptionText;
         switch (rc) {
-        case javax.transaction.xa.XAException.XA_RBROLLBACK:
+        case XAException.XA_RBROLLBACK:
             xaExceptionText = "XA_RBROLLBACK";
             break;
-        case javax.transaction.xa.XAException.XA_RBCOMMFAIL:
+        case XAException.XA_RBCOMMFAIL:
             xaExceptionText = "XA_RBCOMMFAIL";
             break;
-        case javax.transaction.xa.XAException.XA_RBDEADLOCK:
+        case XAException.XA_RBDEADLOCK:
             xaExceptionText = "XA_RBDEADLOCK";
             break;
-        case javax.transaction.xa.XAException.XA_RBINTEGRITY:
+        case XAException.XA_RBINTEGRITY:
             xaExceptionText = "XA_RBINTEGRITY";
             break;
-        case javax.transaction.xa.XAException.XA_RBOTHER:
+        case XAException.XA_RBOTHER:
             xaExceptionText = "XA_RBOTHER";
             break;
-        case javax.transaction.xa.XAException.XA_RBPROTO:
+        case XAException.XA_RBPROTO:
             xaExceptionText = "XA_RBPROTO";
             break;
-        case javax.transaction.xa.XAException.XA_RBTIMEOUT:
+        case XAException.XA_RBTIMEOUT:
             xaExceptionText = "XA_RBTIMEOUT";
             break;
-        case javax.transaction.xa.XAException.XA_RBTRANSIENT:
+        case XAException.XA_RBTRANSIENT:
             xaExceptionText = "XA_RBTRANSIENT";
             break;
-        case javax.transaction.xa.XAException.XA_NOMIGRATE:
+        case XAException.XA_NOMIGRATE:
             xaExceptionText = "XA_NOMIGRATE";
             break;
-        case javax.transaction.xa.XAException.XA_HEURHAZ:
+        case XAException.XA_HEURHAZ:
             xaExceptionText = "XA_HEURHAZ";
             break;
-        case javax.transaction.xa.XAException.XA_HEURCOM:
+        case XAException.XA_HEURCOM:
             xaExceptionText = "XA_HEURCOM";
             break;
-        case javax.transaction.xa.XAException.XA_HEURRB:
+        case XAException.XA_HEURRB:
             xaExceptionText = "XA_HEURRB";
             break;
-        case javax.transaction.xa.XAException.XA_HEURMIX:
+        case XAException.XA_HEURMIX:
             xaExceptionText = "XA_HEURMIX";
             break;
-        case javax.transaction.xa.XAException.XA_RETRY:
+        case XAException.XA_RETRY:
             xaExceptionText = "XA_RETRY";
             break;
-        case javax.transaction.xa.XAException.XA_RDONLY:
+        case XAException.XA_RDONLY:
             xaExceptionText = "XA_RDONLY";
             break;
-        case javax.transaction.xa.XAException.XAER_ASYNC:
+        case XAException.XAER_ASYNC:
             xaExceptionText = "XAER_ASYNC";
             break;
-        case javax.transaction.xa.XAException.XAER_RMERR:
+        case XAException.XAER_RMERR:
             xaExceptionText = "XAER_RMERR";
             break;
-        case javax.transaction.xa.XAException.XAER_NOTA:
+        case XAException.XAER_NOTA:
             xaExceptionText = "XAER_NOTA";
             break;
-        case javax.transaction.xa.XAException.XAER_INVAL:
+        case XAException.XAER_INVAL:
             xaExceptionText = "XAER_INVAL";
             break;
-        case javax.transaction.xa.XAException.XAER_PROTO:
+        case XAException.XAER_PROTO:
             xaExceptionText = "XAER_PROTO";
             break;
-        case javax.transaction.xa.XAException.XAER_RMFAIL:
+        case XAException.XAER_RMFAIL:
             xaExceptionText = "XAER_RMFAIL";
             break;
-        case javax.transaction.xa.XAException.XAER_DUPID:
+        case XAException.XAER_DUPID:
             xaExceptionText = "XAER_DUPID";
             break;
-        case javax.transaction.xa.XAException.XAER_OUTSIDE:
+        case XAException.XAER_OUTSIDE:
             xaExceptionText = "XAER_OUTSIDE";
             break;
         case XAResource.XA_OK:
@@ -746,28 +681,26 @@ public class NetXAResource implements XAResource {
         return xaExceptionText;
     }
 
-    protected void throwXAException(int rc, boolean resetFlag) throws XAException { // ~~~
-        StringBuffer xaExceptionText = new StringBuffer(64);
-        if (resetFlag) {
-            // reset the state of the failed connection
-            NetXACallInfo callInfo = callInfoArray_[conn_.currXACallInfoOffset_];
-            callInfo.xaInProgress_ = false;
-            callInfo.xaWasSuspended = false;
-        }
+    private void throwXAException(int rc)
+            throws XAException {
+
+        StringBuilder xaExceptionText = new StringBuilder(64);
 
         xaExceptionText.append(getXAExceptionText(rc));
         // save the SqlException chain to add it to the XAException
-        org.apache.derby.client.am.SqlException sqlExceptions = exceptionsOnXA;
+        SqlException sqlExceptions = exceptionsOnXA;
 
         while (exceptionsOnXA != null) { // one or more SqlExceptions received, format them
             xaExceptionText.append(" : ").append(exceptionsOnXA.getMessage());
-            exceptionsOnXA = (org.apache.derby.client.am.SqlException)
+            exceptionsOnXA = (SqlException)
                     exceptionsOnXA.getNextException();
         }
-        org.apache.derby.client.am.XaException xaException =
-                new org.apache.derby.client.am.XaException(conn_.agent_.logWriter_,
-                        sqlExceptions,
-                        xaExceptionText.toString());
+
+        XaException xaException = new XaException(
+                conn_.agent_.logWriter_,
+                sqlExceptions,
+                xaExceptionText.toString());
+
         xaException.errorCode = rc;
         setXaStateForXAException(rc); 
         throw xaException;
@@ -776,7 +709,7 @@ public class NetXAResource implements XAResource {
 
     /**
      * Reset the transaction branch association state  to XA_T0_NOT_ASSOCIATED
-     * for XAER_RM* and XA_RB* Exceptions. All other exeptions leave the state 
+     * for XAER_RM* and XA_RB* Exceptions. All other exceptions leave the state
      * unchanged
      * 
      * @param rc  // return code from XAException
@@ -788,32 +721,32 @@ public class NetXAResource implements XAResource {
             // Reset to T0, not  associated for XA_RB*, RM*
            // XAER_RMFAIL and XAER_RMERR will be fatal to the connection
            // but that is not dealt with here
-           case javax.transaction.xa.XAException.XAER_RMFAIL:
-           case javax.transaction.xa.XAException.XAER_RMERR:
-           case javax.transaction.xa.XAException.XA_RBROLLBACK:
-           case javax.transaction.xa.XAException.XA_RBCOMMFAIL:
-           case javax.transaction.xa.XAException.XA_RBDEADLOCK:
-           case javax.transaction.xa.XAException.XA_RBINTEGRITY:
-           case javax.transaction.xa.XAException.XA_RBOTHER:
-           case javax.transaction.xa.XAException.XA_RBPROTO:
-           case javax.transaction.xa.XAException.XA_RBTIMEOUT:
-           case javax.transaction.xa.XAException.XA_RBTRANSIENT:
-               conn_.setXAState(Connection.XA_T0_NOT_ASSOCIATED);
+           case XAException.XAER_RMFAIL:
+           case XAException.XAER_RMERR:
+           case XAException.XA_RBROLLBACK:
+           case XAException.XA_RBCOMMFAIL:
+           case XAException.XA_RBDEADLOCK:
+           case XAException.XA_RBINTEGRITY:
+           case XAException.XA_RBOTHER:
+           case XAException.XA_RBPROTO:
+           case XAException.XA_RBTIMEOUT:
+           case XAException.XA_RBTRANSIENT:
+               conn_.setXAState(ClientConnection.XA_T0_NOT_ASSOCIATED);
            break;
             // No change for other XAExceptions
-            // javax.transaction.xa.XAException.XA_NOMIGRATE
-           //javax.transaction.xa.XAException.XA_HEURHAZ
-           // javax.transaction.xa.XAException.XA_HEURCOM
-           // javax.transaction.xa.XAException.XA_HEURRB
-           // javax.transaction.xa.XAException.XA_HEURMIX
-           // javax.transaction.xa.XAException.XA_RETRY
-           // javax.transaction.xa.XAException.XA_RDONLY
-           // javax.transaction.xa.XAException.XAER_ASYNC
-           // javax.transaction.xa.XAException.XAER_NOTA
-           // javax.transaction.xa.XAException.XAER_INVAL                
-           // javax.transaction.xa.XAException.XAER_PROTO
-           // javax.transaction.xa.XAException.XAER_DUPID
-           // javax.transaction.xa.XAException.XAER_OUTSIDE                
+            // XAException.XA_NOMIGRATE
+           //XAException.XA_HEURHAZ
+           // XAException.XA_HEURCOM
+           // XAException.XA_HEURRB
+           // XAException.XA_HEURMIX
+           // XAException.XA_RETRY
+           // XAException.XA_RDONLY
+           // XAException.XAER_ASYNC
+           // XAException.XAER_NOTA
+           // XAException.XAER_INVAL
+           // XAException.XAER_PROTO
+           // XAException.XAER_DUPID
+           // XAException.XAER_OUTSIDE
             default:
                 return;
         }    
@@ -830,8 +763,9 @@ public class NetXAResource implements XAResource {
             connectionClosedFailure();
         }
 
-        if (xares instanceof org.apache.derby.client.net.NetXAResource) { // both are NetXAResource so check to see if this is the same RM
-            // remember, isSame is initialized to false
+        if (xares instanceof NetXAResource) {
+            // Both are NetXAResource so check to see if this is the same RM.
+            // Remember, isSame is initialized to false
             NetXAResource derbyxares = (NetXAResource) xares;
             while (true) {
                 if (!conn_.databaseName_.equalsIgnoreCase(derbyxares.conn_.databaseName_)) {
@@ -906,32 +840,12 @@ public class NetXAResource implements XAResource {
     }
 
 
-    public List getSpecialRegisters() {
-        return specialRegisters_;
-    }
-
-    public void addSpecialRegisters(String s) {
-        if (s.substring(0, 1).equals("@")) {
-            // SET statement is coming from Client
-            if (specialRegisters_.remove(s.substring(1))) {
-                specialRegisters_.remove(s);
-                specialRegisters_.add(s.substring(1));
-            } else {
-                specialRegisters_.remove(s);
-                specialRegisters_.add(s);
-            }
-        } else { // SET statement is coming from Server
-            specialRegisters_.remove(s);
-            specialRegisters_.add(s);
-        }
-    }
-
     private void connectionClosedFailure() throws XAException { // throw an XAException XAER_RMFAIL, with a chained SqlException - closed
-        exceptionsOnXA = org.apache.derby.client.am.Utils.accumulateSQLException
+        exceptionsOnXA = Utils.accumulateSQLException
                 (new SqlException(null, 
                         new ClientMessageId(SQLState.NO_CURRENT_CONNECTION)),
                         exceptionsOnXA);
-        throwXAException(javax.transaction.xa.XAException.XAER_RMFAIL);
+        throwXAException(XAException.XAER_RMFAIL);
     }
 
     private String getXAFuncStr(int xaFunc) {
@@ -964,10 +878,10 @@ public class NetXAResource implements XAResource {
             // create an SqlException to report this error within
             SqlException accumSql = new SqlException(conn_.netAgent_.logWriter_,
                 new ClientMessageId(SQLState.NET_XARETVAL_ERROR),
+                SqlCode.queuedXAError,
                 getXAFuncStr(callInfo.xaFunction_),
-                getXAExceptionText(rc),
-                org.apache.derby.client.am.SqlCode.queuedXAError);
-            exceptionsOnXA = org.apache.derby.client.am.Utils.accumulateSQLException
+                getXAExceptionText(rc));
+            exceptionsOnXA = Utils.accumulateSQLException
                     (accumSql, exceptionsOnXA);
 
             if (currentRC != XAResource.XA_OK) { // the rc passed into this function had an error also, prioritize error
