@@ -39,7 +39,7 @@ import org.apache.derby.iapi.reference.DRDAConstants;
 import org.apache.derby.iapi.reference.Property;
 import org.apache.derby.iapi.services.io.DynamicByteArrayOutputStream;
 import org.apache.derby.iapi.services.property.PropertyUtil;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 /**
     The DDMWriter is used to write DRDA protocol.   The DRDA Protocol is
@@ -1593,7 +1593,7 @@ class DDMWriter
      * @param precision Precision of decimal or numeric type
      * @param scale declared scale
      *
-     * @exception SQLException Thrown if # digits > 31
+     * @exception SQLException Thrown if # digits &gt; 31
      */
     void writeBigDecimal(BigDecimal b, int precision, int scale)
     throws SQLException
