@@ -103,7 +103,10 @@ class DecryptionManager
   {
     try {
       if (java.security.Security.getProvider ("IBMJCE") == null) // IBMJCE is not installed, install it.
-        java.security.Security.addProvider ((java.security.Provider) Class.forName("IBMJCE").newInstance());
+      {
+          Class<?> clazz = Class.forName("IBMJCE");
+          java.security.Security.addProvider ((java.security.Provider) clazz.getConstructor().newInstance());
+      }
       paramSpec_ = new DHParameterSpec (modulus__, base__, exponential_length__);
       keyPairGenerator_ = KeyPairGenerator.getInstance ("DH", "IBMJCE");
       keyPairGenerator_.initialize ((AlgorithmParameterSpec)paramSpec_);
@@ -123,6 +126,14 @@ class DecryptionManager
       throw new SQLException ("java.lang.InstantiationException is caught" +
                               " when initializing EncryptionManager '" + e.getMessage() + "'");
     }
+    catch (java.lang.NoSuchMethodException e) {
+      throw new SQLException ("java.lang.NoSuchMethodException is caught" +
+                              " when initializing EncryptionManager '" + e.getMessage() + "'");
+    }
+    catch (java.lang.reflect.InvocationTargetException e) {
+      throw new SQLException ("java.lang.reflect.InvocationTargetException is caught" +
+                              " when initializing EncryptionManager '" + e.getMessage() + "'");
+    }
     catch (java.security.NoSuchProviderException e) {
       throw new SQLException ("java.security.NoSuchProviderException is caught" +
                               " when initializing EncryptionManager '" + e.getMessage() + "'");
@@ -428,8 +439,8 @@ class DecryptionManager
         The String may be converted back to a byte array using fromHexString.
         <BR>
         For each byte (b) two characaters are generated, the first character
-        represents the high nibble (4 bits) in hexidecimal (<code>b & 0xf0</code>),
-        the second character represents the low nibble (<code>b & 0x0f</code>).
+        represents the high nibble (4 bits) in hexidecimal (<code>b &amp; 0xf0</code>),
+        the second character represents the low nibble (<code>b &amp; 0x0f</code>).
         <BR>
         The byte at <code>data[offset]</code> is represented by the first two characters in the returned String.
 
@@ -460,9 +471,9 @@ class DecryptionManager
         Convert a string into a byte array in hex format.
         <BR>
         For each character (b) two bytes are generated, the first byte 
-        represents the high nibble (4 bits) in hexidecimal (<code>b & 0xf0</code>),
+        represents the high nibble (4 bits) in hexidecimal (<code>b &amp; 0xf0</code>),
         the second byte 
-        represents the low nibble (<code>b & 0x0f</code>).
+        represents the low nibble (<code>b &amp; 0x0f</code>).
         <BR>
         The character at <code>str.charAt(0)</code> is represented by the first two bytes 
         in the returned String.
