@@ -21,12 +21,15 @@
 
 package org.apache.derby.iapi.error;
 
+import java.lang.reflect.InvocationTargetException;
+import java.security.PrivilegedActionException;
 import java.sql.SQLException;
 import java.sql.SQLWarning;
 
 import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.io.ArrayUtil;
 import org.apache.derby.iapi.services.i18n.MessageService;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 /**
 	StandardException is the root of all exceptions that are handled
@@ -59,6 +62,8 @@ public class StandardException extends Exception
 	private String textMessage;
 	private String sqlState;
 	private transient int report;
+    private transient boolean isForPublicAPI;
+    private transient SQLException next;
 
 	/*
 	** End of constructors
@@ -70,11 +75,6 @@ public class StandardException extends Exception
 
 	}
 
-	protected StandardException(String messageID, Object[] args)
-	{
-		this(messageID, (Throwable) null, args);
-	}
-
 	protected StandardException(String messageID, Throwable t, Object[] args)
 	{
 		super(messageID);
@@ -113,10 +113,7 @@ public class StandardException extends Exception
 	 * Returns the arguments for this exception,
 	 * if there are any.
 	 */
-	public final Object[] getArguments()
-	{
-		return arguments;
-	}
+	public final Object[] getArguments()    { return ArrayUtil.copy( arguments ); }
 
 	/**
 		Yes, report me. Errors that need this method to return
@@ -157,6 +154,27 @@ public class StandardException extends Exception
 		return sqlState;
 	}
 
+    /**
+     * Get the next {@code SQLException} that should be put into the parent
+     * exception when this instance is converted to an {@code SQLException}.
+     * @return the next exception
+     */
+    public final SQLException getNextException() {
+        return next;
+    }
+
+    /**
+     * Mark this exception as one that is thrown by a public API method.
+     * The purpose is to signal that this should be a top-level exception,
+     * so that it doesn't get wrapped inside multiple layers of other
+     * SQLExceptions or StandardExceptions as it travels up through the
+     * code layers.
+     * @see PublicAPI
+     */
+    final void markAsPublicAPI() {
+        isForPublicAPI = true;
+    }
+
 	/**
 		Convert a message identifer from org.apache.derby.iapi.reference.SQLState to
 		a SQLState five character string.
@@ -262,39 +280,18 @@ public class StandardException extends Exception
 		return se;
 	}
 
-	/* 0 arguments */
+    public static StandardException
+            newException(String messageId, Object... args) {
+        return newException(messageId, (Throwable) null, args);
+    }
 
-	public static StandardException newException(String messageID) {
-		return new StandardException(messageID);
-	}
-	public static StandardException newException(String messageID, Throwable t) {
-		return new StandardException(messageID, t, (Object[]) null);
-	}
-
-	/* 1 argument */
-
-	public static StandardException newException(String messageID, Object a1) {
-		Object[] oa = new Object[] {a1};
-		return new StandardException(messageID, oa);
-	}
-
-	public static StandardException newException(String messageID,
-												 Object[] a1) {
-		return new StandardException(messageID, a1);
-	}
-
-	public static StandardException newException(String messageID, Throwable t, Object a1) {
-		Object[] oa = new Object[] {a1};
-		return new StandardException(messageID, t, oa);
-	}
+    public static StandardException
+            newException(String messageId, Throwable t, Object... args) {
+        return new StandardException(messageId, t, args);
+    }
 
 	/* 2 arguments */
 
-	public static StandardException newException(String messageID, Object a1, Object a2) {
-		Object[] oa = new Object[] {a1, a2};
-		return new StandardException(messageID, oa);
-	}
-
     /**
      * Dummy exception to catch incorrect use of
      * StandardException.newException(), at compile-time. If you get a
@@ -323,18 +320,8 @@ public class StandardException extends Exception
         throw new BadMessageArgumentException();
     }
 
-	public static StandardException newException(String messageID, Throwable t, Object a1, Object a2) {
-		Object[] oa = new Object[] {a1, a2};
-		return new StandardException(messageID, t, oa);
-	}
-
 	/* 3 arguments */
 
-	public static StandardException newException(String messageID, Object a1, Object a2, Object a3) {
-		Object[] oa = new Object[] {a1, a2, a3};
-		return new StandardException(messageID, oa);
-	}
-    
     /**
      * Dummy overload which should never be called. Only used to
      * detect incorrect usage, at compile time.
@@ -353,62 +340,6 @@ public class StandardException extends Exception
         throw new BadMessageArgumentException(); 
     }
 
-	public static StandardException newException(String messageID, Throwable t, Object a1, Object a2, Object a3) {
-		Object[] oa = new Object[] {a1, a2, a3};
-		return new StandardException(messageID, t, oa);
-	}
-
-	/* 4 arguments */
-
-	public static StandardException newException(String messageID, Object a1, Object a2, Object a3, Object a4) {
-		Object[] oa = new Object[] {a1, a2, a3, a4};
-		return new StandardException(messageID, oa);
-	}
-	public static StandardException newException(String messageID, Throwable t, Object a1, Object a2, Object a3, Object a4) {
-		Object[] oa = new Object[] {a1, a2, a3, a4};
-		return new StandardException(messageID, t, oa);
-	}
- 
-	/* 5 arguments */
-	public static StandardException newException(String messageID, Object a1, Object a2, Object a3, Object a4, Object a5) {
-		Object[] oa = new Object[] {a1, a2, a3, a4, a5};
-		return new StandardException(messageID, oa);
-	}
-	public static StandardException newException(String messageID, Throwable t, Object a1, Object a2, Object a3, Object a4, Object a5) {
-		Object[] oa = new Object[] {a1, a2, a3, a4, a5};
-		return new StandardException(messageID, t, oa);
-	}
-
-	/* 6 arguments */
-	public static StandardException newException(String messageID, Object a1, Object a2, Object a3, Object a4, Object a5, Object a6) {
-		Object[] oa = new Object[] {a1, a2, a3, a4, a5, a6};
-		return new StandardException(messageID, oa);
-	}
-	public static StandardException newException(String messageID, Throwable t, Object a1, Object a2, Object a3, Object a4, Object a5, Object a6) {
-		Object[] oa = new Object[] {a1, a2, a3, a4, a5, a6};
-		return new StandardException(messageID, t, oa);
-	}
-
-	/* 7 arguments */
-	public static StandardException newException(String messageID, Object a1, Object a2, Object a3, Object a4, Object a5, Object a6, Object a7) {
-		Object[] oa = new Object[] {a1, a2, a3, a4, a5, a6, a7};
-		return new StandardException(messageID, oa);
-	}
-	public static StandardException newException(String messageID, Throwable t, Object a1, Object a2, Object a3, Object a4, Object a5, Object a6, Object a7) {
-		Object[] oa = new Object[] {a1, a2, a3, a4, a5, a6, a7};
-		return new StandardException(messageID, t, oa);
-	}
-
-	/* 8 arguments */
-	public static StandardException newException(String messageID, Object a1, Object a2, Object a3, Object a4, Object a5, Object a6, Object a7, Object a8) {
-		Object[] oa = new Object[] {a1, a2, a3, a4, a5, a6, a7, a8};
-		return new StandardException(messageID, oa);
-	}
-	public static StandardException newException(String messageID, Throwable t, Object a1, Object a2, Object a3, Object a4, Object a5, Object a6, Object a7, Object a8) {
-		Object[] oa = new Object[] {a1, a2, a3, a4, a5, a6, a7, a8};
-		return new StandardException(messageID, t, oa);
-	}
-
     /**
      * Creates a new StandardException using message text that has already been localized.
      *
@@ -431,48 +362,72 @@ public class StandardException extends Exception
     
     
 	/**
-	 * Unpack the exception, looking for an DerbySQLException, which carries
+     * Unpack the exception, looking for a StandardException, which carries
 	 * the Derby messageID and arguments. 
 	 * @see org.apache.derby.impl.jdbc.SQLExceptionFactory
-	 * @see org.apache.derby.impl.jdbc.SQLExceptionFactory40
 	 * @see org.apache.derby.impl.jdbc.Util
 	 */
-	public static SQLException	getArgumentFerry(SQLException se)
+    public static StandardException getArgumentFerry(SQLException se)
 	{
-		if (se instanceof DerbySQLException) {
-			/*
-			 * Cater for pre-JDBC4 scenario.
-			 */
-			return se;
-		}
-		/*
-		 * See DERBY-1178 for background information.
-		 * In JDBC4, the DerbySQLException may be wrapped by a SQLException.
-		 */
-		Throwable	cause = se.getCause();
-
-		if ( (cause == null) || !(cause instanceof DerbySQLException ))	{ return se; }
-		else	{ return (SQLException) cause; }
+        Throwable cause = se.getCause();
+        return (cause instanceof StandardException)
+                ? (StandardException) cause : null;
 	}
 
+    /**
+     * Check if the top-level throwable is just a vacuous wrapper that does
+     * not carry any useful information except what's returned by the
+     * {@link Throwable#getCause()} method.
+     *
+     * @param t the throwable to check
+     * @return {@code true} if the throwable is a vacuous wrapper and the
+     *   only useful piece of information is the cause, or {@code false}
+     *   otherwise
+     */
+    private static boolean isVacuousWrapper(Throwable t) {
+        // The only interesting information in an InvocationTargetException
+        // or a PrivilegedActionException is the cause, so consider them
+        // vacuous if they have a cause.
+        if (t instanceof InvocationTargetException
+                || t instanceof PrivilegedActionException) {
+            return (t.getCause() != null);
+        }
+
+        // All other exceptions are non-vacuous.
+        return false;
+    }
 
 	public static StandardException unexpectedUserException(Throwable t)
 	{
+
+        // If there is no useful information in the top-level throwable,
+        // peel it off and only report the cause.
+        if (isVacuousWrapper(t)) {
+            return unexpectedUserException(t.getCause());
+        }
+
         // If the exception is an SQLException generated by Derby, it has an
-        // argument ferry which is an DerbySQLException. Use this to check
+        // argument ferry which is a StandardException. Use this to check
         // whether the exception was generated by Derby.
-        DerbySQLException ferry = null;
+        StandardException ferry = null;
         if (t instanceof SQLException) {
-            SQLException sqle =
-                getArgumentFerry((SQLException) t);
-            if (sqle instanceof DerbySQLException) {
-                ferry = (DerbySQLException) sqle;
+            SQLException sqle = (SQLException) t;
+            ferry = getArgumentFerry(sqle);
+
+            // If the ferry is marked for public API, it means we shouldn't
+            // wrap it inside an "unexpected user exception", so just peel
+            // off the parent SQLException and return the ferry.
+            if (ferry != null && ferry.isForPublicAPI) {
+                // If the parent SQLException has any next exceptions, we
+                // need to store a reference to them before the parent is
+                // discarded.
+                ferry.next = sqle.getNextException();
+                return ferry;
             }
         }
-        
+
 		/*
-		** If we have a SQLException that isn't an EmbedSQLException
-		** (i.e. it didn't come from Derby), then we check
+        ** If we have a SQLException that didn't come from Derby, then we check
 		** to see if it is a valid user defined exception range 
 		** (38001-38XXX).  If so, then we convert it into a 
 		** StandardException without further ado.
@@ -495,16 +450,6 @@ public class StandardException extends Exception
 			}
 		}
 
-		// Look for simple wrappers for 3.0.1 - will be cleaned up in main
-		if (ferry != null) {
-			if (ferry.isSimpleWrapper()) {
-				Throwable wrapped = ((SQLException)ferry).getCause();
-				if (wrapped instanceof StandardException)
-					return (StandardException) wrapped;
-			}
-		}
-
-
 		// no need to wrap a StandardException
 		if (t instanceof StandardException) 
 		{
@@ -536,23 +481,13 @@ public class StandardException extends Exception
 			** In the past we didn't want to place the class name in
 			** an exception because Cloudscape builds were
 			** obfuscated, so the class name would change from build
-                        ** to build. This is no longer true for Derby, but for
-			** exceptions that are Derby's, i.e. EmbedSQLException,
-			** we use toString(). If this returns an empty or null
+            ** to build. This is no longer true for Derby.
+            ** If the exception has no detail message
 			** then we use the class name to make tracking the 
-                        ** problem down easier, though the lack of a message 
+            ** problem down easier, though the lack of a message
 			** should be seen as a bug.
 			*/
-			String	detailMessage;
-			boolean derbyException = false;
-
-			if (ferry != null) {
-				detailMessage = ferry.toString();
-				derbyException = true;
-			}
-			else {
-				detailMessage = t.getMessage();
-			}
+            String detailMessage = t.getMessage();
 
 			if (detailMessage == null)
 			{
@@ -565,16 +500,13 @@ public class StandardException extends Exception
 			if (detailMessage.length() == 0) {
 				detailMessage = t.getClass().getName();
 			}
-			else {
-
-				if (!derbyException) {
-					detailMessage = t.getClass().getName() + ": " + detailMessage;
-				}
+            else
+            {
+                detailMessage = t.getClass().getName() + ": " + detailMessage;
 			}
 
-			StandardException se =
+            return
 				newException(SQLState.LANG_UNEXPECTED_USER_EXCEPTION, t, detailMessage);
-			return se;
 		}
 	}
 
@@ -586,6 +518,12 @@ public class StandardException extends Exception
 
 	public static StandardException plainWrapException(Throwable t) {
 
+        // If there is no useful information in the top-level throwable,
+        // peel it off and only report the cause.
+        if (isVacuousWrapper(t)) {
+            return plainWrapException(t.getCause());
+        }
+
 		if (t instanceof StandardException)
 			return (StandardException) t;
 
@@ -646,8 +584,10 @@ public class StandardException extends Exception
 	*/
 
 	public String getMessage() {
-		if (textMessage == null)
-			textMessage = MessageService.getCompleteMessage(getMessageId(), getArguments());
+        if (textMessage == null) {
+            textMessage = MessageService.getTextMessage(
+                                getMessageId(), getArguments());
+        }
 
 		return textMessage;
 	}
@@ -674,8 +614,9 @@ public class StandardException extends Exception
 		Don't print the class name in the toString() method.
 	*/
 	public String toString() {
+        // Add the SQLState to the message. This should be kept consistent
+        // with SqlException.toString() in the client driver.
 		String msg = getMessage();
-
 		return "ERROR " + getSQLState() + ": " + msg;
 	}
 
@@ -695,29 +636,9 @@ public class StandardException extends Exception
 	** SQL warnings
 	*/
 
-	public static SQLWarning newWarning(String messageId) {
-
-		return newWarningCommon( messageId, (Object[]) null );
-
-	}
-
-	public static SQLWarning newWarning(String messageId, Object a1) {
-
-		Object[] oa = new Object[] {a1};
-
-		return newWarningCommon( messageId, oa );
-	}
-
-	public static SQLWarning newWarning(String messageId, Object a1, Object a2) {
-
-		Object[] oa = new Object[] {a1, a2};
-
-		return newWarningCommon( messageId, oa );
-	}
-
-	private	static	SQLWarning	newWarningCommon( String messageId, Object[] oa )
+    public static SQLWarning newWarning(String messageId, Object... oa)
 	{
-		String		message = MessageService.getCompleteMessage(messageId, oa);
+        String      message = MessageService.getTextMessage(messageId, oa);
 		String		state = StandardException.getSQLStateFromIdentifier(messageId);
 		SQLWarning	sqlw = new SQLWarning(message, state, ExceptionSeverity.WARNING_SEVERITY);
 
@@ -736,6 +657,19 @@ public class StandardException extends Exception
         return(SQLState.LOCK_TIMEOUT.equals(getSQLState()));
     }
 
+    /**
+     * Is this a self-deadlock exception caused by a nested transaction
+     * being blocked by its parent's locks.
+     * <p>
+     *
+     * @return true if this exception is a self-deadlock.
+     *
+     **/
+    public final boolean isSelfDeadlock() {
+
+        return(SQLState.SELF_DEADLOCK.equals(getSQLState()));
+    }
+
     /**
      * Is this a lock timeout or lock deadlock exception.
      * <p>
