@@ -31,6 +31,7 @@ import java.sql.Statement;
 
 /**
 	A Statement implementation that forwards all of its requests to an underlying Statement.
+    This class implements the JDBC 4.1 interface.
  */
 public class BrokeredStatement implements EngineStatement
 {
@@ -289,7 +290,7 @@ public class BrokeredStatement implements EngineStatement
      * this result is a ResultSet.  getMoreResults also implicitly
      * closes any current ResultSet obtained with getResultSet.
      *
-     * There are no more results when (!getMoreResults() &&
+     * There are no more results when (!getMoreResults() &amp;&amp;
      * (getUpdateCount() == -1)
      *
      * @return true if the next result is a ResultSet; false if it is
@@ -344,7 +345,7 @@ public class BrokeredStatement implements EngineStatement
      *
      * @param rows the number of rows to fetch
      * @exception SQLException if a database-access error occurs, or the
-     * condition 0 <= rows <= this.getMaxRows() is not satisfied.
+     * condition 0 &lt;= rows &lt;= this.getMaxRows() is not satisfied.
      */
     public final void setFetchSize(int rows) throws SQLException
     {
@@ -432,7 +433,7 @@ public class BrokeredStatement implements EngineStatement
      */
 	public final boolean getMoreResults(int current) throws SQLException
     {
-        return ((EngineStatement) getStatement()).getMoreResults( current);
+        return getStatement().getMoreResults(current);
 	}
 
     /**
@@ -462,8 +463,7 @@ public class BrokeredStatement implements EngineStatement
 	public final int getResultSetHoldability()
         throws SQLException
 	{
-        int holdability =
-            ((EngineStatement) getStatement()).getResultSetHoldability();
+        int holdability = getStatement().getResultSetHoldability();
         
         // Holdability might be downgraded.
         return controlCheck().checkHoldCursors(holdability);
@@ -570,7 +570,7 @@ public class BrokeredStatement implements EngineStatement
      * @exception SQLException if an error occurs
      */
     public final boolean isClosed() throws SQLException {
-        return ((EngineStatement) getStatement()).isClosed();
+        return getStatement().isClosed();
     }
 
     /**
@@ -583,7 +583,7 @@ public class BrokeredStatement implements EngineStatement
         throws SQLException
     {
         if (isClosed()) {
-            throw control.getExceptionFactory().getSQLException(
+            throw ExceptionFactory.getInstance().getSQLException(
                     SQLState.ALREADY_CLOSED, null, null,
                     new Object[]{ "Statement" });
         }
@@ -597,11 +597,31 @@ public class BrokeredStatement implements EngineStatement
      * @return an exception indicating that unwrap failed
      */
     final SQLException unableToUnwrap(Class iface) {
-        return control.getExceptionFactory().getSQLException(
+        return ExceptionFactory.getInstance().getSQLException(
                 SQLState.UNABLE_TO_UNWRAP, null, null,
                 new Object[]{ iface });
     }
-    
+
+    /**
+     * Forwards to the real Statement.
+     *
+     * @return true if the underlying Statement is poolable, false otherwise.
+     * @throws SQLException if the forwarding call fails.
+     */
+    public final boolean isPoolable() throws SQLException {
+        return getStatement().isPoolable();
+    }
+
+    /**
+     * Forwards to the real Statement.
+     *
+     * @param poolable the new value for the poolable hint.
+     * @throws SQLException if the forwarding call fails.
+     */
+    public final void setPoolable(boolean poolable) throws SQLException {
+        getStatement().setPoolable(poolable);
+    }
+
     ////////////////////////////////////////////////////////////////////
     //
     // INTRODUCED BY JDBC 4.1 IN JAVA 7
