@@ -21,12 +21,12 @@
 
 package org.apache.derby.iapi.services.context;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.stream.HeaderPrintWriter;
 
 import org.apache.derby.iapi.error.ErrorStringBuilder;
 import org.apache.derby.iapi.error.PassThroughException;
-import org.apache.derby.iapi.error.ShutdownException;
+import org.apache.derby.shared.common.error.ShutdownException;
 
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.error.ExceptionUtil;
@@ -45,6 +45,7 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Collections;
 import java.util.Locale;
+import org.apache.derby.iapi.reference.ContextId;
 
 /**
  *
@@ -71,9 +72,9 @@ public class ContextManager
 	 */
 	private static final class CtxStack {
 		/** Internal list with all the elements of the stack. */
-		private final ArrayList stack_ = new ArrayList();
+		private final ArrayList<Context> stack_ = new ArrayList<Context>();
 		/** Read-only view of the internal list. */
-		private final List view_ = Collections.unmodifiableList(stack_);
+		private final List<Context> view_ = Collections.unmodifiableList(stack_);
 
 		// Keeping a reference to the top element on the stack
 		// optimizes the frequent accesses to this element. The
@@ -88,7 +89,7 @@ public class ContextManager
 		void pop() {
 			stack_.remove(stack_.size()-1);
 			top_ = stack_.isEmpty() ? 
-				null : (Context) stack_.get(stack_.size()-1); 
+                null : stack_.get(stack_.size()-1);
 		}
 		void remove(Context context) {
 			if (context == top_) {
@@ -101,7 +102,8 @@ public class ContextManager
 			return top_; 
 		}
 		boolean isEmpty() { return stack_.isEmpty(); }
-		List getUnmodifiableList() { 
+
+        List<Context> getUnmodifiableList() {
 			return view_;
 		}
 	}
@@ -116,12 +118,12 @@ public class ContextManager
 	 * with a String key.
 	 * @see ContextManager#pushContext(Context)
 	 */
-	private final HashMap ctxTable = new HashMap();
+	private final HashMap<String,CtxStack> ctxTable = new HashMap<String,CtxStack>();
 
 	/**
 	 * List of all Contexts
 	 */
-	private final ArrayList holder = new ArrayList();
+	private final ArrayList<Context> holder = new ArrayList<Context>();
 
 	/**
 	 * Add a Context object to the ContextManager. The object is added
@@ -133,7 +135,7 @@ public class ContextManager
 	{
 		checkInterrupt();
 		final String contextId = newContext.getIdName();
-		CtxStack idStack = (CtxStack) ctxTable.get(contextId);
+		CtxStack idStack = ctxTable.get(contextId);
 
 		// if the stack is null, create a new one.
 		if (idStack == null) {
@@ -157,7 +159,7 @@ public class ContextManager
 	public Context getContext(String contextId) {
 		checkInterrupt();
 		
-		final CtxStack idStack = (CtxStack) ctxTable.get(contextId);
+		final CtxStack idStack = ctxTable.get(contextId);
 		if (SanityManager.DEBUG)
 			SanityManager.ASSERT( idStack == null ||
 								  idStack.isEmpty() ||
@@ -178,11 +180,11 @@ public class ContextManager
 		}
 
 		// remove the top context from the global stack
-		Context theContext = (Context) holder.remove(holder.size()-1);
+		Context theContext = holder.remove(holder.size()-1);
 
 		// now find its id and remove it from there, too
 		final String contextId = theContext.getIdName();
-		final CtxStack idStack = (CtxStack) ctxTable.get(contextId);
+		final CtxStack idStack = ctxTable.get(contextId);
 
 		if (SanityManager.DEBUG) {
 			SanityManager.ASSERT( idStack != null &&
@@ -207,7 +209,7 @@ public class ContextManager
 		holder.remove(holder.lastIndexOf(theContext));
 
 		final String contextId = theContext.getIdName();
-		final CtxStack idStack = (CtxStack) ctxTable.get(contextId);
+		final CtxStack idStack = ctxTable.get(contextId);
 
 		// now remove it from its id's stack.
 		idStack.remove(theContext);
@@ -233,9 +235,11 @@ public class ContextManager
 	 * @see org.apache.derby.impl.sql.conn.GenericLanguageConnectionContext#resetSavepoints()
 	 * @see org.apache.derby.iapi.sql.conn.StatementContext#resetSavePoint()
 	 */
-	public final List getContextStack(String contextId) {
-		final CtxStack cs = (CtxStack) ctxTable.get(contextId);
-		return (cs==null?Collections.EMPTY_LIST:cs.getUnmodifiableList());
+    public final List<Context> getContextStack(String contextId) {
+		final CtxStack cs = ctxTable.get(contextId);
+        return cs == null ?
+               Collections.<Context>emptyList() :
+               cs.getUnmodifiableList();
 	}
     
     /**
@@ -271,12 +275,12 @@ public class ContextManager
 
 		if (reportError) 
 		{
-			ContextImpl lcc = null;
 			StringBuffer sb = null;
 			if (! shutdown)
 			{
 				// report an id for the message if possible
-				lcc = (ContextImpl) getContext(org.apache.derby.iapi.reference.ContextId.LANG_CONNECTION);
+                ContextImpl lcc =
+                        (ContextImpl)getContext(ContextId.LANG_CONNECTION);
 				if (lcc != null) {
 					sb = lcc.appendErrorInfo();
 				}
@@ -337,7 +341,7 @@ cleanup:	for (int index = holder.size() - 1; index >= 0; index--) {
 						break;
 					}
 
-					Context ctx = ((Context) holder.get(index));
+					Context ctx = (holder.get(index));
 					lastHandler = ctx.isLastHandler(errorSeverity);
 
 					ctx.cleanupOnError(error);
