@@ -25,11 +25,7 @@ import java.io.InputStream;
 import java.io.IOException;
 import java.io.EOFException;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
-import org.apache.derby.iapi.services.io.LimitObjectInput;
-import org.apache.derby.iapi.services.io.ErrorObjectInput;
-import org.apache.derby.iapi.util.ReuseFactory;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import java.io.UTFDataFormatException;
 
@@ -52,7 +48,7 @@ public final class ArrayInputStream extends InputStream implements LimitObjectIn
 	 *
 	 */
 	public ArrayInputStream() {
-		this(ReuseFactory.getZeroLenByteArray());
+		this(ArrayUtil.EMPTY_BYTE_ARRAY);
 	}
 
 	private ErrorObjectInput oi;
@@ -389,7 +385,7 @@ public final class ArrayInputStream extends InputStream implements LimitObjectIn
      *                      one passed in.
      * @param utflen the byte length of the value, or {@code 0} if unknown
      *
-	 * @exception  IOException  if an I/O error happens
+     * @exception IOException if an I/O error happens
      **/
     public final int readDerbyUTF(char[][] rawData_array, int utflen)
         throws IOException
@@ -553,12 +549,12 @@ public final class ArrayInputStream extends InputStream implements LimitObjectIn
      *
      * Formats are (with x representing value bits):
      * <PRE>
-     * 1 Byte- 00xxxxxx                            val <= 63 (0x3f)
-     * 2 Byte- 01xxxxxx xxxxxxxx                   val > 63 && <= 16383 (0x3fff)
-     * 4 byte- 1xxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx val > 16383 && <= MAX_INT
+     * 1 Byte- 00xxxxxx                            val &lt;= 63 (0x3f)
+     * 2 Byte- 01xxxxxx xxxxxxxx                   val &gt; 63 &amp;&amp; &lt;= 16383 (0x3fff)
+     * 4 byte- 1xxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx val &gt; 16383 &amp;&amp; &lt;= MAX_INT
      * </PRE>
      *
-	 * @exception  IOException  if an I/O error happens
+     * @exception IOException if an I/O error happens
      **/
     public final int readCompressedInt()
         throws IOException
@@ -641,13 +637,13 @@ public final class ArrayInputStream extends InputStream implements LimitObjectIn
      *
      * Formats are (with x representing value bits):
      * <PRE>
-     * value <= 16383 (0x3fff): 
+     * value &gl;= 16383 (0x3fff): 
      *     2 byte - 00xxxxxx xxxxxxxx 
      *
-     * value > 16383 && <= 0x3fffffff:
+     * value &gt; 16383 &amp;&amp; &lt;= 0x3fffffff:
      *     4 byte - 01xxxxxx xxxxxxxx xxxxxxxx xxxxxxxx
      *
-     * value > 0x3fffffff && <= MAX_LONG:
+     * value &lt; 0x3fffffff &amp;&lt; &lt;= MAX_LONG:
      *     8 byte - 1xxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx xxxxxxxx
      * </PRE>
      *
