@@ -21,8 +21,7 @@
 
 package org.apache.derby.iapi.services.io;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
-import org.apache.derby.iapi.util.ReuseFactory;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import java.io.ObjectOutput;
 import java.io.ObjectInput;
@@ -99,7 +98,7 @@ public final class FormatableBitSet implements Formatable, Cloneable
 	 */
 	public FormatableBitSet()
 	{
-		value = ReuseFactory.getZeroLenByteArray();
+		value = ArrayUtil.EMPTY_BYTE_ARRAY;
 	}
 
 	/**
@@ -133,7 +132,7 @@ public final class FormatableBitSet implements Formatable, Cloneable
 	 */
 	public FormatableBitSet(byte[] newValue)
 	{
-		value = newValue;
+		value = ArrayUtil.copy( newValue );
 		bitsInLastByte = 8;
 		lengthAsBits = calculateLength(newValue.length);
 	}
@@ -222,7 +221,7 @@ public final class FormatableBitSet implements Formatable, Cloneable
 	** @return The length in bits for this value
 	**
 	** NOTE: could possibly be changed to a long.  As is
-	** we are restricted to 2^(31-3) -> 256meg instead
+	** we are restricted to 2^(31-3) -&gt; 256meg instead
 	** of 2^31 (Integer.MAX_VALUE) like other datatypes
 	** (or 2 gig).  If it is ever changed to a long
 	** be sure to change read/writeExternal which write
@@ -273,7 +272,7 @@ public final class FormatableBitSet implements Formatable, Cloneable
 			value = data;
 		}
 
-		return value;
+		return ArrayUtil.copy( value );
 	}
 
 	/**
@@ -393,18 +392,18 @@ public final class FormatableBitSet implements Formatable, Cloneable
 	 * Will always do a byte by byte compare.
 	 *
 	 * Given 2 similar bits of unequal lengths (x and y),
-	 * where x.getLength() < y.getLength() but where:
+	 * where x.getLength() &lt; y.getLength() but where:
 	 *
 	 *	 x[0..x.getLength()] == y[0..x.getLength()]
 	 *
-	 * then x < y.
+	 * then x &lt; y.
 	 *
 	 *
 	 * @param other the other bit to compare to
 	 *
-	 * @return -1	- if other <  this
+	 * @return -1	- if other &lt;  this
 	 *			0	- if other == this
-	 *			1	- if other >  this
+	 *			1	- if other &gt;  this
 	 *
 	 */
 	public int compare(FormatableBitSet other)
@@ -746,6 +745,15 @@ public final class FormatableBitSet implements Formatable, Cloneable
 		}
 	}
 
+    /**
+     * Copy the bytes from another FormatableBitSet. Assumes that this bit set
+     * is at least as large as the argument's bit set.
+     */
+    public  void    copyFrom( FormatableBitSet that )
+    {
+        System.arraycopy( that.getByteArray(), 0, value, 0, that.getLengthInBytes());
+    }
+
 	/**
 	 * Bitwise AND this FormatableBitSet with another
 	 * FormatableBitSet. The result is stored in this bitset. The
