@@ -28,7 +28,8 @@ import org.apache.derby.iapi.sql.depend.Provider;
 import org.apache.derby.catalog.UUID;
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.iapi.services.io.ArrayUtil;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.StatementType;
 import org.apache.derby.iapi.store.access.TransactionController;
 import org.apache.derby.catalog.DependableFinder;
@@ -58,8 +59,8 @@ import org.apache.derby.iapi.services.monitor.Monitor;
  * @version 0.1
  */
 
-public final class ConglomerateDescriptor extends TupleDescriptor
-	implements UniqueTupleDescriptor, Provider
+public final class ConglomerateDescriptor extends UniqueTupleDescriptor
+	implements Provider
 {
 	// Implementation
 	private long	conglomerateNumber;
@@ -108,7 +109,7 @@ public final class ConglomerateDescriptor extends TupleDescriptor
 		this.forConstraint = forConstraint;
 		if (uuid == null)
 		{
-			UUIDFactory uuidFactory = Monitor.getMonitor().getUUIDFactory();
+			UUIDFactory uuidFactory = DataDescriptorGenerator.getMonitor().getUUIDFactory();
 			uuid = uuidFactory.createUUID();
 		}
 		this.uuid = uuid;
@@ -229,7 +230,7 @@ public final class ConglomerateDescriptor extends TupleDescriptor
 	 */
 	public void setColumnNames(String[] columnNames)
 	{
-		this.columnNames = columnNames;
+		this.columnNames = ArrayUtil.copy( columnNames );
 	}
 
 	/**
@@ -240,7 +241,7 @@ public final class ConglomerateDescriptor extends TupleDescriptor
 	 */
 	public String[] getColumnNames()
 	{
-		return columnNames;
+		return ArrayUtil.copy( columnNames );
 	}
 
 	//
