@@ -21,14 +21,12 @@
 
 package org.apache.derby.iapi.sql.dictionary;
 
-import org.apache.derby.iapi.error.StandardException;
-
-import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.sql.StatementType;
-import java.util.Hashtable;
-import org.apache.derby.iapi.services.sanity.SanityManager;
-import org.apache.derby.iapi.services.i18n.MessageService;
 import java.util.Enumeration;
+import java.util.Hashtable;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.i18n.MessageService;
+import org.apache.derby.iapi.sql.StatementType;
 
 /**
  *	Static Data dictionary utilities.
@@ -63,16 +61,17 @@ public	class	DDUtils
 		}
 
 
-		ReferencedKeyConstraintDescriptor refCd = null;
+        ReferencedKeyConstraintDescriptor refCd;
 
-		/*
-		** There were no column names specified, just find
-		** the primary key on the table in question
-		*/
 		String[]	refColumnNames = otherConstraintInfo.getReferencedColumnNames();
+
 		if (refColumnNames == null ||
 			refColumnNames.length == 0)
 		{
+			/*
+			** There were no column names specified, just find
+			** the primary key on the table in question
+			*/
 			refCd = refTd.getPrimaryKey();
 			if (refCd == null)
 			{
@@ -104,14 +103,13 @@ public	class	DDUtils
 			}
 
 			return refCd;	
-		}
-
-		/*
-		** Check the referenced columns vs. each unique or primary key to
-		** see if they match the foreign key.
-		*/
+		} 
 		else
 		{
+			/*
+			** Check the referenced columns vs. each unique or primary key to
+			** see if they match the foreign key.
+			*/
 			ConstraintDescriptor cd;
 
 			ColumnDescriptorList colDl = getColumnDescriptors(dd, td, myColumnNames);
@@ -131,6 +129,19 @@ public	class	DDUtils
 					 columnNamesMatch(refColumnNames, 
 										cd.getColumnDescriptors()))
 				{
+                    if (cd.deferrable()) {
+                        final int onDelete = otherConstraintInfo.
+                                getReferentialActionDeleteRule();
+
+                        if (onDelete == StatementType.RA_CASCADE ||
+                            onDelete == StatementType.RA_SETNULL) {
+                            // DERBY-532: Not yet implemented
+                            throw StandardException.newException(
+                                    SQLState.LANG_INVALID_FK_REF_KEY,
+                                    myConstraintName,
+                                    refTd.getQualifiedName());
+                        }
+                    }
 					return (ReferencedKeyConstraintDescriptor)cd;
 				}
 			}
@@ -168,10 +179,9 @@ public	class	DDUtils
 			return false;
 		}
 		
-		String name;
 		for (int index = 0; index < columnNames.length; index++)
 		{
-			name = ((ColumnDescriptor) cdl.elementAt(index)).getColumnName();
+            String name = cdl.elementAt(index).getColumnName();
 			if (!name.equals(columnNames[index]))
 			{
 				return false;
@@ -186,6 +196,7 @@ public	class	DDUtils
 	**checks whether the foreign key relation ships referential action
 	**is violating the restrictions we have in the current system.
 	**/
+    @SuppressWarnings("UseOfObsoleteCollectionType")
 	public static void validateReferentialActions
     (
 		DataDictionary	dd,
@@ -226,7 +237,7 @@ public	class	DDUtils
 		//check whether the foreign key relation ships referential action
 		//is not violating the restrictions we have in the current system.
 		TableDescriptor refTd = otherConstraintInfo.getReferencedTableDescriptor(dd);
-		Hashtable deleteConnHashtable = new Hashtable();
+		Hashtable<String,Integer> deleteConnHashtable = new Hashtable<String,Integer>();
 		//find whether the foreign key is self referencing.
 		boolean isSelfReferencingFk = (refTd.getUUID().equals(td.getUUID()));
 		String refTableName = refTd.getSchemaName() + "." + refTd.getName();
@@ -234,7 +245,8 @@ public	class	DDUtils
 		int currentSelfRefValue = getCurrentDeleteConnections(dd, td, -1, deleteConnHashtable, false, true);
 		validateDeleteConnection(dd, td, refTd, 
 								 refAction, 
-								 deleteConnHashtable, (Hashtable) deleteConnHashtable.clone(),
+                                 deleteConnHashtable,
+                                 new Hashtable<String, Integer>(deleteConnHashtable),
 								 true, myConstraintName, false , 
 								 new StringBuffer(0), refTableName,
 								 isSelfReferencingFk,
@@ -250,20 +262,21 @@ public	class	DDUtils
 		}	
 	}
 
-	/*
-	** Finds the existing delete connection for the table and the referential
-	** actions that will occur  and stores the information in the hash table.
-	** HashTable (key , value) = ( table name that this table is delete
-	** connected to, referential action that will occur if there is a delete on
-	** the table this table connected to[CASACDE, SETNULL , RESTRICT ...etc).)
-	**/
-
+    /**
+     * Finds the existing delete connection for the table and the referential
+     * actions that will occur and stores the information in the hash table.
+     * HashTable (key , value) = ( table name that this table is delete
+     * connected to, referential action that will occur if there is a delete on
+     * the table this table connected to [CASCADE, SET NULL, RESTRICT, NO
+     * ACTION] )
+     */
+    @SuppressWarnings("UseOfObsoleteCollectionType")
 	private	static int  getCurrentDeleteConnections
 	(
 	 DataDictionary	dd,
 	 TableDescriptor	td,
 	 int refActionType,
-	 Hashtable dch,
+	 Hashtable<String,Integer> dch,
 	 boolean prevNotCascade,
 	 boolean findSelfRef
 	 )
@@ -304,7 +317,8 @@ public	class	DDUtils
 				   
 					String refTableName = refTd.getSchemaName() + "." + refTd.getName();
 					//check with  the existing references.
-					Integer rAction = ((Integer)dch.get(refTableName));
+                    Integer rAction = dch.get(refTableName);
+
 					if(rAction != null) // we already looked at this table
 					{
 						prevNotCascade = passedInPrevNotCascade;
@@ -331,7 +345,7 @@ public	class	DDUtils
 					//not specified on the current link. It is actually the 
 					//value of what happens to the table whose delete
 					// connections we are finding.
-					dch.put(refTableName, (new Integer(childRefAction)));
+                    dch.put(refTableName, Integer.valueOf(childRefAction));
 					
 					//find the next delete conectiions on this path for non
 					//self referencig delete connections.
@@ -346,28 +360,28 @@ public	class	DDUtils
 	}
 
 
-	/*
-	** Following function validates whether the new foreign key relation ship
-	** violates any restriction on the referential actions. Current refAction
-	** implementation does not allow cases where we can possible land up
-	** having multiple action for the same row in a table, this happens becase
-	** user can possibly define differential action through multiple paths.
-	** Following function throws error while creating foreign keys if the new
-	** releations ship leads to any such conditions.
-	** NOTE : SQL99 standard also does not cleary says what we are suppose to do
-	** in these non determenistic cases. 
-	** Our implementation just follows what is did in DB2 and throws error
-	** messaged similar to DB2 (sql0632N, sql0633N, sql0634N)
-	*/
-
+    /**
+     * The following function validates whether the new foreign key relationship
+     * violates any restriction on the referential actions. The current
+     * refAction implementation does not allow cases where we can possible land
+     * up having multiple action for the same row in a table. This could happen
+     * because the user can possibly define different actions through multiple
+     * foreign key paths.  The following function throws an error while creating
+     * foreign keys if the new releationship leads to any such conditions.
+     * NOTE: The SQL99 standard also does not clearly says what we are supposed
+     * to do in these cases.  Our implementation just follows what is done in
+     * DB2 and throws error messaged similar to DB2 (sql0632N, sql0633N,
+     * sql0634N).
+     */
+    @SuppressWarnings("UseOfObsoleteCollectionType")
 	private	static void validateDeleteConnection
 	(
 		DataDictionary	dd,
 		TableDescriptor actualTd,  // the table we are adding the foriegn key.
 		TableDescriptor	refTd,
 		int refActionType,
-		Hashtable dch,
-		Hashtable ech,  //existing delete connections
+		Hashtable<String,Integer> dch,
+		Hashtable<String,Integer> ech,  //existing delete connections
 		boolean checkImmediateRefTable,
 		String myConstraintName,
 		boolean prevNotCascade,
@@ -390,7 +404,7 @@ public	class	DDUtils
 		if(checkImmediateRefTable)
 		{
 		    String refTableName = refTd.getSchemaName() + "." + refTd.getName();
-			rAction = ((Integer)dch.get(refTableName));
+            rAction = dch.get(refTableName);
 			
 			// check possible invalide cases incase of self referencing foreign key
 			if(isSelfReferencingFk)
@@ -441,8 +455,9 @@ public	class	DDUtils
 				** CASCADE, otherwise we should throw error.
 				*/
 
-				if(isSelfReferencingFk && dch.contains(new Integer(StatementType.RA_CASCADE)) && 
-				   refActionType!=  StatementType.RA_CASCADE)
+                if( isSelfReferencingFk &&
+                    dch.contains(Integer.valueOf(StatementType.RA_CASCADE)) &&
+                    refActionType !=  StatementType.RA_CASCADE)
 				{
 					throw
 						generateError(SQLState.LANG_DELETE_RULE_MUSTBE_ECASCADE,
@@ -494,7 +509,7 @@ public	class	DDUtils
 			** of cycles , all the nodes in the cycle have same type of
 			** referential action.
 			**/
-			cycleString = cycleString.append(refActionType);
+			cycleString.append(refActionType);
 		}
 
 
@@ -552,7 +567,7 @@ public	class	DDUtils
 				//check for this is non self referencing cycles case
 				//In cases of cycle, whole cycle should have the same refAction
 				// value. Other wise we should throw an exception
-				cycleString = cycleString.append(raDeleteRule);
+				cycleString.append(raDeleteRule);
 				boolean isFormingCycle = (nextRefTd.getUUID().equals(actualTd.getUUID()));
 				if(isFormingCycle)
 				{
@@ -587,7 +602,7 @@ public	class	DDUtils
 				
 
 				String nextRefTableName =  nextRefTd.getSchemaName() + "." + nextRefTd.getName();
-				rAction = ((Integer)ech.get(nextRefTableName));
+                rAction = ech.get(nextRefTableName);
 				if(rAction != null)
 				{
 					/*
@@ -605,11 +620,14 @@ public	class	DDUtils
 
 				}else
 				{
-					rAction = ((Integer)dch.get(nextRefTableName));
+                    rAction = dch.get(nextRefTableName);
 					if(rAction == null)
 					{
-						if(multiPathCheck)
-							dch.put(nextRefTableName, (new Integer(refActionType)));
+                        if (multiPathCheck) {
+                            dch.put(nextRefTableName,
+                                    Integer.valueOf(refActionType));
+                        }
+
 						if(!isSelfRefLink)
 						{
 							validateDeleteConnection(dd, actualTd,  nextRefTd,
@@ -750,12 +768,13 @@ public	class	DDUtils
 	**/
 
 
-	private static void checkForAnyExistingDeleteConnectionViolations
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+    private static void checkForAnyExistingDeleteConnectionViolations
 	(
 	 DataDictionary	dd,
 	 TableDescriptor td,
 	 int refActionType,
-	 Hashtable newDconnHashTable,
+	 Hashtable<String,Integer> newDconnHashTable,
 	 String myConstraintName
 	 )
 	throws StandardException
@@ -769,12 +788,9 @@ public	class	DDUtils
 		//find the tables that are referring to the table we 
 		//are adding the foreign key and check whether we violate their existing rules.
 		String addTableName = td.getSchemaName() + "." + td.getName();;
-		ConstraintDescriptorList refCDL = dd.getConstraintDescriptors(td);
-		int refCDLSize = refCDL.size();
-		for (int index = 0; index < refCDLSize; index++)
-		{
-			ConstraintDescriptor cd = refCDL.elementAt(index);
 
+        for (ConstraintDescriptor cd : dd.getConstraintDescriptors(td))
+        {
 			if ((cd instanceof ReferencedKeyConstraintDescriptor))
 			{
 				ConstraintDescriptorList fkcdl = dd.getActiveConstraintDescriptors
@@ -788,7 +804,7 @@ public	class	DDUtils
 				
 				//Note: More than one table can refer to the same
 				//ReferencedKeyConstraintDescriptor, so we need to find all the tables.
-				Hashtable dConnHashtable = new Hashtable();
+				Hashtable<String,Integer> dConnHashtable = new Hashtable<String,Integer>();
 				for (int inner = 0; inner < size; inner++)
 				{
 					ForeignKeyConstraintDescriptor fkcd = (ForeignKeyConstraintDescriptor) fkcdl.elementAt(inner);
@@ -814,16 +830,16 @@ public	class	DDUtils
 						**referential action and only one SET NULL path.
 						**/
 
-						for (Enumeration e = dConnHashtable.keys() ; e.hasMoreElements() ;) 
+						for (Enumeration<String> e = dConnHashtable.keys() ; e.hasMoreElements() ;) 
 						{
-							String tName = (String) e.nextElement();
+							String tName = e.nextElement();
 							//we should not check for the table name to which  we are
 							//adding the foreign key relation ship.
 							if(!tName.equals(addTableName))
 							{
 								if(newDconnHashTable.containsKey(tName))
 								{
-									int currentDeleteRule = ((Integer)	dConnHashtable.get(tName)).intValue();
+									int currentDeleteRule = (dConnHashtable.get(tName)).intValue();
 									if((currentDeleteRule == StatementType.RA_SETNULL
 										&& raDeleteRuleToAddTable == StatementType.RA_SETNULL) ||
 									   currentDeleteRule  != raDeleteRuleToAddTable)
