@@ -21,31 +21,18 @@
 
 package org.apache.derby.iapi.sql.dictionary;
 
-import org.apache.derby.iapi.services.monitor.Monitor;
-import org.apache.derby.iapi.error.StandardException;
-
-import org.apache.derby.iapi.sql.dictionary.*;
-
-import org.apache.derby.iapi.types.TypeId;
-import org.apache.derby.iapi.sql.depend.Dependent;
-import org.apache.derby.iapi.sql.depend.Provider;
-import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.sql.execute.ConstantAction;
-import org.apache.derby.iapi.sql.execute.ExecPreparedStatement;
-import org.apache.derby.iapi.services.uuid.UUIDFactory;
-import org.apache.derby.iapi.services.io.FormatableBitSet;
-
-import org.apache.derby.catalog.AliasInfo;
-import org.apache.derby.catalog.DefaultInfo;
-import org.apache.derby.catalog.Dependable;
-import org.apache.derby.catalog.DependableFinder;
-import org.apache.derby.catalog.ReferencedColumns;
-import org.apache.derby.catalog.types.ReferencedColumnsDescriptorImpl;
-import org.apache.derby.catalog.UUID;
-import org.apache.derby.catalog.Statistics;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.sql.Timestamp;
-import java.io.InputStream;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.catalog.ReferencedColumns;
+import org.apache.derby.catalog.UUID;
+import org.apache.derby.catalog.types.ReferencedColumnsDescriptorImpl;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.services.io.FormatableBitSet;
+import org.apache.derby.iapi.services.monitor.ModuleFactory;
+import org.apache.derby.iapi.services.monitor.Monitor;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.services.uuid.UUIDFactory;
 import org.apache.derby.iapi.types.DataTypeDescriptor;
 
 /**
@@ -332,15 +319,15 @@ public class DataDescriptorGenerator
 						UUID schemaID
 						)
 	{
-		return (ConglomerateDescriptor)
-				new ConglomerateDescriptor(dataDictionary, conglomerateId,
-												name,
-												indexable,
-												indexRowGenerator,
-												isConstraint,
-												uuid,
-											    tableID,
-												schemaID);
+        return new ConglomerateDescriptor(dataDictionary,
+                                          conglomerateId,
+                                          name,
+                                          indexable,
+                                          indexRowGenerator,
+                                          isConstraint,
+                                          uuid,
+                                          tableID,
+                                          schemaID);
 	}
 
 	/**
@@ -366,6 +353,8 @@ public class DataDescriptorGenerator
 	 * @param referencingNew whether or not NEW appears in REFERENCING clause
 	 * @param oldReferencingName old referencing table name, if any, that appears in REFERCING clause
 	 * @param newReferencingName new referencing table name, if any, that appears in REFERCING clause
+     * @param whenClauseText the SQL text of the WHEN clause (may be null)
+     * @return a trigger descriptor
 	 *
 	 * @exception StandardException on error
 	 */
@@ -388,10 +377,11 @@ public class DataDescriptorGenerator
 		boolean				referencingOld,
 		boolean				referencingNew,
 		String				oldReferencingName,
-		String				newReferencingName
+        String              newReferencingName,
+        String              whenClauseText
 	) throws StandardException
 	{
-		return new TriggerDescriptor(
+        return new TriggerDescriptor(
 					dataDictionary,
 					sd,
 					uuid,
@@ -410,7 +400,8 @@ public class DataDescriptorGenerator
 					referencingOld,
 					referencingNew,
 					oldReferencingName,
-					newReferencingName
+                    newReferencingName,
+                    whenClauseText
 					);
 	}
 		
@@ -421,7 +412,7 @@ public class DataDescriptorGenerator
 	protected UUIDFactory getUUIDFactory()
 	{
 		if (uuidf == null)
-			uuidf = Monitor.getMonitor().getUUIDFactory();
+			uuidf = getMonitor().getUUIDFactory();
 		return uuidf;
 	}
 
@@ -610,4 +601,23 @@ public class DataDescriptorGenerator
                 grantee,
                 grantable);
     }
+    
+    /**
+     * Privileged Monitor lookup. Must be package private so that user code
+     * can't call this entry point.
+     */
+    static  ModuleFactory  getMonitor()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ModuleFactory>()
+             {
+                 public ModuleFactory run()
+                 {
+                     return Monitor.getMonitor();
+                 }
+             }
+             );
+    }
+
 }
