@@ -21,6 +21,10 @@
 
 package org.apache.derby.iapi.sql.dictionary;
 
+import java.security.PrivilegedAction;
+import java.security.AccessController;
+import java.security.AccessControlException;
+import java.security.AccessControlContext;
 import java.sql.Timestamp;
 import java.util.ArrayList;
 import java.util.List;
@@ -32,9 +36,10 @@ import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.context.ContextManager;
 import org.apache.derby.iapi.services.context.ContextService;
+import org.apache.derby.iapi.services.io.ArrayUtil;
 import org.apache.derby.iapi.services.io.StoredFormatIds;
 import org.apache.derby.iapi.services.monitor.Monitor;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.uuid.UUIDFactory;
 import org.apache.derby.iapi.sql.Statement;
 import org.apache.derby.iapi.sql.StorablePreparedStatement;
@@ -46,6 +51,8 @@ import org.apache.derby.iapi.sql.depend.Provider;
 import org.apache.derby.iapi.sql.execute.ExecPreparedStatement;
 import org.apache.derby.iapi.store.access.TransactionController;
 import org.apache.derby.iapi.types.DataTypeDescriptor;
+import org.apache.derby.iapi.types.DataTypeUtilities;
+import org.apache.derby.iapi.types.DataValueDescriptor;
 
 /**
  * A SPSDescriptor describes a Stored Prepared Statement.
@@ -66,8 +73,8 @@ import org.apache.derby.iapi.types.DataTypeDescriptor;
  *
  *
  */
-public class SPSDescriptor extends TupleDescriptor
-	implements UniqueSQLObjectDescriptor, Dependent, Provider
+public class SPSDescriptor extends UniqueSQLObjectDescriptor
+	implements Dependent, Provider
 {
 	/**
 	 * Statement types.  
@@ -218,7 +225,7 @@ public class SPSDescriptor extends TupleDescriptor
 		this.text = text;
 		this.usingText = usingText;
 		this.valid = valid;
-		this.compileTime = compileTime;
+		this.compileTime = DataTypeUtilities.clone( compileTime );
 		this.sd = dataDictionary.getSchemaDescriptor(suuid, null);
 		this.preparedStatement = preparedStatement;
 		this.compSchemaId = compSchemaUUID;
@@ -510,7 +517,7 @@ public class SPSDescriptor extends TupleDescriptor
 	 */
 	public final synchronized Timestamp getCompileTime()
 	{
-		return compileTime;
+		return DataTypeUtilities.clone( compileTime );
 	}
 
 	/**
@@ -587,13 +594,13 @@ public class SPSDescriptor extends TupleDescriptor
 		throws StandardException
 	{
         if (params == null && !lookedUpParams) {
-            List tmpDefaults = new ArrayList();
+            List<DataValueDescriptor> tmpDefaults = new ArrayList<DataValueDescriptor>();
             params = getDataDictionary().getSPSParams(this, tmpDefaults);
             paramDefaults = tmpDefaults.toArray();
             lookedUpParams = true;
         }
 
-		return params;
+        return ArrayUtil.copy(params);
 	}
 
 	/**
@@ -603,7 +610,7 @@ public class SPSDescriptor extends TupleDescriptor
 	 */
 	public final synchronized void setParams(DataTypeDescriptor params[])
 	{
-		this.params = params;
+        this.params = ArrayUtil.copy(params);
 	}
 
 	/**
@@ -620,9 +627,11 @@ public class SPSDescriptor extends TupleDescriptor
 		throws StandardException
 	{
 		if (paramDefaults == null)
+        {
 			getParams();
+        }
 
-		return paramDefaults;
+		return ArrayUtil.copy( paramDefaults );
 	}
 
 	/**
@@ -632,7 +641,7 @@ public class SPSDescriptor extends TupleDescriptor
 	 */
 	public final synchronized void setParameterDefaults(Object[] values)
 	{
-		this.paramDefaults = values;
+		this.paramDefaults = ArrayUtil.copy( values );
 	}
 	
 	/**
@@ -678,7 +687,7 @@ public class SPSDescriptor extends TupleDescriptor
 			(!valid ||
 			 (preparedStatement == null)))
 		{
-			ContextManager cm = ContextService.getFactory().getCurrentContextManager();
+			ContextManager cm = getContextService().getCurrentContextManager();
 
 			/*
 			** Find the language connection context.  Get
@@ -745,7 +754,10 @@ public class SPSDescriptor extends TupleDescriptor
                         nestedTC.rollbackToSavePoint(savepoint, false, null);
                     }
 
-                    if (nestedTC != null && se.isLockTimeout())
+                    if (
+                        (nestedTC != null) &&
+                        ( se.isLockTimeout() || se.isSelfDeadlock() )
+                        )
 					{
                         // Locks were set nowait, so a lock timeout here
                         // means that some lock request in the nested 
@@ -802,6 +814,7 @@ public class SPSDescriptor extends TupleDescriptor
 	 *
 	 * @return The contents as a String
 	 */
+    @Override
 	public final String toString()
 	{
 		if (SanityManager.DEBUG)
@@ -1020,7 +1033,8 @@ public class SPSDescriptor extends TupleDescriptor
                     preparedStatement = null;
 					updateSYSSTATEMENTS(lcc, INVALIDATE, null);
 				}
-				dm.invalidateFor(this, dm.USER_RECOMPILE_REQUEST, lcc);
+                dm.invalidateFor(
+                        this, DependencyManager.USER_RECOMPILE_REQUEST, lcc);
 				break;
 			case DependencyManager.DROP_SPS:
 				//System.out.println("SPSD " + preparedStatement);
@@ -1134,19 +1148,45 @@ public class SPSDescriptor extends TupleDescriptor
 	{
 		if (uuidFactory == null)
 		{
-			uuidFactory = Monitor.getMonitor().getUUIDFactory();
+			uuidFactory = DataDescriptorGenerator.getMonitor().getUUIDFactory();
 		}
 		return uuidFactory.recreateUUID(idString);
 	}
 
 	/** @see TupleDescriptor#getDescriptorType */
+    @Override
 	public String getDescriptorType() { return "Statement"; }
 
 	/** @see TupleDescriptor#getDescriptorName */
 	// RESOLVE: some descriptors have getName.  some descriptors have
 	// getTableName, getColumnName whatever! try and unify all of this to one
 	// getDescriptorName! 
+    @Override
 	public String getDescriptorName() { return name; }
-	
+
+    /**
+     * Privileged lookup of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ContextService    getContextService()
+    {
+        if ( System.getSecurityManager() == null )
+        {
+            return ContextService.getFactory();
+        }
+        else
+        {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedAction<ContextService>()
+                 {
+                     public ContextService run()
+                     {
+                         return ContextService.getFactory();
+                     }
+                 }
+                 );
+        }
+    }
 }
 
