@@ -115,11 +115,15 @@ public interface ResultSetFactory {
 		@param checkGM	The code to enforce the check constraints, if any
         @param fullTemplate Saved item for a row template used by bulk insert,
                             or -1 if this is not a bulk insert
+        @param schemaNameName schema name of table
+        @param tableName table name
 		@return the insert operation as a result set.
 		@exception StandardException thrown when unable to perform the insert
 	 */
 	ResultSet getInsertResultSet(NoPutResultSet source, GeneratedMethod generationClauses,
-                                 GeneratedMethod checkGM, int fullTemplate)
+                                 GeneratedMethod checkGM, int fullTemplate,
+                                 String schemaNameName,
+                                 String tableName)
         throws StandardException;
 
 	/**
@@ -168,6 +172,20 @@ public interface ResultSetFactory {
 	ResultSet getDeleteResultSet(NoPutResultSet source)
 							throws StandardException;
 
+	/**
+		A MERGE result set simply reports that it completed, and
+		the number of rows that it INSERTed/UPDATEd/DELETEdd.  It does not return rows.
+		The delete has been completed once the
+		MERGE result set is available.
+
+		@param drivingLeftJoin the result set from which to take rows to
+			be drive the INSERT/UPDATE/DELETE operations.
+		@return the MERGE operation as a result set.
+		@exception StandardException thrown when unable to perform the work
+	 */
+	ResultSet getMergeResultSet(NoPutResultSet drivingLeftJoin)
+							throws StandardException;
+
 	/**
 		A delete Cascade result set simply reports that it completed, and
 		the number of rows deleted.  It does not return rows.
@@ -299,6 +317,10 @@ public interface ResultSetFactory {
         @param cloneMapItem Item # for columns that need cloning
         @param reuseResult  Whether or not to reuse the result row.
 		@param doesProjection	Whether or not this PRN does a projection
+        @param validatingCheckConstraint {@code true if this PRN is used to
+            for validating a deferred check constraint}.
+        @param validatingBaseTableUUIDString The uuid for the table being
+            validated.
 		@param optimizerEstimatedRowCount	Estimated total # of rows by
 											optimizer
 		@param optimizerEstimatedCost		Estimated total cost by optimizer
@@ -314,6 +336,8 @@ public interface ResultSetFactory {
         int cloneMapItem,
 		boolean reuseResult,
 		boolean doesProjection,
+        boolean validatingCheckConstraint,
+        String validatingBaseTableUUIDString,
 		double optimizerEstimatedRowCount,
 		double optimizerEstimatedCost) throws StandardException;
 
@@ -572,7 +596,7 @@ public interface ResultSetFactory {
 
 	/**
 		A once result set iterates over its source,
-		raising an error if the source returns > 1 row and
+		raising an error if the source returns &gt; 1 row and
 		returning a row with all columns set to nulls
 		if the source returns no rows.
 
@@ -654,6 +678,8 @@ public interface ResultSetFactory {
 		@param returnTypeNumber	Which saved object contains the return type (a multi-set) serialized as a byte array
 		@param vtiProjectionNumber	Which saved object contains the projection for a RestrictedVTI
 		@param vtiRestrictionNumber	Which saved object contains the restriction for a RestrictedVTI
+        @param   vtiSchema   Name of the schema holding the table function.
+        @param   vtiName      Name of the table function.
 		@return the row as a result set.
 		@exception StandardException thrown when unable to create the
 			result set
@@ -673,7 +699,9 @@ public interface ResultSetFactory {
                                      boolean isDerbyStyleTableFunction,
                                      int returnTypeNumber,
                                      int vtiProjectionNumber,
-                                     int vtiRestrictionNumber
+                                     int vtiRestrictionNumber,
+                                     String vtiSchema,
+                                     String vtiName
                                           )
 		 throws StandardException;
 
@@ -873,6 +901,33 @@ public interface ResultSetFactory {
 		@exception StandardException thrown when unable to create the
 			result set
 	 */
+    NoPutResultSet getValidateCheckConstraintResultSet(
+                                Activation activation,
+                                long conglomId,
+                                int scociItem,
+                                int resultRowTemplate,
+                                int resultSetNumber,
+                                GeneratedMethod startKeyGetter,
+                                int startSearchOperator,
+                                GeneratedMethod stopKeyGetter,
+                                int stopSearchOperator,
+                                boolean sameStartStopPosition,
+                                Qualifier[][] qualifiers,
+                                String tableName,
+                                String userSuppliedOptimizerOverrides,
+                                String indexName,
+                                boolean isConstraint,
+                                boolean forUpdate,
+                                int colRefItem,
+                                int indexColItem,
+                                int lockMode,
+                                boolean tableLocked,
+                                int isolationLevel,
+                                boolean oneRowScan,
+                                double optimizerEstimatedRowCount,
+                                double optimizerEstimatedCost)
+            throws StandardException;
+
 	NoPutResultSet getTableScanResultSet(
 			                    Activation activation,
 								long conglomId,
@@ -900,7 +955,7 @@ public interface ResultSetFactory {
 								double optimizerEstimatedCost)
 			throws StandardException;
 
-	/**
+    /**
 		A table scan result set forms a result set on a scan
 		of a table.
 		The rows can be constructed as they are requested from the
@@ -1068,6 +1123,7 @@ public interface ResultSetFactory {
 		@param optimizerEstimatedRowCount	Estimated total # of rows by
 											optimizer
 		@param optimizerEstimatedCost		Estimated total cost by optimizer
+		@param baseColumnCount		Number of columns in the base table
 
 		@return the index row to base row operation as a result set.
 		@exception StandardException thrown when unable to create the
@@ -1087,7 +1143,8 @@ public interface ResultSetFactory {
 								GeneratedMethod restriction,
 								boolean forUpdate,
 								double optimizerEstimatedRowCount,
-								double optimizerEstimatedCost)
+								double optimizerEstimatedCost,
+								int baseColumnCount )
 			throws StandardException;
 
 
