@@ -161,8 +161,7 @@ public interface Transaction {
         occur of lower severity than Transaction severity are caught, the 
         transaction is then aborted and then an exception of Transaction
 		severity is thrown nesting the original exception.
-
-		Any exception more severe than a
+        Any exception more severe than a
         Transaction exception is not caught and the transaction is not aborted.
         The transaction will be aborted by the standard context mechanism.
 
@@ -186,8 +185,7 @@ public interface Transaction {
         occur of lower severity than Transaction severity are caught, the 
         transaction is then aborted and then an exception of Transaction
 		severity is thrown nesting the original exception.
-
-		Any exception more severe than a
+        Any exception more severe than a
         Transaction exception is not caught and the transaction is not aborted.
         The transaction will be aborted by the standard context mechanism.
 	*/
@@ -493,6 +491,14 @@ public interface Transaction {
 	*/
 	public void addPostCommitWork(Serviceable work);
 
+	/**
+		Add to the list of post abort work that may be processed after this
+		transaction aborts.  
+
+		@param work the post commit work that is added
+	*/
+	public void addPostAbortWork(Serviceable work);
+
 	/**
 		Add to the list of post termination work that may be processed after this
 		transaction commits or aborts.
