@@ -31,7 +31,7 @@ import org.apache.derby.iapi.services.context.ContextService;
 
 import org.apache.derby.iapi.services.io.StoredFormatIds;
  
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.services.cache.ClassSize;
 import org.apache.derby.iapi.services.i18n.LocaleFinder;
@@ -57,7 +57,7 @@ import java.text.ParseException;
 /**
  * This contains an instance of a SQL Date.
  * <p>
- * The date is stored as int (year << 16 + month << 8 + day)
+ * The date is stored as int (year &lt;&lt; 16 + month &lt;&lt; 8 + day)
  * Null is represented by an encodedDate value of 0.
  * Some of the static methods in this class are also used by SQLTime and SQLTimestamp
  * so check those classes if you change the date encoding
@@ -377,8 +377,8 @@ public final class SQLDate extends DataType
      * @param isJdbcEscape if true then only the JDBC date escape syntax is allowed
      * @param localeFinder
      *
-     * @exception StandardException if the syntax is invalid or the value 
-     *     is out of range.
+     * @exception StandardException if the syntax is invalid or the value is
+     * out of range
      */
     public SQLDate( String dateStr, boolean isJdbcEscape, LocaleFinder localeFinder)
         throws StandardException
@@ -399,8 +399,8 @@ public final class SQLDate extends DataType
      * @param isJdbcEscape if true then only the JDBC date escape syntax is allowed
      * @param localeFinder
      *
-     * @exception StandardException if the syntax is invalid or the value 
-     *     is out of range.
+     * @exception StandardException if the syntax is invalid or the value is
+     * out of range
      */
     public SQLDate( String dateStr, boolean isJdbcEscape, LocaleFinder localeFinder, Calendar cal)
         throws StandardException
@@ -554,7 +554,7 @@ public final class SQLDate extends DataType
 
 		if (theValue != null)
 		{
-            DatabaseContext databaseContext = (DatabaseContext) ContextService.getContext(DatabaseContext.CONTEXT_ID);
+            DatabaseContext databaseContext = (DatabaseContext) DataValueFactoryImpl.getContext(DatabaseContext.CONTEXT_ID);
             parseDate( theValue,
                        false,
                        (databaseContext == null) ? null : databaseContext.getDatabase(),
@@ -743,7 +743,7 @@ public final class SQLDate extends DataType
 	/**
 	 *	computeEncodedDate extracts the year, month and date from
 	 *	a Calendar value and encodes them as
-	 *		year << 16 + month << 8 + date
+	 *		year &lt;&lt; 16 + month &lt;&lt; 8 + date
 	 *	Use this function will help to remember to add 1 to month
 	 *  which is 0 based in the Calendar class
 	 *	@param cal	the Calendar 
