@@ -26,7 +26,7 @@ import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.io.StoredFormatIds;
 import org.apache.derby.iapi.services.io.Storable;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.error.StandardException;
 
@@ -164,7 +164,7 @@ public final class SQLDouble extends NumberDataType
 		if (isNull())
 			return null;
 		else
-			return new Double(value);
+			return value;
 	}
 
 
@@ -435,7 +435,7 @@ public final class SQLDouble extends NumberDataType
 	/**
 		Called for an application setting this value using a BigDecimal 
 	*/
-	public  void setBigDecimal(Number bigDecimal) throws StandardException
+	public  void setBigDecimal(BigDecimal bigDecimal) throws StandardException
 	{
 		if (objectNull(bigDecimal)) 
 			return;
@@ -449,8 +449,7 @@ public final class SQLDouble extends NumberDataType
         if (v == 0) {
             // We need to catch underflow here, since BigDecimal#doubleValue it
             // just returns 0 (i.e. no exception).
-            boolean isZero =
-                ((BigDecimal) bigDecimal).compareTo(BigDecimal.ZERO) == 0;
+            boolean isZero = bigDecimal.compareTo(BigDecimal.ZERO) == 0;
 
             if (!isZero) {
                 throw StandardException.
@@ -512,11 +511,11 @@ public final class SQLDouble extends NumberDataType
 	}
 
 	/**
-	 * The <> operator as called from the language module, as opposed to
+	 * The &lt;&gt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <>
-	 * @param right			The value on the right side of the <>
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *						is not.
 	 *
 	 * @return	A SQL boolean value telling whether the two parameters
@@ -535,11 +534,11 @@ public final class SQLDouble extends NumberDataType
 	}
 
 	/**
-	 * The < operator as called from the language module, as opposed to
+	 * The &lt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <
-	 * @param right			The value on the right side of the <
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is less
 	 *			than the second operand
@@ -557,11 +556,11 @@ public final class SQLDouble extends NumberDataType
 	}
 
 	/**
-	 * The > operator as called from the language module, as opposed to
+	 * The &gt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the >
-	 * @param right			The value on the right side of the >
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is greater
 	 *			than the second operand
@@ -579,11 +578,11 @@ public final class SQLDouble extends NumberDataType
 	}
 
 	/**
-	 * The <= operator as called from the language module, as opposed to
+	 * The &lt;= operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <=
-	 * @param right			The value on the right side of the <=
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is less
 	 *			than or equal to the second operand
@@ -601,11 +600,11 @@ public final class SQLDouble extends NumberDataType
 	}
 
 	/**
-	 * The >= operator as called from the language module, as opposed to
+	 * The &gt;= operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the >=
-	 * @param right			The value on the right side of the >=
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is greater
 	 *			than or equal to the second operand
