@@ -28,7 +28,7 @@ import org.apache.derby.iapi.services.io.StoredFormatIds;
 
 import org.apache.derby.iapi.error.StandardException;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.services.cache.ClassSize;
 
@@ -185,7 +185,7 @@ public final class SQLReal
 		if (isNull())
 			return null;
 		else
-			return new Float(value);
+			return value;
 	}
 
 	// this is for DataType's error generator
@@ -392,7 +392,7 @@ public final class SQLReal
 	/**
 		Called for an application setting this value using a BigDecimal 
 	*/
-	public  void setBigDecimal(Number bigDecimal) throws StandardException
+	public  void setBigDecimal(BigDecimal bigDecimal) throws StandardException
 	{
 		if (objectNull(bigDecimal)) 
 			return;
@@ -406,8 +406,7 @@ public final class SQLReal
         if (v == 0) {
             // We need to catch underflow here, since BigDecimal#floatValue it
             // just returns 0 (i.e. no exception).
-            boolean isZero =
-                ((BigDecimal) bigDecimal).compareTo(BigDecimal.ZERO) == 0;
+            boolean isZero = bigDecimal.compareTo(BigDecimal.ZERO) == 0;
 
             if (!isZero) {
                 throw StandardException.
@@ -529,11 +528,11 @@ public final class SQLReal
 	}
 
 	/**
-	 * The <> operator as called from the language module, as opposed to
+	 * The &lt;&gt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <>
-	 * @param right			The value on the right side of the <>
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the two parameters
 	 *			are not equal
@@ -551,11 +550,11 @@ public final class SQLReal
 	}
 
 	/**
-	 * The < operator as called from the language module, as opposed to
+	 * The &lt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <
-	 * @param right			The value on the right side of the <
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is less
 	 *			than the second operand
@@ -573,11 +572,11 @@ public final class SQLReal
 	}
 
 	/**
-	 * The > operator as called from the language module, as opposed to
+	 * The &gt; operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the >
-	 * @param right			The value on the right side of the >
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is greater
 	 *			than the second operand
@@ -595,11 +594,11 @@ public final class SQLReal
 	}
 
 	/**
-	 * The <= operator as called from the language module, as opposed to
+	 * The &lt;= operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the <=
-	 * @param right			The value on the right side of the <=
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is less
 	 *			than or equal to the second operand
@@ -617,11 +616,11 @@ public final class SQLReal
 	}
 
 	/**
-	 * The >= operator as called from the language module, as opposed to
+	 * The &gt;= operator as called from the language module, as opposed to
 	 * the storage module.
 	 *
-	 * @param left			The value on the left side of the >=
-	 * @param right			The value on the right side of the >=
+	 * @param left			The value on the left side of the operator
+	 * @param right			The value on the right side of the operator
 	 *
 	 * @return	A SQL boolean value telling whether the first operand is greater
 	 *			than or equal to the second operand
