@@ -25,7 +25,7 @@ import org.apache.derby.iapi.reference.SQLState;
 
 import org.apache.derby.iapi.services.context.ContextService;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.io.StoredFormatIds;
 
 import org.apache.derby.iapi.error.StandardException;
@@ -381,11 +381,11 @@ public final class SQLTime extends DataType
      *<ol>
      *<li>old ISO and IBM European standard: hh.mm[.ss]
      *<li>IBM USA standard: hh[:mm] {AM | PM}
-     *<li>JIS & current ISO: hh:mm[:ss]
+     *<li>JIS and current ISO: hh:mm[:ss]
      *</ol>
      * 
      * @exception StandardException if the syntax is invalid or the value is
-     *     out of range.
+     * out of range
      */
     public SQLTime( String timeStr, boolean isJdbcEscape, LocaleFinder localeFinder)
         throws StandardException
@@ -398,11 +398,11 @@ public final class SQLTime extends DataType
      *<ol>
      *<li>old ISO and IBM European standard: hh.mm[.ss]
      *<li>IBM USA standard: hh[:mm] {AM | PM}
-     *<li>JIS & current ISO: hh:mm[:ss]
+     *<li>JIS and current ISO: hh:mm[:ss]
      *</ol>
      * 
      * @exception StandardException if the syntax is invalid or the value is
-     *     out of range.
+     * out of range
      */
     public SQLTime( String timeStr, boolean isJdbcEscape, LocaleFinder localeFinder, Calendar cal)
         throws StandardException
@@ -605,7 +605,7 @@ public final class SQLTime extends DataType
 		restoreToNull();
 		if (theValue != null)
         {
-            DatabaseContext databaseContext = (DatabaseContext) ContextService.getContext(DatabaseContext.CONTEXT_ID);
+            DatabaseContext databaseContext = (DatabaseContext) DataValueFactoryImpl.getContext(DatabaseContext.CONTEXT_ID);
             parseTime( theValue,
                        false,
                        (databaseContext == null) ? null : databaseContext.getDatabase(),
@@ -841,7 +841,7 @@ public final class SQLTime extends DataType
 	}
 	/**
 	 *	Calculate the encoded time from a Calendar object
-	 *	encoded time is hour << 16 + min << 8 + sec
+	 *	encoded time is hour &lt;&lt; 16 + min &lt;&lt; 8 + sec
 	 *  this function is also used by SQLTimestamp 
 	 *
 	 * @param	cal calendar with time set
@@ -909,7 +909,7 @@ public final class SQLTime extends DataType
 
 	/**
 	 * Compute encoded time value
-	 * Time is represented by hour << 16 + minute << 8 + seconds
+	 * Time is represented by hour &lt;&lt; 16 + minute &lt;&lt; 8 + seconds
 	 */
 	private	int computeEncodedTime(java.util.Date value) throws StandardException
 	{
