@@ -38,14 +38,15 @@ import org.apache.derby.iapi.services.monitor.ModuleControl;
 import org.apache.derby.iapi.services.monitor.ModuleSupportable;
 import org.apache.derby.iapi.services.monitor.Monitor;
 import org.apache.derby.iapi.services.monitor.ModuleFactory;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.db.Database;
 import org.apache.derby.iapi.db.DatabaseContext;
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.sql.execute.ExecutionFactory;
+import org.apache.derby.iapi.store.raw.data.DataFactory;
 import org.apache.derby.iapi.types.DataValueFactory;
 import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
-
+import org.apache.derby.iapi.util.StringUtil;
 import org.apache.derby.iapi.sql.conn.LanguageConnectionFactory;
 
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
@@ -62,12 +63,20 @@ import org.apache.derby.iapi.store.access.TransactionController;
 import org.apache.derby.iapi.jdbc.AuthenticationService;
 import org.apache.derby.iapi.services.uuid.UUIDFactory;
 import org.apache.derby.impl.sql.execute.JarUtil;
+import org.apache.derby.iapi.services.io.FileUtil;
 import org.apache.derby.io.StorageFile;
+import org.apache.derby.io.StorageFactory;
 import org.apache.derby.catalog.UUID;
 
 import org.apache.derby.iapi.store.replication.slave.SlaveFactory;
 
+import java.io.File;
+import java.io.IOException;
 import java.io.Serializable;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
+import java.security.PrivilegedActionException;
+import java.security.PrivilegedExceptionAction;
 import java.sql.SQLException;
 import java.util.Properties;
 import java.util.Dictionary;
@@ -140,7 +149,7 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 		throws StandardException
 	{
 
-		ModuleFactory monitor = Monitor.getMonitor();
+		ModuleFactory monitor = getMonitor();
 		if (create)
 		{
 			if (startParams.getProperty(Property.CREATE_WITH_NO_LOG) == null)
@@ -168,7 +177,7 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 		// registered types (DECIMAL) are there before logical undo recovery 
         // might need them.
 		DataValueFactory dvf = (DataValueFactory) 
-            Monitor.bootServiceModule(
+            bootServiceModule(
                 create, 
                 this,
 				org.apache.derby.iapi.reference.ClassName.DataValueFactory, 
@@ -193,15 +202,15 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 		bootClassFactory(create, allParams);
         
         dd = (DataDictionary)
-            Monitor.bootServiceModule(create, this,
+            bootServiceModule(create, this,
                     DataDictionary.MODULE, allParams);
 
 		lcf = (LanguageConnectionFactory) 
-            Monitor.bootServiceModule(
+            bootServiceModule(
                 create, this, LanguageConnectionFactory.MODULE, allParams);
 
 		lf = (LanguageFactory) 
-            Monitor.bootServiceModule(
+            bootServiceModule(
                 create, this, LanguageFactory.MODULE, allParams);
 
 		bootResourceAdapter(create, allParams);
@@ -428,6 +437,10 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
     {
 		try {
 			af.backup(backupDir, wait);
+            if ( luceneLoaded() )
+            {
+                backupLucene( backupDir );
+            }
 		} catch (StandardException se) {
 			throw PublicAPI.wrapStandardException(se);
 		}
@@ -443,6 +456,10 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 			af.backupAndEnableLogArchiveMode(backupDir, 
                                              deleteOnlineArchivedLogFiles,
                                              wait); 
+            if ( luceneLoaded() )
+            {
+                backupLucene( backupDir );
+            }
 		} catch (StandardException se) {
 			throw PublicAPI.wrapStandardException(se);
 		}
@@ -568,7 +585,7 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 	{
 		
 		TransactionController tc = af.getTransaction(
-				ContextService.getFactory().getCurrentContextManager());
+				getContextService().getCurrentContextManager());
 
 		String  upgradeID = null;
 		UUID	databaseID;
@@ -578,7 +595,7 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 			// no property defined in the Transaction set
 			// this could be an upgrade, see if it's stored in the service set
 
-			UUIDFactory	uuidFactory  = Monitor.getMonitor().getUUIDFactory();
+			UUIDFactory	uuidFactory  = getMonitor().getUUIDFactory();
 
 			
 			upgradeID = startParams.getProperty(DataDictionary.DATABASE_ID);
@@ -728,7 +745,7 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 			IdUtil.parseDbClassPath(classpath);
 
 			startParams.put(Property.BOOT_DB_CLASSPATH, classpath);
-			cfDB = (ClassFactory) Monitor.bootServiceModule(create, this,
+			cfDB = (ClassFactory) bootServiceModule(create, this,
 					org.apache.derby.iapi.reference.Module.ClassFactory, startParams);
 	}
 
@@ -746,18 +763,18 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 
 	protected AuthenticationService bootAuthenticationService(boolean create, Properties props) throws StandardException {
 		return (AuthenticationService)
-				Monitor.bootServiceModule(create, this, AuthenticationService.MODULE, props);
+				bootServiceModule(create, this, AuthenticationService.MODULE, props);
 	}
 
 	protected void bootValidation(boolean create, Properties startParams)
 		throws StandardException {
-		pf = (PropertyFactory) Monitor.bootServiceModule(create, this,
+		pf = (PropertyFactory) bootServiceModule(create, this,
 			org.apache.derby.iapi.reference.Module.PropertyFactory, startParams);
 	}
 
 	protected void bootStore(boolean create, Properties startParams)
 		throws StandardException {
-		af = (AccessFactory) Monitor.bootServiceModule(create, this, AccessFactory.MODULE, startParams);
+		af = (AccessFactory) bootServiceModule(create, this, AccessFactory.MODULE, startParams);
 	}
 
     /**
@@ -768,7 +785,7 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 		throws StandardException {
 
 		TransactionController tc = af.getTransaction(
-                    ContextService.getFactory().getCurrentContextManager());
+                    getContextService().getCurrentContextManager());
 		Properties dbProps = tc.getProperties();
 		tc.commit();
 		tc.destroy();
@@ -791,7 +808,7 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 		try
 		{
 			resourceAdapter = 
-				Monitor.bootServiceModule(create, this,
+				bootServiceModule(create, this,
 										 org.apache.derby.iapi.reference.Module.ResourceAdapter,
 										 allParams);
 		}
@@ -818,7 +835,7 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 
 		long generationId = fid.getGenerationId();
 
-        ContextManager cm = ContextService.getFactory().getCurrentContextManager();
+        ContextManager cm = getContextService().getCurrentContextManager();
 		FileResource fr = af.getTransaction(cm).getFileHandler();
 
         String externalName = JarUtil.mkExternalName(
@@ -827,4 +844,193 @@ public class BasicDatabase implements ModuleControl, ModuleSupportable, Property
 		return fr.getAsFile(externalName, generationId);
 	}
 
+    ////////////////////////////////////////////////////////////////////////
+    //
+    // SUPPORT FOR BACKING UP LUCENE DIRECTORY
+    //
+    ////////////////////////////////////////////////////////////////////////
+
+    /**
+     * <p>
+     * Return true if the Lucene plugin is loaded.
+     * </p>
+     */
+    private boolean luceneLoaded()
+        throws StandardException
+    {
+        try {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedExceptionAction<Boolean>()
+                 {
+                     public Boolean run()
+                         throws StandardException
+                     {
+                         return getLuceneDir().exists();
+                     }
+                 }
+                 ).booleanValue();
+        }
+        catch (PrivilegedActionException pae) { throw StandardException.plainWrapException( pae ); }
+    }
+
+    /** Get the location of the Lucene indexes */
+    private StorageFile getLuceneDir()
+        throws StandardException
+    {
+        StorageFactory  dir = getStorageFactory();
+		
+		return dir.newStorageFile( Database.LUCENE_DIR );
+    }
+
+    /**
+     * <p>
+     * Get the database StorageFactory.
+     * </p>
+     */
+    private StorageFactory  getStorageFactory()
+        throws StandardException
+    {
+        DataFactory dataFactory = (DataFactory) findServiceModule( this, DataFactory.MODULE );
+
+        return dataFactory.getStorageFactory();
+    }
+
+    /**
+     * <p>
+     * Backup Lucene indexes to the backup directory. This assumes
+     * that the rest of the database has been backup up and sanity
+     * checks have been run.
+     * </p>
+     */
+    private void    backupLucene( String backupDir )
+        throws StandardException
+    {
+        try {
+            File            backupRoot = new File( backupDir );
+            StorageFactory  storageFactory = getStorageFactory();
+            String      canonicalDbName = storageFactory.getCanonicalName();
+            String      dbname = StringUtil.shortDBName( canonicalDbName, storageFactory.getSeparator() );
+            File        backupDB = new File( backupRoot, dbname );
+        
+            final   File            targetDir = new File( backupDB, Database.LUCENE_DIR );
+            final   StorageFile sourceDir = getLuceneDir();
+
+            AccessController.doPrivileged
+                (
+                 new PrivilegedExceptionAction<Object>()
+                 {
+                     public Boolean run()
+                         throws StandardException
+                     {
+                         if ( !FileUtil.copyDirectory( getStorageFactory(), sourceDir, targetDir, null, null, true ) )
+                         {
+                             throw StandardException.newException
+                                 (
+                                  SQLState.UNABLE_TO_COPY_FILE_FROM_BACKUP,
+                                  sourceDir.getPath(),
+                                  targetDir.getAbsolutePath()
+                                  );
+                         }
+                         
+                         return null;
+                     }
+                 }
+                 );
+        }
+        catch (IOException ioe) { throw StandardException.plainWrapException( ioe ); }
+        catch (PrivilegedActionException pae) { throw StandardException.plainWrapException( pae ); }
+    }
+
+    /**
+     * Privileged lookup of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ContextService    getContextService()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ContextService>()
+             {
+                 public ContextService run()
+                 {
+                     return ContextService.getFactory();
+                 }
+             }
+             );
+    }
+
+    /**
+     * Privileged Monitor lookup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ModuleFactory  getMonitor()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ModuleFactory>()
+             {
+                 public ModuleFactory run()
+                 {
+                     return Monitor.getMonitor();
+                 }
+             }
+             );
+    }
+
+    
+    /**
+     * Privileged startup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Object bootServiceModule
+        (
+         final boolean create, final Object serviceModule,
+         final String factoryInterface, final Properties properties
+         )
+        throws StandardException
+    {
+        try {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedExceptionAction<Object>()
+                 {
+                     public Object run()
+                         throws StandardException
+                     {
+                         return Monitor.bootServiceModule( create, serviceModule, factoryInterface, properties );
+                     }
+                 }
+                 );
+        } catch (PrivilegedActionException pae)
+        {
+            throw StandardException.plainWrapException( pae );
+        }
+    }
+
+    /**
+     * Privileged startup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Object findServiceModule( final Object serviceModule, final String factoryInterface)
+        throws StandardException
+    {
+        try {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedExceptionAction<Object>()
+                 {
+                     public Object run()
+                         throws StandardException
+                     {
+                         return Monitor.findServiceModule( serviceModule, factoryInterface );
+                     }
+                 }
+                 );
+        } catch (PrivilegedActionException pae)
+        {
+            throw StandardException.plainWrapException( pae );
+        }
+    }
+
 }
