@@ -21,7 +21,7 @@
 
 package org.apache.derby.impl.jdbc;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.types.VariableSizeDataValue;
 
@@ -60,10 +60,15 @@ import java.sql.Blob;
 
 import java.io.InputStream;
 import java.io.Reader;
+import java.sql.Array;
+import java.sql.NClob;
+import java.sql.ParameterMetaData;
+import java.sql.Ref;
+import java.sql.RowId;
+import java.sql.SQLXML;
 import java.sql.Types;
 
 import org.apache.derby.iapi.jdbc.BrokeredConnectionControl;
-import org.apache.derby.iapi.jdbc.EngineParameterMetaData;
 import org.apache.derby.iapi.jdbc.EnginePreparedStatement;
 import org.apache.derby.iapi.services.loader.GeneratedClass;
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
@@ -71,15 +76,10 @@ import org.apache.derby.iapi.types.StringDataValue;
 import org.apache.derby.iapi.util.InterruptStatus;
 
 /**
- *
  * EmbedPreparedStatement is a local JDBC statement.
-  <P><B>Supports</B>
-   <UL>
-   <LI> JSR169
-   </UL>
+ * It supports JDBC 4.1.
  */
-public class EmbedPreparedStatement
-	extends EmbedStatement
+public class EmbedPreparedStatement extends EmbedStatement
 	implements EnginePreparedStatement
 {
 
@@ -143,7 +143,7 @@ public class EmbedPreparedStatement
 				checkRequiresCallableStatement(activation);
 
 			//bug 4838 - save the auto-generated key information in activation. keeping this
-			//information in lcc will not work work as it can be tampered by a nested trasaction
+			//information in lcc will not work as it can be tampered by a nested transaction
   				if (autoGeneratedKeys == Statement.RETURN_GENERATED_KEYS)
   					activation.setAutoGeneratedKeysResultsetInfo(columnIndexes, columnNames);
 
@@ -227,7 +227,7 @@ public class EmbedPreparedStatement
 	 *      </code>
 	 *
 	 * In the above case the PreparedStatement will actually be a 
-	 * BrokeredPreparedStatement40 object. Hence when we call 
+     * BrokeredPreparedStatement object. Hence when we call
 	 * bcc.onStatementClose and pass the PreparedStatement that caused it
 	 * applicationStatement will be the appropriate choice since it will 
 	 * contain the appropriate instance of PreparedStatement in each case
@@ -455,11 +455,6 @@ public class EmbedPreparedStatement
 
 	}
 
-	/*
-	** Methods using BigDecimal, moved back into EmbedPreparedStatement
-	** since our small device implementation now requires CDC/FP 1.1, which
-    ** supports BigDecimal.
-	*/
 	/**
      * Set a parameter to a java.lang.BigDecimal value.  
      * The driver converts this to a SQL NUMERIC value when
@@ -958,7 +953,7 @@ public class EmbedPreparedStatement
         // in that case the cast to int would not be appropriate.
         if ( !lengthLess && length > Integer.MAX_VALUE ) {
             throw newSQLException(SQLState.LANG_OUTSIDE_RANGE_FOR_DATATYPE,
-               getEmbedParameterSetMetaData().getParameterTypeName(
+               getParameterMetaData().getParameterTypeName(
                    parameterIndex));
         }
 
@@ -1019,9 +1014,8 @@ public class EmbedPreparedStatement
      *  ignored if the parameter is not a user-defined type or REF
      * @exception SQLException if a database access error occurs or
      * this method is called on a closed <code>PreparedStatement</code>
-     * @exception SQLFeatureNotSupportedException if Derby does not
-     *   support the type specified in the {@code sqlType} parameter
-     *
+     * @exception java.sql.SQLFeatureNotSupportedException if Derby does not
+     * support the type specified in the {@code sqlType} parameter
      */
     public void setNull(int paramIndex,
         int sqlType,
@@ -1046,7 +1040,7 @@ public class EmbedPreparedStatement
 	  // away
   	  synchronized (getConnectionSynchronization()) {
   			if (batchStatements == null)
-  				batchStatements = new Vector();
+  				batchStatements = new Vector<Object>();
 
           //get a clone of the parameterValueSet and save it in the vector
           //which will be used later on at the time of batch execution.
@@ -1161,6 +1155,32 @@ public class EmbedPreparedStatement
 		return rMetaData;
 	}
 
+    /**
+     * JDBC 2.0
+     *
+     * Set a REF(&lt;structured-type&gt;) parameter.
+     *
+     * @param i the first parameter is 1, the second is 2, ...
+     * @param x an object representing data of an SQL REF Type
+     * @exception SQLException Feature not implemented for now.
+     */
+    public final void setRef(int i, Ref x) throws SQLException {
+        throw Util.notImplemented();
+    }
+
+    /**
+     * JDBC 2.0
+     *
+     * Set an Array parameter.
+     *
+     * @param i the first parameter is 1, the second is 2, ...
+     * @param x an object representing an SQL array
+     * @exception SQLException Feature not implemented for now.
+     */
+    public final void setArray(int i, Array x) throws SQLException {
+        throw Util.notImplemented();
+    }
+
     //----------------------------------------------------------------------
     // Advanced features:
 
@@ -1546,55 +1566,21 @@ public class EmbedPreparedStatement
 	}
 
 
-	/**
-		Check the parameterINdex is in range and return the
-		array of type descriptors.
-
-		@exception SQLException parameter is out of range
-	*/
-	protected final DataTypeDescriptor[] getTypes(int parameterIndex)
-		throws SQLException {
-
-		DataTypeDescriptor[] types = preparedStatement.getParameterTypes();
-
-		if (types == null) {
-			throw newSQLException(SQLState.NO_INPUT_PARAMETERS);
-		}
-
-		/* Check that the parameterIndex is in range. */
-		if (parameterIndex < 1 ||
-				parameterIndex > types.length) {
-
-			/* This message matches the one used by the DBMS */
-			throw newSQLException(SQLState.LANG_INVALID_PARAM_POSITION, 
-            new Integer(parameterIndex), new Integer(types.length));
-		}
-		return types;
-	}
-
 	/**
 		Get the target JDBC type for a parameter. Will throw exceptions
-		if the parameter index is out of range
+		if the parameter index is out of range. The parameterIndex is 1-based.
 
 		@exception SQLException parameter is out of range
 	*/
 	protected int getParameterJDBCType(int parameterIndex)
-		throws SQLException {
+		throws SQLException
+    {
+        try {
+            DataTypeDescriptor dtd = preparedStatement.getParameterType( parameterIndex-1 );
 
-		DataTypeDescriptor[] types = getTypes(parameterIndex);
-
-		int type = types[parameterIndex -1] == null ? 
-			Types.OTHER :
-			types[parameterIndex - 1].getTypeId().getJDBCTypeId();
-
-		if (SanityManager.DEBUG) {
-			//int pmType = getEmbedParameterSetMetaData().getParameterType(parameterIndex);
-			//if (type != pmType) {
-				//SanityManager.THROWASSERT("MISMATCH PARAMETER META DATA param " + parameterIndex + " types " + type + " != " + pmType + "\n" + SQLText);
-			//}
-		}
-
-		return type;
+            return (dtd == null) ? Types.OTHER : dtd.getTypeId().getJDBCTypeId();
+            
+		} catch (StandardException t) { throw EmbedResultSet.noStateChangeException(t); }
 	}
 
     /**
@@ -1605,9 +1591,14 @@ public class EmbedPreparedStatement
      * @throws SQLException if parameter is out of range
      */
     protected final String getParameterSQLType(int parameterIndex)
-            throws SQLException {
-        DataTypeDescriptor[] pTypes = getTypes(parameterIndex);
-        return pTypes[parameterIndex-1].getTypeName();
+            throws SQLException
+    {
+        try {
+            DataTypeDescriptor dtd = preparedStatement.getParameterType( parameterIndex-1 );
+
+            return (dtd == null) ? null : dtd.getTypeName();
+            
+		} catch (StandardException t) { throw EmbedResultSet.noStateChangeException(t); }
     }
 
     /**
@@ -1623,7 +1614,7 @@ public class EmbedPreparedStatement
 		checkStatus();
 
 		if (scale < 0)
-			throw newSQLException(SQLState.BAD_SCALE_VALUE, new Integer(scale));
+			throw newSQLException(SQLState.BAD_SCALE_VALUE, scale);
 		
 		try {
 
@@ -1649,25 +1640,6 @@ public class EmbedPreparedStatement
 		}
 	}
 
-
-	/**
-    * Immitate the function in JDBC 3.0
-    *
-    * Retrieves the number, types and properties of this PreparedStatement
-    * object's parameters.
-    *
-    * @return a EngineParameterMetaData object that contains information about the
-    * number, types and properties of this PreparedStatement object's parameters.
-    * @exception SQLException if a database access error occurs
-	*/
-	private EngineParameterMetaData getEmbedParameterSetMetaData()
-    	throws SQLException
-	{
-	  checkExecStatus();
-	  return new EmbedParameterSetMetaData(
-				getParms(), preparedStatement.getParameterTypes());
-
-	}
 	/**
     * JDBC 3.0
     *
@@ -1684,6 +1656,24 @@ public class EmbedPreparedStatement
 		throw Util.notImplemented();
 	}
 
+    /**
+     * JDBC 3.0
+     *
+     * Retrieves the number, types and properties of this PreparedStatement
+     * object's parameters.
+     *
+     * @return a ParameterMetaData object that contains information about the
+     * number, types and properties of this PreparedStatement object's
+     * parameters.
+     * @exception SQLException if a database access error occurs
+     */
+    public final ParameterMetaData getParameterMetaData()
+            throws SQLException {
+        checkStatus();
+        return new EmbedParameterSetMetaData(
+                getParms(), preparedStatement.getParameterTypes());
+    }
+
 	//
 	// methods to be overridden in subimplementations
 	// that want to stay within their subimplementation.
@@ -1721,9 +1711,9 @@ public class EmbedPreparedStatement
 
 	final SQLException dataTypeConversion(int column, String sourceType)
 		throws SQLException {
-		SQLException se = newSQLException(SQLState.LANG_DATA_TYPE_GET_MISMATCH, getEmbedParameterSetMetaData().getParameterTypeName(column),
+        return newSQLException(SQLState.LANG_DATA_TYPE_GET_MISMATCH,
+            getParameterMetaData().getParameterTypeName(column),
 			sourceType);
-		return se;
 	}
         /**
          * This method is used to initialize the BrokeredConnectionControl 
@@ -1755,7 +1745,7 @@ public class EmbedPreparedStatement
          *      </code>
          *
          * In the above case the PreparedStatement will actually be a 
-         * BrokeredPreparedStatement40 object. Hence when we call 
+         * BrokeredPreparedStatement object. Hence when we call
          * bcc.onStatementClose and pass the PreparedStatement that caused it
          * applicationStatement will be the appropriate choice since it will 
          * contain the appropriate instance of PreparedStatement in each case
@@ -1939,6 +1929,41 @@ public class EmbedPreparedStatement
         }
     }
 
+    public final void setRowId(int parameterIndex, RowId x) throws SQLException {
+        throw Util.notImplemented();
+    }
+
+    public final void setNString(int index, String value) throws SQLException {
+        throw Util.notImplemented();
+    }
+
+    public final void setNCharacterStream(int parameterIndex, Reader value)
+            throws SQLException {
+        throw Util.notImplemented();
+    }
+
+    public final void setNCharacterStream(int index, Reader value, long length) throws SQLException {
+        throw Util.notImplemented();
+    }
+
+    public final void setNClob(int parameterIndex, Reader reader)
+            throws SQLException {
+        throw Util.notImplemented();
+    }
+
+    public final void setNClob(int index, NClob value) throws SQLException {
+        throw Util.notImplemented();
+    }
+
+    public final void setNClob(int parameterIndex, Reader reader, long length)
+            throws SQLException {
+        throw Util.notImplemented();
+    }
+
+    public final void setSQLXML(int parameterIndex, SQLXML xmlObject) throws SQLException {
+        throw Util.notImplemented();
+    }
+
     public final long getVersionCounter() throws SQLException {
         return preparedStatement.getVersionCounter();
     }
