@@ -489,6 +489,11 @@ final class LOBStreamControl {
         return charCount;
     }
 
+    //
+    // This method in java.lang.Object was deprecated as of build 167
+    // of JDK 9. See DERBY-6932.
+    //
+    @SuppressWarnings("deprecation")
     protected void finalize() throws Throwable {
         free();
     }
