@@ -29,6 +29,7 @@ import org.apache.derby.iapi.reference.Limits;
 
 import org.apache.derby.iapi.error.StandardException;
 
+import org.apache.derby.iapi.services.context.Context;
 import org.apache.derby.iapi.services.context.ContextService;
 import org.apache.derby.iapi.services.daemon.Serviceable;
 
@@ -40,7 +41,7 @@ import org.apache.derby.iapi.services.property.PropertyFactory;
 import org.apache.derby.iapi.store.access.TransactionController;
 import org.apache.derby.iapi.services.property.PropertySetCallback;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.reference.Attribute;
 
@@ -54,6 +55,8 @@ import org.apache.derby.iapi.sql.dictionary.UserDescriptor;
 
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
+import java.security.PrivilegedAction;
+import java.security.AccessController;
 
 import java.io.Serializable;
 import java.io.UnsupportedEncodingException;
@@ -187,12 +190,12 @@ public abstract class AuthenticationServiceBase
 			// at boot stage.
 			//
 			store = (AccessFactory)
-				Monitor.getServiceModule(this, AccessFactory.MODULE);
+				getServiceModule(this, AccessFactory.MODULE);
 			// register to be notified upon db properties changes
 			// _only_ if we're on a database context of course :)
 
 			PropertyFactory pf = (PropertyFactory)
-				Monitor.getServiceModule(this, org.apache.derby.iapi.reference.Module.PropertyFactory);
+				getServiceModule(this, org.apache.derby.iapi.reference.Module.PropertyFactory);
 			if (pf != null)
 				pf.addPropertySetNotification(this);
 
@@ -300,7 +303,7 @@ public abstract class AuthenticationServiceBase
         if ( store == null ) { return null; }
         else
         {
-            return store.getTransaction( ContextService.getFactory().getCurrentContextManager() );
+            return store.getTransaction( getContextService().getCurrentContextManager() );
         }
     }
 
@@ -332,7 +335,7 @@ public abstract class AuthenticationServiceBase
     protected   String  getServiceName()
     {
         if ( store == null ) { return null; }
-        else { return Monitor.getServiceName( store ); }
+        else { return getServiceName( store ); }
     }
 
 	public String getDatabaseProperty(String key) {
@@ -344,7 +347,7 @@ public abstract class AuthenticationServiceBase
 
 		  if (store != null)
 			tc = store.getTransaction(
-                ContextService.getFactory().getCurrentContextManager());
+                getContextService().getCurrentContextManager());
 
 		  propertyValue =
 			PropertyUtil.getDatabaseProperty(tc, key);
@@ -457,7 +460,7 @@ public abstract class AuthenticationServiceBase
 
                 if ( passwordLifetime < 0L ) { passwordLifetime = 0L; }
 
-                return new Long( passwordLifetime );
+                return passwordLifetime;
             } catch (Exception e) { return null; }
     }
     /** Parse the value of the password expiration threshold property. Return null if it is bad. */
@@ -467,7 +470,7 @@ public abstract class AuthenticationServiceBase
                 double  expirationThreshold = Double.parseDouble( expirationThresholdString );
 
                 if ( expirationThreshold <= 0L ) { return null; }
-                else { return new Double( expirationThreshold ); }
+                else { return expirationThreshold; }
             } catch (Exception e) { return null; }
     }
     
@@ -667,7 +670,7 @@ public abstract class AuthenticationServiceBase
      */
     private static DataDictionary getDataDictionary() {
         LanguageConnectionContext lcc = (LanguageConnectionContext)
-            ContextService.getContext(LanguageConnectionContext.CONTEXT_ID);
+            getContext(LanguageConnectionContext.CONTEXT_ID);
         return lcc.getDataDictionary();
     }
 
@@ -817,4 +820,77 @@ public abstract class AuthenticationServiceBase
         return StringUtil.toHexString(passwordSubstitute, 0,
                                       passwordSubstitute.length);
     }
+
+    /**
+     * Privileged lookup of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ContextService    getContextService()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ContextService>()
+             {
+                 public ContextService run()
+                 {
+                     return ContextService.getFactory();
+                 }
+             }
+             );
+    }
+
+    /**
+     * Privileged lookup of a Context. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Context    getContext( final String contextID )
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<Context>()
+             {
+                 public Context run()
+                 {
+                     return ContextService.getContext( contextID );
+                 }
+             }
+             );
+    }
+
+    /**
+     * Privileged service name lookup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  String getServiceName( final Object serviceModule )
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<String>()
+             {
+                 public String run()
+                 {
+                     return Monitor.getServiceName( serviceModule );
+                 }
+             }
+             );
+    }
+
+    /**
+     * Privileged module lookup. Must be package protected so that user code
+     * can't call this entry point.
+     */
+    static  Object getServiceModule( final Object serviceModule, final String factoryInterface )
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<Object>()
+             {
+                 public Object run()
+                 {
+                     return Monitor.getServiceModule( serviceModule, factoryInterface );
+                 }
+             }
+             );
+    }
+
 }
