@@ -30,13 +30,14 @@ import java.io.IOException;
 import java.io.File;
 import java.net.MalformedURLException;
 import java.net.URL;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import java.sql.SQLException;
 
-final class ImportReadData implements java.security.PrivilegedExceptionAction {
+final class ImportReadData implements java.security.PrivilegedExceptionAction<Object> {
   //Read data from this file
   private String inputFileName;
-
+  //The number of header lines to be skipped.
+  private short skipLines;
   private int[] columnWidths;
   private int rowWidth;
   private char[] tempString;
@@ -108,7 +109,7 @@ final class ImportReadData implements java.security.PrivilegedExceptionAction {
   // variables realted to reading lob data from files.
   private ImportLobFile[] lobFileHandles; // lob file handle object 
   private String lobFileName; // current file name
-  private int lobOffset; // offset of the current large object
+  private long lobOffset; // offset of the current large object
   private int lobLength; //length of the current large object
 
 
@@ -143,8 +144,9 @@ final class ImportReadData implements java.security.PrivilegedExceptionAction {
   }
   //inputFileName: File to read data from
   //controlFileReader: File used to interpret data in the inputFileName
-  ImportReadData(String inputFileName, ControlInfo controlFileReader)
+  ImportReadData(String inputFileName, ControlInfo controlFileReader,short skipLines)
   throws Exception {
+    this.skipLines=skipLines;
     this.inputFileName = inputFileName;
     this.controlFileReader = controlFileReader;
 
@@ -171,6 +173,20 @@ final class ImportReadData implements java.security.PrivilegedExceptionAction {
     readNextToken(recordSeparator, 0, recordSeparatorLength, true);
   }
 
+
+  /**if skipHeaderLines is greater than 0, ignore skipHeaderLines number of lines. The way to do that is to just
+  *  look for the record separator
+ 	* @exception	Exception if there is an error
+	*/
+  protected void ignoreHeaderLines() throws Exception {
+    for(int i =0;i<skipLines;i++){
+       if(!readNextToken(recordSeparator, 0, recordSeparatorLength, true))
+          throw LoadError.unexpectedEndOfFile(lineNumber+1);
+       lineNumber++;
+    }
+  }
+
+
   /** load the column types from the meta data line to be analyzed
     * later in the constructor of the ImportResultSetMetaData.
 	*/
@@ -649,7 +665,8 @@ final class ImportReadData implements java.security.PrivilegedExceptionAction {
        //do uppercase because the ui shows the values as True and False
        if (hasColumnDefinition){
           ignoreFirstRow();
-	   }
+       }
+       ignoreHeaderLines();
     }
     if (formatCode == DEFAULT_FORMAT_CODE)
        readVal=readNextDelimitedRow(returnStringArray);
@@ -1037,7 +1054,7 @@ final class ImportReadData implements java.security.PrivilegedExceptionAction {
         lobLength = Integer.parseInt(lobLocationStr.substring(
                                      lengthIndex + 1, 
                                      lobLocationStr.length() -1));
-        lobOffset = Integer.parseInt(lobLocationStr.substring(
+        lobOffset = Long.parseLong(lobLocationStr.substring(
                                      offsetIndex+1, 
                                      lengthIndex));
         lobFileName = lobLocationStr.substring(0 , offsetIndex);
