@@ -21,17 +21,24 @@
 
 package org.apache.derby.impl.services.cache;
 
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.atomic.AtomicLong;
 import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.reference.Module;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.cache.CacheManager;
 import org.apache.derby.iapi.services.cache.Cacheable;
 import org.apache.derby.iapi.services.cache.CacheableFactory;
 import org.apache.derby.iapi.services.daemon.DaemonService;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.iapi.services.jmx.ManagementService;
+import org.apache.derby.iapi.services.monitor.Monitor;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.util.Matchable;
+import org.apache.derby.mbeans.CacheManagerMBean;
 
 /**
  * A cache manager based on the utilities found in the
@@ -62,6 +69,19 @@ final class ConcurrentCache implements CacheManager {
     /** Replacement policy to be used for this cache. */
     private final ReplacementPolicy replacementPolicy;
 
+    // Fields used by the MBean that monitors this instance.
+
+    /** The identifier of the MBean that allows monitoring of this instance. */
+    private Object mbean;
+    /** Flag that tells if hit/miss/eviction counts should be collected. */
+    private volatile boolean collectAccessCounts;
+    /** The number of cache hits. */
+    private final AtomicLong hits = new AtomicLong();
+    /** The number of cache misses. */
+    private final AtomicLong misses = new AtomicLong();
+    /** The number of evictions from the cache. */
+    private final AtomicLong evictions = new AtomicLong();
+
     /**
      * Flag that indicates whether this cache instance has been shut down. When
      * it has been stopped, <code>find()</code>, <code>findCached()</code> and
@@ -187,6 +207,7 @@ final class ConcurrentCache implements CacheManager {
         CacheEntry entry = cache.remove(key);
         entry.getCacheable().clearIdentity();
         entry.setCacheable(null);
+        countEviction();
     }
 
     /**
@@ -277,11 +298,13 @@ final class ConcurrentCache implements CacheManager {
                 // The object is already cached. Increase the use count and
                 // return it.
                 entry.keep(true);
+                countHit();
                 return item;
             } else {
                 // The object is not cached. Insert the entry into a free
                 // slot and retrieve a reusable Cacheable.
                 item = insertIntoFreeSlot(key, entry);
+                countMiss();
             }
         } finally {
             entry.unlock();
@@ -322,6 +345,7 @@ final class ConcurrentCache implements CacheManager {
         CacheEntry entry = cache.get(key);
         if (entry == null) {
             // No such object was found in the cache.
+            countMiss();
             return null;
         }
 
@@ -332,11 +356,15 @@ final class ConcurrentCache implements CacheManager {
             // for it to complete so that we don't return a cacheable that
             // isn't fully initialized.
             entry.waitUntilIdentityIsSet();
+
             // Return the cacheable. If the entry was removed right before we
             // locked it, getCacheable() returns null and so should we do.
             Cacheable item = entry.getCacheable();
             if (item != null) {
+                countHit();
                 entry.keep(true);
+            } else {
+                countMiss();
             }
             return item;
         } finally {
@@ -599,6 +627,7 @@ final class ConcurrentCache implements CacheManager {
         if (cleaner != null) {
             cleaner.unsubscribe();
         }
+        deregisterMBean();
     }
 
     /**
@@ -681,4 +710,114 @@ final class ConcurrentCache implements CacheManager {
         }
         return values;
     }
+
+    @Override
+    public void registerMBean(String dbName) throws StandardException {
+        if (SanityManager.DEBUG) {
+            SanityManager.ASSERT(mbean == null, "registerMBean() called twice");
+        }
+
+        ManagementService managementService =
+                (ManagementService) getSystemModule(Module.JMX);
+
+        if (managementService != null) {
+            mbean = managementService.registerMBean(
+                    new ConcurrentCacheMBeanImpl(this),
+                    CacheManagerMBean.class,
+                    "type=CacheManager,name=" + name +
+                    ",db=" + managementService.quotePropertyValue(dbName));
+        }
+    }
+
+    @Override
+    public void deregisterMBean() {
+        if (mbean != null) {
+            ManagementService managementService =
+                (ManagementService) getSystemModule(Module.JMX);
+            if (managementService != null) {
+                managementService.unregisterMBean(mbean);
+            }
+            mbean = null;
+        }
+    }
+
+    /** Count a cache hit. */
+    private void countHit() {
+        if (collectAccessCounts) {
+            hits.getAndIncrement();
+        }
+    }
+
+    /** Count a cache miss. */
+    private void countMiss() {
+        if (collectAccessCounts) {
+            misses.getAndIncrement();
+        }
+    }
+
+    /** Count an eviction from the cache. */
+    private void countEviction() {
+        if (collectAccessCounts) {
+            evictions.getAndIncrement();
+        }
+    }
+
+    /** Enable or disable collection of hit/miss/eviction counts. */
+    void setCollectAccessCounts(boolean collect) {
+        collectAccessCounts = collect;
+    }
+
+    /** Check if collection of hit/miss/eviction counts is enabled. */
+    boolean getCollectAccessCounts() {
+        return collectAccessCounts;
+    }
+
+    /** Get the number of cache hits. */
+    long getHitCount() {
+        return hits.get();
+    }
+
+    /** Get the number of cache misses. */
+    long getMissCount() {
+        return misses.get();
+    }
+
+    /** Get the number of evictions from the cache. */
+    long getEvictionCount() {
+        return evictions.get();
+    }
+
+    /** Get the maximum number of entries in the cache. */
+    long getMaxEntries() {
+        return maxSize;
+    }
+
+    /** Get the number of allocated entries in the cache. */
+    long getAllocatedEntries() {
+        return replacementPolicy.size();
+    }
+
+    /** Get the number of cached objects. */
+    long getUsedEntries() {
+        return cache.size();
+    }
+    
+    /**
+     * Privileged module lookup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private static  Object getSystemModule( final String factoryInterface )
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<Object>()
+             {
+                 public Object run()
+                 {
+                     return Monitor.getSystemModule( factoryInterface );
+                 }
+             }
+             );
+    }
+
 }
