@@ -21,6 +21,8 @@
 package org.apache.derby.impl.services.daemon;
 
 import java.io.PrintWriter;
+import java.security.PrivilegedAction;
+import java.security.AccessController;
 import java.sql.Connection;
 import java.util.ArrayList;
 import java.util.List;
@@ -30,14 +32,14 @@ import org.apache.derby.catalog.types.StatisticsImpl;
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.db.Database;
 import org.apache.derby.iapi.error.ExceptionSeverity;
-import org.apache.derby.iapi.error.ShutdownException;
+import org.apache.derby.shared.common.error.ShutdownException;
 import org.apache.derby.iapi.reference.Property;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.context.ContextManager;
 import org.apache.derby.iapi.services.context.ContextService;
 import org.apache.derby.iapi.services.daemon.IndexStatisticsDaemon;
 import org.apache.derby.iapi.services.property.PropertyUtil;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.stream.HeaderPrintWriter;
 import org.apache.derby.iapi.services.uuid.UUIDFactory;
 import org.apache.derby.iapi.services.monitor.Monitor;
@@ -160,7 +162,7 @@ public class IndexStatisticsDaemonImpl
      * Note that the descriptor isn't removed before the work has
      * been completed.
      */
-    private final ArrayList queue = new ArrayList(MAX_QUEUE_LENGTH);
+    private final ArrayList<TableDescriptor> queue = new ArrayList<TableDescriptor>(MAX_QUEUE_LENGTH);
     /**
      * The thread in which the index statistics refresh operation is being
      * executed, if any. Created as needed, but there will only be one
@@ -227,7 +229,7 @@ public class IndexStatisticsDaemonImpl
         this.db = db;
         this.dbOwner = userName;
         this.databaseName = databaseName;
-        this.ctxMgr = ContextService.getFactory().newContextManager();
+        this.ctxMgr = getContextService().newContextManager();
         this.timeOfCreation = System.currentTimeMillis();
         trace(0, "created{log=" + doLog + ", traceLog=" +
                 traceToDerbyLog + ", traceOut=" + traceToStdOut +
@@ -280,7 +282,7 @@ public class IndexStatisticsDaemonImpl
                 if (runningThread == null) {
                     //DERBY-5582. Make sure the thread is in the derby group
                     // to avoid potential security manager issues
-                    runningThread = Monitor.getMonitor().getDaemonThread(this, "index-stat-thread", false);
+                    runningThread = BasicDaemon.getMonitor().getDaemonThread(this, "index-stat-thread", false);
                     runningThread.start();
                 }
             }
@@ -306,7 +308,7 @@ public class IndexStatisticsDaemonImpl
             // duplicates should yield acceptable performance. Also, we don't
             // look for duplicates if the queue is already full.
             for (int i=0; i < queue.size(); i++) {
-                TableDescriptor work = (TableDescriptor)queue.get(i);
+                TableDescriptor work = queue.get(i);
                 if (work.tableNameEquals(table, schema)) {
                     accept = false;
                     break;
@@ -498,9 +500,9 @@ public class IndexStatisticsDaemonImpl
             // flag.  The above loop controls if we drop single column unique
             // index stats or not.  In all cases we are going to drop 
             // stats with no associated index (orphaned stats).
-
-            List existingStats = td.getStatistics();
             
+
+            List<StatisticsDescriptor> existingStats = td.getStatistics();
             StatisticsDescriptor[] stats = (StatisticsDescriptor[])
                     existingStats.toArray(
                         new StatisticsDescriptor[existingStats.size()]);
@@ -843,7 +845,7 @@ public class IndexStatisticsDaemonImpl
         // Implement the outer-level exception handling here.
         try {
             // DERBY-5088: Factory-call may fail.
-            ctxService = ContextService.getFactory();
+            ctxService = getContextService();
             ctxService.setCurrentContextManager(ctxMgr);
             processingLoop();
         } catch (ShutdownException se) {
@@ -925,7 +927,7 @@ public class IndexStatisticsDaemonImpl
                         trace(1, "queue empty");
                         break;
                     }
-                    td = (TableDescriptor)queue.get(0);
+                    td = queue.get(0);
                 }
                 try {
                     start = System.currentTimeMillis();
@@ -1334,6 +1336,31 @@ public class IndexStatisticsDaemonImpl
                 "> " + trace;
     }
 
+    /**
+     * Privileged lookup of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ContextService    getContextService()
+    {
+        if ( System.getSecurityManager() == null )
+        {
+            return ContextService.getFactory();
+        }
+        else
+        {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedAction<ContextService>()
+                 {
+                     public ContextService run()
+                     {
+                         return ContextService.getFactory();
+                     }
+                 }
+                 );
+        }
+    }    
+
     /**
      * Support class used to compare keys when scanning indexes.
      */
@@ -1389,7 +1416,7 @@ public class IndexStatisticsDaemonImpl
          * @param index row index
          * @return {@code -1} if the current and previous key are identical,
          *      the index of the changed part of the key otherwise
-         *      ([0, key length>)
+         *      ([0, key length&gt;)
          * @throws StandardException if comparing the two keys fails
          */
         public int compareWithPrevKey(int index)
