@@ -21,61 +21,50 @@
 
 package org.apache.derby.impl.sql;
 
-import	org.apache.derby.catalog.Dependable;
-import	org.apache.derby.catalog.DependableFinder;
-
-import org.apache.derby.iapi.services.context.ContextService;
-import org.apache.derby.iapi.services.context.ContextManager;
-
-import org.apache.derby.iapi.services.monitor.Monitor;
-
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
-import org.apache.derby.iapi.services.stream.HeaderPrintWriter;
-import org.apache.derby.iapi.services.cache.Cacheable;
-
-import org.apache.derby.catalog.UUID;
-import org.apache.derby.iapi.services.uuid.UUIDFactory;
-import org.apache.derby.iapi.util.ByteArray;
-import org.apache.derby.iapi.util.ReuseFactory;
-
-import org.apache.derby.iapi.sql.dictionary.DataDictionary;
-import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
-import org.apache.derby.iapi.sql.dictionary.SPSDescriptor;
-
-import org.apache.derby.iapi.sql.ParameterValueSet;
-import org.apache.derby.iapi.sql.PreparedStatement;
-import org.apache.derby.iapi.sql.Statement;
-import org.apache.derby.iapi.types.DataTypeDescriptor;
-import org.apache.derby.iapi.sql.ResultColumnDescriptor;
-import org.apache.derby.iapi.sql.ResultDescription;
-import org.apache.derby.iapi.sql.ResultSet;
-import org.apache.derby.iapi.sql.Activation;
-
-import org.apache.derby.iapi.sql.execute.ConstantAction;
-import org.apache.derby.iapi.sql.execute.ExecCursorTableReference;
-import org.apache.derby.iapi.sql.execute.ExecPreparedStatement;
-
-import org.apache.derby.iapi.sql.depend.DependencyManager;
-import org.apache.derby.iapi.sql.depend.Provider;
-
-import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
-import org.apache.derby.iapi.sql.conn.StatementContext;
-
-import org.apache.derby.impl.sql.compile.CursorNode;
-import org.apache.derby.impl.sql.compile.StatementNode;
-
-import org.apache.derby.iapi.error.StandardException;
-
-import org.apache.derby.iapi.reference.SQLState;
-
-import org.apache.derby.iapi.services.loader.GeneratedClass;
-
-import java.sql.Timestamp;
+import java.security.PrivilegedAction;
+import java.security.AccessController;
 import java.sql.SQLWarning;
+import java.sql.Timestamp;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import org.apache.derby.catalog.Dependable;
+import org.apache.derby.catalog.DependableFinder;
+import org.apache.derby.catalog.UUID;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.cache.Cacheable;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.services.context.ContextService;
+import org.apache.derby.iapi.services.io.ArrayUtil;
+import org.apache.derby.iapi.services.loader.GeneratedClass;
+import org.apache.derby.iapi.services.monitor.ModuleFactory;
+import org.apache.derby.iapi.services.monitor.Monitor;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.services.stream.HeaderPrintWriter;
+import org.apache.derby.iapi.services.uuid.UUIDFactory;
+import org.apache.derby.iapi.sql.Activation;
+import org.apache.derby.iapi.sql.ParameterValueSet;
+import org.apache.derby.iapi.sql.PreparedStatement;
+import org.apache.derby.iapi.sql.ResultDescription;
+import org.apache.derby.iapi.sql.ResultSet;
+import org.apache.derby.iapi.sql.Statement;
+import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
+import org.apache.derby.iapi.sql.conn.StatementContext;
+import org.apache.derby.iapi.sql.depend.DependencyManager;
+import org.apache.derby.iapi.sql.depend.Provider;
+import org.apache.derby.iapi.sql.dictionary.DataDictionary;
+import org.apache.derby.iapi.sql.dictionary.SPSDescriptor;
+import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
+import org.apache.derby.iapi.sql.dictionary.StatementPermission;
+import org.apache.derby.iapi.sql.execute.ConstantAction;
+import org.apache.derby.iapi.sql.execute.ExecCursorTableReference;
+import org.apache.derby.iapi.sql.execute.ExecPreparedStatement;
+import org.apache.derby.iapi.types.DataTypeDescriptor;
+import org.apache.derby.iapi.types.DataTypeUtilities;
+import org.apache.derby.iapi.util.ByteArray;
+import org.apache.derby.impl.sql.compile.CursorNode;
+import org.apache.derby.impl.sql.compile.StatementNode;
 
 /**
  * Basic implementation of prepared statement.
@@ -126,13 +115,12 @@ public class GenericPreparedStatement
 
 	// fields used for cursors
 	protected ExecCursorTableReference	targetTable; 
-	protected ResultColumnDescriptor[]	targetColumns; 
-	protected String[] 					updateColumns; 
+    protected List<String>           updateColumns;
 	protected int 						updateMode;
 
 	protected ConstantAction	executionConstants;
 	protected Object[]	savedObjects;
-	protected List requiredPermissionsList;
+    protected List<StatementPermission> requiredPermissionsList;
 
 	// fields for dependency tracking
 	protected String UUIDString;
@@ -148,7 +136,7 @@ public class GenericPreparedStatement
 	private int inUseCount;
 
 	// true if the statement is being compiled.
-	boolean compilingStatement;
+    private boolean compilingStatement;
 
     /** True if the statement was invalidated while it was being compiled. */
     boolean invalidatedWhileCompiling;
@@ -194,7 +182,7 @@ public class GenericPreparedStatement
 	GenericPreparedStatement() {
 		/* Get the UUID for this prepared statement */
 		UUIDFactory uuidFactory = 
-			Monitor.getMonitor().getUUIDFactory();
+			getMonitor().getUUIDFactory();
 
 		UUIDValue = uuidFactory.createUUID();
 		UUIDString = UUIDValue.toString();
@@ -239,6 +227,29 @@ public class GenericPreparedStatement
         return isValid && (activationClass != null) && !compilingStatement;
     }
 
+    /** Check if this statement is currently being compiled. */
+    final synchronized boolean isCompiling() {
+        return compilingStatement;
+    }
+
+    /**
+     * Signal that the statement is about to be compiled. This will block
+     * others from attempting to compile it.
+     */
+    final synchronized void beginCompiling() {
+        compilingStatement = true;
+        setActivationClass(null);
+    }
+
+    /**
+     * Signal that we're done compiling the statement and unblock others
+     * that are waiting for the compilation to finish.
+     */
+    final synchronized void endCompiling() {
+        compilingStatement = false;
+        notifyAll();
+    }
+
 	public void rePrepare(LanguageConnectionContext lcc) 
 		throws StandardException {
         rePrepare(lcc, false);
@@ -453,7 +464,7 @@ recompileOutOfDatePlan:
 				lccToUse.validateStmtExecution(executionConstants);
 			}
 
-			ResultSet resultSet = null;
+            ResultSet resultSet;
 			try {
 	
 				resultSet = activation.execute();
@@ -476,7 +487,11 @@ recompileOutOfDatePlan:
 				statementContext.clearSavePoint();
 			}
 
-			lccToUse.popStatementContext(statementContext, null);					
+            lccToUse.popStatementContext(statementContext, null);
+
+            if (activation.getSQLSessionContextForChildren() != null) {
+                lccToUse.popNestedSessionContext(activation);
+            }
 
 			if (activation.isSingleExecution() && resultSet.isClosed())
 			{
@@ -498,9 +513,33 @@ recompileOutOfDatePlan:
 	}
 
 	public DataTypeDescriptor[]	getParameterTypes()	{
-		return paramTypeDescriptors;
+        return ArrayUtil.copy(paramTypeDescriptors);
 	}
 
+    /** Return the type of the parameter (0-based indexing) */
+    public DataTypeDescriptor  getParameterType( int idx ) throws StandardException
+    {
+		if ( paramTypeDescriptors == null )
+        {
+			throw StandardException.newException( SQLState.NO_INPUT_PARAMETERS );
+		}
+
+		/* Check that the parameterIndex is in range. */
+		if ( (idx < 0) || (idx >= paramTypeDescriptors.length) )
+        {
+			/* This message matches the one used by the DBMS */
+			throw StandardException.newException
+                (
+                 SQLState.LANG_INVALID_PARAM_POSITION, 
+                 idx+1,
+                 paramTypeDescriptors.length
+                 );
+		}
+
+        return paramTypeDescriptors[ idx ];
+    }
+    
+
 	public String getSource() {
 		return (sourceTxt != null) ?
 			sourceTxt : 
@@ -581,7 +620,7 @@ recompileOutOfDatePlan:
 	 */
 	public Timestamp getBeginCompileTimestamp()
 	{
-		return beginCompileTimestamp;
+		return DataTypeUtilities.clone( beginCompileTimestamp );
 	}
 
 	/**
@@ -591,7 +630,7 @@ recompileOutOfDatePlan:
 	 */
 	public Timestamp getEndCompileTimestamp()
 	{
-		return endCompileTimestamp;
+		return DataTypeUtilities.clone( endCompileTimestamp );
 	}
 
 	void setCompileTimeWarnings(SQLWarning warnings) {
@@ -717,9 +756,11 @@ recompileOutOfDatePlan:
 	 *
 	 *	@return all the saved objects
 	 */
-	public	final Object[]	getSavedObjects()
+    public final List<Object> getSavedObjects()
 	{
-		return	savedObjects;
+        // Return an unmodifiable view of the underlying array, so that
+        // the caller cannot modify the internal state.
+        return ArrayUtil.asReadOnlyList(savedObjects);
 	}
 
 	//
@@ -827,7 +868,7 @@ recompileOutOfDatePlan:
 			isValid = false;
 
 			// block compiles while we are invalidating
-			compilingStatement = true;
+            beginCompiling();
 		}
 
 		try {
@@ -862,10 +903,7 @@ recompileOutOfDatePlan:
 				}
 			}
 		} finally {
-			synchronized (this) {
-				compilingStatement = false;
-				notifyAll();
-			}
+            endCompiling();
 		}
 	}
 
@@ -992,7 +1030,6 @@ recompileOutOfDatePlan:
 			targetTable = null;
 			updateMode = 0;
 			updateColumns = null;
-			targetColumns = null;
 		}
 
 		// get the result description (null for non-cursor statements)
@@ -1009,14 +1046,7 @@ recompileOutOfDatePlan:
 			/*
 				For cursors, we carry around some extra information.
 			 */
-			CursorInfo cursorInfo = (CursorInfo)qt.getCursorInfo();
-			if (cursorInfo != null)
-			{
-				targetTable = cursorInfo.targetTable;
-				targetColumns = cursorInfo.targetColumns;
-				updateColumns = cursorInfo.updateColumns;
-				updateMode = cursorInfo.updateMode;
-			}
+            setCursorInfo((CursorInfo) qt.getCursorInfo());
 		}
 		isValid = true;
 
@@ -1061,24 +1091,13 @@ recompileOutOfDatePlan:
 		return targetTable;
 	}
 
-	/**
-	 * the target columns of the cursor as a result column list
-	 *
-	 * @return	target columns of the cursor as a result column list
-	 */
-	public ResultColumnDescriptor[]	getTargetColumns() {
-		return targetColumns;
-	}
+    public boolean hasUpdateColumns() {
+        return updateColumns != null && !updateColumns.isEmpty();
+    }
 
-	/**
-	 * the update columns of the cursor as a update column list
-	 *
-	 * @return	update columns of the cursor as a array of strings
-	 */
-	public String[]	getUpdateColumns() 
-	{
-		return updateColumns;
-	}
+    public boolean isUpdateColumn(String columnName) {
+        return updateColumns != null && updateColumns.contains(columnName);
+    }
 
 	/**
 	 * Return the cursor info in a single chunk.  Used
@@ -1089,7 +1108,6 @@ recompileOutOfDatePlan:
 		return new CursorInfo(
 			updateMode,
 			targetTable, 
-			targetColumns,
 			updateColumns);
 	}
 
@@ -1099,7 +1117,6 @@ recompileOutOfDatePlan:
 		{
 			updateMode = cursorInfo.updateMode;
 			targetTable = cursorInfo.targetTable;
-			targetColumns = cursorInfo.targetColumns;
 			updateColumns = cursorInfo.updateColumns;
 		}
 	}
@@ -1203,7 +1220,6 @@ recompileOutOfDatePlan:
 		clone.isAtomic = isAtomic;
 		clone.sourceTxt = sourceTxt;
 		clone.targetTable = targetTable;
-		clone.targetColumns = targetColumns;
 		clone.updateColumns = updateColumns;
 		clone.updateMode = updateMode;	
 		clone.needsSavepoint = needsSavepoint;
@@ -1223,7 +1239,7 @@ recompileOutOfDatePlan:
 			if (!isValid || (inUseCount != 0))
 				return;
 
-			ContextManager cm = ContextService.getFactory().getCurrentContextManager();
+			ContextManager cm = getContextService().getCurrentContextManager();
 			LanguageConnectionContext lcc = 
 				(LanguageConnectionContext) 
 				(cm.getContext(LanguageConnectionContext.CONTEXT_ID));
@@ -1248,6 +1264,7 @@ recompileOutOfDatePlan:
 		}
 	}
 
+    @Override
 	public String toString() {
 		return getObjectName();
 	}
@@ -1256,12 +1273,13 @@ recompileOutOfDatePlan:
 		return false;
 	}
 
-	public void setRequiredPermissionsList( List requiredPermissionsList)
+    public void setRequiredPermissionsList(
+            List<StatementPermission> requiredPermissionsList)
 	{
 		this.requiredPermissionsList = requiredPermissionsList;
 	}
 
-	public List getRequiredPermissionsList()
+    public List<StatementPermission> getRequiredPermissionsList()
 	{
 		return requiredPermissionsList;
 	}
@@ -1290,7 +1308,7 @@ recompileOutOfDatePlan:
     private static class RowCountStatistics {
         private int stalePlanCheckInterval;
         private int executionCount;
-        private ArrayList rowCounts;
+        private ArrayList<Long> rowCounts;
 
         // No synchronization for executionCount. Since it's accessed on
         // every execution, we want to avoid synchronization. Nothing serious
@@ -1311,7 +1329,7 @@ recompileOutOfDatePlan:
         synchronized long getInitialRowCount(int rsNum, long rowCount) {
             // Allocate the list of row counts lazily.
             if (rowCounts == null) {
-                rowCounts = new ArrayList();
+                rowCounts = new ArrayList<Long>();
             }
 
             // Make sure the list is big enough to hold the row count for
@@ -1319,14 +1337,14 @@ recompileOutOfDatePlan:
             if (rsNum >= rowCounts.size()) {
                 int newSize = rsNum + 1;
                 rowCounts.addAll(
-                        Collections.nCopies(newSize - rowCounts.size(), null));
+                                 Collections.nCopies(newSize - rowCounts.size(), (Long) null));
             }
 
             // Get the initial row count for the specified result set, and
             // set it if it is not already set.
-            Long initialCount = (Long) rowCounts.get(rsNum);
+            Long initialCount = rowCounts.get(rsNum);
             if (initialCount == null) {
-                rowCounts.set(rsNum, ReuseFactory.getLong(rowCount));
+                rowCounts.set(rsNum, rowCount);
                 return rowCount;
             } else {
                 return initialCount.longValue();
@@ -1376,4 +1394,42 @@ recompileOutOfDatePlan:
     public long getInitialRowCount(int rsNum, long currentRowCount) {
         return rowCountStats.getInitialRowCount(rsNum, currentRowCount);
     }
+    
+    /**
+     * Privileged lookup of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ContextService    getContextService()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ContextService>()
+             {
+                 public ContextService run()
+                 {
+                     return ContextService.getFactory();
+                 }
+             }
+             );
+    }
+
+    
+    /**
+     * Privileged Monitor lookup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ModuleFactory  getMonitor()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ModuleFactory>()
+             {
+                 public ModuleFactory run()
+                 {
+                     return Monitor.getMonitor();
+                 }
+             }
+             );
+    }
+
 }
