@@ -20,6 +20,10 @@
  */
 package org.apache.derby.impl.sql.catalog;
 
+import java.security.AccessController;
+import java.security.PrivilegedAction;
+import java.util.HashMap;
+
 import org.apache.derby.catalog.SequencePreallocator;
 import org.apache.derby.iapi.db.Database;
 import org.apache.derby.iapi.error.StandardException;
@@ -28,12 +32,15 @@ import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.cache.Cacheable;
 import org.apache.derby.iapi.services.cache.CacheManager;
 import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.services.context.Context;
 import org.apache.derby.iapi.services.context.ContextService;
 import org.apache.derby.iapi.services.i18n.MessageService;
 import org.apache.derby.iapi.services.monitor.Monitor;
 import org.apache.derby.iapi.services.property.PropertyUtil;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
+import org.apache.derby.iapi.sql.dictionary.BulkInsertCounter;
+import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
 import org.apache.derby.iapi.sql.dictionary.SequenceDescriptor;
 import org.apache.derby.iapi.store.access.AccessFactory;
 import org.apache.derby.iapi.store.access.TransactionController;
@@ -104,10 +111,14 @@ public abstract class SequenceUpdater implements Cacheable
     // DataDictionary where this generator is cached.
     protected DataDictionaryImpl _dd;
 
-    // This is the key used to lookup this generator in the cache.
+    //
+    // The following state needs to be reset whenever this Cachable is re-used.
+    //
+    
+    /** This is the key used to lookup this generator in the cache. */
     protected String _uuidString;
 
-    // This is the object which allocates ranges of sequence values
+    /** This is the object which allocates ranges of sequence values */
     protected SequenceGenerator _sequenceGenerator;
 
     ///////////////////////////////////////////////////////////////////////////////////
@@ -169,10 +180,18 @@ public abstract class SequenceUpdater implements Cacheable
      * and identities. See DERBY-5426.
      * </p>
      */
-    protected   StandardException   tooMuchContentionException()
+    private   StandardException   tooMuchContentionException()
     {
-        return StandardException.newException
-            ( SQLState.LANG_TOO_MUCH_CONTENTION_ON_SEQUENCE, _sequenceGenerator.getName() );
+        // If the sequence lives in the SYS schema, then it is used to back an identity column.
+        if ( SchemaDescriptor.STD_SYSTEM_SCHEMA_NAME.equals( _sequenceGenerator.getSchemaName() ) )
+        {
+            return StandardException.newException( SQLState.LOCK_TIMEOUT );
+        }
+        else
+        {
+            return StandardException.newException
+                ( SQLState.LANG_TOO_MUCH_CONTENTION_ON_SEQUENCE, _sequenceGenerator.getName() );
+        }
     }
     
     ///////////////////////////////////////////////////////////////////////////////////
@@ -187,11 +206,32 @@ public abstract class SequenceUpdater implements Cacheable
         // Flush current value to disk. This prevents us from leaking values when DDL
         // is performed. The metadata caches are invalidated and cleared when DDL is performed.
         // We flush the current value to disk on database shutdown also.
+        // The call to updateCurrentValueOnDisk can fail if someone is holding a lock
+        // on the SYS.SYSSEQUENCES row. This can happen if the user disregards our
+        // advice and scans that catalog. This can also happen if the transaction which
+        // creates the sequences is open for a long time and some later sequence creation
+        // causes us to evict the old, uncommitted sequence generator from the cache.
         //
-        if ( _sequenceGenerator != null )
-        {
-            boolean gapClosed = updateCurrentValueOnDisk( null, peekAtCurrentValue() );
+        boolean gapClosed = false;
 
+        try {
+            if ( _sequenceGenerator == null ) { gapClosed = true; }
+            else
+            {
+                gapClosed = updateCurrentValueOnDisk( null, peekAtCurrentValue() );
+            }
+        }
+        catch (StandardException se)
+        {
+            // The too much contention exception is redundant because the problem is logged
+            // by the message below
+            if ( !SQLState.LANG_TOO_MUCH_CONTENTION_ON_SEQUENCE.equals( se.getMessageId() ) )
+            {
+                throw se;
+            }
+        }
+        finally
+        {
             // log an error message if we failed to flush the preallocated values.
             if ( !gapClosed )
             {
@@ -204,10 +244,10 @@ public abstract class SequenceUpdater implements Cacheable
 
                 Monitor.getStream().println( errorMessage );
             }
-        }
 
-        _uuidString = null;
-        _sequenceGenerator = null;
+            _uuidString = null;
+            _sequenceGenerator = null;
+        }
 	}
     
 	public boolean isDirty() { return false; }
@@ -298,6 +338,35 @@ public abstract class SequenceUpdater implements Cacheable
     //
     ///////////////////////////////////////////////////////////////////////////////////
 
+    /**
+     * <p>
+     * Reset the sequence generator to a new start value. This is used by the special
+     * bulk-insert optimization in InsertResultSet.
+     * </p>
+     */
+    public synchronized void reset( Long newValue )
+        throws StandardException
+    {
+        // first try to reset on disk
+        updateCurrentValueOnDisk( null, newValue );
+
+        // now reset the sequence generator
+        _sequenceGenerator = _sequenceGenerator.clone( newValue );
+    }
+    
+    /**
+     * <p>
+     * Get the SequenceUpdater used for the bulk-insert optimization in InsertResultSet.
+     * </p>
+     *
+     * @param restart   True if the counter should be re-initialized to its start position.
+     */
+    public synchronized BulkInsertUpdater   getBulkInsertUpdater( boolean restart )
+        throws StandardException
+    {
+        return new BulkInsertUpdater( this, restart );
+    }
+    
     /**
      * <p>
      * Get the next sequence number managed by this generator and advance the number. Could raise an
@@ -338,13 +407,13 @@ public abstract class SequenceUpdater implements Cacheable
                 return;
                 
             case SequenceGenerator.RET_MARK_EXHAUSTED:
-                updateCurrentValueOnDisk( new Long( currentValue ), null );
+                updateCurrentValueOnDisk( currentValue, null );
                 returnValue.setValue( currentValue );
                 return;
                 
             case SequenceGenerator.RET_ALLOCATE_NEW_VALUES:
                 
-                if ( updateCurrentValueOnDisk( new Long( currentValue ), new Long( lastAllocatedValue ) ) )
+                if ( updateCurrentValueOnDisk( currentValue, lastAllocatedValue ) )
                 {
                     _sequenceGenerator.allocateNewRange( currentValue, numberOfValuesAllocated );
                 }
@@ -403,7 +472,7 @@ public abstract class SequenceUpdater implements Cacheable
 				SanityManager.ASSERT( oldValue == null, "We should be flushing unused sequence values here." );
 			}
             
-            ContextService csf = ContextService.getFactory();
+            ContextService csf = getContextService();
             ContextManager cm = csf.getCurrentContextManager();
             AccessFactory af = _dd.af;
             TransactionController   dummyTransaction = af.getTransaction( cm );
@@ -421,13 +490,29 @@ public abstract class SequenceUpdater implements Cacheable
 
         if ( nestedTransaction != null )
         {
+            boolean retval = false;
+            boolean escalateToParentTransaction = false;
+            
             try
             {
-                return updateCurrentValueOnDisk( nestedTransaction, oldValue, newValue, false );
+                retval = updateCurrentValueOnDisk( nestedTransaction, oldValue, newValue, false );
             }
             catch (StandardException se)
             {
-                if ( !se.isLockTimeout() ) { throw se; }
+                if ( !se.isLockTimeout() )
+                {
+                    if ( se.isSelfDeadlock() )
+                    {
+                        // We're blocked by a lock held by our parent transaction.
+                        // Escalate into the parent transaction now. See DERBY-6554.
+                        escalateToParentTransaction = true;
+                    }
+                    else
+                    {
+                        Monitor.logThrowable( se );
+                        throw se;
+                    }
+                }
             }
             finally
             {
@@ -437,6 +522,13 @@ public abstract class SequenceUpdater implements Cacheable
                 // transaction commits by default.
                 nestedTransaction.commit();
                 nestedTransaction.destroy();
+
+                if ( escalateToParentTransaction )
+                {
+                    retval = updateCurrentValueOnDisk( executionTransaction, oldValue, newValue, false );
+                }
+
+                return retval;
             }
         }
         
@@ -469,14 +561,22 @@ public abstract class SequenceUpdater implements Cacheable
             {
                 return new SequenceRange( Integer.parseInt( className ) );
             }
-            
-            return (SequencePreallocator) Class.forName( className ).newInstance();
+
+            Class<?> klass = Class.forName(className);
+            if (!SequencePreallocator.class.isAssignableFrom(klass)) {
+                throw StandardException.newException(
+                    SQLState.LANG_NOT_A_SEQUENCE_PREALLOCATOR, propertyName);
+            }
+
+            return (SequencePreallocator) klass.getConstructor().newInstance();
         }
         catch (ClassNotFoundException e) { throw missingAllocator( propertyName, className, e ); }
         catch (ClassCastException e) { throw missingAllocator( propertyName, className, e ); }
         catch (InstantiationException e) { throw missingAllocator( propertyName, className, e ); }
         catch (IllegalAccessException e) { throw missingAllocator( propertyName, className, e ); }
         catch (NumberFormatException e) { throw missingAllocator( propertyName, className, e ); }
+        catch (NoSuchMethodException e) { throw missingAllocator( propertyName, className, e ); }
+        catch (java.lang.reflect.InvocationTargetException e) { throw missingAllocator( propertyName, className, e ); }
     }
     private StandardException   missingAllocator( String propertyName, String className, Exception e )
     {
@@ -497,7 +597,7 @@ public abstract class SequenceUpdater implements Cacheable
 	private static LanguageConnectionContext getLCC()
     {
 		return (LanguageConnectionContext) 
-					ContextService.getContextOrNull(LanguageConnectionContext.CONTEXT_ID);
+					getContextOrNull(LanguageConnectionContext.CONTEXT_ID);
 	}
 
     /** Report an unimplemented feature */
@@ -506,6 +606,57 @@ public abstract class SequenceUpdater implements Cacheable
         return StandardException.newException( SQLState.BTREE_UNIMPLEMENTED_FEATURE );
     }
 
+    /**
+     * Privileged lookup of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ContextService    getContextService()
+    {
+        if ( System.getSecurityManager() == null )
+        {
+            return ContextService.getFactory();
+        }
+        else
+        {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedAction<ContextService>()
+                 {
+                     public ContextService run()
+                     {
+                         return ContextService.getFactory();
+                     }
+                 }
+                 );
+        }
+    }
+
+    
+    /**
+     * Privileged lookup of a Context. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Context    getContextOrNull( final String contextID )
+    {
+        if ( System.getSecurityManager() == null )
+        {
+            return ContextService.getContextOrNull( contextID );
+        }
+        else
+        {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedAction<Context>()
+                 {
+                     public Context run()
+                     {
+                         return ContextService.getContextOrNull( contextID );
+                     }
+                 }
+                 );
+        }
+    }
+
     ///////////////////////////////////////////////////////////////////////////////////
     //
     // INNER CLASSES
@@ -561,5 +712,41 @@ public abstract class SequenceUpdater implements Cacheable
         }
     }
 
+    /**
+     * <p>
+     * Implementation of SequenceUpdater for use with the bulk-insert optimization
+     * used by InsertResultSet. This BulkInsertUpdater doesn't really write to disk. It is assumed
+     * that the BulkInsertUpdater will only be used by the bulk-insert code, where the
+     * user has exclusive write-access on the table whose identity column is backed by
+     * the original SequenceUpdater. At the end of bulk-insert, the current value of the
+     * BulkInsertUpdater is written to disk by other code.
+     * </p>
+     */
+    public static final class BulkInsertUpdater extends SequenceUpdater implements BulkInsertCounter
+    {
+        public BulkInsertUpdater() { super(); }
+        public BulkInsertUpdater( SequenceUpdater originalUpdater, boolean restart )
+        {
+            _sequenceGenerator = originalUpdater._sequenceGenerator.clone( restart );
+        }
+    
+        //
+        // SequenceUpdater BEHAVIOR
+        //
+
+        protected SequenceGenerator createSequenceGenerator( TransactionController readOnlyTC )
+            throws StandardException
+        {
+            return _sequenceGenerator;
+        }
+
+        protected boolean updateCurrentValueOnDisk( TransactionController tc, Long oldValue, Long newValue, boolean wait )
+            throws StandardException
+        {
+            // always succeeds
+            return true;
+        }
+    }
+
 }
 
