@@ -21,25 +21,19 @@
 
 package	org.apache.derby.impl.sql.compile;
 
+import java.lang.reflect.Modifier;
+import java.util.List;
+import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.ClassName;
 import org.apache.derby.iapi.reference.SQLState;
-
 import org.apache.derby.iapi.services.classfile.VMOpcode;
-
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
-import org.apache.derby.iapi.error.StandardException;
-
 import org.apache.derby.iapi.services.compiler.LocalField;
 import org.apache.derby.iapi.services.compiler.MethodBuilder;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.compile.Visitor;
-
 import org.apache.derby.iapi.util.JBitSet;
 
-import java.lang.reflect.Modifier;
-
-import java.util.List;
-
 /**
  * This node represents coalesce/value function which returns the first argument that is not null.
  * The arguments are evaluated in the order in which they are specified, and the result of the
@@ -54,15 +48,15 @@ import java.util.List;
  * This table has FOR BIT DATA TYPES broken out into separate columns for clarity
  *
  * Note that are few differences between Derby and DB2
- * 1)there are few differences between what datatypes are consdiered compatible
- * In DB2, CHAR FOR BIT DATA datatypes are compatible with CHAR datatypes
+ * 1)there are few differences between what data types are considered compatible
+ * In DB2, CHAR FOR BIT DATA data types are compatible with CHAR data types
  * ie in addition to following table, CHAR is compatible with CHAR FOR BIT DATA, VARCHAR FOR BIT DATA and LONG VARCHAR FOR BIT DATA
  * ie in addition to following table, VARCHAR is compatible with CHAR FOR BIT DATA, VARCHAR FOR BIT DATA and LONG VARCHAR FOR BIT DATA
  * ie in addition to following table, LONG VARCHAR is compatible with CHAR FOR BIT DATA, VARCHAR FOR BIT DATA and LONG VARCHAR FOR BIT DATA
  * ie in addition to following table, CHAR FOR BIT DATA is compatible with DATE, TIME, TIMESTAMP
  * ie in addition to following table, VARCHAR FOR BIT DATA is compatible with DATE, TIME, TIMESTAMP
  *
- * 2)few datatypes donot have matching precision in Derby and DB2
+ * 2)few data types do not have matching precision in Derby and DB2
  * In DB2, precision of TIME is 8. In Derby, precision of TIME is 0.
  * In DB2, precision,scale of TIMESTAMP is 26,6. In Derby, precision of TIMESTAMP is 0,0.
  * In DB2, precision of DOUBLE is 15. In Derby, precision of DOUBLE is 52.
@@ -100,7 +94,7 @@ import java.util.List;
  * BLOB             { "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "ERROR", "BLOB" }
  */
 
-public class CoalesceFunctionNode extends ValueNode
+class CoalesceFunctionNode extends ValueNode
 {
 	String	functionName; //Are we here because of COALESCE function or VALUE function
 	ValueNodeList	argumentsList; //this is the list of arguments to the function. We are interested in the first not-null argument
@@ -112,15 +106,20 @@ public class CoalesceFunctionNode extends ValueNode
 	private int firstNonParameterNodeIdx = -1;
 
 	/**
-	 * Initializer for a CalesceFunctionNode
+     * Constructor for a CoalesceFunctionNode
 	 *
 	 * @param functionName	Tells if the function was called with name COALESCE or with name VALUE
 	 * @param argumentsList	The list of arguments to the coalesce/value function
+     * @param cm            The context manager
 	 */
-	public void init(Object functionName, Object argumentsList)
+    CoalesceFunctionNode(
+            String functionName,
+            ValueNodeList argumentsList,
+            ContextManager cm)
 	{
-		this.functionName = (String) functionName;
-		this.argumentsList = (ValueNodeList) argumentsList;
+        super(cm);
+        this.functionName = functionName;
+        this.argumentsList = argumentsList;
 	}
 
 	/**
@@ -129,18 +128,18 @@ public class CoalesceFunctionNode extends ValueNode
 	 *
 	 * @param fromList			The FROM list for the statement.
 	 * @param subqueryList		The subquery list being built as we find SubqueryNodes.
-	 * @param aggregateVector	The aggregate vector being built as we find AggregateNodes.
+     * @param aggregates        The aggregate list being built as we find AggregateNodes.
 	 *
 	 * @return	The new top of the expression tree.
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ValueNode bindExpression(FromList fromList, SubqueryList subqueryList,
-							List aggregateVector)
+    @Override
+    ValueNode bindExpression(FromList fromList, SubqueryList subqueryList, List<AggregateNode> aggregates)
 					throws StandardException
 	{
 		//bind all the arguments
-		argumentsList.bindExpression(fromList, subqueryList, aggregateVector);
+        argumentsList.bindExpression(fromList, subqueryList, aggregates);
 
 		//There should be more than one argument
 		if (argumentsList.size() < 2)
@@ -154,46 +153,49 @@ public class CoalesceFunctionNode extends ValueNode
 		//find the first non-param argument. The generated method will generate code to call coalesce on this argument
 		for (int index = 0; index < argumentsListSize; index++)
 		{
-			if (!(((ValueNode) argumentsList.elementAt(index)).requiresTypeFromContext()))
+            if (!argumentsList.elementAt(index).requiresTypeFromContext())
 			{
 				firstNonParameterNodeIdx = index;
 				break;
 			}
 		}
 
-		//make sure these arguments are compatible to each other before coalesce can be allowed
-		for (int index = 0; index < argumentsListSize; index++)
-		{
-			if (((ValueNode) argumentsList.elementAt(index)).requiresTypeFromContext()) //since we don't know the type of param, can't check for compatibility
+        // Make sure these arguments are compatible to each other before
+        // coalesce can be allowed.
+        for (ValueNode vn : argumentsList) {
+            if (vn.requiresTypeFromContext()) {
+                // Since we don't know the type of param, can't check for
+                // compatibility.
 				continue;
-				argumentsList.compatible((ValueNode) argumentsList.elementAt(index));
+            }
+            argumentsList.compatible(vn);
 		}
 
-		//set the result type to the most dominant datatype in the arguments list and based on the table listed above
+        // Set the result type to the most dominant datatype in the arguments
+        // list and based on the table listed above.
 		setType(argumentsList.getDominantTypeServices());
 
-		//set all the parameter types to the type of the result type
-		for (int index = 0; index < argumentsListSize; index++)
+        // Set all the parameter types to the type of the result type.
+        for (ValueNode vn : argumentsList)
 		{
-			if (((ValueNode) argumentsList.elementAt(index)).requiresTypeFromContext())
+            if (vn.requiresTypeFromContext())
 			{
-				((ValueNode)argumentsList.elementAt(index)).setType(getTypeServices());
+                vn.setType(getTypeServices());
 			}
 		}
 		return this;
 	}
 
 	/**
-	 * Do code generation for coalese/value
+     * Do code generation for coalesce/value
 	 *
 	 * @param acb	The ExpressionClassBuilder for the class we're generating
 	 * @param mb	The method the expression will go into
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public void generateExpression(ExpressionClassBuilder acb,
-											MethodBuilder mb)
+    @Override
+    void generateExpression(ExpressionClassBuilder acb, MethodBuilder mb)
 									throws StandardException
 	{
 		int			argumentsListSize = argumentsList.size();
@@ -257,7 +259,7 @@ public class CoalesceFunctionNode extends ValueNode
 			}
 
 			setArrayMethod.getField(arrayField); 
-			((ValueNode) argumentsList.elementAt(index)).generateExpression(acb, setArrayMethod);
+            argumentsList.elementAt(index).generateExpression(acb, setArrayMethod);
 			setArrayMethod.upCast(receiverType);
 			setArrayMethod.setArrayElement(index);
 		}
@@ -283,7 +285,7 @@ public class CoalesceFunctionNode extends ValueNode
 		*/
 
 		// coalesce will be called on this non-parameter argument
-		((ValueNode) argumentsList.elementAt(firstNonParameterNodeIdx)).
+        argumentsList.elementAt(firstNonParameterNodeIdx).
 			generateExpression(acb, mb);
 
 		mb.upCast(ClassName.DataValueDescriptor);
@@ -314,6 +316,7 @@ public class CoalesceFunctionNode extends ValueNode
 	/*
 		print the non-node subfields
 	 */
+    @Override
 	public String toString() 
 	{
 		if (SanityManager.DEBUG)
@@ -335,8 +338,8 @@ public class CoalesceFunctionNode extends ValueNode
 	 *
 	 * @param depth		The depth of this node in the tree
 	 */
-
-	public void printSubNodes(int depth)
+    @Override
+    void printSubNodes(int depth)
 	{
 		if (SanityManager.DEBUG)
 		{
@@ -351,10 +354,9 @@ public class CoalesceFunctionNode extends ValueNode
 	/**
 	 * {@inheritDoc}
 	 */
-	protected boolean isEquivalent(ValueNode o) throws StandardException
+    boolean isEquivalent(ValueNode o) throws StandardException
 	{
-		if (!isSameNodeType(o))
-		{
+        if (! isSameNodeKind(o)) {
 			return false;
 		}
 		
@@ -374,6 +376,7 @@ public class CoalesceFunctionNode extends ValueNode
 	 * @param v the visitor
 	 * @throws StandardException on error in the visitor
 	 */
+    @Override
 	void acceptChildren(Visitor v) throws StandardException
 	{
 		super.acceptChildren(v);
@@ -386,6 +389,7 @@ public class CoalesceFunctionNode extends ValueNode
      *
      * @see ValueNode#categorize(JBitSet, boolean)
      */
+    @Override
     public boolean categorize(JBitSet referencedTabs, boolean simplePredsOnly)
         throws StandardException
     {
@@ -407,7 +411,8 @@ public class CoalesceFunctionNode extends ValueNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ValueNode preprocess(int numTables,
+    @Override
+    ValueNode preprocess(int numTables,
 								FromList outerFromList,
 								SubqueryList outerSubqueryList,
 								PredicateList outerPredicateList) 
@@ -429,6 +434,7 @@ public class CoalesceFunctionNode extends ValueNode
      * @return the remapped tree
      * @throws StandardException if an error occurs
      */
+    @Override
     public ValueNode remapColumnReferencesToExpressions()
             throws StandardException
     {
