@@ -21,6 +21,7 @@
 
 package	org.apache.derby.impl.sql.compile;
 
+import java.sql.Types;
 import java.util.List;
 import org.apache.derby.catalog.AliasInfo;
 import org.apache.derby.catalog.TypeDescriptor;
@@ -29,9 +30,9 @@ import org.apache.derby.catalog.types.RoutineAliasInfo;
 import org.apache.derby.catalog.types.SynonymAliasInfo;
 import org.apache.derby.catalog.types.UDTAliasInfo;
 import org.apache.derby.iapi.error.StandardException;
-import org.apache.derby.iapi.reference.JDBC40Translation;
 import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.dictionary.AliasDescriptor;
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
 import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
@@ -44,7 +45,7 @@ import org.apache.derby.iapi.types.TypeId;
  *
  */
 
-public class CreateAliasNode extends DDLStatementNode
+class CreateAliasNode extends DDLStatementNode
 {
     // indexes into routineElements
     public static final int PARAMETER_ARRAY = 0;
@@ -118,27 +119,27 @@ public class CreateAliasNode extends DDLStatementNode
 
 
 	/**
-	 * Initializer for a CreateAliasNode
+     * Constructor
 	 *
 	 * @param aliasName				The name of the alias
-	 * @param targetObject			Target name
+     * @param targetObject          Target name string or, if
+     *        aliasType == ALIAS_TYPE_SYNONYM_AS_CHAR, a TableName
 	 * @param methodName		    The method name
-	 * @param aliasType				The alias type
-     *
-	 * @exception StandardException		Thrown on error
+     * @param aliasSpecificInfo     An array of objects, see code for
+     *                              interpretation
+     * @param cm                    The context manager
+     * @exception StandardException Thrown on error
 	 */
-	public void init(
-						Object aliasName,
+    CreateAliasNode(    TableName aliasName,
 						Object targetObject,
-						Object methodName,
+                        String methodName,
 						Object aliasSpecificInfo,
-                        Object aliasType)
+                        char aliasType,
+                        ContextManager cm)
 		throws StandardException
 	{		
-		TableName qn = (TableName) aliasName;
-		this.aliasType = ((Character) aliasType).charValue();
-
-		initAndCheck(qn);
+        super(aliasName, cm);
+        this.aliasType = aliasType;
 
 		switch (this.aliasType)
 		{
@@ -151,8 +152,8 @@ public class CreateAliasNode extends DDLStatementNode
 
                 // XML not allowed because SQLXML support has not been implemented
                 if (
-                    (aggForType.getJDBCTypeId() == JDBC40Translation.SQLXML) ||
-                    (aggReturnType.getJDBCTypeId() == JDBC40Translation.SQLXML)
+                    (aggForType.getJDBCTypeId() == Types.SQLXML) ||
+                    (aggReturnType.getJDBCTypeId() == Types.SQLXML)
                    )
                 {
                     throw StandardException.newException( SQLState.LANG_XML_NOT_ALLOWED_DJRS );
@@ -173,7 +174,7 @@ public class CreateAliasNode extends DDLStatementNode
 			case AliasInfo.ALIAS_TYPE_FUNCTION_AS_CHAR:
 			{
 				this.javaClassName = (String) targetObject;
-				this.methodName = (String) methodName;
+                this.methodName = methodName;
 
 				//routineElements contains the description of the procedure.
 				// 
@@ -211,14 +212,13 @@ public class CreateAliasNode extends DDLStatementNode
 				
 				if (paramCount != 0) {
 
-                    names = (String[]) ((List) parameters[0]).toArray(
-                            new String[paramCount]);
-
-                    types = (TypeDescriptor[]) ((List) parameters[1]).toArray(
-                            new TypeDescriptor[paramCount]);
-
+                    names = new String[paramCount];
+                    types = new TypeDescriptor[paramCount];
 					modes = new int[paramCount];
+
 					for (int i = 0; i < paramCount; i++) {
+                        names[i] = (String) ((List) parameters[0]).get(i);
+                        types[i] = (TypeDescriptor) ((List) parameters[1]).get(i);
                         int currentMode =  ((Integer) (((List) parameters[2]).get(i))).intValue();
                         modes[i] = currentMode;
   
@@ -325,7 +325,7 @@ public class CreateAliasNode extends DDLStatementNode
 		}
 	}
 
-	public String statementToString()
+    String statementToString()
 	{
 		switch (this.aliasType)
 		{
@@ -354,7 +354,7 @@ public class CreateAliasNode extends DDLStatementNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
+    @Override
 	public void bindStatement() throws StandardException
 	{
 		//Are we dealing with user defined function or procedure?
@@ -474,15 +474,15 @@ public class CreateAliasNode extends DDLStatementNode
         // A user-defined aggregate cannot have the name of a builtin function which takes 1 argument.
         //
         SchemaDescriptor    sysfun = getSchemaDescriptor( "SYSFUN", true );
-        List                        systemFunctions = getDataDictionary().getRoutineList
-            (
-             sysfun.getUUID().toString(), unqualifiedName,
-             AliasInfo.ALIAS_NAME_SPACE_FUNCTION_AS_CHAR
-             );
+        List<AliasDescriptor> systemFunctions =
+            getDataDictionary().getRoutineList(
+                sysfun.getUUID().toString(),
+                unqualifiedName,
+                AliasInfo.ALIAS_NAME_SPACE_FUNCTION_AS_CHAR);
 
         for ( int i = 0; i < systemFunctions.size(); i++ )
         {
-			AliasDescriptor function = (AliasDescriptor) systemFunctions.get(i);
+            AliasDescriptor function = systemFunctions.get(i);
 
 			RoutineAliasInfo routineInfo = (RoutineAliasInfo) function.getAliasInfo();
 			int parameterCount = routineInfo.getParameterCount();
@@ -531,6 +531,8 @@ public class CreateAliasNode extends DDLStatementNode
         {
             parameterTypes[ i ] = bindUserCatalogType( parameterTypes[ i ] );
         }
+
+        aliasInfo.setParameterTypes( parameterTypes );
     }
 
 	/**
@@ -538,7 +540,8 @@ public class CreateAliasNode extends DDLStatementNode
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
-	public ConstantAction	makeConstantAction() throws StandardException
+    @Override
+    public ConstantAction makeConstantAction() throws StandardException
 	{
 		String schemaName = getSchemaDescriptor().getSchemaName();
 
