@@ -21,20 +21,25 @@
 
 package	org.apache.derby.impl.sql.compile;
 
-import java.sql.Timestamp;
-import java.util.Collections;
+import java.util.ArrayList;
 import java.util.Comparator;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
-
+import java.util.SortedSet;
+import java.util.TreeSet;
 import org.apache.derby.catalog.UUID;
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.compile.CompilerContext;
+import org.apache.derby.iapi.sql.compile.Visitable;
+import org.apache.derby.iapi.sql.compile.Visitor;
 import org.apache.derby.iapi.sql.conn.Authorizer;
 import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
+import org.apache.derby.iapi.sql.depend.DependencyManager;
+import org.apache.derby.iapi.sql.depend.ProviderInfo;
+import org.apache.derby.iapi.sql.depend.ProviderList;
 import org.apache.derby.iapi.sql.dictionary.ColumnDescriptor;
 import org.apache.derby.iapi.sql.dictionary.ColumnDescriptorList;
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
@@ -50,7 +55,7 @@ import org.apache.derby.iapi.sql.execute.ConstantAction;
  *
  */
 
-public class CreateTriggerNode extends DDLStatementNode
+class CreateTriggerNode extends DDLStatementNode
 {
 	private	TableName			triggerName;
 	private	TableName			tableName;
@@ -59,14 +64,14 @@ public class CreateTriggerNode extends DDLStatementNode
 	private	boolean				isBefore;
 	private	boolean				isRow;
 	private	boolean				isEnabled;
-	private	List				refClause;
+    private List<TriggerReferencingStruct> refClause;
 	private	ValueNode		    whenClause;
 	private	String				whenText;
-	private	int					whenOffset;
 	private	StatementNode		actionNode;
 	private	String				actionText;
-	private	String				originalActionText; // text w/o trim of spaces
-	private	int					actionOffset;
+    private String              originalWhenText;
+    private String              originalActionText;
+    private ProviderInfo[]      providerInfo;
 
 	private SchemaDescriptor	triggerSchemaDescriptor;
 	private SchemaDescriptor	compSchemaDescriptor;
@@ -197,7 +202,6 @@ public class CreateTriggerNode extends DDLStatementNode
 	 */
 	private int[]				referencedColsInTriggerAction;
 	private TableDescriptor		triggerTableDescriptor;
-	private	UUID				actionCompSchemaId;
 
 	/*
 	** Names of old and new table.  By default we have
@@ -215,9 +219,35 @@ public class CreateTriggerNode extends DDLStatementNode
 	private boolean oldTableInReferencingClause;
 	private boolean newTableInReferencingClause;
 
+    /**
+     * <p>
+     * A list that describes how the original SQL text of the trigger action
+     * statement was modified when transition tables and transition variables
+     * were replaced by VTI calls. Each element in the list contains four
+     * integers describing positions where modifications have happened. The
+     * first two integers are begin and end positions of a transition table
+     * or transition variable in {@link #originalActionText the original SQL
+     * text}. The last two integers are begin and end positions of the
+     * corresponding replacement in {@link #actionText the transformed SQL
+     * text}.
+     * </p>
+     *
+     * <p>
+     * Begin positions are inclusive and end positions are exclusive.
+     * </p>
+     */
+    private final ArrayList<int[]>
+            actionTransformations = new ArrayList<int[]>();
+
+    /**
+     * Structure that has the same shape as {@code actionTransformations},
+     * except that it describes the transformations in the WHEN clause.
+     */
+    private final ArrayList<int[]>
+            whenClauseTransformations = new ArrayList<int[]>();
 
 	/**
-	 * Initializer for a CreateTriggerNode
+     * Constructor for a CreateTriggerNode
 	 *
 	 * @param triggerName			name of the trigger	
 	 * @param tableName				name of the table which the trigger is declared upon	
@@ -230,53 +260,49 @@ public class CreateTriggerNode extends DDLStatementNode
 	 * @param refClause				the referencing clause
 	 * @param whenClause			the WHEN clause tree
 	 * @param whenText				the text of the WHEN clause
-	 * @param whenOffset			offset of start of WHEN clause
 	 * @param actionNode			the trigger action tree
 	 * @param actionText			the text of the trigger action
-	 * @param actionOffset			offset of start of action clause
+     * @param cm                    context manager
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void init 
+    CreateTriggerNode
 	(
-		Object		triggerName,
-		Object		tableName,
-		Object				triggerEventMask,
-		Object triggerCols,
-		Object			isBefore,
-		Object			isRow,
-		Object			isEnabled,
-		Object			refClause,
-		Object	whenClause,
-		Object			whenText,
-		Object				whenOffset,
-		Object	actionNode,
-		Object			actionText,
-		Object				actionOffset
+        TableName       triggerName,
+        TableName       tableName,
+        int             triggerEventMask,
+        ResultColumnList triggerCols,
+        boolean         isBefore,
+        boolean         isRow,
+        boolean         isEnabled,
+        List<TriggerReferencingStruct> refClause,
+        ValueNode       whenClause,
+        String          whenText,
+        StatementNode   actionNode,
+        String          actionText,
+        ContextManager  cm
 	) throws StandardException
 	{
-		initAndCheck(triggerName);
-		this.triggerName = (TableName) triggerName;
-		this.tableName = (TableName) tableName;
-		this.triggerEventMask = ((Integer) triggerEventMask).intValue();
-		this.triggerCols = (ResultColumnList) triggerCols;
-		this.isBefore = ((Boolean) isBefore).booleanValue();
-		this.isRow = ((Boolean) isRow).booleanValue();
-		this.isEnabled = ((Boolean) isEnabled).booleanValue();
-		this.refClause = (List) refClause;
-		this.whenClause = (ValueNode) whenClause;
-		this.whenText = (whenText == null) ? null : ((String) whenText).trim();
-		this.whenOffset = ((Integer) whenOffset).intValue();
-		this.actionNode = (StatementNode) actionNode;
-		this.originalActionText = (String) actionText;
-		this.actionText =
-					(actionText == null) ? null : ((String) actionText).trim();
-		this.actionOffset = ((Integer) actionOffset).intValue();
+        super(triggerName, cm);
 
-		implicitCreateSchema = true;
+        this.triggerName = triggerName;
+        this.tableName = tableName;
+        this.triggerEventMask = triggerEventMask;
+        this.triggerCols = triggerCols;
+        this.isBefore = isBefore;
+        this.isRow = isRow;
+        this.isEnabled = isEnabled;
+        this.refClause = refClause;
+        this.whenClause = whenClause;
+        this.originalWhenText = whenText;
+        this.whenText = (whenText == null) ? null : whenText.trim();
+        this.actionNode = actionNode;
+        this.originalActionText = actionText;
+        this.actionText = (actionText == null) ? null : actionText.trim();
+        this.implicitCreateSchema = true;
 	}
 
-	public String statementToString()
+    String statementToString()
 	{
 		return "CREATE TRIGGER";
 	}
@@ -287,8 +313,8 @@ public class CreateTriggerNode extends DDLStatementNode
 	 *
 	 * @param depth		The depth of this node in the tree
 	 */
-
-	public void printSubNodes(int depth)
+    @Override
+    void printSubNodes(int depth)
 	{
 		if (SanityManager.DEBUG)
 		{
@@ -325,6 +351,7 @@ public class CreateTriggerNode extends DDLStatementNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	public void bindStatement() throws StandardException
 	{
 		CompilerContext compilerContext = getCompilerContext();
@@ -366,9 +393,27 @@ public class CreateTriggerNode extends DDLStatementNode
 		*/
 		boolean needInternalSQL = bindReferencesClause(dd);
 
+        // Get all the names of SQL objects referenced by the triggered
+        // SQL statement and the WHEN clause. Since some of the TableName
+        // nodes may be eliminated from the node tree during the bind phase,
+        // we collect the nodes before the nodes have been bound. The
+        // names will be used later when we normalize the trigger text
+        // that will be stored in the system tables.
+        SortedSet<TableName> actionNames =
+                actionNode.getOffsetOrderedNodes(TableName.class);
+        SortedSet<TableName> whenNames = (whenClause != null)
+                ? whenClause.getOffsetOrderedNodes(TableName.class)
+                : null;
+
+        ProviderList prevAPL =
+                compilerContext.getCurrentAuxiliaryProviderList();
+        ProviderList apl = new ProviderList();
+
 		lcc.pushTriggerTable(triggerTableDescriptor);
 		try
 		{	
+            compilerContext.setCurrentAuxiliaryProviderList(apl);
+
 			/*
 			** Bind the trigger action and the trigger
 			** when clause to make sure that they are
@@ -388,18 +433,26 @@ public class CreateTriggerNode extends DDLStatementNode
 					
 			actionNode.bindStatement();
 
-			/* when clause is always null
 			if (whenClause != null)
 			{
-				whenClause.bind();
+                ContextManager cm = getContextManager();
+                whenClause = whenClause.bindExpression(
+                        new FromList(cm), new SubqueryList(cm),
+                        new ArrayList<AggregateNode>(0));
+
+                // The WHEN clause must be a BOOLEAN expression.
+                whenClause.checkIsBoolean();
 			}
-			*/
 		}
 		finally
 		{
 			lcc.popTriggerTable(triggerTableDescriptor);
+            compilerContext.setCurrentAuxiliaryProviderList(prevAPL);
 		}
 
+        // Qualify identifiers before storing them (DERBY-5901/DERBY-6370).
+        qualifyNames(actionNames, whenNames);
+
 		/* 
 		** Statement is dependent on the TableDescriptor 
 		*/
@@ -411,11 +464,10 @@ public class CreateTriggerNode extends DDLStatementNode
 		*/
 		if (triggerCols != null && triggerCols.size() != 0)
 		{
-            HashSet columnNames = new HashSet();
-			int tcSize = triggerCols.size();
-			for (int i = 0; i < tcSize; i++)
+            HashSet<String> columnNames = new HashSet<String>();
+
+            for (ResultColumn rc : triggerCols)
 			{
-				ResultColumn rc  = (ResultColumn) triggerCols.elementAt(i);
 				if (!columnNames.add(rc.getName()))
 				{
 					throw StandardException.newException(SQLState.LANG_DUPLICATE_COLUMN_IN_TRIGGER_UPDATE, 
@@ -433,9 +485,15 @@ public class CreateTriggerNode extends DDLStatementNode
 			}
 		}
 
-		//If attempting to reference a SESSION schema table (temporary or permanent) in the trigger action, throw an exception
-		if (actionNode.referencesSessionSchema())
+        // Throw an exception if the WHEN clause or the triggered SQL
+        // statement references a table in the SESSION schema.
+        if (referencesSessionSchema()) {
 			throw StandardException.newException(SQLState.LANG_OPERATION_NOT_ALLOWED_ON_SESSION_SCHEMA_TABLES);
+        }
+
+        DependencyManager dm = dd.getDependencyManager();
+        providerInfo = dm.getPersistentProviderInfos(apl);
+        dm.clearColumnInfoInProviders(apl);
 
 	}
 
@@ -446,25 +504,28 @@ public class CreateTriggerNode extends DDLStatementNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	public boolean referencesSessionSchema()
 		throws StandardException
 	{
 		//If create trigger is part of create statement and the trigger is defined on or it references SESSION schema tables,
 		//it will get caught in the bind phase of trigger and exception will be thrown by the trigger bind. 
-		return (isSessionSchema(triggerTableDescriptor.getSchemaName()) || actionNode.referencesSessionSchema());
+        return isSessionSchema(triggerTableDescriptor.getSchemaName())
+                || actionNode.referencesSessionSchema()
+                || (whenClause != null && whenClause.referencesSessionSchema());
 	}
 
     /**
      * Comparator that can be used for sorting lists of FromBaseTables
      * on the position they have in the SQL query string.
      */
-    private static final Comparator OFFSET_COMPARATOR = new Comparator() {
-        public int compare(Object o1, Object o2) {
+    private static final Comparator<FromBaseTable> OFFSET_COMPARATOR = new Comparator<FromBaseTable>() {
+        public int compare(FromBaseTable o1, FromBaseTable o2) {
             // Return negative int, zero, or positive int if the offset of the
             // first table is less than, equal to, or greater than the offset
             // of the second table.
-            return ((FromBaseTable) o1).getTableNameField().getBeginOffset() -
-                    ((FromBaseTable) o2).getTableNameField().getBeginOffset();
+            return o1.getTableNameField().getBeginOffset() -
+                    o2.getTableNameField().getBeginOffset();
         }
     };
 
@@ -474,7 +535,7 @@ public class CreateTriggerNode extends DDLStatementNode
 	**
 	** 1) validate the referencing clause (if any)
 	**
-	** 2) convert trigger action text.  e.g. 
+    ** 2) convert trigger action text and WHEN clause text.  e.g.
 	**	DELETE FROM t WHERE c = old.c
 	** turns into
 	**	DELETE FROM t WHERE c = org.apache.derby.iapi.db.Factory::
@@ -519,14 +580,13 @@ public class CreateTriggerNode extends DDLStatementNode
         if ( isBefore ) { forbidActionsOnGenCols(); }
 
 		String transformedActionText;
-		int start = 0;
+        String transformedWhenText = null;
 		if (triggerCols != null && triggerCols.size() != 0) {
 			//If the trigger is defined on speific columns, then collect
 			//their column positions and ensure that those columns do
 			//indeed exist in the trigger table.
 			referencedColInts = new int[triggerCols.size()];
-			ResultColumn rc;
-			ColumnDescriptor cd;
+
 			//This is the most interesting case for us. If we are here, 
 			//then it means that a set of trigger columns are specified
 			//in the CREATE TRIGGER statement. This can only happen for
@@ -537,8 +597,9 @@ public class CreateTriggerNode extends DDLStatementNode
 			//    FOR EACH ROW UPDATE table2 SET c24=oldt.c14;
 			
 			for (int i=0; i < triggerCols.size(); i++){
-				rc = (ResultColumn)triggerCols.elementAt(i);
-				cd = triggerTableDescriptor.getColumnDescriptor(rc.getName());
+                ResultColumn rc = triggerCols.elementAt(i);
+                ColumnDescriptor cd =
+                    triggerTableDescriptor.getColumnDescriptor(rc.getName());
 				//Following will catch the case where an invalid trigger column
 				//has been specified in CREATE TRIGGER statement.
 				//CREATE TRIGGER tr1 AFTER UPDATE OF c1678 ON table1 
@@ -565,6 +626,37 @@ public class CreateTriggerNode extends DDLStatementNode
 			//column positions in the array.
 			referencedColsInTriggerAction = new int[triggerTableDescriptor.getNumberOfColumns()];
 			java.util.Arrays.fill(referencedColsInTriggerAction, -1);
+
+			int[] cols;
+
+			cols = getDataDictionary().examineTriggerNodeAndCols(actionNode,
+					oldTableName,
+					newTableName,
+					originalActionText,
+					referencedColInts,
+					referencedColsInTriggerAction,
+                    actionNode.getBeginOffset(),
+					triggerTableDescriptor,
+					triggerEventMask,
+                    true,
+                    actionTransformations);
+
+    		if (whenClause != null)
+    		{
+        		cols = getDataDictionary().examineTriggerNodeAndCols(whenClause,
+        			oldTableName,
+					newTableName,
+					originalActionText,
+					referencedColInts,
+					referencedColsInTriggerAction,
+                    actionNode.getBeginOffset(),
+					triggerTableDescriptor,
+					triggerEventMask,
+                    true,
+                    actionTransformations);
+    		}
+
+
 			//Now that we have verified that are no invalid column references
 			//for trigger columns, let's go ahead and transform the OLD/NEW
 			//transient table references in the trigger action sql.
@@ -574,11 +666,24 @@ public class CreateTriggerNode extends DDLStatementNode
 					originalActionText,
 					referencedColInts,
 					referencedColsInTriggerAction,
-					actionOffset,
+                    actionNode.getBeginOffset(),
 					triggerTableDescriptor,
 					triggerEventMask,
-					true
-					);			
+                    true,
+                    actionTransformations, cols);
+
+            // If there is a WHEN clause, we need to transform its text too.
+            if (whenClause != null) {
+                transformedWhenText =
+                    getDataDictionary().getTriggerActionString(
+                            whenClause, oldTableName, newTableName,
+                            originalWhenText, referencedColInts,
+                            referencedColsInTriggerAction,
+                            whenClause.getBeginOffset(),
+                            triggerTableDescriptor, triggerEventMask, true,
+                            whenClauseTransformations, cols);
+            }
+
 			//Now that we know what columns we need for REFERENCEd columns in
 			//trigger action, we can get rid of -1 entries for the remaining 
 			//columns from trigger table. This information will be saved in
@@ -590,56 +695,13 @@ public class CreateTriggerNode extends DDLStatementNode
 		else
 		{
 			//This is a table level trigger	        
-
-			StringBuffer newText = new StringBuffer();
-			/*
-			** For a statement trigger, we find all FromBaseTable nodes.  If
-			** the from table is NEW or OLD (or user designated alternates
-			** REFERENCING), we turn them into a trigger table VTI.
-			*/
-			CollectNodesVisitor visitor = new CollectNodesVisitor(FromBaseTable.class);
-			actionNode.accept(visitor);
-			List tabs = visitor.getList();
-			Collections.sort(tabs, OFFSET_COMPARATOR);
-			for (int i = 0; i < tabs.size(); i++)
-			{
-				FromBaseTable fromTable = (FromBaseTable) tabs.get(i);
-				String baseTableName = fromTable.getBaseTableName();
-				if ((baseTableName == null) ||
-					((oldTableName == null || !oldTableName.equals(baseTableName)) &&
-					(newTableName == null || !newTableName.equals(baseTableName))))
-				{
-					continue;
-				}
-				int tokBeginOffset = fromTable.getTableNameField().getBeginOffset();
-				int tokEndOffset = fromTable.getTableNameField().getEndOffset();
-				if (tokBeginOffset == -1)
-				{
-					continue;
-				}
-
-				checkInvalidTriggerReference(baseTableName);
-
-				newText.append(originalActionText.substring(start, tokBeginOffset-actionOffset));
-				newText.append(baseTableName.equals(oldTableName) ?
-								"new org.apache.derby.catalog.TriggerOldTransitionRows() " :
-								"new org.apache.derby.catalog.TriggerNewTransitionRows() ");
-				/*
-				** If the user supplied a correlation, then just
-				** pick it up automatically; otherwise, supply
-				** the default.
-				*/
-                if (fromTable.getCorrelationName() == null)
-				{
-					newText.append(baseTableName).append(" ");
-				}
-				start=tokEndOffset-actionOffset+1;
-			}
-			if (start < originalActionText.length())
-			{
-				newText.append(originalActionText.substring(start));
-			}
-			transformedActionText = newText.toString();
+            transformedActionText = transformStatementTriggerText(
+                    actionNode, originalActionText, actionTransformations);
+            if (whenClause != null) {
+                transformedWhenText = transformStatementTriggerText(
+                        whenClause, originalWhenText,
+                        whenClauseTransformations);
+            }
 		}
 
 		if (referencedColsInTriggerAction != null)
@@ -658,9 +720,155 @@ public class CreateTriggerNode extends DDLStatementNode
 			actionNode = parseStatement(actionText, true);
 		}
 
+        if (whenClause != null && !transformedWhenText.equals(whenText)) {
+            regenNode = true;
+            whenText = transformedWhenText;
+            whenClause = parseSearchCondition(whenText, true);
+        }
+
 		return regenNode;
 	}
 
+    /**
+     * Make sure all references to SQL schema objects (such as tables and
+     * functions) in the SQL fragments that will be stored in the SPS and
+     * in the trigger descriptor, are fully qualified with a schema name.
+     *
+     * @param actionNames all the TableName nodes found in the triggered
+     *                    SQL statement
+     * @param whenNames   all the Table Name nodes found in the WHEN clause
+     */
+    private void qualifyNames(SortedSet<TableName> actionNames,
+                              SortedSet<TableName> whenNames)
+            throws StandardException {
+
+        StringBuilder original = new StringBuilder();
+        StringBuilder transformed = new StringBuilder();
+
+        // Qualify the names in the action text.
+        qualifyNames(actionNode, actionNames, originalActionText, actionText,
+                     actionTransformations, original, transformed);
+        originalActionText = original.toString();
+        actionText = transformed.toString();
+
+        // Do the same for the WHEN clause, if there is one.
+        if (whenClause != null) {
+            original.setLength(0);
+            transformed.setLength(0);
+            qualifyNames(whenClause, whenNames, originalWhenText, whenText,
+                         whenClauseTransformations, original, transformed);
+            originalWhenText = original.toString();
+            whenText = transformed.toString();
+        }
+    }
+
+    /**
+     * Qualify all names SQL object names in original and transformed SQL
+     * text for an action or a WHEN clause.
+     *
+     * @param node the query tree node for the transformed version of the
+     *   SQL text, in a bound state
+     * @param tableNames all the TableName nodes in the transformed text,
+     *   in the order in which they appear in the SQL text
+     * @param originalText the original SQL text
+     * @param transformedText the transformed SQL text (with VTI calls for
+     *   transition tables or transition variables)
+     * @param replacements a data structure that describes how {@code
+     *   originalText} was transformed into {@code transformedText}
+     * @param newOriginal where to store the normalized version of the
+     *   original text
+     * @param newTransformed where to store the normalized version of the
+     *   transformed text
+     */
+    private void qualifyNames(
+            QueryTreeNode node,
+            SortedSet<TableName> tableNames,
+            String originalText,
+            String transformedText,
+            List<int[]> replacements,
+            StringBuilder newOriginal,
+            StringBuilder newTransformed) throws StandardException {
+
+        int originalPos = 0;
+        int transformedPos = 0;
+
+        for (TableName name : tableNames) {
+
+            String qualifiedName = name.getFullSQLName();
+
+            int beginOffset = name.getBeginOffset() - node.getBeginOffset();
+            int tokenLength = name.getEndOffset() + 1 - name.getBeginOffset();
+
+            // For the transformed text, use the positions from the node.
+            newTransformed.append(transformedText, transformedPos, beginOffset);
+            newTransformed.append(qualifiedName);
+            transformedPos = beginOffset + tokenLength;
+
+            // For the original text, we need to adjust the positions to
+            // compensate for the changes in the transformed text.
+            Integer origBeginOffset =
+                    getOriginalPosition(replacements, beginOffset);
+            if (origBeginOffset != null) {
+                newOriginal.append(originalText, originalPos, origBeginOffset);
+                newOriginal.append(qualifiedName);
+                originalPos = origBeginOffset + tokenLength;
+            }
+        }
+
+        newTransformed.append(
+                transformedText, transformedPos, transformedText.length());
+        newOriginal.append(originalText, originalPos, originalText.length());
+    }
+
+    /**
+     * Translate a position from the transformed trigger text
+     * ({@link #actionText} or {@link #whenText}) to the corresponding
+     * position in the original trigger text ({@link #originalActionText}
+     * or {@link #originalWhenText}).
+     *
+     * @param replacements a data structure that describes the relationship
+     *   between positions in the original and the transformed text
+     * @param transformedPosition the position to translate
+     * @return the position in the original text, or {@code null} if there
+     *   is no corresponding position in the original text (for example if
+     *   it points to a token that was added to the transformed text and
+     *   does not exist in the original text)
+     */
+    private static Integer getOriginalPosition(
+            List<int[]> replacements, int transformedPosition) {
+
+        // Find the last change before the position we want to translate.
+        for (int i = replacements.size() - 1; i >= 0; i--) {
+            int[] offsets = replacements.get(i);
+
+            // offset[0] is the begin offset of the replaced text
+            // offset[1] is the end offset of the replaced text
+            // offset[2] is the begin offset of the replacement text
+            // offset[3] is the end offset of the replacement text
+
+            // Skip those changes that come after the position we
+            // want to translate.
+            if (transformedPosition >= offsets[2]) {
+                if (transformedPosition < offsets[3]) {
+                    // The position points inside a changed portion of the
+                    // SQL text, so there's no corresponding position in the
+                    // original text. Return null.
+                    return null;
+                } else {
+                    // The position points after the end of the changed text,
+                    // which means it's in a portion that's common to the
+                    // original and the transformed text. Translate between
+                    // the two.
+                    return offsets[1] + (transformedPosition - offsets[3]);
+                }
+            }
+        }
+
+        // The position is before any of the transformations, so the position
+        // is the same in the original and the transformed text.
+        return transformedPosition;
+    }
+
 	/*
 	 * The arrary passed will have either -1 or a column position as it's 
 	 * elements. If the array only has -1 as for all it's elements, then
@@ -689,6 +897,141 @@ public class CreateTriggerNode extends DDLStatementNode
 			return null;
 	}
 
+    /**
+     * Transform the WHEN clause or the triggered SQL statement of a
+     * statement trigger from its original shape to internal syntax where
+     * references to transition tables are replaced with VTIs that return
+     * the before or after image of the changed rows.
+     *
+     * @param node the syntax tree of the WHEN clause or the triggered
+     *   SQL statement
+     * @param originalText the original text of the WHEN clause or the
+     *   triggered SQL statement
+     * @param replacements list that will be populated with int arrays that
+     *   describe how the original text was transformed. The int arrays
+     *   contain the begin (inclusive) and end (exclusive) positions of the
+     *   original text that got replaced and of the replacement text, so that
+     *   positions in the transformed text can be mapped to positions in the
+     *   original text.
+     * @return internal syntax for accessing before or after image of
+     *   the changed rows
+     * @throws StandardException if an error happens while performing the
+     *   transformation
+     */
+    private String transformStatementTriggerText(
+            QueryTreeNode node, String originalText, List<int[]> replacements)
+        throws StandardException
+    {
+        final int offset = node.getBeginOffset();
+        int start = 0;
+        StringBuilder newText = new StringBuilder();
+
+        // For a statement trigger, we find all FromBaseTable nodes. If
+        // the from table is NEW or OLD (or user designated alternates
+        // REFERENCING), we turn them into a trigger table VTI.
+        for (FromBaseTable fromTable : getTransitionTables(node)) {
+            String baseTableName = fromTable.getBaseTableName();
+            int tokBeginOffset = fromTable.getTableNameField().getBeginOffset();
+            int tokEndOffset = fromTable.getTableNameField().getEndOffset();
+            int nextTokenStart = tokEndOffset - offset + 1;
+
+            // Check if this transition table is allowed in this trigger type.
+            checkInvalidTriggerReference(baseTableName);
+
+            // The text up to the transition table name should be kept.
+            newText.append(originalText, start, tokBeginOffset - offset);
+
+            // Replace the transition table name with a VTI.
+            final int replacementOffset = newText.length();
+            newText.append(baseTableName.equals(oldTableName)
+                ? "new org.apache.derby.catalog.TriggerOldTransitionRows() "
+                : "new org.apache.derby.catalog.TriggerNewTransitionRows() ");
+
+            // If the user supplied a correlation, then just
+            // pick it up automatically; otherwise, supply
+            // the default.
+            if (fromTable.getCorrelationName() == null) {
+                newText.append(baseTableName).append(' ');
+            }
+
+            // Record that we have made a change.
+            replacements.add(new int[] {
+                tokBeginOffset - offset,  // offset to original token
+                nextTokenStart,           // offset to next token
+                replacementOffset,        // offset to replacement
+                newText.length()          // offset to token after replacement
+            });
+
+            start = nextTokenStart;
+        }
+
+        // Finally, add everything found after the last transition table
+        // unchanged.
+        newText.append(originalText, start, originalText.length());
+
+        return newText.toString();
+    }
+
+    /**
+     * Get all transition tables referenced by a given node, sorted in the
+     * order in which they appear in the SQL text.
+     *
+     * @param node the node in which to search for transition tables
+     * @return a sorted set of {@code FromBaseTable}s that represent
+     *   transition tables
+     * @throws StandardException if an error occurs
+     */
+    private SortedSet<FromBaseTable> getTransitionTables(Visitable node)
+            throws StandardException {
+
+        CollectNodesVisitor<FromBaseTable> visitor =
+                new CollectNodesVisitor<FromBaseTable>(FromBaseTable.class);
+        node.accept(visitor);
+
+        TreeSet<FromBaseTable> tables =
+                new TreeSet<FromBaseTable>(OFFSET_COMPARATOR);
+
+        for (FromBaseTable fbt : visitor.getList()) {
+            if (!isTransitionTable(fbt)) {
+                // The from table is not the NEW or OLD table, so no need
+                // to do anything. Skip this table.
+                continue;
+            }
+
+            int tokBeginOffset = fbt.getTableNameField().getBeginOffset();
+            if (tokBeginOffset == -1) {
+                // Unknown offset. Skip this table.
+                continue;
+            }
+
+            tables.add(fbt);
+        }
+
+        return tables;
+    }
+
+    /**
+     * Check if a table represents one of the transition tables.
+     *
+     * @param fbt the table to check
+     * @return {@code true} if {@code fbt} represents either the old or
+     *   the new transition table, {@code false} otherwise
+     */
+    private boolean isTransitionTable(FromBaseTable fbt) {
+        // DERBY-6540: It can only be a transition table if the name
+        // is not schema qualified.
+        if (!fbt.getOrigTableName().hasSchema()) {
+            String baseTableName = fbt.getBaseTableName();
+            if (baseTableName != null) {
+                return baseTableName.equals(oldTableName) ||
+                        baseTableName.equals(newTableName);
+            }
+        }
+
+        // Table name didn't match a transition table.
+        return false;
+    }
+
     /*
      * Forbid references to generated columns in the actions of BEFORE triggers.
      * This is DERBY-3948, enforcing the following section of the SQL standard:
@@ -710,16 +1053,17 @@ public class CreateTriggerNode extends DDLStatementNode
 
         if ( genColCount == 0 ) { return; }
 
-        CollectNodesVisitor     visitor = new CollectNodesVisitor( ColumnReference.class );
+        CollectNodesVisitor<ColumnReference> visitor =
+            new CollectNodesVisitor<ColumnReference>(ColumnReference.class);
 
         actionNode.accept( visitor );
 
-        List columnRefs = visitor.getList();
-        int                             colRefCount = columnRefs.size();
+        if (whenClause != null) {
+            whenClause.accept(visitor);
+        }
 
-        for ( int crf_idx = 0; crf_idx < colRefCount; crf_idx++ )
+        for (ColumnReference cr : visitor.getList())
         {
-            ColumnReference     cr = (ColumnReference) columnRefs.get( crf_idx );
             String  colRefName = cr.getColumnName();
             String  tabRefName = cr.getTableName();
 
@@ -777,10 +1121,8 @@ public class CreateTriggerNode extends DDLStatementNode
 			return;
 		}
 
-		for (Iterator it = refClause.iterator(); it.hasNext(); )
+        for (TriggerReferencingStruct trn : refClause)
 		{
-			TriggerReferencingStruct trn = (TriggerReferencingStruct) it.next();
-
 			/*
 			** 1) Make sure that we don't try to refer
 			** to a table for a row trigger or a row for
@@ -851,6 +1193,7 @@ public class CreateTriggerNode extends DDLStatementNode
 	 *
 	 * @exception StandardException		Thrown on failure
 	 */
+    @Override
 	public ConstantAction makeConstantAction() throws StandardException
 	{
 		String oldReferencingName = (oldTableInReferencingClause) ? oldTableName : null;
@@ -868,16 +1211,16 @@ public class CreateTriggerNode extends DDLStatementNode
 											whenText,
 											(UUID)null,			// action SPSid 
 											actionText,
-											(actionCompSchemaId == null) ?
-												compSchemaDescriptor.getUUID() :
-												actionCompSchemaId,
+                                            compSchemaDescriptor.getUUID(),
 											referencedColInts,
 											referencedColsInTriggerAction,
+                                            originalWhenText,
 											originalActionText,
 											oldTableInReferencingClause,
 											newTableInReferencingClause,
 											oldReferencingName,
-											newReferencingName
+                                            newReferencingName,
+                                            providerInfo
 											);
 	}
 
@@ -888,6 +1231,7 @@ public class CreateTriggerNode extends DDLStatementNode
 	 *
 	 * @return	This object as a String
 	 */
+    @Override
 	public String toString()
 	{
 		if (SanityManager.DEBUG)
@@ -895,12 +1239,10 @@ public class CreateTriggerNode extends DDLStatementNode
 			String refString = "null";
 			if (refClause != null)
 			{
-				StringBuffer buf = new StringBuffer();
-				for (Iterator it = refClause.iterator(); it.hasNext(); )
+                StringBuilder buf = new StringBuilder();
+                for (TriggerReferencingStruct trn : refClause)
 				{
 					buf.append("\t");
-					TriggerReferencingStruct trn = 	
-							(TriggerReferencingStruct) it.next();
 					buf.append(trn.toString());
 					buf.append("\n");
 				}
@@ -924,4 +1266,16 @@ public class CreateTriggerNode extends DDLStatementNode
 		}
 	}
 
+    @Override
+    void acceptChildren(Visitor v) throws StandardException {
+        super.acceptChildren(v);
+
+        if (triggerName != null) {
+            triggerName = (TableName) triggerName.accept(v);
+        }
+
+        if (tableName != null) {
+            tableName = (TableName) tableName.accept(v);
+        }
+    }
 }
