@@ -21,8 +21,6 @@
 
 package	org.apache.derby.impl.sql.compile;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
 import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.Properties;
@@ -30,25 +28,15 @@ import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.context.ContextManager;
 import org.apache.derby.iapi.sql.compile.CompilerContext;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.compile.Optimizable;
 import org.apache.derby.iapi.sql.compile.OptimizableList;
 import org.apache.derby.iapi.sql.compile.Optimizer;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
 import org.apache.derby.iapi.sql.conn.Authorizer;
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
-
-import org.apache.derby.iapi.error.StandardException;
-
-import org.apache.derby.iapi.reference.SQLState;
-
 import org.apache.derby.iapi.util.JBitSet;
-import org.apache.derby.iapi.util.ReuseFactory;
 import org.apache.derby.iapi.util.StringUtil;
 
-import java.util.ArrayList;
-import java.util.Properties;
-import java.util.Enumeration;
-
 
 /**
  * A FromList represents the list of tables in a FROM clause in a DML
@@ -56,7 +44,8 @@ import java.util.Enumeration;
  *
  */
 
-public class FromList extends QueryTreeNodeVector implements OptimizableList
+class FromList extends    QueryTreeNodeVector<ResultSetNode>
+               implements OptimizableList
 {
 	Properties	properties;
 	// RESOLVE: The default should be false
@@ -85,28 +74,52 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 */
 	private WindowList windows;
 
+    /**
+     *  Does not change the default for join order optimization, i.e.
+     * {@code false}.
+     * @param cm context manager
+     */
+    FromList(ContextManager cm) {
+        super(ResultSetNode.class, cm);
+        this.isTransparent = false;
+    }
 
-	/** Initializer for a FromList */
+    /**
+     * Constructor for a FromList
+     *
+     * @param optimizeJoinOrder {@code true} if join order optimization is to
+     *                          be performed
+     * @param cm                context manager
+     */
 
-	public void init(Object optimizeJoinOrder)
+    FromList(boolean optimizeJoinOrder, ContextManager cm)
 	{
-		fixedJoinOrder = ! (((Boolean) optimizeJoinOrder).booleanValue());
-		isTransparent = false;
+        super(ResultSetNode.class, cm);
+        constructorMinion(optimizeJoinOrder);
 	}
 
 	/**
-	 * Initializer for a FromList
+     * Constructor for a FromList
 	 *
+     * @param optimizeJoinOrder {@code true} if join order optimization is to
+     *                          be performed
+     * @param fromTable         initialize list with this table
+     * @param cm                context manager
 	 * @exception StandardException		Thrown on error
 	 */
-	public void init(Object optimizeJoinOrder, Object fromTable)
-				throws StandardException
+    FromList(boolean optimizeJoinOrder,
+             FromTable fromTable,
+             ContextManager cm) throws StandardException
 	{
-		init(optimizeJoinOrder);
-
-		addFromTable((FromTable) fromTable);
+        super(ResultSetNode.class, cm);
+        constructorMinion(optimizeJoinOrder);
+        addFromTable(fromTable);
 	}
 
+    private void constructorMinion(boolean optimizeJoinOrder) {
+        this.fixedJoinOrder = !optimizeJoinOrder;
+        this.isTransparent = false;
+    }
 	/*
 	 * OptimizableList interface
 	 */
@@ -149,7 +162,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 * @exception StandardException		Thrown on error
 	 */
 
-	public void addFromTable(FromTable fromTable) throws StandardException
+    final void addFromTable(FromTable fromTable) throws StandardException
 	{
 		/* Don't worry about checking TableOperatorNodes since
 		 * they don't have exposed names.  This will potentially
@@ -160,8 +173,9 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 		 * user is executing a really dumb query and we won't throw
 		 * and exception - consider it an ANSI extension.
 		 */
-        TableName leftTable = null;
-        TableName rightTable = null;
+        TableName leftTable;
+        TableName rightTable;
+
 		if (! (fromTable instanceof TableOperatorNode))
 		{
 			/* Check for duplicate table name in FROM list */
@@ -184,7 +198,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 			}
 		}
 
-		addElement(fromTable);
+        addElement(fromTable);
 	}
 
 	/**
@@ -197,7 +211,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public boolean referencesTarget(String name, boolean baseTable)
+    boolean referencesTarget(String name, boolean baseTable)
 		throws StandardException
 	{
 		FromTable		fromTable;
@@ -226,6 +240,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	public boolean referencesSessionSchema()
 		throws StandardException
 	{
@@ -269,10 +284,9 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	protected FromTable getFromTableByName(String name, String schemaName, boolean exactMatch)
+    FromTable getFromTableByName(String name, String schemaName, boolean exactMatch)
 		throws StandardException
 	{
-		boolean			found = false;
 		FromTable		fromTable;
 		FromTable		result = null;
 
@@ -297,7 +311,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 * @see HalfOuterJoinNode#isJoinColumnForRightOuterJoin
 	 */
 
-	public void isJoinColumnForRightOuterJoin(ResultColumn rc) 
+    void isJoinColumnForRightOuterJoin(ResultColumn rc)
 	{
 		FromTable	fromTable;
 		int size = size();
@@ -307,10 +321,16 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 			fromTable.isJoinColumnForRightOuterJoin(rc);
 		}
 	}
-	public void bindTables(DataDictionary dataDictionary, 
+    void bindTables(DataDictionary dataDictionary,
 							FromList fromListParam) 
 			throws StandardException
 	{
+        //
+        // Don't add USAGE privilege on user-defined types just because we're
+        // binding tables.
+        //
+        boolean wasSkippingTypePrivileges = getCompilerContext().skipTypePrivileges( true );
+        
 		FromTable	fromTable;
 
 		/* Now we bind the tables - this is a 2 step process.
@@ -325,21 +345,27 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 		for (int index = 0; index < size; index++)
 		{
 			fromTable = (FromTable) elementAt(index);
-			ResultSetNode newNode = fromTable.bindNonVTITables(dataDictionary, fromListParam);
+			FromTable   newNode = (FromTable) fromTable.bindNonVTITables(dataDictionary, fromListParam);
 			// If the fromTable is a view in the SESSION schema, then we need to save that information
 			// in referencesSessionSchema element. The reason for this is that the view will get
 			// replaced by it's view definition and we will loose the information that the statement
 			// was referencing a SESSION schema object. 
 			if (fromTable.referencesSessionSchema())
+            {
 				referencesSessionSchema = true;
+            }
+            newNode.setMergeTableID( fromTable.getMergeTableID() );
 			setElementAt(newNode, index);
 		}
 		for (int index = 0; index < size; index++)
 		{
 			fromTable = (FromTable) elementAt(index);
-			ResultSetNode newNode = fromTable.bindVTITables(fromListParam);
+			FromTable   newNode = (FromTable) fromTable.bindVTITables(fromListParam);
 			if (fromTable.referencesSessionSchema())
+            {
 				referencesSessionSchema = true;
+            }
+            newNode.setMergeTableID( fromTable.getMergeTableID() );
 			setElementAt(newNode, index);
 		}
 
@@ -370,6 +396,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
             }
         }
         cc.popCurrentPrivType();
+        getCompilerContext().skipTypePrivileges( wasSkippingTypePrivileges );
 	}
 
 	/**
@@ -380,7 +407,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 * @exception StandardException		Thrown on error
 	 */
 
-	public void bindExpressions( FromList fromListParam )
+    void bindExpressions( FromList fromListParam )
 					throws StandardException
 	{
 		FromTable	fromTable;
@@ -409,7 +436,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 * @exception StandardException		Thrown on error
 	 */
 
-	public void bindResultColumns(FromList fromListParam)
+    void bindResultColumns(FromList fromListParam)
 				throws StandardException
 	{
 		FromTable	fromTable;
@@ -448,7 +475,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 *  select j from onerow where exists
 	 *    (select 1 from somerow
-	 *      union select * from diffrow where onerow.j < diffrow.k)
+	 *      union select * from diffrow where onerow.j &lt; diffrow.k)
 	 *
 	 * If "this" is the FromList for the right child of the UNION then it will
 	 * contain both "diffrow" and "onerow", the latter of which was passed
@@ -464,11 +491,11 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ResultColumnList expandAll(TableName allTableName)
+    ResultColumnList expandAll(TableName allTableName)
 			throws StandardException
 	{
 		ResultColumnList resultColumnList = null;
-		ResultColumnList tempRCList = null;
+        ResultColumnList tempRCList;
 		boolean			 matchfound = false;
 		FromTable	 fromTable;
  
@@ -557,39 +584,57 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	}
 
 	/**
+     * <p>
 	 * Bind a column reference to one of the tables in this FromList.  The column name
 	 * must be unique within the tables in the FromList.  An exception is thrown
-	 * if a column name is not unique.
+	 * if a column name is not unique. This method fills in various fields
+     * in the column reference.
+     * </p>
 	 *
+     * <p>
 	 * NOTE: Callers are responsible for ordering the FromList by nesting level,
 	 * with tables at the deepest (current) nesting level first.  We will try to 
 	 * match against all FromTables at a given nesting level.  If no match is
 	 * found at a nesting level, then we proceed to the next level.  We stop
 	 * walking the list when the nesting level changes and we have found a match.
+     * </p>
 	 *
+     * <p>
 	 * NOTE: If the ColumnReference is qualified, then we will stop the search
 	 * at the first nesting level where there is a match on the exposed table name.
-	 * For example, s (a, b, c), t (d, e, f)
+	 * For example,
+     * <p>
+     *
+     * <pre>
+     *s (a, b, c), t (d, e, f)
 	 *		select * from s where exists (select * from t s where s.c = a)
+     * </pre>
+     *
+     * <p>
 	 * will not find a match for s.c, which is the expected ANSI behavior.
+     * </p>
 	 *
+     * <p>
 	 * bindTables() must have already been called on this FromList before
 	 * calling this method.
+     * </p>
 	 *
 	 * @param columnReference	The ColumnReference describing the column to bind
 	 *
-	 * @return	ResultColumn	The matching ResultColumn
+     * @return ResultColumn     The matching ResultColumn, or {@code null} if
+     *                          there is no matching column
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
 
-	public ResultColumn bindColumnReference(ColumnReference columnReference)
+    ResultColumn bindColumnReference(ColumnReference columnReference)
 				throws StandardException
 	{
 		boolean			columnNameMatch = false;
 		boolean			tableNameMatch = false;
-		FromTable		fromTable;
-		int				currentLevel = -1;
+		FromTable		fromTable = null;
+		FromTable		matchingTable = null;
+        int             currentLevel;
 		int				previousLevel = -1;
 		ResultColumn	matchingRC = null;
 		ResultColumn	resultColumn;
@@ -605,6 +650,20 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 		{
 			fromTable = (FromTable) elementAt(index);
 
+            //
+            // If the MERGE statement has marked its variables, we expect to follow
+            // its judgment. Make sure that we only match SOURCE columns to
+            // SOURCE tables and TARGET columns to TARGET tables.
+            //
+            if (
+                (fromTable.getMergeTableID() != ColumnReference.MERGE_UNKNOWN) &&
+                (columnReference.getMergeTableID() != ColumnReference.MERGE_UNKNOWN) &&
+                (fromTable.getMergeTableID() != columnReference.getMergeTableID())
+                )
+            {
+                continue;
+            }
+
 			/* We can stop if we've found a matching column or table name 
 			 * at the previous nesting level.
 			 */
@@ -625,7 +684,6 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 			previousLevel = currentLevel;
 
 			resultColumn = fromTable.getMatchingColumn(columnReference);
-
 			if (resultColumn != null)
 			{
 				if (! columnNameMatch)
@@ -646,7 +704,11 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 					columnNameMatch = true;
 
 					if (fromTable.isPrivilegeCollectionRequired())
-						getCompilerContext().addRequiredColumnPriv( resultColumn.getTableColumnDescriptor());						
+                    {
+						getCompilerContext().addRequiredColumnPriv( resultColumn.getTableColumnDescriptor());
+                    }
+
+                    matchingTable = fromTable;
 				}
 				else
 				{
@@ -663,6 +725,21 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 						 crTableName.equals(fromTable.getExposedName()) );
 		}
 
+        // fill in the table name
+        if ( (matchingTable != null) && (matchingRC != null) && (columnReference.getTableName() == null) )
+        {
+            TableName   crtn = matchingTable.getTableName();
+            if ( matchingTable instanceof FromBaseTable )
+            {
+                FromBaseTable   fbt = (FromBaseTable) matchingTable;
+                if ( fbt.getExposedTableName() !=  null )
+                {
+                    crtn = fbt.getExposedTableName();
+                }
+            }
+            columnReference.setQualifiedTableName( crtn );
+        }
+        
 		return matchingRC;
 	}
 
@@ -674,7 +751,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *									directly under a ResultColumn
 	 */
 
-	public void rejectParameters() throws StandardException
+    void rejectParameters() throws StandardException
 	{
 		FromTable	fromTable;
 
@@ -689,7 +766,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	// This method reorders LOJs in the FROM clause.
 	// For now, we process only a LOJ.  For example, "... from LOJ_1, LOJ2 ..."
 	// will not be processed. 
-	public boolean LOJ_reorderable(int numTables) throws StandardException
+    boolean LOJ_reorderable(int numTables) throws StandardException
 	{
 		boolean anyChange = false;
 
@@ -719,7 +796,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void preprocess(int numTables,
+    void preprocess(int numTables,
 						   GroupByList gbl,
 						   ValueNode predicateTree)
 								throws StandardException
@@ -751,7 +828,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void flattenFromTables(ResultColumnList rcl,
+    void flattenFromTables(ResultColumnList rcl,
 								  PredicateList predicateList,
 								  SubqueryList sql,
                                   GroupByList gbl,
@@ -759,7 +836,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 									throws StandardException
 	{
 		boolean			flattened = true;
-		ArrayList		flattenedTableNumbers = new ArrayList();
+		ArrayList<Integer>		flattenedTableNumbers = new ArrayList<Integer>();
 
 		if (SanityManager.DEBUG)
 		{
@@ -789,8 +866,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 					ft.isFlattenableJoinNode())
 				{
 					//save the table number of the node to be flattened
-                    flattenedTableNumbers.add(
-                            ReuseFactory.getInteger(ft.getTableNumber()));
+                    flattenedTableNumbers.add(ft.getTableNumber());
 
 					/* Remove the node from the list and insert its
 					 * FromList here.
@@ -889,7 +965,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 * @param level		The query block level for this table.
 	 */
-	public void setLevel(int level)
+    void setLevel(int level)
 	{
 		int size = size();
 		for (int index = 0; index < size; index++)
@@ -907,7 +983,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 * @return FromTable	The matching FromTable.
 	 */
-	public FromTable getFromTableByResultColumn(ResultColumn rc)
+    FromTable getFromTableByResultColumn(ResultColumn rc)
 	{
 		FromTable	fromTable = null;
 
@@ -935,7 +1011,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void setProperties(Properties props) throws StandardException
+    void setProperties(Properties props) throws StandardException
 	{
 		properties = props;
 
@@ -944,7 +1020,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 		** there is nothing in this properties list that relies on binding
 		** or optimization to validate.
 		*/
-		Enumeration e = properties.keys();
+        Enumeration<?> e = properties.keys();
 		while (e.hasMoreElements())
 		{
 			String key = (String) e.nextElement();
@@ -1025,15 +1101,17 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 			*/
 			if (sum != ( ( joinOrder.length * (joinOrder.length - 1) ) / 2) )
 			{
-				String arrayVals = "";
-				for (int i = 0; i < joinOrder.length; i++)
-					arrayVals = arrayVals + joinOrder[i] + " ";
+                StringBuilder arrayVals = new StringBuilder();
+                for (int i : joinOrder) {
+                    arrayVals.append(i);
+                    arrayVals.append(' ');
+                }
 				SanityManager.THROWASSERT("joinOrder array has some duplicate value: " + arrayVals);
 			}
 		}
 
 		/* Form a list that's in the order we want */
-		QueryTreeNode[] orderedFL = new FromTable[joinOrder.length];
+        ResultSetNode[] orderedFL = new FromTable[joinOrder.length];
 		for (posn = 0; posn < joinOrder.length; posn++)
 		{
 			/*
@@ -1098,7 +1176,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void bindUntypedNullsToResultColumns(ResultColumnList bindingRCL)
+    void bindUntypedNullsToResultColumns(ResultColumnList bindingRCL)
 				throws StandardException
 	{
 		int size = size();
@@ -1229,11 +1307,11 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 		ColumnReference	additionalCR = null;
 
 		PredicateList predicatesTemp;
-		predicatesTemp = (PredicateList) getNodeFactory().getNode(
-			C_NodeTypes.PREDICATE_LIST,	getContextManager());
-		int wherePredicatesSize = wherePredicates.size();
-		for (int index = 0; index < wherePredicatesSize; index++)
-			predicatesTemp.addPredicate((Predicate)wherePredicates.elementAt(index));
+        predicatesTemp = new PredicateList(getContextManager());
+
+        for (Predicate p : wherePredicates) {
+            predicatesTemp.addPredicate(p);
+        }
 
 		/* When considering subquery flattening, we are interested
 		 * in the 1st (and only) entry in the RCL.  (The RCL will be
@@ -1243,7 +1321,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 		 */
 		if (rcl != null)
 		{
-			ResultColumn rc = (ResultColumn) rcl.elementAt(0);
+            ResultColumn rc = rcl.elementAt(0);
 			if (rc.getExpression() instanceof ColumnReference)
 			{
 				additionalCR = (ColumnReference) rc.getExpression();
@@ -1308,12 +1386,11 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 			if (fbt.getExistsBaseTable())
 			{
 				int existsTableNumber = fbt.getTableNumber();
-				int predicatesTempSize = predicatesTemp.size();
-				for (int predicatesTempIndex = predicatesTempSize-1;
-					predicatesTempIndex >= 0; predicatesTempIndex--)
+
+                for (int i = predicatesTemp.size() - 1; i >= 0; i--)
 				{
-					AndNode topAndNode = (AndNode)
-						((Predicate) predicatesTemp.elementAt(predicatesTempIndex)).getAndNode();
+                    AndNode topAndNode =
+                            predicatesTemp.elementAt(i).getAndNode();
 
 					for (ValueNode whereWalker = topAndNode; whereWalker instanceof AndNode;
 						whereWalker = ((AndNode) whereWalker).getRightOperand())
@@ -1333,7 +1410,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 						JBitSet referencedTables = and.getLeftOperand().getTablesReferenced();
 						if (referencedTables.get(existsTableNumber))
 						{
-							predicatesTemp.removeElementAt(predicatesTempIndex);
+                            predicatesTemp.removeElementAt(i);
 							break;
 						}
 					}
@@ -1349,7 +1426,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 		tableNumbers = getTableNumbers();
 		JBitSet[][] tableColMap = new JBitSet[size][size];
 		boolean[] oneRow = new boolean[size];
-		boolean oneRowResult = false;
+        boolean oneRowResult;
 
 		/* See if each table has a uniqueness condition */
 		for (int index = 0; index < size; index++)
@@ -1597,17 +1674,17 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 *
 	 * @return	The lock mode
 	 */
-	public int updateTargetLockMode()
+    int updateTargetLockMode()
 	{
 		if (SanityManager.DEBUG)
 		{
 			if (size() != 1)
 			{
-				SanityManager.THROWASSERT(
-					"size() expected to be 1");
+				SanityManager.THROWASSERT
+                    ( "size() is " + size() + " but should be 1");
 			}
 		}
-		return ((ResultSetNode) elementAt(0)).updateTargetLockMode();
+        return elementAt(0).updateTargetLockMode();
 	}
 
 	/**
@@ -1650,7 +1727,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	 * Set windows field to the supplied value.
 	 * @param windows list of window definitions associated with a SELECT.
 	 */
-	public void setWindows(WindowList windows) {
+    void setWindows(WindowList windows) {
 		this.windows = windows;
 	}
 
@@ -1658,7 +1735,7 @@ public class FromList extends QueryTreeNodeVector implements OptimizableList
 	/**
 	 * @return list of window definitions associated with a SELECT.
 	 */
-	public WindowList getWindows() {
+    WindowList getWindows() {
 		return windows;
 	}
 }
