@@ -21,17 +21,14 @@
 
 package	org.apache.derby.impl.sql.compile;
 
+import java.util.Properties;
+import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.services.compiler.MethodBuilder;
-
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.sql.compile.CostEstimate;
 import org.apache.derby.iapi.sql.compile.Optimizable;
 import org.apache.derby.iapi.sql.compile.OptimizablePredicate;
-import org.apache.derby.iapi.sql.compile.CostEstimate;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
-
-import org.apache.derby.iapi.error.StandardException;
-
 import org.apache.derby.iapi.util.JBitSet;
 
 /**
@@ -41,13 +38,13 @@ import org.apache.derby.iapi.util.JBitSet;
  *
  */
 
-public class HalfOuterJoinNode extends JoinNode
+class HalfOuterJoinNode extends JoinNode
 {
 	private boolean rightOuterJoin;
 	private boolean transformed = false;
 
 	/**
-	 * Initializer for a HalfOuterJoinNode.
+     * Constructor for a HalfOuterJoinNode.
 	 *
 	 * @param leftResult		The ResultSetNode on the left side of this join
 	 * @param rightResult		The ResultSetNode on the right side of this join
@@ -56,28 +53,28 @@ public class HalfOuterJoinNode extends JoinNode
 	 * @param rightOuterJoin	Whether or not this node represents a user
 	 *							specified right outer join
 	 * @param tableProperties	Properties list associated with the table
+     * @param cm                The context manager
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public void init(
-							Object leftResult,
-							Object rightResult,
-							Object onClause,
-							Object usingClause,
-							Object rightOuterJoin,
-							Object tableProperties)
+    HalfOuterJoinNode(ResultSetNode leftResult,
+                      ResultSetNode rightResult,
+                      ValueNode onClause,
+                      ResultColumnList usingClause,
+                      boolean rightOuterJoin,
+                      Properties tableProperties,
+                      ContextManager cm)
 		throws StandardException
 	{
-		super.init(
-				leftResult,
-				rightResult,
-				onClause,
-				usingClause,
-				null,
+        super(  leftResult,
+                rightResult,
+                onClause,
+                usingClause,
+                null,
 				tableProperties,
-				null);
-		this.rightOuterJoin = ((Boolean) rightOuterJoin).booleanValue();
+                null,
+                cm);
+        this.rightOuterJoin = rightOuterJoin;
 
 		/* We can only flatten an outer join
 		 * using the null intolerant predicate xform.
@@ -95,7 +92,7 @@ public class HalfOuterJoinNode extends JoinNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
+    @Override
 	public boolean pushOptPredicate(OptimizablePredicate optimizablePredicate)
 			throws StandardException
 	{
@@ -118,7 +115,7 @@ public class HalfOuterJoinNode extends JoinNode
 	 *
 	 * @return	This object as a String
 	 */
-
+    @Override
 	public String toString()
 	{
 		if (SanityManager.DEBUG)
@@ -157,7 +154,8 @@ public class HalfOuterJoinNode extends JoinNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ResultSetNode preprocess(int numTables,
+    @Override
+    ResultSetNode preprocess(int numTables,
 									GroupByList gbl,
 									FromList fromList)
 								throws StandardException
@@ -199,7 +197,8 @@ public class HalfOuterJoinNode extends JoinNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void pushExpressions(PredicateList outerPredicateList)
+    @Override
+    void pushExpressions(PredicateList outerPredicateList)
 					throws StandardException
 	{
 		FromTable		leftFromTable = (FromTable) leftResultSet;
@@ -221,7 +220,7 @@ public class HalfOuterJoinNode extends JoinNode
 		{
 			Predicate predicate;
 
-			predicate = (Predicate) joinPredicates.elementAt(index);
+            predicate = joinPredicates.elementAt(index);
 			if (! predicate.getPushable())
 			{
 				continue;
@@ -234,10 +233,7 @@ public class HalfOuterJoinNode extends JoinNode
 		}
 
 		/* Recurse down both sides of tree */
-		PredicateList	noPredicates =
-						(PredicateList) getNodeFactory().getNode(
-											C_NodeTypes.PREDICATE_LIST,
-											getContextManager());
+        PredicateList noPredicates = new PredicateList(getContextManager());
 		leftFromTable.pushExpressions(getLeftPredicateList());
 		rightFromTable.pushExpressions(noPredicates);
 	}
@@ -284,7 +280,8 @@ public class HalfOuterJoinNode extends JoinNode
      *                 so caller can know whether rebinding may be necessary
      * @throws StandardException standard error policy
      */
-	public boolean LOJ_reorderable(int numTables)
+    @Override
+    boolean LOJ_reorderable(int numTables)
 		throws StandardException
 	{
 		boolean anyChange = false;
@@ -463,17 +460,16 @@ public class HalfOuterJoinNode extends JoinNode
                 // ((HalfOuterJoinNode)logicalRightResultSet).transformed =
                 //     local_transformed;
 
-                FromList localFromList = (FromList) getNodeFactory().getNode(
-                    C_NodeTypes.FROM_LIST,
-                    getNodeFactory().doJoinOrderOptimization(),
-                    getContextManager());
+                FromList localFromList = new FromList(
+                        getOptimizerFactory().doJoinOrderOptimization(),
+                        getContextManager());
 
                 // switch OJ nodes: by handling the current OJ node
                 leftResultSet  = logicalRightResultSet;
                 rightResultSet = RChild;
 
                 // rebuild the result columns and re-bind column references
-                ((HalfOuterJoinNode)leftResultSet).resultColumns = null;
+                ((HalfOuterJoinNode)leftResultSet).setResultColumns( null );
                  // localFromList is empty:
                 ((JoinNode)leftResultSet).bindResultColumns(localFromList);
 
@@ -618,15 +614,15 @@ private boolean isNullRejecting (
 
 	// This method re-binds the result columns which may be referenced in the ON
 	// clause in this node.
-	public boolean LOJ_bindResultColumns(boolean anyChange)
+    boolean LOJ_bindResultColumns(boolean anyChange)
 		throws StandardException
 	{
 		if (anyChange)
 		{
-			this.resultColumns = null;
-			FromList localFromList = (FromList) getNodeFactory().getNode(C_NodeTypes.FROM_LIST,
-																		 getNodeFactory().doJoinOrderOptimization(),
-																		 getContextManager());
+			setResultColumns( null );
+            FromList localFromList = new FromList(
+                    getOptimizerFactory().doJoinOrderOptimization(),
+                    getContextManager());
 			((JoinNode)this).bindResultColumns(localFromList);
 		}
 		return anyChange;
@@ -644,7 +640,8 @@ private boolean isNullRejecting (
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public FromTable transformOuterJoins(ValueNode predicateTree, int numTables)
+    @Override
+    FromTable transformOuterJoins(ValueNode predicateTree, int numTables)
 		throws StandardException
 	{
 		ResultSetNode innerRS;
@@ -691,7 +688,8 @@ private boolean isNullRejecting (
 			ValueNode left = and.getLeftOperand();
 
 			/* Skip IS NULL predicates as they are not null intolerant */
-			if (left.isInstanceOf(C_NodeTypes.IS_NULL_NODE))
+            if (left.getClass().equals(IsNullNode.class) &&
+                ((IsNullNode)left).isNullNode())
 			{
 				vn = and.getRightOperand();
 				continue;
@@ -719,20 +717,18 @@ private boolean isNullRejecting (
 					if (refMap.get(bit) && innerMap.get(bit))
 					{
 						// OJ -> IJ
-						JoinNode ij =  (JoinNode)
-											getNodeFactory().getNode(
-												C_NodeTypes.JOIN_NODE,
+                        JoinNode ij =  new JoinNode(
 												leftResultSet,
 												rightResultSet,
 												joinClause,
 												null,
-												resultColumns,
+												getResultColumns(),
 												null,
 												null,
 												getContextManager());
 						ij.setTableNumber(tableNumber);
 						ij.setSubqueryList(subqueryList);
-						ij.setAggregateVector(aggregateVector);
+                        ij.setAggregates(aggregates);
 						return ij;
 					}
 				}
@@ -751,6 +747,7 @@ private boolean isNullRejecting (
 	}
 
 	/** @see JoinNode#adjustNumberOfRowsReturned */
+    @Override
 	protected void adjustNumberOfRowsReturned(CostEstimate costEstimate)
 	{
 		/*
@@ -773,8 +770,8 @@ private boolean isNullRejecting (
 	 *
 	 * @exception StandardException		Thrown on error
      */
-	public void generate(ActivationClassBuilder acb,
-								MethodBuilder mb)
+    @Override
+    void generate(ActivationClassBuilder acb, MethodBuilder mb)
 						throws StandardException
 	{
 		/* Verify that a user specifed right outer join is transformed into
@@ -803,8 +800,8 @@ private boolean isNullRejecting (
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	 protected int addOuterJoinArguments(ActivationClassBuilder acb,
-											MethodBuilder mb)
+    @Override
+    int addOuterJoinArguments(ActivationClassBuilder acb, MethodBuilder mb)
 		 throws StandardException
 	 {
 		/* Nulls always generated from the right */
@@ -819,14 +816,15 @@ private boolean isNullRejecting (
 	/**
 	 * Return the number of arguments to the join result set.
 	 */
+    @Override
 	protected int getNumJoinArguments()
 	{
 		/* We add two more arguments than the superclass does */
 		return super.getNumJoinArguments() + 2;
 	}
 
-	protected void oneRowRightSide(ActivationClassBuilder acb,
-									   MethodBuilder mb)
+    @Override
+    void oneRowRightSide(ActivationClassBuilder acb, MethodBuilder mb)
 	{
 		// always return false for now
 		mb.push(false);
@@ -839,6 +837,7 @@ private boolean isNullRejecting (
 	 * (For RIGHT OUTER JOIN, the left is the right
 	 * and the right is the left and the JOIN is the NIOJ).
 	 */
+    @Override
 	ResultSetNode getLogicalLeftResultSet()
 	{
 		if (rightOuterJoin)
@@ -857,6 +856,7 @@ private boolean isNullRejecting (
 	 * (For RIGHT OUTER JOIN, the left is the right
 	 * and the right is the left and the JOIN is the NIOJ).
 	 */
+    @Override
 	ResultSetNode getLogicalRightResultSet()
 	{
 		if (rightOuterJoin)
@@ -873,7 +873,7 @@ private boolean isNullRejecting (
 	 * Return true if right outer join or false if left outer join
 	 * Used to set Nullability correctly in JoinNode
 	 */
-	public boolean isRightOuterJoin()
+    boolean isRightOuterJoin()
 	{
 		return rightOuterJoin;
 	}
@@ -883,7 +883,8 @@ private boolean isNullRejecting (
 	 *  check if the passed ResultColumn is a join column. If yes, then 
 	 *  ResultColumn should be marked such. DERBY-4631
 	 */
-	public void isJoinColumnForRightOuterJoin(ResultColumn rc) 
+    @Override
+    void isJoinColumnForRightOuterJoin(ResultColumn rc)
 	{
 		if (isRightOuterJoin() && usingClause != null &&  
 				usingClause.getResultColumn(rc.getUnderlyingOrAliasName()) != null) {
@@ -893,13 +894,13 @@ private boolean isNullRejecting (
 	}
 
 	// return the Null-producing table references
-	public JBitSet LOJgetNPReferencedTables(int numTables)
+    JBitSet LOJgetNPReferencedTables(int numTables)
 				throws StandardException
 	{
 		if (rightOuterJoin && !transformed)
-			return (JBitSet) leftResultSet.LOJgetReferencedTables(numTables);
+            return leftResultSet.LOJgetReferencedTables(numTables);
 		else
-			return (JBitSet) rightResultSet.LOJgetReferencedTables(numTables);
+            return rightResultSet.LOJgetReferencedTables(numTables);
 	}
 
     // return the row-preserving table references
@@ -907,8 +908,8 @@ private boolean isNullRejecting (
                 throws StandardException
     {
         if (rightOuterJoin && !transformed)
-            return (JBitSet) rightResultSet.LOJgetReferencedTables(numTables);
+            return rightResultSet.LOJgetReferencedTables(numTables);
         else
-            return (JBitSet) leftResultSet.LOJgetReferencedTables(numTables);
+            return leftResultSet.LOJgetReferencedTables(numTables);
     }
 }
