@@ -21,28 +21,19 @@
 
 package	org.apache.derby.impl.sql.compile;
 
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
-
+import java.lang.reflect.Modifier;
 import org.apache.derby.iapi.error.StandardException;
-
 import org.apache.derby.iapi.reference.ClassName;
-
-import org.apache.derby.iapi.types.TypeId;
+import org.apache.derby.iapi.services.classfile.VMOpcode;
+import org.apache.derby.iapi.services.compiler.LocalField;
+import org.apache.derby.iapi.services.compiler.MethodBuilder;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.services.loader.ClassFactory;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.sql.compile.Optimizable;
 import org.apache.derby.iapi.types.DataTypeDescriptor;
 import org.apache.derby.iapi.types.DataValueDescriptor;
-
-import org.apache.derby.iapi.services.compiler.MethodBuilder;
-import org.apache.derby.iapi.services.compiler.LocalField;
-import org.apache.derby.iapi.services.loader.ClassFactory;
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
-import org.apache.derby.iapi.sql.compile.Optimizable;
-
-import org.apache.derby.impl.sql.compile.ExpressionClassBuilder;
-
-import org.apache.derby.iapi.services.classfile.VMOpcode;
-
-import java.lang.reflect.Modifier;
+import org.apache.derby.iapi.types.TypeId;
 
 /**
  * An InListOperatorNode represents an IN list.
@@ -54,25 +45,26 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 	private boolean isOrdered;
 	private boolean sortDescending;
 
-	/**
-	 * Initializer for a InListOperatorNode
-	 *
-	 * @param leftOperand		The left operand of the node
-	 * @param rightOperandList	The right operand list of the node
-	 */
+    /**
+     * @param leftOperand The left operand of the node
+     * @param rightOperandList The right operand list of the node
+     * @param cm Context manager
+     * @throws StandardException
+     */
+    InListOperatorNode(
+            ValueNode leftOperand,
+            ValueNodeList rightOperandList,
+            ContextManager cm) throws StandardException {
+        super(leftOperand, rightOperandList, "IN", "in", cm);
+    }
 
-	public void init(Object leftOperand, Object rightOperandList)
-	{
-		init(leftOperand, rightOperandList, "IN", "in");
-	}
-
-	/**
+    /**
 	 * Convert this object to a String.  See comments in QueryTreeNode.java
 	 * for how this should be done for tree printing.
 	 *
 	 * @return	This object as a String
 	 */
-
+    @Override
 	public String toString()
 	{
 		if (SanityManager.DEBUG)
@@ -93,9 +85,7 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 	 */
 	protected InListOperatorNode shallowCopy() throws StandardException
 	{
-		InListOperatorNode ilon =
-			 (InListOperatorNode)getNodeFactory().getNode(
-				C_NodeTypes.IN_LIST_OPERATOR_NODE,
+        InListOperatorNode ilon = new InListOperatorNode(
 				leftOperand,
 				rightOperandList,
 				getContextManager());
@@ -125,7 +115,8 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ValueNode preprocess(int numTables,
+    @Override
+    ValueNode preprocess(int numTables,
 								FromList outerFromList,
 								SubqueryList outerSubqueryList,
 								PredicateList outerPredicateList) 
@@ -141,12 +132,12 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 		if (rightOperandList.size() == 1)
 		{
 			BinaryComparisonOperatorNode equal = 
-				(BinaryComparisonOperatorNode) getNodeFactory().getNode(
-						C_NodeTypes.BINARY_EQUALS_OPERATOR_NODE,
-						leftOperand, 
-						(ValueNode) rightOperandList.elementAt(0),
-						Boolean.FALSE,
-						getContextManager());
+                new BinaryRelationalOperatorNode(
+                    BinaryRelationalOperatorNode.K_EQUALS,
+                    leftOperand,
+                    rightOperandList.elementAt(0),
+                    false,
+                    getContextManager());
 			/* Set type info for the operator node */
 			equal.bindComparisonOperator();
 			return equal;
@@ -164,11 +155,8 @@ public final class InListOperatorNode extends BinaryListOperatorNode
         if ((leftOperand.getTypeServices().getTypeId().typePrecedence() !=
                     targetTypePrecedence) &&
                 !rightOperandList.allSamePrecendence(targetTypePrecedence)) {
-            CastNode cn = (CastNode) getNodeFactory().getNode(
-                    C_NodeTypes.CAST_NODE,
-                    leftOperand,
-                    targetType,
-                    getContextManager());
+            CastNode cn =
+                    new CastNode(leftOperand, targetType, getContextManager());
             cn.bindCastNodeOnly();
             leftOperand = cn;
         }
@@ -267,9 +255,8 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 				rightOperandList.sortInAscendingOrder(judgeODV);
 				isOrdered = true;
 
-				ValueNode minValue = (ValueNode)rightOperandList.elementAt(0);
-				ValueNode maxValue =
-					(ValueNode)rightOperandList.elementAt(
+                ValueNode minValue = rightOperandList.elementAt(0);
+                ValueNode maxValue = rightOperandList.elementAt(
 						rightOperandList.size() - 1);
 
 				/* Handle the degenerate case where the min and the max
@@ -284,11 +271,11 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 				if (judgeODV.equals(minODV, maxODV).equals(true))
 				{
 					BinaryComparisonOperatorNode equal = 
-						(BinaryComparisonOperatorNode)getNodeFactory().getNode(
-							C_NodeTypes.BINARY_EQUALS_OPERATOR_NODE,
+                        new BinaryRelationalOperatorNode(
+                            BinaryRelationalOperatorNode.K_EQUALS,
 							leftOperand, 
 							minValue,
-							Boolean.FALSE,
+                            false,
 							getContextManager());
 					/* Set type info for the operator node */
 					equal.bindComparisonOperator();
@@ -315,11 +302,9 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 			 * the list.  This is arbitrary and should not matter in the
 			 * big picture.
 			 */
-			ValueNode srcVal = (ValueNode) rightOperandList.elementAt(0);
-			ParameterNode pNode =
-				(ParameterNode) getNodeFactory().getNode(
-					C_NodeTypes.PARAMETER_NODE,
-					new Integer(0),
+            ValueNode srcVal = rightOperandList.elementAt(0);
+            ParameterNode pNode = new ParameterNode(
+                    0,
 					null, // default value
 					getContextManager());
 
@@ -353,12 +338,12 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 			 * to "this").
 			 */
 			BinaryComparisonOperatorNode equal = 
-				(BinaryComparisonOperatorNode) getNodeFactory().getNode(
-					C_NodeTypes.BINARY_EQUALS_OPERATOR_NODE,
+                new BinaryRelationalOperatorNode(
+                    BinaryRelationalOperatorNode.K_EQUALS,
 					leftOperand, 
 					pNode,
 					this,
-					Boolean.FALSE,
+                    false,
 					getContextManager());
 
 			/* Set type info for the operator node */
@@ -385,9 +370,8 @@ public final class InListOperatorNode extends BinaryListOperatorNode
             // Iterate through the entire list of values to find out
             // what the dominant type is.
             ClassFactory cf = getClassFactory();
-            int sz = rightOperandList.size();
-            for (int i = 0; i < sz; i++) {
-                ValueNode vn = (ValueNode) rightOperandList.elementAt(i);
+
+            for (ValueNode vn : rightOperandList) {
                 targetType = targetType.getDominantType(
                         vn.getTypeServices(), cf);
             }
@@ -412,10 +396,10 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	ValueNode eliminateNots(boolean underNotNode) 
 					throws StandardException
 	{
-		AndNode						 newAnd = null;
 		BinaryComparisonOperatorNode leftBCO;
 		BinaryComparisonOperatorNode rightBCO;
 		int							 listSize = rightOperandList.size();
@@ -448,13 +432,12 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 		 * requires each <> node to have a separate object.
 		 */
 		ValueNode leftClone = (leftOperand instanceof ColumnReference) ? leftOperand.getClone() : leftOperand;
-		leftBCO = (BinaryComparisonOperatorNode) 
-					getNodeFactory().getNode(
-						C_NodeTypes.BINARY_NOT_EQUALS_OPERATOR_NODE,
-						leftClone,
-						(ValueNode) rightOperandList.elementAt(0),
-						Boolean.FALSE,
-						getContextManager());
+        leftBCO = new BinaryRelationalOperatorNode(
+                BinaryRelationalOperatorNode.K_NOT_EQUALS,
+                leftClone,
+                rightOperandList.elementAt(0),
+                false,
+                getContextManager());
 		/* Set type info for the operator node */
 		leftBCO.bindComparisonOperator();
 
@@ -465,22 +448,18 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 
 			/* leftO <> rightOList.elementAt(elemsDone) */
 			leftClone = (leftOperand instanceof ColumnReference) ? leftOperand.getClone() : leftOperand;
-			rightBCO = (BinaryComparisonOperatorNode) 
-						getNodeFactory().getNode(
-							C_NodeTypes.BINARY_NOT_EQUALS_OPERATOR_NODE,
-							leftClone,
-							(ValueNode) rightOperandList.elementAt(elemsDone),
-							Boolean.FALSE,
-							getContextManager());
+            rightBCO = new BinaryRelationalOperatorNode(
+                    BinaryRelationalOperatorNode.K_NOT_EQUALS,
+                    leftClone,
+                    rightOperandList.elementAt(elemsDone),
+                    false,
+                    getContextManager());
 			/* Set type info for the operator node */
 			rightBCO.bindComparisonOperator();
 
 			/* Create the AND */
-			newAnd = (AndNode) getNodeFactory().getNode(
-												C_NodeTypes.AND_NODE,
-												leftSide,
-												rightBCO,
-												getContextManager());
+            AndNode newAnd =
+                    new AndNode(leftSide, rightBCO, getContextManager());
 			newAnd.postBindFixup();
 
 			leftSide = newAnd;
@@ -498,15 +477,14 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public boolean selfReference(ColumnReference cr)
+    boolean selfReference(ColumnReference cr)
 		throws StandardException
 	{
-		int size = rightOperandList.size();
-		for (int i = 0; i < size; i++)
+        for (ValueNode vn : rightOperandList)
 		{
-			ValueNode vn = (ValueNode) rightOperandList.elementAt(i);
-			if (vn.getTablesReferenced().get(cr.getTableNumber()))
+            if (vn.getTablesReferenced().get(cr.getTableNumber())) {
 				return true;
+            }
 		}
 		return false;
 	}
@@ -515,6 +493,7 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 	 * The selectivity for an "IN" predicate is generally very small.
 	 * This is an estimate applicable when in list are not all constants.
 	 */
+    @Override
 	public double selectivity(Optimizable optTable)
 	{
 		return 0.3d;
@@ -527,10 +506,9 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 	 * @param mb The MethodBuilder the expression will go into
 	 *
 	 * @exception StandardException		Thrown on error
-	 */
-
-	public void generateExpression(ExpressionClassBuilder acb,
-											MethodBuilder mb)
+     */
+    @Override
+    void generateExpression(ExpressionClassBuilder acb, MethodBuilder mb)
 									throws StandardException
 	{
 		int			listSize = rightOperandList.size();
@@ -564,8 +542,6 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 		** to the new method.
 		*/
 
-		receiver = leftOperand;
-
 		/* Figure out the result type name */
 		resultTypeName = getTypeCompiler().interfaceName();
 
@@ -661,7 +637,7 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 			}
 
 			setArrayMethod.getField(arrayField); // first arg
-			((ValueNode) rightOperandList.elementAt(index)).generateExpression(acb, setArrayMethod);
+            rightOperandList.elementAt(index).generateExpression(acb, setArrayMethod);
 			setArrayMethod.upCast(ClassName.DataValueDescriptor); // second arg
 			setArrayMethod.setArrayElement(index);
 		}
@@ -693,7 +669,7 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void generateStartStopKey(boolean isAsc, boolean isStartKey,
+    void generateStartStopKey(boolean isAsc, boolean isStartKey,
 									 ExpressionClassBuilder acb,
 									 MethodBuilder mb)
 											   throws StandardException
@@ -747,7 +723,7 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 							  ((i == 0) ? 4 : 3);
 			for (int j = 0; j < numVals; j++)
 			{
-				ValueNode vn = (ValueNode) rightOperandList.elementAt(currentOpnd++);
+                ValueNode vn = rightOperandList.elementAt(currentOpnd++);
 				vn.generateExpression(acb, mb);
 				mb.upCast(ClassName.DataValueDescriptor);
 			}
@@ -773,14 +749,18 @@ public final class InListOperatorNode extends BinaryListOperatorNode
 
 			/* decide to get min or max value
 			 */
-			String methodName;
+            String methodNam;
 			if ((isAsc && isStartKey) || (! isAsc && ! isStartKey))
-				methodName = "minValue";
+                methodNam = "minValue";
 			else
-				methodName = "maxValue";
+                methodNam = "maxValue";
 		
-			mb.callMethod(VMOpcode.INVOKESTATIC, ClassName.BaseExpressionActivation, methodName, ClassName.DataValueDescriptor, 12);
-
+            mb.callMethod(
+                VMOpcode.INVOKESTATIC,
+                ClassName.BaseExpressionActivation,
+                methodNam,
+                ClassName.DataValueDescriptor,
+                12);
 		}
 	}
 
