@@ -21,39 +21,26 @@
 
 package org.apache.derby.impl.sql.compile;
 
-import org.apache.derby.iapi.services.compiler.MethodBuilder;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
-
-
-import org.apache.derby.iapi.error.StandardException;
-
-import org.apache.derby.iapi.types.TypeId;
-import org.apache.derby.iapi.types.StringDataValue;
-import org.apache.derby.iapi.types.DataTypeDescriptor;
-
-import org.apache.derby.iapi.sql.compile.TypeCompiler;
-
-import org.apache.derby.iapi.reference.Limits;
-import org.apache.derby.iapi.reference.SQLState;
-
-
-import org.apache.derby.iapi.util.ReuseFactory;
-
-import org.apache.derby.iapi.services.classfile.VMOpcode;
-
-import org.apache.derby.iapi.types.Like;
-
 import java.sql.Types;
 import java.util.Arrays;
 import java.util.List;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.reference.Limits;
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.classfile.VMOpcode;
+import org.apache.derby.iapi.services.compiler.MethodBuilder;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.types.DataTypeDescriptor;
+import org.apache.derby.iapi.types.Like;
+import org.apache.derby.iapi.types.StringDataValue;
+import org.apache.derby.iapi.types.TypeId;
 
-  
 /**
     This node represents a like comparison operator (no escape)
 
     If the like pattern is a constant or a parameter then if possible
-    the like is modified to include a >= and < operator. In some cases
-    the like can be eliminated.  By adding =, >= or < operators it may
+    the like is modified to include a &gt;= and &lt; operator. In some cases
+    the like can be eliminated.  By adding =, &gt;= or &lt; operators it may
     allow indexes to be used to greatly narrow the search range of the
     query, and allow optimizer to estimate number of rows to affected.
 
@@ -61,36 +48,36 @@ import java.util.List;
     constant or parameter LIKE pattern with prefix followed by optional wild 
     card e.g. Derby%
 
-    CHAR(n), VARCHAR(n) where n < 255
+    CHAR(n), VARCHAR(n) where n &lt; 255
 
-        >=   prefix padded with '\u0000' to length n -- e.g. Derby\u0000\u0000
-        <=   prefix appended with '\uffff' -- e.g. Derby\uffff
+        &gt;=   prefix padded with '\u0000' to length n -- e.g. Derby\u0000\u0000
+        &lt;=   prefix appended with '\uffff' -- e.g. Derby\uffff
 
         [ can eliminate LIKE if constant. ]
 
 
-    CHAR(n), VARCHAR(n), LONG VARCHAR where n >= 255
+    CHAR(n), VARCHAR(n), LONG VARCHAR where n &gt;= 255
 
-        >= prefix backed up one characer
-        <= prefix appended with '\uffff'
+        &gt;= prefix backed up one characer
+        &lt;= prefix appended with '\uffff'
 
         no elimination of like
 
 
     parameter like pattern starts with wild card e.g. %Derby
 
-    CHAR(n), VARCHAR(n) where n <= 256
+    CHAR(n), VARCHAR(n) where n &lt;= 256
 
-        >= '\u0000' padded with '\u0000' to length n
-        <= '\uffff'
+        &gt;= '\u0000' padded with '\u0000' to length n
+        &lt;= '\uffff'
 
         no elimination of like
 
-    CHAR(n), VARCHAR(n), LONG VARCHAR where n > 256
+    CHAR(n), VARCHAR(n), LONG VARCHAR where n &gt; 256
 
-        >= NULL
+        &gt;= NULL
 
-        <= '\uffff'
+        &lt;= '\uffff'
 
 
     Note that the Unicode value '\uffff' is defined as not a character value
@@ -111,7 +98,7 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
     String  escape;
 
     /**
-     * Initializer for a LikeEscapeOperatorNode
+     * Constructor for a LikeEscapeOperatorNode
      *
      * receiver like pattern [ escape escapeValue ]
      *
@@ -119,16 +106,20 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
      *                              column, CharConstant or Parameter
      * @param leftOperand   The right operand of the like: the pattern
      * @param rightOperand  The optional escape clause, null if not present
+     * @param cm            The context manager
      */
-    public void init(
-    Object receiver,
-    Object leftOperand,
-    Object rightOperand)
+    LikeEscapeOperatorNode(
+            ValueNode receiver,
+            ValueNode leftOperand,
+            ValueNode rightOperand,
+            ContextManager cm)
     {
         /* By convention, the method name for the like operator is "like" */
-        super.init(
-            receiver, leftOperand, rightOperand, 
-            ReuseFactory.getInteger(TernaryOperatorNode.LIKE), null); 
+        super(receiver,
+              leftOperand,
+              rightOperand,
+              TernaryOperatorNode.K_LIKE,
+              cm);
     }
 
     /**
@@ -141,13 +132,12 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
      *
      * @exception StandardException thrown on failure
      */
-    public ValueNode bindExpression(
-    FromList        fromList, 
-    SubqueryList    subqueryList,
-    List            aggregateVector)
+    @Override
+    ValueNode bindExpression(
+    FromList fromList, SubqueryList subqueryList, List<AggregateNode> aggregates)
         throws StandardException
     {
-        super.bindExpression(fromList, subqueryList, aggregateVector);
+        super.bindExpression(fromList, subqueryList, aggregates);
 
         String pattern = null;
 
@@ -257,9 +247,6 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
 
         bindToBuiltIn();
 
-        TypeCompiler receiverTC = receiver.getTypeCompiler();
-        TypeCompiler leftTC     = leftOperand.getTypeCompiler();
-
         /* The receiver must be a string type
         */
         if (! receiver.getTypeId().isStringTypeId())
@@ -274,7 +261,6 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
         if (!leftOperand.getTypeId().isStringTypeId())
         {
             leftOperand = castArgToString(leftOperand);
-            leftTC      = leftOperand.getTypeCompiler();
         }
 
         if (rightOperand != null)
@@ -392,14 +378,12 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
                     //     /   \
                     //  column  'Derby'
                     BinaryComparisonOperatorNode equals = 
-                        (BinaryComparisonOperatorNode) getNodeFactory().getNode(
-                            C_NodeTypes.BINARY_EQUALS_OPERATOR_NODE,
+                        new BinaryRelationalOperatorNode(
+                            BinaryRelationalOperatorNode.K_EQUALS,
                             leftClone, 
-                            (ValueNode) getNodeFactory().getNode(
-                                C_NodeTypes.CHAR_CONSTANT_NODE,
-                                newPattern,
-                                getContextManager()),
-                                Boolean.FALSE,
+                            new CharConstantNode(newPattern,
+                                                 getContextManager()),
+                            false,
                             getContextManager());
 
                     // Set forQueryRewrite to bypass comparability checks
@@ -407,7 +391,7 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
 
                     equals = (BinaryComparisonOperatorNode) 
                         equals.bindExpression(
-                            fromList, subqueryList, aggregateVector);
+                            fromList, subqueryList, aggregates);
 
                     // create new and node and hook in "equals" the new "=' node
                     //
@@ -417,12 +401,8 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
                     //           / \
                     //       column 'Derby'
 
-                    AndNode newAnd = 
-                        (AndNode) getNodeFactory().getNode(
-                                    C_NodeTypes.AND_NODE,
-                                    this,
-                                    equals,
-                                    getContextManager());
+                    AndNode newAnd =
+                            new AndNode(this, equals, getContextManager());
 
                     finishBindExpr();
                     newAnd.postBindFixup();
@@ -513,7 +493,8 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
     *
     * @exception StandardException  Thrown on error
     */
-    public ValueNode preprocess(
+    @Override
+    ValueNode preprocess(
     int             numTables,
     FromList        outerFromList,
     SubqueryList    outerSubqueryList,
@@ -650,11 +631,7 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
          */
 
         AndNode   newAnd   = null;
-        ValueNode trueNode = 
-            (ValueNode) getNodeFactory().getNode(
-                            C_NodeTypes.BOOLEAN_CONSTANT_NODE,
-                            Boolean.TRUE,
-                            getContextManager());
+        ValueNode trueNode = new BooleanConstantNode(true, getContextManager());
 
         /* Create the AND <, if lessThanString is non-null or 
          * leftOperand is a parameter.
@@ -662,7 +639,7 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
         if (lessThanString != null || 
             leftOperand.requiresTypeFromContext())
         {
-            QueryTreeNode likeLTopt;
+            ValueNode likeLTopt;
             if (leftOperand.requiresTypeFromContext())
             {
                 // pattern string is a parameter 
@@ -678,19 +655,16 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
             {
                 // pattern string is a constant
                 likeLTopt = 
-                    (QueryTreeNode) getNodeFactory().getNode(
-                        C_NodeTypes.CHAR_CONSTANT_NODE,
-                        lessThanString,
-                        getContextManager());
+                    new CharConstantNode(lessThanString, getContextManager());
             }
 
             BinaryComparisonOperatorNode lessThan = 
-                (BinaryComparisonOperatorNode) getNodeFactory().getNode(
-                    C_NodeTypes.BINARY_LESS_THAN_OPERATOR_NODE,
+                new BinaryRelationalOperatorNode(
+                    BinaryRelationalOperatorNode.K_LESS_THAN,
                     receiver.getClone(), 
                     likeLTopt,
-                    Boolean.FALSE,
-                   getContextManager());
+                    false,
+                    getContextManager());
 
             // Disable comparability checks
             lessThan.setForQueryRewrite(true);
@@ -701,11 +675,7 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
             lessThan.setBetweenSelectivity();
 
             /* Create the AND */
-            newAnd = (AndNode) getNodeFactory().getNode(
-                C_NodeTypes.AND_NODE,
-                lessThan,
-                trueNode,
-                getContextManager());
+            newAnd = new AndNode(lessThan, trueNode, getContextManager());
 
             newAnd.postBindFixup();
         }
@@ -734,10 +704,7 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
             // the pattern is a constant, eg. c1 LIKE 'Derby'
 
             likeGEopt = 
-                (ValueNode) getNodeFactory().getNode(
-                    C_NodeTypes.CHAR_CONSTANT_NODE,
-                    greaterEqualString,
-                    getContextManager());
+                new CharConstantNode(greaterEqualString, getContextManager());
         }
 
         // greaterEqual from (reciever LIKE pattern):
@@ -745,11 +712,11 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
         //      /   \
         //  reciever pattern
         BinaryComparisonOperatorNode greaterEqual = 
-            (BinaryComparisonOperatorNode) getNodeFactory().getNode(
-                C_NodeTypes.BINARY_GREATER_EQUALS_OPERATOR_NODE,
+            new BinaryRelationalOperatorNode(
+                BinaryRelationalOperatorNode.K_GREATER_EQUALS,
                 receiver.getClone(), 
                 likeGEopt,
-                Boolean.FALSE,
+                false,
                 getContextManager());
 
 
@@ -764,19 +731,11 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
         /* Create the AND */
         if (newAnd == null)
         {
-            newAnd = (AndNode) getNodeFactory().getNode(
-                C_NodeTypes.AND_NODE,
-                greaterEqual,
-                trueNode,
-                getContextManager());
+            newAnd = new AndNode(greaterEqual, trueNode, getContextManager());
         }
         else
         {
-            newAnd = (AndNode) getNodeFactory().getNode(
-                C_NodeTypes.AND_NODE,
-                greaterEqual,
-                newAnd,
-                getContextManager());
+            newAnd = new AndNode(greaterEqual, newAnd, getContextManager());
         }
         newAnd.postBindFixup();
 
@@ -785,13 +744,7 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
          */
         if (!eliminateLikeComparison)
         {
-            newAnd = (AndNode) 
-                getNodeFactory().getNode(
-                    C_NodeTypes.AND_NODE,
-                    this,
-                    newAnd,
-                    getContextManager());
-
+            newAnd = new AndNode(this, newAnd, getContextManager());
             newAnd.postBindFixup();
         }
 
@@ -814,10 +767,9 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
      *
      * @exception StandardException Thrown on error
      */
-
-    public void generateExpression(
-    ExpressionClassBuilder  acb,
-    MethodBuilder           mb)
+    @Override
+    void generateExpression(
+    ExpressionClassBuilder acb, MethodBuilder mb)
         throws StandardException
     {
 
@@ -887,9 +839,7 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
             methodName += "WithEsc";
         }
 
-        StaticMethodCallNode methodCall = (StaticMethodCallNode)
-            getNodeFactory().getNode(
-                C_NodeTypes.STATIC_METHOD_CALL_NODE,
+        StaticMethodCallNode methodCall = new StaticMethodCallNode(
                 methodName,
                 "org.apache.derby.iapi.types.Like",
                 getContextManager());
@@ -897,29 +847,23 @@ public final class LikeEscapeOperatorNode extends TernaryOperatorNode
         // using a method call directly, thus need internal sql capability
         methodCall.internalCall = true;
 
-        QueryTreeNode maxWidthNode = (QueryTreeNode) getNodeFactory().getNode(
-            C_NodeTypes.INT_CONSTANT_NODE,
-            new Integer(maxWidth),
+        NumericConstantNode maxWidthNode = new NumericConstantNode(
+            TypeId.getBuiltInTypeId(Types.INTEGER),
+            Integer.valueOf(maxWidth),
             getContextManager());
 
-        QueryTreeNode[] param = (escapeNode == null) ?
-            new QueryTreeNode[] { parameterNode, maxWidthNode } :
-            new QueryTreeNode[] { parameterNode, escapeNode, maxWidthNode };
+        ValueNode[] param = (escapeNode == null) ?
+            new ValueNode[] { parameterNode, maxWidthNode } :
+            new ValueNode[] { parameterNode, escapeNode, maxWidthNode };
 
         methodCall.addParms(Arrays.asList(param));
 
         ValueNode java2SQL = 
-            (ValueNode) getNodeFactory().getNode(
-                C_NodeTypes.JAVA_TO_SQL_VALUE_NODE,
-                methodCall,
-                getContextManager());
+                new JavaToSQLValueNode(methodCall, getContextManager());
 
+        java2SQL = java2SQL.bindExpression(null, null, null);
 
-        java2SQL = (ValueNode) java2SQL.bindExpression(null, null, null);
-
-        CastNode likeOpt = (CastNode)
-        getNodeFactory().getNode(
-            C_NodeTypes.CAST_NODE,
+        CastNode likeOpt = new CastNode(
             java2SQL,
             parameterNode.getTypeServices(),
             getContextManager());
