@@ -21,40 +21,33 @@
 
 package org.apache.derby.impl.sql.compile;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
+import java.util.HashMap;
 import org.apache.derby.iapi.error.StandardException;
-
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.services.io.ArrayUtil;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.sql.compile.AccessPath;
+import org.apache.derby.iapi.sql.compile.CostEstimate;
 import org.apache.derby.iapi.sql.compile.JoinStrategy;
+import org.apache.derby.iapi.sql.compile.OptTrace;
 import org.apache.derby.iapi.sql.compile.Optimizable;
 import org.apache.derby.iapi.sql.compile.OptimizableList;
 import org.apache.derby.iapi.sql.compile.OptimizablePredicate;
 import org.apache.derby.iapi.sql.compile.OptimizablePredicateList;
 import org.apache.derby.iapi.sql.compile.Optimizer;
-import org.apache.derby.iapi.sql.compile.CostEstimate;
+import org.apache.derby.iapi.sql.compile.OptimizerPlan;
 import org.apache.derby.iapi.sql.compile.RequiredRowOrdering;
 import org.apache.derby.iapi.sql.compile.RowOrdering;
-import org.apache.derby.iapi.sql.compile.AccessPath;
-
-
+import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
 import org.apache.derby.iapi.sql.dictionary.ConglomerateDescriptor;
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
 import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
-
-import org.apache.derby.iapi.reference.SQLState;
-
+import org.apache.derby.iapi.sql.dictionary.UniqueTupleDescriptor;
 import org.apache.derby.iapi.util.JBitSet;
 import org.apache.derby.iapi.util.StringUtil;
 
-import java.util.HashMap;
-
 /**
- * This will be the Level 1 Optimizer.
- * RESOLVE - it's a level 0 optimizer right now.
- * Current State:
- *	o  No costing services
- *	o  We can only cost a derived table with a join once.
- *  
  * Optimizer uses OptimizableList to keep track of the best join order as it
  * builds it.  For each available slot in the join order, we cost all of the
  * Optimizables from that slot til the end of the OptimizableList.  Later,
@@ -66,36 +59,39 @@ import java.util.HashMap;
  * clauses between permutations yet.)
  */
 
-public class OptimizerImpl implements Optimizer 
+class OptimizerImpl implements Optimizer 
 {
+	private LanguageConnectionContext lcc;
 
-	DataDictionary			 dDictionary;
+	private DataDictionary			 dDictionary;
 	/* The number of tables in the query as a whole.  (Size of bit maps.) */
-	int						 numTablesInQuery;
+	private int						 numTablesInQuery;
 	/* The number of optimizables in the list to optimize */
-	int						 numOptimizables;
+	private int						 numOptimizables;
 
 	/* Bit map of tables that have already been assigned to slots.
 	 * Useful for pushing join clauses as slots are assigned.
      * Enforcement of ordering dependencies is done through
      * assignedTableMap.
      */
-	protected JBitSet		 assignedTableMap;
-	protected OptimizableList optimizableList;
-	OptimizablePredicateList predicateList;
-	JBitSet					 nonCorrelatedTableMap;
+	private JBitSet                     assignedTableMap;
+	private OptimizableList             optimizableList;
+    private OptimizerPlan               overridingPlan;
+    private OptimizerPlan               currentPlan;
+	private OptimizablePredicateList    predicateList;
+	private JBitSet					    nonCorrelatedTableMap;
 
-	protected int[]			 proposedJoinOrder;
-	protected int[]					 bestJoinOrder;
-	protected int			 joinPosition;
-	boolean					 desiredJoinOrderFound;
+	private int[]			            proposedJoinOrder;
+	private int[]					    bestJoinOrder;
+	private int			                joinPosition;
+	private boolean					    desiredJoinOrderFound;
 
 	/* This implements a state machine to jump start to a appearingly good join
 	 * order, when the number of tables is high, and the optimization could take
-	 * a long time.  A good start can prune better, and timeout sooner.  Otherwise,
-	 * it may take forever to exhaust or timeout (see beetle 5870).  Basically after
-	 * we jump, we walk the high part, then fall when we reach the peak, finally we
-	 * walk the low part til where we jumped to.
+	 * a long time.  A good start can prune better, and timeout sooner.  
+     * Otherwise, it may take forever to exhaust or timeout (see beetle 5870).  
+     * Basically after we jump, we walk the high part, then fall when we reach 
+     * the peak, finally we walk the low part until where we jumped to.
 	 */
 	private static final int NO_JUMP = 0;
 	private static final int READY_TO_JUMP = 1;
@@ -108,36 +104,30 @@ public class OptimizerImpl implements Optimizer
 	private boolean			 ruleBasedOptimization;
 
 	private CostEstimateImpl outermostCostEstimate;
-	protected CostEstimateImpl currentCost;
-	protected CostEstimateImpl currentSortAvoidanceCost;
-	protected CostEstimateImpl bestCost;
+	private CostEstimateImpl currentCost;
+	private CostEstimateImpl currentSortAvoidanceCost;
+	private CostEstimateImpl bestCost;
 
-	protected long			 timeOptimizationStarted;
-	protected long			 currentTime;
-	protected boolean		 timeExceeded;
+	private long			 timeOptimizationStarted;
+	private long			 currentTime;
+	private boolean		 timeExceeded;
 	private boolean			 noTimeout;
 	private boolean 		 useStatistics;
 	private int				 tableLockThreshold;
 
 	private JoinStrategy[]	joinStrategies;
 
-	protected RequiredRowOrdering	requiredRowOrdering;
+	private RequiredRowOrdering	requiredRowOrdering;
 
 	private boolean			 foundABestPlan;
 
-	protected CostEstimate sortCost;
+	private CostEstimate sortCost;
 
 	private RowOrdering currentRowOrdering = new RowOrderingImpl();
 	private RowOrdering bestRowOrdering = new RowOrderingImpl();
 
-	private boolean	conglomerate_OneRowResultSet;
-
-	// optimizer trace
-	protected boolean optimizerTrace;
-	protected boolean optimizerTraceHtml;
-
 	// max memory use per table
-	protected int maxMemoryPerTable;
+	private int maxMemoryPerTable;
 
 	// Whether or not we need to reload the best plan for an Optimizable
     // when we "pull" [1] it.  If the latest complete join order was the
@@ -171,15 +161,15 @@ public class OptimizerImpl implements Optimizer
 	// of Optimizer.  Each outer query could potentially have a different
 	// idea of what this OptimizerImpl's "best join order" is, so we have
 	// to keep track of them all.
-	private HashMap savedJoinOrders;
+	private HashMap<Object,int[]> savedJoinOrders;
 
 	// Value used to figure out when/if we've timed out for this
 	// Optimizable.
-	protected double timeLimit;
+	private double timeLimit;
 
 	// Cost estimate for the final "best join order" that we chose--i.e.
 	// the one that's actually going to be generated.
-	CostEstimate finalCostEstimate;
+	private CostEstimate finalCostEstimate;
 
 	/* Status variables used for "jumping" to previous best join
 	 * order when possible.  In particular, this helps when this
@@ -199,7 +189,7 @@ public class OptimizerImpl implements Optimizer
 	private boolean usingPredsPushedFromAbove;
 	private boolean bestJoinOrderUsedPredsFromAbove;
 
-	protected  OptimizerImpl(OptimizableList optimizableList, 
+	OptimizerImpl(OptimizableList optimizableList, 
 				  OptimizablePredicateList predicateList,
 				  DataDictionary dDictionary,
 				  boolean ruleBasedOptimization,
@@ -209,7 +199,9 @@ public class OptimizerImpl implements Optimizer
 				  JoinStrategy[] joinStrategies,
 				  int tableLockThreshold,
 				  RequiredRowOrdering requiredRowOrdering,
-				  int numTablesInQuery)
+                  int numTablesInQuery,
+                  OptimizerPlan overridingPlan,
+                  LanguageConnectionContext lcc )
 		throws StandardException
 	{
 		if (SanityManager.DEBUG) {
@@ -223,7 +215,9 @@ public class OptimizerImpl implements Optimizer
 
 		currentSortAvoidanceCost = getNewCostEstimate(0.0d, 0.0d, 0.0d);
 
-		bestCost = getNewCostEstimate(Double.MAX_VALUE, Double.MAX_VALUE, Double.MAX_VALUE);
+		bestCost = 
+            getNewCostEstimate(
+                Double.MAX_VALUE, Double.MAX_VALUE, Double.MAX_VALUE);
 
 		// Verify that any Properties lists for user overrides are valid
 		optimizableList.verifyProperties(dDictionary);
@@ -240,16 +234,19 @@ public class OptimizerImpl implements Optimizer
 
 		bestJoinOrder = new int[numOptimizables];
 		joinPosition = -1;
-		this.optimizableList = optimizableList;
-		this.predicateList = predicateList;
-		this.dDictionary = dDictionary;
-		this.ruleBasedOptimization = ruleBasedOptimization;
-		this.noTimeout = noTimeout;
-		this.maxMemoryPerTable = maxMemoryPerTable;
-		this.joinStrategies = joinStrategies;
-		this.tableLockThreshold = tableLockThreshold;
-		this.requiredRowOrdering = requiredRowOrdering;
-		this.useStatistics = useStatistics;
+
+		this.optimizableList         = optimizableList;
+		this.overridingPlan          = overridingPlan;
+		this.predicateList           = predicateList;
+		this.dDictionary             = dDictionary;
+		this.ruleBasedOptimization   = ruleBasedOptimization;
+		this.noTimeout               = noTimeout;
+		this.maxMemoryPerTable       = maxMemoryPerTable;
+		this.joinStrategies          = joinStrategies;
+		this.tableLockThreshold      = tableLockThreshold;
+		this.requiredRowOrdering     = requiredRowOrdering;
+		this.useStatistics           = useStatistics;
+        this.lcc                     = lcc;
 
 		/* initialize variables for tracking permutations */
 		assignedTableMap = new JBitSet(numTablesInQuery);
@@ -275,6 +272,33 @@ public class OptimizerImpl implements Optimizer
 
 		usingPredsPushedFromAbove = false;
 		bestJoinOrderUsedPredsFromAbove = false;
+
+		// Optimization started
+		if (tracingIsOn()) 
+        { 
+            tracer().traceStartQueryBlock( 
+                timeOptimizationStarted, hashCode(), optimizableList ); 
+        }
+
+        // make sure that optimizer overrides are bound and left-deep
+        if ( overridingPlan != null )
+        {
+            if ( !overridingPlan.isBound() )
+            {
+                throw StandardException.newException(
+                        SQLState.LANG_UNRESOLVED_ROW_SOURCE );
+            }
+            
+            int     actualRowSourceCount = optimizableList.size();
+            int     overriddenRowSourceCount = overridingPlan.countLeafNodes();
+            if ( actualRowSourceCount != overriddenRowSourceCount )
+            {
+                throw StandardException.newException( 
+                    SQLState.LANG_BAD_ROW_SOURCE_COUNT, 
+                    overriddenRowSourceCount, 
+                    actualRowSourceCount );
+            }
+        }
 	}
 
 	/**
@@ -388,6 +412,8 @@ public class OptimizerImpl implements Optimizer
         return permuteState;
     }
 
+    private  boolean tracingIsOn() { return lcc.optimizerTracingIsOn(); }
+
     public int getMaxMemoryPerTable()
     {
         return maxMemoryPerTable;
@@ -404,10 +430,7 @@ public class OptimizerImpl implements Optimizer
 		/* Don't get any permutations if there is nothing to optimize */
 		if (numOptimizables < 1)
 		{
-			if (optimizerTrace)
-			{
-				trace(NO_TABLES, 0, 0, 0.0, null);
-			}
+			if ( tracingIsOn() ) { tracer().traceVacuous(); }
 
 			endOfRoundCleanup();
 			return false;
@@ -437,10 +460,10 @@ public class OptimizerImpl implements Optimizer
 			currentTime = System.currentTimeMillis();
 			timeExceeded = (currentTime - timeOptimizationStarted) > timeLimit;
 
-			if (optimizerTrace && timeExceeded)
-			{
-				trace(TIME_EXCEEDED, 0, 0, 0.0, null);
-			}
+			if (tracingIsOn() && timeExceeded) 
+            { 
+                tracer().traceTimeout( currentTime, bestCost ); 
+            }
 		}
 
 		if (bestCost.isUninitialized() && foundABestPlan &&
@@ -493,8 +516,8 @@ public class OptimizerImpl implements Optimizer
 				// proceed with normal timeout logic.
 				if (firstLookOrder == null)
 					firstLookOrder = new int[numOptimizables];
-				for (int i = 0; i < numOptimizables; i++)
-					firstLookOrder[i] = bestJoinOrder[i];
+                System.arraycopy(
+                        bestJoinOrder, 0, firstLookOrder, 0, numOptimizables);
 				permuteState = JUMPING;
 
 				/* If we already assigned at least one position in the
@@ -587,18 +610,6 @@ public class OptimizerImpl implements Optimizer
 		}
 		else
 		{
-			if (optimizerTrace)
- 			{
-				/*
-				** Not considered short-circuiting if all slots in join
-				** order are taken.
-				*/
-				if (joinPosition < (numOptimizables - 1))
-				{
-					trace(SHORT_CIRCUITING, 0, 0, 0.0, null);
-				}
-			}
-
 			// If we short-circuited the current join order then we need
 			// to make sure that, when we start pulling optimizables to find
 			// a new join order, we reload the best plans for those
@@ -607,7 +618,17 @@ public class OptimizerImpl implements Optimizer
 			// was part of a short-circuited (and thus rejected) join
 			// order.
 			if (joinPosition < (numOptimizables - 1))
-				reloadBestPlan = true;
+            {
+                if (tracingIsOn())
+                {
+                    tracer().traceShortCircuiting
+                        (
+                         timeExceeded, optimizableList.getOptimizable( proposedJoinOrder[joinPosition] ), joinPosition
+                         );
+                }
+
+                reloadBestPlan = true;
+            }
 		}
 
 		if (permuteState == JUMPING && !joinPosAdvanced && joinPosition >= 0)
@@ -827,20 +848,18 @@ public class OptimizerImpl implements Optimizer
 					if ((nextOptimizable < numOptimizables) &&
 						!joinOrderMeetsDependencies(nextOptimizable))
 					{
-						if (optimizerTrace)
-						{
-							trace(SKIPPING_JOIN_ORDER, nextOptimizable, 0, 0.0, null);
-						}
+						if (tracingIsOn())
+                        {
+                            tracer().traceSkippingJoinOrder
+                                ( nextOptimizable, joinPosition, ArrayUtil.copy( proposedJoinOrder ), (JBitSet) assignedTableMap.clone() );
+                        }
 
 						/*
 						** If this is a user specified join order then it is illegal.
 						*/
 						if ( ! optimizableList.optimizeJoinOrder())
 						{
-							if (optimizerTrace)
-							{
-								trace(ILLEGAL_USER_JOIN_ORDER, 0, 0, 0.0, null);
-							}
+							if (tracingIsOn()) { tracer().traceIllegalUserJoinOrder(); }
 
 							throw StandardException.newException(
 								SQLState.LANG_ILLEGAL_FORCED_JOIN_ORDER);
@@ -865,18 +884,12 @@ public class OptimizerImpl implements Optimizer
 					// Verify that the user specified a legal join order
 					if ( ! optimizableList.legalJoinOrder(numTablesInQuery))
 					{
-						if (optimizerTrace)
-						{
-							trace(ILLEGAL_USER_JOIN_ORDER, 0, 0, 0.0, null);
-						}
+						if (tracingIsOn())  { tracer().traceIllegalUserJoinOrder(); }
 
 						throw StandardException.newException(SQLState.LANG_ILLEGAL_FORCED_JOIN_ORDER);
 					}
 
-					if (optimizerTrace)
-					{
-						trace(USER_JOIN_ORDER_OPTIMIZED, 0, 0, 0.0, null);
-					}
+					if (tracingIsOn()) { tracer().traceUserJoinOrderOptimized(); }
 
 					desiredJoinOrderFound = true;
 				}
@@ -885,8 +898,9 @@ public class OptimizerImpl implements Optimizer
 				{
 					permuteState = JUMPING;
 
-					/* A simple heuristics is that the row count we got indicates a potentially
-					 * good join order.  We'd like row count to get big as late as possible, so
+					/* A simple heuristics is that the row count we got 
+                     * indicates a potentially good join order.  We'd like row 
+                     * count to get big as late as possible, so
 					 * that less load is carried over.
 					 */
 					double rc[] = new double[numOptimizables];
@@ -996,10 +1010,11 @@ public class OptimizerImpl implements Optimizer
 			optimizableList.getOptimizable(nextOptimizable).
 				getBestAccessPath().setCostEstimate((CostEstimate) null);
 
-			if (optimizerTrace)
-			{
-				trace(CONSIDERING_JOIN_ORDER, 0, 0, 0.0, null);
-			}
+			if (tracingIsOn())
+            {
+                tracer().traceJoinOrderConsideration
+                    ( joinPosition, ArrayUtil.copy( proposedJoinOrder ), (JBitSet) assignedTableMap.clone() );
+            }
 
 			Optimizable nextOpt =
 							optimizableList.getOptimizable(nextOptimizable);
@@ -1084,7 +1099,7 @@ public class OptimizerImpl implements Optimizer
 	 * is used to 'recover' cost estimate sums that have been lost due to
 	 * the addition/subtraction of the cost estimate for the Optimizable
 	 * at position "joinPosition".  Ex. If the total cost for Optimizables
-	 * at positions < joinPosition is 1500, and then the Optimizable at
+	 * at positions &lt; joinPosition is 1500, and then the Optimizable at
 	 * joinPosition has an estimated cost of 3.14E40, adding those two
 	 * numbers effectively "loses" the 1500. When we later subtract 3.14E40
 	 * from the total cost estimate (as part of "pull" processing), we'll
@@ -1178,12 +1193,11 @@ public class OptimizerImpl implements Optimizer
 		** cost.
 		*/
 		double newCost = currentCost.getEstimatedCost();
-		double pullCost = 0.0;
 		CostEstimate pullCostEstimate =
 						pullMe.getBestAccessPath().getCostEstimate();
 		if (pullCostEstimate != null)
 		{
-			pullCost = pullCostEstimate.getEstimatedCost();
+            double pullCost = pullCostEstimate.getEstimatedCost();
 
 			newCost -= pullCost;
 
@@ -1247,7 +1261,7 @@ public class OptimizerImpl implements Optimizer
 			if (pullMe.considerSortAvoidancePath())
 			{
 				AccessPath ap = pullMe.getBestSortAvoidancePath();
-				double	   prevEstimatedCost = 0.0d;
+                double     prevEstimatedCost;
 
 				/*
 				** Subtract the sort avoidance cost estimate of the
@@ -1407,7 +1421,6 @@ public class OptimizerImpl implements Optimizer
 		JBitSet	                predMap         = new JBitSet(numTablesInQuery);
 		JBitSet                 curTableNums    = null;
 		BaseTableNumbersVisitor btnVis          = null;
-		boolean                 pushPredNow     = false;
 		int                     tNum;
 		Predicate               pred;
 
@@ -1474,7 +1487,7 @@ public class OptimizerImpl implements Optimizer
 			 * We can check for this condition by seeing if predMap is empty,
 			 * which is what the following line does.
 			 */
-			pushPredNow = (predMap.getFirstSetBit() == -1);
+            boolean pushPredNow = (predMap.getFirstSetBit() == -1);
 
 			/* If the predicate is scoped, there's more work to do. A
 			 * scoped predicate's "referenced map" may not be in sync
@@ -1579,10 +1592,52 @@ public class OptimizerImpl implements Optimizer
 		
 		// RESOLVE: Should we step through the different join strategies here?
 
-		/* Returns true until all access paths are exhausted */
-		retval =  curOpt.nextAccessPath(this,
-										(OptimizablePredicateList) null,
-										currentRowOrdering);
+        while ( true )
+        {
+            /* Returns true until all access paths are exhausted */
+            retval =  curOpt.nextAccessPath(this, (OptimizablePredicateList) null, currentRowOrdering);
+
+            // if the user didn't specify an explicit plan, we're ok
+            if ( overridingPlan == null ) { break; }
+            if ( !retval ) { break; }
+
+            // if we've already found the right access path for this location in
+            // the join order, then we move on to the next position
+            if ( currentPlan != null )
+            {
+                if ( currentPlan.countLeafNodes() == (joinPosition+1) )
+                {
+                    retval = false;
+                    break;
+                }
+            }
+
+            // at this point, figure out if the plan so far is a prefix of the desired plan
+            OptimizerPlan   candidate = OptimizerPlan.makeRowSource( getTupleDescriptor( curOpt ), dDictionary );
+            if ( candidate == null )
+            {
+                retval = false;
+                break;
+            }
+            if ( currentPlan != null )
+            {
+                candidate = new OptimizerPlan.Join
+                    (
+                     curOpt.getCurrentAccessPath().getJoinStrategy(),
+                     currentPlan,
+                     candidate
+                     );
+            }
+
+            if ( candidate.isLeftPrefixOf( overridingPlan ) )
+            {
+                currentPlan = candidate;
+                break;
+            }
+
+            // well, that decoration didn't match up with the user-specified plan.
+            // try again
+        }
 
 		// If the previous path that we considered for curOpt was _not_ the best
 		// path for this round, then we need to revert back to whatever the
@@ -1656,22 +1711,16 @@ public class OptimizerImpl implements Optimizer
 					ce.singleScanRowCount());
 			}
 
-			if (optimizerTrace)
+			if (tracingIsOn())
 			{
-				trace(TOTAL_COST_NON_SA_PLAN, 0, 0, 0.0, null);
-				if (curOpt.considerSortAvoidancePath())
-				{
-					trace(TOTAL_COST_SA_PLAN, 0, 0, 0.0, null);
-				}
+                tracer().traceCostWithoutSortAvoidance( currentCost );
+				if (curOpt.considerSortAvoidancePath()) { tracer().traceCostWithSortAvoidance( currentSortAvoidanceCost ); }
 			}
 				
 			/* Do we have a complete join order? */
 			if ( joinPosition == (numOptimizables - 1) )
 			{
-				if (optimizerTrace)
-				{
-					trace(COMPLETE_JOIN_ORDER, 0, 0, 0.0, null);
-				}
+				if (tracingIsOn()) { tracer().traceCompleteJoinOrder(); }
 
 				/* Add cost of sorting to non-sort-avoidance cost */
 				if (requiredRowOrdering != null)
@@ -1753,11 +1802,7 @@ public class OptimizerImpl implements Optimizer
 										currentCost.singleScanRowCount()
 										);
 					
-					if (optimizerTrace)
-					{
-						trace(COST_OF_SORTING, 0, 0, 0.0, null);
-						trace(TOTAL_COST_WITH_SORTING, 0, 0, 0.0, null);
-					}
+					if (tracingIsOn()) { tracer().traceSortCost( sortCost, currentCost ); }
 				}
 
 				/*
@@ -1827,10 +1872,7 @@ public class OptimizerImpl implements Optimizer
                             proposedJoinOrder) ==
                         RequiredRowOrdering.NOTHING_REQUIRED)
 					{
-						if (optimizerTrace)
-						{
-							trace(CURRENT_PLAN_IS_SA_PLAN, 0, 0, 0.0, null);
-						}
+						if (tracingIsOn()) { tracer().traceCurrentPlanAvoidsSort( bestCost, currentSortAvoidanceCost ); }
 
 						if ((currentSortAvoidanceCost.compare(bestCost) <= 0)
 							|| bestCost.isUninitialized())
@@ -1846,6 +1888,34 @@ public class OptimizerImpl implements Optimizer
 		return retval;
 	}
 
+    /**
+     * Get the unique tuple descriptor of the current access path for an Optimizable.
+     */
+    private UniqueTupleDescriptor   getTupleDescriptor( Optimizable optimizable )
+        throws StandardException
+    {
+        if ( isTableFunction( optimizable ) )
+        {
+            ProjectRestrictNode prn = (ProjectRestrictNode) optimizable;
+            return ((StaticMethodCallNode) ((FromVTI) prn.getChildResult()).getMethodCall() ).ad;
+        }
+        else
+        {
+            return optimizable.getCurrentAccessPath().getConglomerateDescriptor();
+        }
+    }
+
+    /** Return true if the optimizable is a table function */
+    static  boolean isTableFunction( Optimizable optimizable )
+    {
+        if ( !( optimizable instanceof ProjectRestrictNode ) ) { return false; }
+
+        ResultSetNode   rsn = ((ProjectRestrictNode) optimizable).getChildResult();
+        if ( !( rsn instanceof FromVTI ) ) { return false; }
+
+        return ( ((FromVTI) rsn).getMethodCall() instanceof StaticMethodCallNode );
+    }
+
 	/**
 	 * Is the cost of this join order lower than the best one we've
 	 * found so far?  If so, remember it.
@@ -1860,12 +1930,7 @@ public class OptimizerImpl implements Optimizer
 	{
 		foundABestPlan = true;
 
-		if (optimizerTrace)
-		{
-			trace(CHEAPEST_PLAN_SO_FAR, 0, 0, 0.0, null);
-			trace(PLAN_TYPE, planType, 0, 0.0, null);
-			trace(COST_OF_CHEAPEST_PLAN_SO_FAR, 0, 0, 0.0, null);
-		}
+		if (tracingIsOn()) { tracer().traceCheapestPlanSoFar( planType, currentCost ); }
 
 		/* Remember the current cost as best */
 		bestCost.setCost(currentCost);
@@ -1887,11 +1952,11 @@ public class OptimizerImpl implements Optimizer
 		** we use 2 loops.
 		*/
 		bestJoinOrderUsedPredsFromAbove = usingPredsPushedFromAbove;
-		for (int i = 0; i < numOptimizables; i++)
-		{
-			bestJoinOrder[i] = proposedJoinOrder[i];
-		}
-		for (int i = 0; i < numOptimizables; i++)
+
+        System.arraycopy(
+                proposedJoinOrder, 0, bestJoinOrder, 0, numOptimizables);
+
+        for (int i = 0; i < numOptimizables; i++)
 		{
 			optimizableList.getOptimizable(bestJoinOrder[i]).
 				rememberAsBest(planType, this);
@@ -1906,13 +1971,11 @@ public class OptimizerImpl implements Optimizer
 				requiredRowOrdering.sortNeeded();
 		}
 
-		if (optimizerTrace)
+		if (tracingIsOn())
 		{
-			if (requiredRowOrdering != null)
-			{
-				trace(SORT_NEEDED_FOR_ORDERING, planType, 0, 0.0, null);
-			}
-			trace(REMEMBERING_BEST_JOIN_ORDER, 0, 0, 0.0, null);
+			if (requiredRowOrdering != null)    { tracer().traceSortNeededForOrdering( planType, requiredRowOrdering ); }
+            tracer().traceRememberingBestJoinOrder
+                ( joinPosition, ArrayUtil.copy( bestJoinOrder ), planType, currentCost, (JBitSet) assignedTableMap.clone() );
 		}
 	}
 
@@ -2055,11 +2118,7 @@ public class OptimizerImpl implements Optimizer
 				throws StandardException
 	{
 		/* CHOOSE BEST CONGLOMERATE HERE */
-		ConglomerateDescriptor	conglomerateDescriptor = null;
-		ConglomerateDescriptor	bestConglomerateDescriptor = null;
 		AccessPath bestAp = optimizable.getBestAccessPath();
-		int lockMode = optimizable.getCurrentAccessPath().getLockMode();
-
 
 		/*
 		** If the current conglomerate better than the best so far?
@@ -2163,7 +2222,9 @@ public class OptimizerImpl implements Optimizer
 		** If all else fails, and no conglomerate has been picked yet,
 		** pick this one.
 		*/
-		bestConglomerateDescriptor = bestAp.getConglomerateDescriptor();
+        ConglomerateDescriptor bestConglomerateDescriptor =
+            bestAp.getConglomerateDescriptor();
+
 		if (bestConglomerateDescriptor == null)
 		{
 			bestAp.setCostEstimate(
@@ -2186,8 +2247,6 @@ public class OptimizerImpl implements Optimizer
 
 			optimizable.rememberJoinStrategyAsBest(bestAp);
 		}
-
-		return;
 	}
 
 	/**
@@ -2229,10 +2288,7 @@ public class OptimizerImpl implements Optimizer
 		// DERBY-1259.
 		if( ! optimizable.memoryUsageOK( estimatedCost.rowCount() / outerCost.rowCount(), maxMemoryPerTable))
 		{
-			if (optimizerTrace)
-			{
-				trace(SKIPPING_DUE_TO_EXCESS_MEMORY, 0, 0, 0.0, null);
-			}
+			if (tracingIsOn()) { tracer().traceSkippingBecauseTooMuchMemory( maxMemoryPerTable ); }
 			return;
 		}
 
@@ -2367,10 +2423,7 @@ public class OptimizerImpl implements Optimizer
         if( ! optimizable.memoryUsageOK( estimatedCost.rowCount() / outerCost.rowCount(),
                                          maxMemoryPerTable))
 		{
-			if (optimizerTrace)
-			{
-				trace(SKIPPING_DUE_TO_EXCESS_MEMORY, 0, 0, 0.0, null);
-			}
+			if (tracingIsOn()) { tracer().traceSkippingBecauseTooMuchMemory( maxMemoryPerTable ); }
 			return;
 		}
 
@@ -2457,17 +2510,11 @@ public class OptimizerImpl implements Optimizer
 	 */
 	public void modifyAccessPaths() throws StandardException
 	{
-		if (optimizerTrace)
-		{
-			trace(MODIFYING_ACCESS_PATHS, 0, 0, 0.0, null);
-		}
+		if (tracingIsOn()) { tracer().traceModifyingAccessPaths( hashCode() ); }
 
 		if ( ! foundABestPlan)
 		{
-			if (optimizerTrace)
-			{
-				trace(NO_BEST_PLAN, 0, 0, 0.0, null);
-			}
+			if (tracingIsOn()) { tracer().traceNoBestPlan(); }
 
 			throw StandardException.newException(SQLState.LANG_NO_BEST_PLAN_FOUND);
 		}
@@ -2498,8 +2545,7 @@ public class OptimizerImpl implements Optimizer
 		}
 	}
 
-	/** @see Optimizer#newCostEstimate */
-	public CostEstimate newCostEstimate()
+	private CostEstimate newCostEstimate()
 	{
 		return new CostEstimateImpl();
 	}
@@ -2528,10 +2574,10 @@ public class OptimizerImpl implements Optimizer
 		// number of rows is the number of rows returned by the innermost
 		// optimizable.
 		finalCostEstimate = getNewCostEstimate(0.0d, 0.0d, 0.0d);
-		CostEstimate ce = null;
-		for (int i = 0; i < bestJoinOrder.length; i++)
+
+        for (int i = 0; i < bestJoinOrder.length; i++)
 		{
-			ce = optimizableList.getOptimizable(bestJoinOrder[i])
+            CostEstimate ce = optimizableList.getOptimizable(bestJoinOrder[i])
 					.getTrulyTheBestAccessPath().getCostEstimate();
 
 			finalCostEstimate.setCost(
@@ -2672,22 +2718,16 @@ public class OptimizerImpl implements Optimizer
 	/** @see Optimizer#getLevel */
 	public int getLevel()
 	{
-		return 1;
+		return 2;
 	}
 
-	public CostEstimateImpl getNewCostEstimate(double theCost,
+    CostEstimateImpl getNewCostEstimate(double theCost,
 							double theRowCount,
 							double theSingleScanRowCount)
 	{
 		return new CostEstimateImpl(theCost, theRowCount, theSingleScanRowCount);
 	}
 
-	// Optimzer trace
-	public void trace(int traceFlag, int intParam1, int intParam2,
-					  double doubleParam, Object objectParam1)
-	{
-	}
-	
 	/** @see Optimizer#useStatistics */
 	public boolean useStatistics() { return useStatistics && optimizableList.useStatistics(); }
 
@@ -2706,7 +2746,7 @@ public class OptimizerImpl implements Optimizer
 	 *  OptimizerImpl that could potentially reject plans chosen by this
 	 *  OptimizerImpl.
 	 */
-	protected void updateBestPlanMaps(short action,
+	public void updateBestPlanMaps(short action,
 		Object planKey) throws StandardException
 	{
 		// First we process this OptimizerImpl's best join order.  If there's
@@ -2720,8 +2760,9 @@ public class OptimizerImpl implements Optimizer
 				if (savedJoinOrders != null)
 				{
 					savedJoinOrders.remove(planKey);
-					if (savedJoinOrders.size() == 0)
+                    if (savedJoinOrders.isEmpty()) {
 						savedJoinOrders = null;
+                    }
 				}
 			}
 			else if (action == FromTable.ADD_PLAN)
@@ -2729,18 +2770,17 @@ public class OptimizerImpl implements Optimizer
 				// If the savedJoinOrder map already exists, search for the
 				// join order for the target optimizer and reuse that.
 				if (savedJoinOrders == null)
-					savedJoinOrders = new HashMap();
+					savedJoinOrders = new HashMap<Object,int[]>();
 				else
-					joinOrder = (int[])savedJoinOrders.get(planKey);
+					joinOrder = savedJoinOrders.get(planKey);
 
 				// If we don't already have a join order array for the optimizer,
 				// create a new one.
 				if (joinOrder == null)
 					joinOrder = new int[numOptimizables];
 
-				// Now copy current bestJoinOrder and save it.
-				for (int i = 0; i < bestJoinOrder.length; i++)
-					joinOrder[i] = bestJoinOrder[i];
+                System.arraycopy(
+                        bestJoinOrder, 0, joinOrder, 0, bestJoinOrder.length);
 
 				savedJoinOrders.put(planKey, joinOrder);
 			}
@@ -2754,13 +2794,11 @@ public class OptimizerImpl implements Optimizer
 				// for which there was no valid plan.
 				if (savedJoinOrders != null)
 				{
-					joinOrder = (int[])savedJoinOrders.get(planKey);
+					joinOrder = savedJoinOrders.get(planKey);
 					if (joinOrder != null)
 					{
-						// Load the join order we found into our
-						// bestJoinOrder array.
-						for (int i = 0; i < joinOrder.length; i++)
-							bestJoinOrder[i] = joinOrder[i];
+                        System.arraycopy(
+                            joinOrder, 0, bestJoinOrder, 0, joinOrder.length);
 					}
 				}
 			}
@@ -2789,11 +2827,14 @@ public class OptimizerImpl implements Optimizer
 	 * Before adding the received predicates, this method also
 	 * clears out any scoped predicates that might be sitting in
 	 * OptimizerImpl's list from the last round of optimizing.
+	 *
+	 * This method should be in the Optimizer interface, but it relies
+     * on an argument type (PredicateList) which lives in an impl package.
 	 *
 	 * @param pList List of predicates to add to this OptimizerImpl's
 	 *  own list for pushing.
 	 */
-	protected void addScopedPredicatesToList(PredicateList pList)
+    void addScopedPredicatesToList(PredicateList pList, ContextManager cm)
 		throws StandardException
 	{
 		if ((pList == null) || (pList == predicateList))
@@ -2803,7 +2844,7 @@ public class OptimizerImpl implements Optimizer
 		if (predicateList == null)
 		// in this case, there is no 'original' predicateList, so we
 		// can just create one.
-			predicateList = new PredicateList();
+            predicateList = new PredicateList(cm);
 
 		// First, we need to go through and remove any predicates in this
 		// optimizer's list that may have been pushed here from outer queries
@@ -2811,7 +2852,7 @@ public class OptimizerImpl implements Optimizer
 		// predicate was pushed from an outer query because it will have
 		// been scoped to the node for which this OptimizerImpl was
 		// created.
-		Predicate pred = null;
+        Predicate pred;
 		for (int i = predicateList.size() - 1; i >= 0; i--) {
 			pred = (Predicate)predicateList.getOptPredicate(i);
 			if (pred.isScopedForPush())
@@ -2833,8 +2874,12 @@ public class OptimizerImpl implements Optimizer
 				pList.removeOptPredicate(i);
 			}
 		}
-
-		return;
 	}
 
+    /** Get the trace machinery */
+    private  OptTrace    tracer()    { return lcc.getOptimizerTracer(); }
+
+    public  int getOptimizableCount() { return optimizableList.size(); }
+
+    public  Optimizable getOptimizable( int idx ) { return optimizableList.getOptimizable( idx ); }
 }
