@@ -21,30 +21,24 @@
 
 package	org.apache.derby.impl.sql.compile;
 
-import org.apache.derby.iapi.sql.compile.CompilerContext;
-import org.apache.derby.iapi.sql.compile.Optimizable;
-import org.apache.derby.iapi.sql.compile.CostEstimate;
-import org.apache.derby.iapi.sql.compile.OptimizableList;
-import org.apache.derby.iapi.sql.compile.RequiredRowOrdering;
-import org.apache.derby.iapi.sql.compile.RowOrdering;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
-
 import org.apache.derby.iapi.error.StandardException;
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
-import org.apache.derby.iapi.services.compiler.MethodBuilder;
-
-import org.apache.derby.iapi.store.access.ColumnOrdering;
-import org.apache.derby.iapi.store.access.SortCostController;
-
-import org.apache.derby.iapi.types.DataValueDescriptor;
-
 import org.apache.derby.iapi.reference.ClassName;
 import org.apache.derby.iapi.reference.Limits;
 import org.apache.derby.iapi.reference.SQLState;
-
-import org.apache.derby.iapi.util.JBitSet;
 import org.apache.derby.iapi.services.classfile.VMOpcode;
+import org.apache.derby.iapi.services.compiler.MethodBuilder;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.sql.compile.CompilerContext;
+import org.apache.derby.iapi.sql.compile.CostEstimate;
+import org.apache.derby.iapi.sql.compile.Optimizable;
+import org.apache.derby.iapi.sql.compile.OptimizableList;
+import org.apache.derby.iapi.sql.compile.RequiredRowOrdering;
+import org.apache.derby.iapi.sql.compile.RowOrdering;
+import org.apache.derby.iapi.store.access.ColumnOrdering;
+import org.apache.derby.iapi.store.access.SortCostController;
+import org.apache.derby.iapi.types.DataValueDescriptor;
+import org.apache.derby.iapi.util.JBitSet;
 
 /**
  * An OrderByList is an ordered list of columns in the ORDER BY clause.
@@ -53,7 +47,7 @@ import org.apache.derby.iapi.services.classfile.VMOpcode;
  * and the last column in the list is the least significant.
  *
  */
-public class OrderByList extends OrderedColumnList
+class OrderByList extends OrderedColumnList<OrderByColumn>
 						implements RequiredRowOrdering {
 
 	private boolean allAscending = true;
@@ -74,12 +68,16 @@ public class OrderByList extends OrderedColumnList
     private boolean isTableValueCtorOrdering;
 
     /**
+     * Constructor.
      * Initialize with the type of the result set this {@code OrderByList} is
      * attached to, e.g. {@code SELECT}, {@code VALUES} or a set operation.
-     * @param rs The result set this {@code OrderByList} is ordering.
+     * @param rs The result set this {@code OrderByList} is ordering. May be
+     *           null
+     * @param cm The context manager
     */
-   public void init(Object rs) {
-        this.isTableValueCtorOrdering =
+   OrderByList(ResultSetNode rs, ContextManager cm) {
+       super(OrderByColumn.class, cm);
+       this.isTableValueCtorOrdering =
                 (rs instanceof UnionNode &&
                 ((UnionNode)rs).tableConstructor()) ||
                 rs instanceof RowResultSetNode;
@@ -90,7 +88,7 @@ public class OrderByList extends OrderedColumnList
 	
 		@param column	The column to add to the list
 	 */
-	public void addOrderByColumn(OrderByColumn column) 
+    void addOrderByColumn(OrderByColumn column)
 	{
 		addElement(column);
 
@@ -113,10 +111,12 @@ public class OrderByList extends OrderedColumnList
 	
 		@param position	The column to get from the list
 	 */
-	public OrderByColumn getOrderByColumn(int position) {
-		if (SanityManager.DEBUG)
-		SanityManager.ASSERT(position >=0 && position < size());
-		return (OrderByColumn) elementAt(position);
+    OrderByColumn getOrderByColumn(int position) {
+        if (SanityManager.DEBUG) {
+            SanityManager.ASSERT(position >=0 && position < size());
+        }
+
+        return elementAt(position);
 	}
 
 	/**
@@ -126,23 +126,20 @@ public class OrderByList extends OrderedColumnList
 	 * 	@param target	The underlying result set
 	 *	@exception StandardException		Thrown on error
 	 */
-	public void bindOrderByColumns(ResultSetNode target)
+    void bindOrderByColumns(ResultSetNode target)
 					throws StandardException {
 
 		/* Remember the target for use in optimization */
 		resultToSort = target;
 
-		int size = size();
-
 		/* Only 1012 columns allowed in ORDER BY clause */
-		if (size > Limits.DB2_MAX_ELEMENTS_IN_ORDER_BY)
+        if (size() > Limits.DB2_MAX_ELEMENTS_IN_ORDER_BY)
 		{
 			throw StandardException.newException(SQLState.LANG_TOO_MANY_ELEMENTS);
 		}
 
-		for (int index = 0; index < size; index++)
+        for (OrderByColumn obc : this)
 		{
-			OrderByColumn obc = (OrderByColumn) elementAt(index);
 			obc.bindOrderByColumn(target, this);
 
 			/*
@@ -174,9 +171,8 @@ public class OrderByList extends OrderedColumnList
 	 */
 	void closeGap(int gap)
 	{
-		for (int index = 0; index < size(); index++)
+        for (OrderByColumn obc : this)
 		{
-			OrderByColumn obc = (OrderByColumn) elementAt(index);
 			obc.collapseAddedColumnGap(gap);
 		}
 	}
@@ -188,16 +184,14 @@ public class OrderByList extends OrderedColumnList
 		@param target	The underlying result set
 	
 	 */
-	public void pullUpOrderByColumns(ResultSetNode target)
+    void pullUpOrderByColumns(ResultSetNode target)
 					throws StandardException {
 
 		/* Remember the target for use in optimization */
 		resultToSort = target;
 
-		int size = size();
-		for (int index = 0; index < size; index++)
+        for (OrderByColumn obc : this)
 		{
-			OrderByColumn obc = (OrderByColumn) elementAt(index);
 			obc.pullUpOrderByColumn(target);
 		}
 
@@ -214,12 +208,10 @@ public class OrderByList extends OrderedColumnList
 	 */
 	boolean isInOrderPrefix(ResultColumnList sourceRCL)
 	{
-		boolean inOrderPrefix = true;
-		int rclSize = sourceRCL.size();
-
 		if (SanityManager.DEBUG)
 		{
-			if (size() > sourceRCL.size())
+            int rclSize = sourceRCL.size();
+            if (size() > rclSize)
 			{
 				SanityManager.THROWASSERT(
 					"size() (" + size() + 
@@ -231,8 +223,8 @@ public class OrderByList extends OrderedColumnList
 		int size = size();
 		for (int index = 0; index < size; index++)
 		{
-			if (((OrderByColumn) elementAt(index)).getResultColumn() !=
-				(ResultColumn) sourceRCL.elementAt(index))
+            if (elementAt(index).getResultColumn() !=
+                sourceRCL.elementAt(index))
 			{
 				return false;
 			}
@@ -248,10 +240,8 @@ public class OrderByList extends OrderedColumnList
 	 */
 	void resetToSourceRCs()
 	{
-		int size = size();
-		for (int index = 0; index < size; index++)
+        for (OrderByColumn obc : this)
 		{
-			OrderByColumn obc = (OrderByColumn) elementAt(index);
 			obc.resetToSourceRC();
 		}
 	}
@@ -267,15 +257,11 @@ public class OrderByList extends OrderedColumnList
 	ResultColumnList reorderRCL(ResultColumnList resultColumns)
 		throws StandardException
 	{
-		ResultColumnList newRCL = (ResultColumnList) getNodeFactory().getNode(
-												C_NodeTypes.RESULT_COLUMN_LIST,
-												getContextManager());
+        ResultColumnList newRCL = new ResultColumnList(getContextManager());
 
 		/* The new RCL starts with the ordering columns */
-		int size = size();
-		for (int index = 0; index < size; index++)
+        for (OrderByColumn obc : this)
 		{
-			OrderByColumn obc = (OrderByColumn) elementAt(index);
 			newRCL.addElement(obc.getResultColumn());
 			resultColumns.removeElement(obc.getResultColumn());
 		}
@@ -300,9 +286,7 @@ public class OrderByList extends OrderedColumnList
 			 loc >= 0;
 			 loc--)
 		{
-			OrderByColumn obc = (OrderByColumn) elementAt(loc);
-
-			if (obc.constantColumn(whereClause))
+            if (elementAt(loc).constantColumn(whereClause))
 			{
 				removeElementAt(loc);
 			}
@@ -320,12 +304,12 @@ public class OrderByList extends OrderedColumnList
 		/* Walk the list backwards so we can remove elements safely */
 		for (int loc = size() - 1; loc > 0; loc--)
 		{
-			OrderByColumn obc = (OrderByColumn) elementAt(loc);
+            OrderByColumn obc = elementAt(loc);
 			int           colPosition = obc.getColumnPosition();
 
 			for (int inner = 0; inner < loc; inner++)
 			{
-				OrderByColumn prev_obc = (OrderByColumn) elementAt(inner);
+                OrderByColumn prev_obc = elementAt(inner);
 				if (colPosition == prev_obc.getColumnPosition())
 				{
 					removeElementAt(loc);
@@ -343,7 +327,7 @@ public class OrderByList extends OrderedColumnList
 		@param mb	the method the generated code is to go into
 		@exception StandardException thrown on failure
 	 */
-	public void generate(ActivationClassBuilder acb, 
+    void generate(ActivationClassBuilder acb,
 								MethodBuilder mb,
 								ResultSetNode child)
 							throws StandardException 
@@ -843,10 +827,10 @@ public class OrderByList extends OrderedColumnList
 		return false;
 	}
 
-
+    @Override
 	public String toString() {
 
-		StringBuffer buff = new StringBuffer();
+        StringBuilder buff = new StringBuilder();
 
 		if (columnOrdering != null) {
 			for (int i = 0; i < columnOrdering.length; i++) {
