@@ -23,85 +23,59 @@ package	org.apache.derby.impl.sql.compile;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Properties;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.reference.ClassName;
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.classfile.VMOpcode;
+import org.apache.derby.iapi.services.compiler.MethodBuilder;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.compile.CostEstimate;
-import org.apache.derby.iapi.sql.compile.Optimizer;
 import org.apache.derby.iapi.sql.compile.Optimizable;
 import org.apache.derby.iapi.sql.compile.OptimizablePredicateList;
+import org.apache.derby.iapi.sql.compile.Optimizer;
 import org.apache.derby.iapi.sql.compile.RequiredRowOrdering;
 import org.apache.derby.iapi.sql.compile.RowOrdering;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
-
-import org.apache.derby.iapi.sql.dictionary.DataDictionary;
 import org.apache.derby.iapi.sql.dictionary.ConglomerateDescriptor;
+import org.apache.derby.iapi.sql.dictionary.DataDictionary;
 import org.apache.derby.iapi.sql.dictionary.TableDescriptor;
-
-import org.apache.derby.iapi.error.StandardException;
-
-import org.apache.derby.iapi.services.compiler.MethodBuilder;
-
 import org.apache.derby.iapi.store.access.Qualifier;
-
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
 import org.apache.derby.iapi.types.DataTypeDescriptor;
 import org.apache.derby.iapi.types.TypeId;
-
 import org.apache.derby.iapi.util.JBitSet;
-import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.reference.ClassName;
-import org.apache.derby.iapi.services.classfile.VMOpcode;
-
 
 /**
  * A RowResultSetNode represents the result set for a VALUES clause.
  *
  */
 
-public class RowResultSetNode extends FromTable
+class RowResultSetNode extends FromTable
 {
 	SubqueryList subquerys;
-	private List aggregateVector;
-	OrderByList	 orderByList;
-    ValueNode    offset; // OFFSET n ROWS
-    ValueNode    fetchFirst; // FETCH FIRST n ROWS ONLY
-    boolean   hasJDBClimitClause; //  were OFFSET/FETCH FIRST specified by a JDBC LIMIT clause?
+    private List<AggregateNode> aggregates;
+
+    QueryExpressionClauses qec = new QueryExpressionClauses();
 
 	/**
-	 * Initializer for a RowResultSetNode.
+     * Constructor for a RowResultSetNode.
 	 *
 	 * @param valuesClause	The result column list for the VALUES clause.
 	 * @param tableProperties	Properties list associated with the table
+     * @param cm            The context manager
 	 */
-	public void init(Object valuesClause, Object tableProperties)
-	{
-		super.init(null, tableProperties);
-		resultColumns = (ResultColumnList) valuesClause;
-		if (resultColumns != null)
-			resultColumns.markInitialSize();
+    RowResultSetNode(ResultColumnList valuesClause,
+                     Properties tableProperties,
+                     ContextManager cm) {
+        super(null /* correlationName */, tableProperties, cm);
+        setResultColumns( valuesClause );
+
+        if (getResultColumns() != null) {
+			getResultColumns().markInitialSize();
+        }
 	}
 
-	/**
-	 * Convert this object to a String.  See comments in QueryTreeNode.java
-	 * for how this should be done for tree printing.
-	 *
-	 * @return	This object as a String
-	 */
-
-	public String toString()
-	{
-		if (SanityManager.DEBUG)
-		{
-			return 	"orderByList: " + 
-				(orderByList != null ? orderByList.toString() : "null") + "\n" +
-				super.toString();
-		}
-		else
-		{
-			return "";
-		}
-	}
-
-	public String statementToString()
+    String statementToString()
 	{
 		return "VALUES";
 	}
@@ -112,8 +86,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @param depth		The depth of this node in the tree
 	 */
-
-	public void printSubNodes(int depth)
+    @Override
+    void printSubNodes(int depth)
 	{
 		if (SanityManager.DEBUG)
 		{
@@ -125,24 +99,11 @@ public class RowResultSetNode extends FromTable
 				subquerys.treePrint(depth + 1);
 			}
 
-            if (orderByList != null) {
-                printLabel(depth, "orderByList:");
-                orderByList.treePrint(depth + 1);
-            }
-
-            if (offset != null) {
-                printLabel(depth, "offset:");
-                offset.treePrint(depth + 1);
-            }
-
-            if (fetchFirst != null) {
-                printLabel(depth, "fetch first/next:");
-                fetchFirst.treePrint(depth + 1);
-            }
-
+            printQueryExpressionSuffixClauses(depth, qec);
 		}
 	}
 
+    @Override
     public boolean referencesSessionSchema() throws StandardException {
         return subquerys != null && subquerys.referencesSessionSchema();
     }
@@ -150,12 +111,13 @@ public class RowResultSetNode extends FromTable
 	/**
 	 * Modify the RCL of this node to match the target of the insert.
 	 */
+    @Override
 	ResultSetNode enhanceRCLForInsert(
 			InsertNode target, boolean inOrder, int[] colMap)
 		throws StandardException
 	{
-		if (!inOrder || resultColumns.size() < target.resultColumnList.size()) {
-			resultColumns = getRCLForInsert(target, colMap);
+		if (!inOrder || getResultColumns().size() < target.resultColumnList.size()) {
+			setResultColumns( getRCLForInsert(target, colMap) );
 		}
 		return this;
 	}
@@ -169,6 +131,7 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	public CostEstimate estimateCost(OptimizablePredicateList predList,
 										ConglomerateDescriptor cd,
 										CostEstimate outerCost,
@@ -181,17 +144,17 @@ public class RowResultSetNode extends FromTable
 		** fetched.  Is this true, and if not, does it make a difference?
 		** There's nothing to optimize in this case.
 		*/
-		if (costEstimate == null)
+		if (getCostEstimate() == null)
 		{
-			costEstimate = optimizer.newCostEstimate();
+			setCostEstimate( getOptimizerFactory().getCostEstimate() );
 		}
 
-		costEstimate.setCost(0.0d, 1.0d, 1.0d);
+		getCostEstimate().setCost(0.0d, 1.0d, 1.0d);
 
 		/* A single row is always ordered */
 		rowOrdering.optimizableAlwaysOrdered(this);
 
-		return costEstimate;
+		return getCostEstimate();
 	}
 
 	/**
@@ -205,8 +168,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public ResultSetNode bindNonVTITables(DataDictionary dataDictionary, 
+    @Override
+    ResultSetNode bindNonVTITables(DataDictionary dataDictionary,
 							FromList fromListParam) 
 					throws StandardException
 	{
@@ -225,17 +188,15 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public void bindExpressions(FromList fromListParam)
+    @Override
+    void bindExpressions(FromList fromListParam)
 					throws StandardException
 	{
 		int nestingLevel;
 
-		subquerys = (SubqueryList) getNodeFactory().getNode(
-										C_NodeTypes.SUBQUERY_LIST,
-										getContextManager());
+        subquerys = new SubqueryList(getContextManager());
 
-		aggregateVector = new ArrayList();
+        aggregates = new ArrayList<AggregateNode>();
 
 		/* Verify that there are no DEFAULTs in the RCL.
 		 * DEFAULT is only valid for an insert, and it has
@@ -244,7 +205,7 @@ public class RowResultSetNode extends FromTable
 		 *		VALUES DEFAULT;
 		 * so we need to check for that here and throw an exception if found.
 		 */
-		resultColumns.checkForInvalidDefaults();
+		getResultColumns().checkForInvalidDefaults();
 
 		/* Believe it or not, a values clause can contain correlated column references
 		 * and subqueries.  In order to get correlated column resolution working 
@@ -263,24 +224,28 @@ public class RowResultSetNode extends FromTable
 		}
 		setLevel(nestingLevel);
 		fromListParam.insertElementAt(this, 0);
-		resultColumns.bindExpressions(fromListParam, subquerys,
-									  aggregateVector);
+		getResultColumns().bindExpressions(fromListParam, subquerys,
+                                      aggregates);
 		// Pop ourselves back out of the FROM list
 		fromListParam.removeElementAt(0);
 
-		if (!aggregateVector.isEmpty())
+        if (!aggregates.isEmpty())
 		{
 			throw StandardException.newException(SQLState.LANG_NO_AGGREGATES_IN_WHERE_CLAUSE);
 		}
 
-		SelectNode.checkNoWindowFunctions(resultColumns, "VALUES");
+		SelectNode.checkNoWindowFunctions(getResultColumns(), "VALUES");
 
-        if (orderByList != null) {
-            orderByList.pullUpOrderByColumns(this);
-            orderByList.bindOrderByColumns(this);
+        for (int i = 0; i < qec.size(); i++) {
+            final OrderByList obl = qec.getOrderByList(i);
+
+            if (obl != null) {
+                obl.pullUpOrderByColumns(this);
+                obl.bindOrderByColumns(this);
+            }
+
+            bindOffsetFetch(qec.getOffset(i), qec.getFetchFirst(i));
         }
-
-        bindOffsetFetch(offset, fetchFirst);
     }
 
 	/**
@@ -292,11 +257,11 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void bindExpressionsWithTables(FromList fromListParam)
+    @Override
+    void bindExpressionsWithTables(FromList fromListParam)
 					throws StandardException
 	{
 		/* We don't have any tables, so just return */
-		return;
 	}
 
 	/**
@@ -308,8 +273,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public void bindTargetExpressions(FromList fromListParam)
+    @Override
+    void bindTargetExpressions(FromList fromListParam)
 					throws StandardException
 	{
 		bindExpressions(fromListParam);
@@ -322,7 +287,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void bindUntypedNullsToResultColumns(ResultColumnList bindingRCL)
+    @Override
+    void bindUntypedNullsToResultColumns(ResultColumnList bindingRCL)
 				throws StandardException
 	{
 		/*
@@ -335,9 +301,9 @@ public class RowResultSetNode extends FromTable
 		** in our RCL.
 		*/
 		if (bindingRCL == null)
-			bindingRCL = resultColumns;
+			bindingRCL = getResultColumns();
 
-		resultColumns.bindUntypedNullsToResultColumns(bindingRCL);
+		getResultColumns().bindUntypedNullsToResultColumns(bindingRCL);
 	}
 
 	/**
@@ -353,8 +319,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public ResultColumn getMatchingColumn(
+    @Override
+    ResultColumn getMatchingColumn(
 						ColumnReference columnReference)
 						throws StandardException
 	{
@@ -369,7 +335,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public String getExposedName() throws StandardException
+    @Override
+    String getExposedName() throws StandardException
 	{
 		return null;
 	}
@@ -382,12 +349,17 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void verifySelectStarSubquery(FromList outerFromList, int subqueryType) 
+    @Override
+    void verifySelectStarSubquery(FromList outerFromList, int subqueryType)
 					throws StandardException
 	{
-		return; 
 	}
 
+    @Override
+    public void pushQueryExpressionSuffix() {
+        qec.push();
+    }
+
 	/**
 	 * Push the order by list down from the cursor node
 	 * into its child result set so that the optimizer
@@ -396,9 +368,10 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @param orderByList	The order by list
 	 */
+    @Override
 	void pushOrderByList(OrderByList orderByList)
 	{
-		this.orderByList = orderByList;
+        qec.setOrderByList(orderByList);
 	}
 
     /**
@@ -408,11 +381,12 @@ public class RowResultSetNode extends FromTable
      * @param fetchFirst the OFFSET FIRST, if any
      * @param hasJDBClimitClause true if the clauses were added by (and have the semantics of) a JDBC limit clause
      */
+    @Override
     void pushOffsetFetchFirst( ValueNode offset, ValueNode fetchFirst, boolean hasJDBClimitClause )
     {
-        this.offset = offset;
-        this.fetchFirst = fetchFirst;
-        this.hasJDBClimitClause = hasJDBClimitClause;
+        qec.setOffset(offset);
+        qec.setFetchFirst(fetchFirst);
+        qec.setHasJDBCLimitClause(hasJDBClimitClause);
     }
 
 
@@ -440,8 +414,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public ResultSetNode preprocess(int numTables,
+    @Override
+    ResultSetNode preprocess(int numTables,
 									GroupByList gbl,
 									FromList fromList)
 								throws StandardException
@@ -450,21 +424,24 @@ public class RowResultSetNode extends FromTable
                 numTables,
                 fromList,
                 subquerys,
-                (PredicateList) getNodeFactory().getNode(
-													C_NodeTypes.PREDICATE_LIST,
-													getContextManager()));
+                new PredicateList(getContextManager()));
 
 		/* Allocate a dummy referenced table map */ 
-		referencedTableMap = new JBitSet(numTables);
-		referencedTableMap.set(tableNumber);
+		setReferencedTableMap( new JBitSet(numTables) );
+		getReferencedTableMap().set(tableNumber);
 
         // If we have more than 1 ORDERBY columns, we may be able to
         // remove duplicate columns, e.g., "ORDER BY 1, 1, 2".
         // Well, not very likely here, since we're here:
         //     VALUES x followed by ORDER BY 1,1,2
         // but for completeness...
-        if (orderByList != null && orderByList.size() > 1) {
-            orderByList.removeDupColumns();
+        for (int i = 0; i < qec.size(); i++) {
+            final OrderByList obl = qec.getOrderByList(i);
+
+
+            if (obl != null && obl.size() > 1) {
+                obl.removeDupColumns();
+            }
         }
 
         return this;
@@ -478,7 +455,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ResultSetNode ensurePredicateList(int numTables) 
+    @Override
+    ResultSetNode ensurePredicateList(int numTables)
 		throws StandardException
 	{
 		return genProjectRestrict(numTables);
@@ -495,7 +473,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ResultSetNode addNewPredicate(Predicate predicate)
+    @Override
+    ResultSetNode addNewPredicate(Predicate predicate)
 			throws StandardException
 	{
 		PredicateList		predList;
@@ -510,24 +489,20 @@ public class RowResultSetNode extends FromTable
 		/* We get a shallow copy of the ResultColumnList and its 
 		 * ResultColumns.  (Copy maintains ResultColumn.expression for now.)
 		 */
-		prRCList = resultColumns;
-		resultColumns = resultColumns.copyListAndObjects();
+		prRCList = getResultColumns();
+		setResultColumns( getResultColumns().copyListAndObjects() );
 
 		/* Replace ResultColumn.expression with new VirtualColumnNodes
 		 * in the ProjectRestrictNode's ResultColumnList.  (VirtualColumnNodes include
 		 * pointers to source ResultSetNode, this, and source ResultColumn.)
 		 */
-		prRCList.genVirtualColumnNodes(this, resultColumns);
+		prRCList.genVirtualColumnNodes(this, getResultColumns());
 
 		/* Put the new predicate in a list */
-		predList = (PredicateList) getNodeFactory().getNode(
-										C_NodeTypes.PREDICATE_LIST,
-										getContextManager());
+        predList = new PredicateList(getContextManager());
 		predList.addPredicate(predicate);
 
-		/* Finally, we create the new ProjectRestrictNode */
-		return (ResultSetNode) getNodeFactory().getNode(
-								C_NodeTypes.PROJECT_RESTRICT_NODE,
+        return new ProjectRestrictNode(
 								this,
 								prRCList,
 								null,	/* Restriction */
@@ -553,7 +528,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @return boolean	Whether or not the FromSubquery is flattenable.
 	 */
-	public boolean flattenableInFromSubquery(FromList fromList)
+    @Override
+    boolean flattenableInFromSubquery(FromList fromList)
 	{
 		if ((subquerys != null) &&
 			(subquerys.size() > 0))
@@ -561,8 +537,7 @@ public class RowResultSetNode extends FromTable
 			return false;
 		}
 
-		if ((aggregateVector != null) &&
-			(aggregateVector.size() > 0))
+        if ((aggregates != null) && !aggregates.isEmpty())
 		{
 			return false;
 		}
@@ -571,7 +546,7 @@ public class RowResultSetNode extends FromTable
 		** Don't flatten if select list contains something
 		** that isn't clonable
 		*/
-		if ( ! resultColumns.isCloneable())
+		if ( ! getResultColumns().isCloneable())
 		{
 			return false;
 		}
@@ -613,7 +588,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ResultSetNode optimize(DataDictionary dataDictionary,
+    @Override
+    ResultSetNode optimize(DataDictionary dataDictionary,
 								  PredicateList	predicateList,
 								  double outerRows) 
 			throws StandardException
@@ -623,19 +599,10 @@ public class RowResultSetNode extends FromTable
 		** CostEstimate object, so we can represent the cost of this node.
 		** This seems like overkill, but it's just an object allocation...
 		*/
-		Optimizer optimizer =
-					getOptimizer(
-								(FromList) getNodeFactory().getNode(
-									C_NodeTypes.FROM_LIST,
-									getNodeFactory().doJoinOrderOptimization(),
-									getContextManager()),
-								predicateList,
-								dataDictionary,
-								(RequiredRowOrdering) null);
-		costEstimate = optimizer.newCostEstimate();
+        setCostEstimate( getOptimizerFactory().getCostEstimate() );
 
 		// RESOLVE: THE COST SHOULD TAKE SUBQUERIES INTO ACCOUNT
-		costEstimate.setCost(0.0d, outerRows, outerRows);
+		getCostEstimate().setCost(0.0d, outerRows, outerRows);
 
 		subquerys.optimize(dataDictionary, outerRows);
 		return this;
@@ -646,6 +613,7 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	public Optimizable modifyAccessPath(JBitSet outerTables) throws StandardException
 	{
 		/* For most types of Optimizable, do nothing */
@@ -657,7 +625,8 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ResultSetNode modifyAccessPaths() throws StandardException
+    @Override
+    ResultSetNode modifyAccessPaths() throws StandardException
 	{
 		ResultSetNode treeTop = this;
 
@@ -666,29 +635,33 @@ public class RowResultSetNode extends FromTable
 		/* Generate the OrderByNode if a sort is still required for
 		 * the order by.
 		 */
-		if (orderByList != null)
-		{
-			treeTop = (ResultSetNode) getNodeFactory().getNode(
-											C_NodeTypes.ORDER_BY_NODE,
-											treeTop,
-											orderByList,
-											tableProperties,
-											getContextManager());
-		}
+        for (int i = 0; i < qec.size(); i++) {
+            final OrderByList obl = qec.getOrderByList(i);
+            if (obl != null) {
+                treeTop = new OrderByNode(treeTop,
+                        obl,
+                        tableProperties,
+                        getContextManager());
+            }
 
-        if (offset != null || fetchFirst != null) {
-            ResultColumnList newRcl =
-                treeTop.getResultColumns().copyListAndObjects();
-            newRcl.genVirtualColumnNodes(treeTop, treeTop.getResultColumns());
+            final ValueNode offset = qec.getOffset(i);
+            final ValueNode fetchFirst = qec.getFetchFirst(i);
+            Boolean hasJDBClimitClause = qec.getHasJDBCLimitClause()[i];
 
-            treeTop = (ResultSetNode)getNodeFactory().getNode(
-                C_NodeTypes.ROW_COUNT_NODE,
-                treeTop,
-                newRcl,
-                offset,
-                fetchFirst,
-                Boolean.valueOf( hasJDBClimitClause ),
-                getContextManager());
+            if (offset != null || fetchFirst != null) {
+                ResultColumnList newRcl =
+                    treeTop.getResultColumns().copyListAndObjects();
+                newRcl.genVirtualColumnNodes(
+                    treeTop, treeTop.getResultColumns());
+
+                treeTop = new RowCountNode(
+                        treeTop,
+                        newRcl,
+                        offset,
+                        fetchFirst,
+                        hasJDBClimitClause.booleanValue(),
+                        getContextManager());
+            }
         }
 
 		return treeTop;
@@ -702,6 +675,7 @@ public class RowResultSetNode extends FromTable
 	 * @return Whether or not this ResultSet tree is guaranteed to return
 	 * at most 1 row based on heuristics.
 	 */
+    @Override
 	boolean returnsAtMostOneRow()
 	{
 		return true;
@@ -715,26 +689,24 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	void setTableConstructorTypes(ResultColumnList typeColumns)
 			throws StandardException
 	{
 		if (SanityManager.DEBUG)
-			SanityManager.ASSERT(resultColumns.visibleSize() <= typeColumns.size(),
+			SanityManager.ASSERT(getResultColumns().visibleSize() <= typeColumns.size(),
 				"More columns in ResultColumnList than in base table");
 
 		/* Look for ? parameters in the result column list */
-		int rclSize = resultColumns.size();
+		int rclSize = getResultColumns().size();
 		for (int index = 0; index < rclSize; index++)
 		{
-			ResultColumn	rc = (ResultColumn) resultColumns.elementAt(index);
-
+            ResultColumn rc = getResultColumns().elementAt(index);
 			ValueNode re = rc.getExpression();
 
 			if (re.requiresTypeFromContext())
 			{
-				ResultColumn	typeCol =
-					(ResultColumn) typeColumns.elementAt(index);
-
+                ResultColumn typeCol = typeColumns.elementAt(index);
 				/*
 				** We found a ? - set its type to the type of the
 				** corresponding column of the target table.
@@ -779,8 +751,7 @@ public class RowResultSetNode extends FromTable
 				// still a normailzation step when the value is inserted into the table.
 				// That will set the correct length and perform truncation checks etc.
 
-				ResultColumn	typeCol =
-					(ResultColumn) typeColumns.elementAt(index);
+                ResultColumn typeCol = typeColumns.elementAt(index);
 
 				TypeId colTypeId = typeCol.getTypeId();
 
@@ -816,10 +787,8 @@ public class RowResultSetNode extends FromTable
 			}
 			else if (re instanceof BitConstantNode)
 			{
-				ResultColumn	typeCol =
-					(ResultColumn) typeColumns.elementAt(index);
-
-				TypeId colTypeId = typeCol.getTypeId();
+                ResultColumn typeCol   = typeColumns.elementAt(index);
+                TypeId       colTypeId = typeCol.getTypeId();
 
 				if (colTypeId.isBitTypeId()) {
 
@@ -865,15 +834,15 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
      */
-	public void generate(ActivationClassBuilder acb,
-								MethodBuilder mb)
+    @Override
+    void generate(ActivationClassBuilder acb, MethodBuilder mb)
 							throws StandardException
 	{
 		if (SanityManager.DEBUG)
-        SanityManager.ASSERT(resultColumns != null, "Tree structure bad");
+            SanityManager.ASSERT(getResultColumns() != null, "Tree structure bad");
 
 		// Get our final cost estimate.
-		costEstimate = getFinalCostEstimate();
+		setCostEstimate( getFinalCostEstimate() );
 
 		/*
 		** Check and see if everything below us is a constant or not.
@@ -906,11 +875,11 @@ public class RowResultSetNode extends FromTable
 		acb.pushGetResultSetFactoryExpression(mb);
 
 		acb.pushThisAsActivation(mb);
-		resultColumns.generate(acb, mb);
+		getResultColumns().generate(acb, mb);
 		mb.push(canCache);
-		mb.push(resultSetNumber);
-		mb.push(costEstimate.rowCount());
-		mb.push(costEstimate.getEstimatedCost());
+		mb.push(getResultSetNumber());
+		mb.push(getCostEstimate().rowCount());
+		mb.push(getCostEstimate().getEstimatedCost());
 		mb.callMethod(VMOpcode.INVOKEINTERFACE, (String) null, "getRowResultSet",
                 ClassName.NoPutResultSet, 6);
 	}
@@ -918,12 +887,13 @@ public class RowResultSetNode extends FromTable
 	/**
      * {@inheritDoc}
 	 */
+    @Override
 	void replaceOrForbidDefaults(TableDescriptor ttd,
                                  ResultColumnList tcl,
                                  boolean allowDefaults)
 		throws StandardException
 	{
-		resultColumns.replaceOrForbidDefaults(ttd, tcl, allowDefaults);
+		getResultColumns().replaceOrForbidDefaults(ttd, tcl, allowDefaults);
 	}
 
 	/**
@@ -933,6 +903,7 @@ public class RowResultSetNode extends FromTable
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	void optimizeSubqueries(DataDictionary dd, double rowCount)
 		throws StandardException
 	{
@@ -942,6 +913,7 @@ public class RowResultSetNode extends FromTable
 	/**
 	 * @see ResultSetNode#adjustForSortElimination
 	 */
+    @Override
 	void adjustForSortElimination()
 	{
 	}
