@@ -21,40 +21,32 @@
 
 package	org.apache.derby.impl.sql.compile;
 
-import org.apache.derby.iapi.services.compiler.MethodBuilder;
-import org.apache.derby.iapi.services.context.ContextManager;
-
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
-import org.apache.derby.iapi.sql.compile.CompilerContext;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
-import org.apache.derby.iapi.sql.compile.NodeFactory;
-import org.apache.derby.iapi.types.JSQLType;
-import org.apache.derby.iapi.types.DataTypeDescriptor;
-import org.apache.derby.iapi.types.StringDataValue;
-import org.apache.derby.iapi.types.TypeId;
-
-import org.apache.derby.iapi.sql.dictionary.AliasDescriptor;
-import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
-
-import org.apache.derby.iapi.reference.ClassName;
-import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.reference.JDBC30Translation;
-import org.apache.derby.iapi.error.StandardException;
-
-import org.apache.derby.iapi.services.compiler.LocalField;
-
-import org.apache.derby.iapi.util.JBitSet;
-import org.apache.derby.iapi.services.classfile.VMOpcode;
-
-import org.apache.derby.iapi.sql.conn.Authorizer;
-
+import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
+import java.sql.ParameterMetaData;
+import java.util.List;
 import org.apache.derby.catalog.AliasInfo;
 import org.apache.derby.catalog.TypeDescriptor;
 import org.apache.derby.catalog.types.RoutineAliasInfo;
-
-import java.lang.reflect.Modifier;
-import java.util.List;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.reference.ClassName;
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.classfile.VMOpcode;
+import org.apache.derby.iapi.services.compiler.LocalField;
+import org.apache.derby.iapi.services.compiler.MethodBuilder;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.sql.compile.CompilerContext;
+import org.apache.derby.iapi.sql.compile.Visitor;
+import org.apache.derby.iapi.sql.conn.Authorizer;
+import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
+import org.apache.derby.iapi.sql.dictionary.AliasDescriptor;
+import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
+import org.apache.derby.iapi.types.DataTypeDescriptor;
+import org.apache.derby.iapi.types.JSQLType;
+import org.apache.derby.iapi.types.StringDataValue;
+import org.apache.derby.iapi.types.TypeId;
+import org.apache.derby.iapi.util.JBitSet;
 
 /**
  * A StaticMethodCallNode represents a static method call from a Class
@@ -92,8 +84,10 @@ import java.util.List;
 		- the parameter is passed directly to the method call (no casts or expressions).
 		- the method's parameter type is a Java array type.
 
-    Since this is a dynmaic decision we compile in code to take both paths, based upon a boolean isINOUT which is dervied from the
-	ParameterValueSet. Code is logically (only single parameter String[] shown here). Note, no casts can exist here.
+    Since this is a dynamic decision we compile in code to take both paths,
+    based upon a boolean is INOUT which is derived from the
+    ParameterValueSet. Code is logically (only single parameter String[] shown
+    here). Note, no casts can exist here.
 
 	boolean isINOUT = getParameterValueSet().getParameterMode(0) == PARAMETER_IN_OUT;
 	if (isINOUT) {
@@ -105,7 +99,7 @@ import java.util.List;
 
  *
  */
-public class StaticMethodCallNode extends MethodCallNode
+class StaticMethodCallNode extends MethodCallNode
 {
 	private TableName procedureName;
 
@@ -142,22 +136,37 @@ public class StaticMethodCallNode extends MethodCallNode
 
 
 	/**
-	 * Intializer for a NonStaticMethodCallNode
+     * Constructor for a NonStaticMethodCallNode
 	 *
 	 * @param methodName		The name of the method to call
-	 * @param javaClassName		The name of the java class that the static method belongs to.
+     * @param javaClassName     The name of the java class that the static
+     *                          method belongs to.
+     * @param cm                The context manager
 	 */
-	public void init(Object methodName, Object javaClassName)
-	{
-		if (methodName instanceof String)
-			init(methodName);
-		else {
-			procedureName = (TableName) methodName;
-			init(procedureName.getTableName());
-		}
+    StaticMethodCallNode(
+            String methodName,
+            String javaClassName,
+            ContextManager cm) {
+        super(methodName, cm);
+        this.javaClassName = javaClassName;
+    }
 
-		this.javaClassName = (String) javaClassName;
-	}
+    /**
+     * Constructor for a StaticMethodCallNode
+     *
+     * @param methodName        The name of the method to call
+     * @param javaClassName     The name of the java class that the static
+     *                          method belongs to.
+     * @param cm                The context manager
+     */
+    StaticMethodCallNode(
+            TableName methodName,
+            String javaClassName,
+            ContextManager cm) {
+        super(methodName.getTableName(), cm);
+        procedureName = methodName;
+        this.javaClassName = javaClassName;
+    }
 
     /**
      * Get the aggregate, if any, which this method call resolves to.
@@ -167,23 +176,27 @@ public class StaticMethodCallNode extends MethodCallNode
     /** Flag that this function invocation appears in a GROUP BY clause */
     public  void    setAppearsInGroupBy() { appearsInGroupBy = true; }
     
+    @Override
+    TableName getFullName()
+	{
+		return  procedureName;
+	}
+
 	/**
 	 * Bind this expression.  This means binding the sub-expressions,
 	 * as well as figuring out what the return type is for this expression.
 	 *
 	 * @param fromList		The FROM list for the query this
-	 *				expression is in, for binding columns.
-	 * @param subqueryList		The subquery list being built as we find SubqueryNodes
-	 * @param aggregateVector	The aggregate vector being built as we find AggregateNodes
+     *                      expression is in, for binding columns.
+     * @param subqueryList  The subquery list being built as we find SubqueryNodes
+     * @param aggregates    The aggregate list being built as we find AggregateNodes
 	 *
 	 * @return	this or an AggregateNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public JavaValueNode bindExpression(
-		FromList fromList, SubqueryList subqueryList,
-		List aggregateVector)
+    JavaValueNode bindExpression(
+        FromList fromList, SubqueryList subqueryList, List<AggregateNode> aggregates)
 			throws StandardException
 	{
 		// for a function we can get called recursively
@@ -193,7 +206,7 @@ public class StaticMethodCallNode extends MethodCallNode
 
         isInsideBind = true;
         try {
-            return bindExpressionMinion(fromList, subqueryList, aggregateVector);
+            return bindExpressionMinion(fromList, subqueryList, aggregates);
         } finally {
             isInsideBind = false;
         }
@@ -202,10 +215,10 @@ public class StaticMethodCallNode extends MethodCallNode
     private JavaValueNode bindExpressionMinion(
             FromList fromList,
             SubqueryList subqueryList,
-            List aggregateVector)
+            List<AggregateNode> aggregates)
         throws StandardException
     {
-		bindParameters(fromList, subqueryList, aggregateVector);
+        bindParameters(fromList, subqueryList, aggregates);
 
 		
 		/* If javaClassName is null then we assume that the current methodName
@@ -226,20 +239,22 @@ public class StaticMethodCallNode extends MethodCallNode
 
             // The field methodName is used by resolveRoutine and
             // is set to the name of the routine (procedureName.getTableName()).
-			resolveRoutine( fromList, subqueryList, aggregateVector, sd );
+            resolveRoutine( fromList, subqueryList, aggregates, sd, noSchema );
 
             if ( (ad != null) && (ad.getAliasType() == AliasInfo.ALIAS_TYPE_AGGREGATE_AS_CHAR) )
             {
-                resolvedAggregate = (AggregateNode) getNodeFactory().getNode
-                    (
-                     C_NodeTypes.AGGREGATE_NODE,
+                resolvedAggregate = new AggregateNode(
                      ((SQLToJavaValueNode) methodParms[ 0 ]).getSQLValueNode(),
                      new UserAggregateDefinition( ad ), 
-                     Boolean.FALSE,
+                     procedureName,
+                     false,
                      ad.getJavaClassName(),
                      getContextManager()
                      );
 
+                // Propagate tags used to flag nodes which need privilege checks. See DERBY-6429.
+                resolvedAggregate.copyTagsFrom( this );
+
                 // The parser may have noticed that this aggregate is invoked in a
                 // GROUP BY clause. That is not allowed.
                 if ( appearsInGroupBy )
@@ -261,7 +276,7 @@ public class StaticMethodCallNode extends MethodCallNode
                 // an in-memory table, set up in DataDictioanryImpl.
                 sd = getSchemaDescriptor("SYSFUN", true);
 
-                resolveRoutine(fromList, subqueryList, aggregateVector, sd);
+                resolveRoutine(fromList, subqueryList, aggregates, sd, noSchema);
             }
 
             if (ad == null) {
@@ -286,7 +301,7 @@ public class StaticMethodCallNode extends MethodCallNode
 
                     forCallStatement = true; // temporarily: resolve
                                              // as procedure
-                    resolveRoutine(fromList, subqueryList, aggregateVector, sd);
+                    resolveRoutine(fromList, subqueryList, aggregates, sd, noSchema);
                     forCallStatement = false; // restore it
 
                     if (ad != null) {
@@ -298,7 +313,7 @@ public class StaticMethodCallNode extends MethodCallNode
                     // Maybe a function is being CALLed ?
                     forCallStatement = false; // temporarily: resolve
                                               // as function
-                    resolveRoutine(fromList, subqueryList, aggregateVector, sd);
+                    resolveRoutine(fromList, subqueryList, aggregates, sd, noSchema);
                     forCallStatement = true; // restore it
 
                     if (ad != null) {
@@ -315,7 +330,13 @@ public class StaticMethodCallNode extends MethodCallNode
 				throw StandardException.newException(
                         SQLState.LANG_NO_SUCH_METHOD_ALIAS, procedureName);
 			}
-	
+
+            if (noSchema) {
+                // If no schema was specified, register where we found the
+                // routine.
+                procedureName.setSchemaName(sd.getSchemaName());
+            }
+
             if ( !routineInfo.isDeterministic() )
             {
                 checkReliability( getMethodName(), CompilerContext.NON_DETERMINISTIC_ILLEGAL );
@@ -348,6 +369,10 @@ public class StaticMethodCallNode extends MethodCallNode
             if (
                 javaClassName.startsWith( "org.apache.derby." ) &&
                 !javaClassName.startsWith( "org.apache.derby.impl.tools.optional." ) &&
+                !javaClassName.startsWith( "org.apache.derby.optional.lucene." ) &&
+                !javaClassName.startsWith( "org.apache.derby.optional.json." ) &&
+                !javaClassName.startsWith( "org.apache.derby.optional.api." ) &&
+                !javaClassName.startsWith( "org.apache.derby.optional.dump." ) &&
                 !javaClassName.startsWith( "org.apache.derby.vti." )
                 )
             {
@@ -396,13 +421,10 @@ public class StaticMethodCallNode extends MethodCallNode
 							);
 							
 
-					ValueNode returnValueToSQL = (ValueNode) getNodeFactory().getNode(
-								C_NodeTypes.JAVA_TO_SQL_VALUE_NODE,
-								this, 
-								getContextManager());
+                    ValueNode returnValueToSQL =
+                            new JavaToSQLValueNode(this, getContextManager());
 
-					ValueNode returnValueCastNode = (ValueNode) getNodeFactory().getNode(
-									C_NodeTypes.CAST_NODE,
+                    ValueNode returnValueCastNode = new CastNode(
 									returnValueToSQL, 
 									returnValueDtd,
 									getContextManager());
@@ -412,13 +434,10 @@ public class StaticMethodCallNode extends MethodCallNode
                             returnType.getCollationType(),
                             StringDataValue.COLLATION_DERIVATION_IMPLICIT);
 
-
-					JavaValueNode returnValueToJava = (JavaValueNode) getNodeFactory().getNode(
-										C_NodeTypes.SQL_TO_JAVA_VALUE_NODE,
-										returnValueCastNode, 
-										getContextManager());
+                    JavaValueNode returnValueToJava = new SQLToJavaValueNode(
+                            returnValueCastNode, getContextManager());
 					returnValueToJava.setCollationType(returnType.getCollationType());
-					return returnValueToJava.bindExpression(fromList, subqueryList, aggregateVector);
+                    return returnValueToJava.bindExpression(fromList, subqueryList, aggregates);
 				}
 
 			}
@@ -512,25 +531,23 @@ public class StaticMethodCallNode extends MethodCallNode
 	 * with a given type and name is allowed, thus if changes are made to
 	 * support overloaded routines, careful code inspection and testing will
 	 * be required.
-	 * @param fromList
-	 * @param subqueryList
-	 * @param aggregateVector
-	 * @param sd
-	 * @throws StandardException
 	 */
     private void resolveRoutine(FromList fromList, SubqueryList subqueryList,
-                                List aggregateVector, SchemaDescriptor sd)
+                                List<AggregateNode> aggregates, SchemaDescriptor sd,
+                                boolean noSchema)
             throws StandardException {
 		if (sd.getUUID() != null) {
 
-		java.util.List list = getDataDictionary().getRoutineList(
-			sd.getUUID().toString(), methodName,
-			forCallStatement ? AliasInfo.ALIAS_NAME_SPACE_PROCEDURE_AS_CHAR : AliasInfo.ALIAS_NAME_SPACE_FUNCTION_AS_CHAR
-			);
+        List<AliasDescriptor> list = getDataDictionary().getRoutineList(
+            sd.getUUID().toString(),
+            methodName,
+            forCallStatement ?
+                AliasInfo.ALIAS_NAME_SPACE_PROCEDURE_AS_CHAR :
+                AliasInfo.ALIAS_NAME_SPACE_FUNCTION_AS_CHAR);
 
 		for (int i = list.size() - 1; i >= 0; i--) {
 
-			AliasDescriptor proc = (AliasDescriptor) list.get(i);
+            AliasDescriptor proc = list.get(i);
 
 			RoutineAliasInfo rai = (RoutineAliasInfo) proc.getAliasInfo();
 			int parameterCount = rai.getParameterCount();
@@ -569,7 +586,7 @@ public class StaticMethodCallNode extends MethodCallNode
 				// if it's an OUT or INOUT parameter we need an array.
 				int parameterMode = rai.getParameterModes()[ getRoutineArgIdx( rai, p ) ];
 
-				if (parameterMode != JDBC30Translation.PARAMETER_MODE_IN) {
+                if (parameterMode != (ParameterMetaData.parameterModeIn)) {
 
 					String arrayType;
 					switch (typeId.getJDBCTypeId()) {
@@ -628,7 +645,7 @@ public class StaticMethodCallNode extends MethodCallNode
                     {
                         coerceMethodParameter
                             (
-                             fromList, subqueryList, aggregateVector,
+                             fromList, subqueryList, aggregates,
                              rai,
                              methodParms.length,
                              paramdtd, parameterTypeId, parameterMode,
@@ -640,7 +657,7 @@ public class StaticMethodCallNode extends MethodCallNode
                 {
                     coerceMethodParameter
                         (
-                         fromList, subqueryList, aggregateVector,
+                         fromList, subqueryList, aggregates,
                          rai,
                          methodParms.length,
                          paramdtd, parameterTypeId, parameterMode,
@@ -683,7 +700,8 @@ public class StaticMethodCallNode extends MethodCallNode
 
         if ( (ad == null) && (methodParms.length == 1) )
         {
-            ad = AggregateNode.resolveAggregate( getDataDictionary(), sd, methodName );
+            ad = AggregateNode.resolveAggregate
+                ( getDataDictionary(), sd, methodName, noSchema );
         }
 	}
 
@@ -697,7 +715,7 @@ public class StaticMethodCallNode extends MethodCallNode
         (
          FromList fromList,
          SubqueryList subqueryList,
-         List aggregateVector,
+         List<AggregateNode> aggregates,
          RoutineAliasInfo rai,
          int    parameterCount, // number of declared routine args
          DataTypeDescriptor paramdtd,   // declared type of routine arg
@@ -720,7 +738,7 @@ public class StaticMethodCallNode extends MethodCallNode
         boolean isParameterMarker = true;
         if ((sqlParamNode == null) || !sqlParamNode.requiresTypeFromContext())
         {
-            if (parameterMode != JDBC30Translation.PARAMETER_MODE_IN)
+            if (parameterMode != (ParameterMetaData.parameterModeIn))
             {
                 throw StandardException.newException
                     (
@@ -802,7 +820,7 @@ public class StaticMethodCallNode extends MethodCallNode
             // correctly as 10 characters long.
             if (parameterTypeId.variableLength())
             {
-                if (parameterMode != JDBC30Translation.PARAMETER_MODE_OUT)
+                if (parameterMode != (ParameterMetaData.parameterModeOut))
                 { needCast = true; }
             }
         }
@@ -819,30 +837,22 @@ public class StaticMethodCallNode extends MethodCallNode
             
             if (sqlParamNode == null)
             {
-                sqlParamNode = (ValueNode) getNodeFactory().getNode
-                    (
-                     C_NodeTypes.JAVA_TO_SQL_VALUE_NODE,
-                     methodParms[p], 
-                     getContextManager()
-                     );
+                sqlParamNode =
+                    new JavaToSQLValueNode(methodParms[p], getContextManager());
             }
 
             ValueNode castNode = makeCast
                 (
                  sqlParamNode,
                  paramdtd,
-                 getNodeFactory(),
                  getContextManager()
                  );
 
-            methodParms[p] = (JavaValueNode) getNodeFactory().getNode
-                (
-                 C_NodeTypes.SQL_TO_JAVA_VALUE_NODE,
-                 castNode, 
-                 getContextManager()
-                 );
+            methodParms[p] =
+                    new SQLToJavaValueNode(castNode, getContextManager());
 
-            methodParms[p] = methodParms[p].bindExpression(fromList, subqueryList, aggregateVector);
+            methodParms[p] = methodParms[p].bindExpression(
+                    fromList, subqueryList, aggregates);
         }
 
         // only force the type for a ? so that the correct type shows up
@@ -853,17 +863,12 @@ public class StaticMethodCallNode extends MethodCallNode
     /**
      * Wrap a parameter in a CAST node.
      */
-    public  static  ValueNode   makeCast
-        ( ValueNode parameterNode, DataTypeDescriptor targetType, NodeFactory nodeFactory, ContextManager cm )
+    public static ValueNode makeCast (ValueNode parameterNode,
+                                      DataTypeDescriptor targetType,
+                                      ContextManager cm)
         throws StandardException
     {
-        ValueNode castNode = (ValueNode) nodeFactory.getNode
-            (
-             C_NodeTypes.CAST_NODE,
-             parameterNode, 
-             targetType,
-             cm
-             );
+        ValueNode castNode = new CastNode(parameterNode, targetType, cm);
 
         // Argument type has the same semantics as assignment:
         // Section 9.2 (Store assignment). There, General Rule 
@@ -880,13 +885,13 @@ public class StaticMethodCallNode extends MethodCallNode
 	 * procedure or function which needs a nested SQL session
 	 * context (only needed for those which can contain SQL).
 	 *
-	 * The generated code calls setupNestedSessionContext.
-	 * @see org.apache.derby.iapi.sql.conn.LanguageConnectionContext#setupNestedSessionContext
+     * The generated code calls pushNestedSessionContext.
+     * @see LanguageConnectionContext#pushNestedSessionContext
 	 *
 	 * @param acb activation class builder
 	 * @param mb  method builder
 	 */
-    private void generateSetupNestedSessionContext(
+    private void generatePushNestedSessionContext(
         ActivationClassBuilder acb,
         MethodBuilder mb,
         boolean hadDefinersRights,
@@ -894,7 +899,7 @@ public class StaticMethodCallNode extends MethodCallNode
 
 		// Generates the following Java code:
 		// ((Activation)this).getLanguageConnectionContext().
-		//       setupNestedSessionContext((Activation)this);
+        //       pushNestedSessionContext((Activation)this);
 
 		acb.pushThisAsActivation(mb);
 		mb.callMethod(VMOpcode.INVOKEINTERFACE, null,
@@ -904,7 +909,7 @@ public class StaticMethodCallNode extends MethodCallNode
         mb.push(hadDefinersRights);
         mb.push(definer);
 		mb.callMethod(VMOpcode.INVOKEINTERFACE, null,
-					  "setupNestedSessionContext",
+                      "pushNestedSessionContext",
                       "void", 3);
 	}
 
@@ -913,7 +918,8 @@ public class StaticMethodCallNode extends MethodCallNode
 		Push extra code to generate the casts within the
 		arrays for the parameters passed as arrays.
 	*/
-	public	void generateOneParameter(ExpressionClassBuilder acb,
+    @Override
+    void generateOneParameter(ExpressionClassBuilder acb,
 											MethodBuilder mb,
 											int parameterNumber )
 			throws StandardException
@@ -931,7 +937,7 @@ public class StaticMethodCallNode extends MethodCallNode
 			// may be registered as an IN OUT parameter. For a static method argument to be
 			// a dynmaically registered out parameter it must be a simple ? parameter
 
-			parameterMode = JDBC30Translation.PARAMETER_MODE_IN;
+            parameterMode = (ParameterMetaData.parameterModeIn);
 
 			if (sql2j != null) {
 				if (sql2j.getSQLValueNode().requiresTypeFromContext()) {
@@ -955,7 +961,7 @@ public class StaticMethodCallNode extends MethodCallNode
 											"getParameterValueSet", ClassName.ParameterValueSet, 0);
 
 						constructor.push(applicationParameterNumber);
-						constructor.push(JDBC30Translation.PARAMETER_MODE_UNKNOWN);
+                        constructor.push(ParameterMetaData.parameterModeUnknown);
 						constructor.callMethod(VMOpcode.INVOKEINTERFACE, null,
 											"setParameterMode", "void", 2);
 						constructor.endStatement();
@@ -965,27 +971,27 @@ public class StaticMethodCallNode extends MethodCallNode
 		}
 
 		switch (parameterMode) {
-		case JDBC30Translation.PARAMETER_MODE_IN:
-		case JDBC30Translation.PARAMETER_MODE_IN_OUT:
-		case JDBC30Translation.PARAMETER_MODE_UNKNOWN:
+        case (ParameterMetaData.parameterModeIn):
+        case (ParameterMetaData.parameterModeInOut):
+        case (ParameterMetaData.parameterModeUnknown):
 			if (sql2j != null)
 				sql2j.returnsNullOnNullState = returnsNullOnNullState;
 			super.generateOneParameter(acb, mb, parameterNumber);
 			break;
 
-		case JDBC30Translation.PARAMETER_MODE_OUT:
+        case (ParameterMetaData.parameterModeOut):
 			// For an OUT parameter we require nothing to be pushed into the
 			// method call from the parameter node.
 			break;
 		}
 
 		switch (parameterMode) {
-		case JDBC30Translation.PARAMETER_MODE_IN:
-		case JDBC30Translation.PARAMETER_MODE_UNKNOWN:
+        case (ParameterMetaData.parameterModeIn):
+        case (ParameterMetaData.parameterModeUnknown):
 			break;
 
-		case JDBC30Translation.PARAMETER_MODE_IN_OUT:
-		case JDBC30Translation.PARAMETER_MODE_OUT:
+        case (ParameterMetaData.parameterModeInOut):
+        case (ParameterMetaData.parameterModeOut):
 		{
 			// Create the array used to pass into the method. We create a
 			// new array for each call as there is a small chance the
@@ -1013,7 +1019,7 @@ public class StaticMethodCallNode extends MethodCallNode
 			mb.putField(lf);
 
 			// set the IN part of the parameter into the INOUT parameter.
-			if (parameterMode != JDBC30Translation.PARAMETER_MODE_OUT) {
+            if (parameterMode != (ParameterMetaData.parameterModeOut)) {
 				mb.swap();
 				mb.setArrayElement(0);
 				mb.getField(lf);
@@ -1050,7 +1056,8 @@ public class StaticMethodCallNode extends MethodCallNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public boolean categorize(JBitSet referencedTabs, boolean simplePredsOnly)
+    @Override
+    boolean categorize(JBitSet referencedTabs, boolean simplePredsOnly)
 		throws StandardException
 	{
 		/* We stop here when only considering simple predicates
@@ -1075,7 +1082,7 @@ public class StaticMethodCallNode extends MethodCallNode
 	 *
 	 * @return	This object as a String
 	 */
-
+    @Override
 	public String toString()
 	{
 		if (SanityManager.DEBUG)
@@ -1099,9 +1106,7 @@ public class StaticMethodCallNode extends MethodCallNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public void generateExpression(ExpressionClassBuilder acb,
-											MethodBuilder mb)
+    void generateExpression(ExpressionClassBuilder acb, MethodBuilder mb)
 									throws StandardException
 	{
 		if (routineInfo != null) {
@@ -1182,7 +1187,7 @@ public class StaticMethodCallNode extends MethodCallNode
 			// If no SQL, there is no need to setup a nested session
 			// context.
 			if (sqlAllowed != RoutineAliasInfo.NO_SQL) {
-                generateSetupNestedSessionContext(
+                generatePushNestedSessionContext(
                     (ActivationClassBuilder) acb,
                     mb,
                     routineInfo.hasDefinersRights(),
@@ -1378,7 +1383,7 @@ public class StaticMethodCallNode extends MethodCallNode
 
 					int parameterMode = parameterModes[ getRoutineArgIdx( i ) ];
                     
-					if (parameterMode != JDBC30Translation.PARAMETER_MODE_IN) {
+                    if (parameterMode != (ParameterMetaData.parameterModeIn)) {
 
 						// must be a parameter if it is INOUT or OUT.
 						ValueNode sqlParamNode = ((SQLToJavaValueNode) methodParms[i]).getSQLValueNode();
@@ -1409,9 +1414,13 @@ public class StaticMethodCallNode extends MethodCallNode
 
 						// is the underlying type for the OUT/INOUT parameter primitive.
                         // if this is a varargs arg then we have to strip off another array level
-                        Class   cellType = ((java.lang.reflect.Method) method).getParameterTypes()[ getRoutineArgIdx( i ) ].
-                            getComponentType();
-                        if ( isVararg( i ) ) { cellType = cellType.getComponentType(); }
+                        Class<?> cellType = ((Method) method).
+                                getParameterTypes()[ getRoutineArgIdx( i ) ].
+                                getComponentType();
+
+                        if ( isVararg( i ) ) {
+                            cellType = cellType.getComponentType();
+                        }
 						boolean isPrimitive = cellType.isPrimitive();
 
 						if (isNumericType) {
@@ -1473,4 +1482,13 @@ public class StaticMethodCallNode extends MethodCallNode
 	{
 		return Authorizer.EXECUTE_PRIV;
 	}
+
+    @Override
+    void acceptChildren(Visitor v) throws StandardException {
+        super.acceptChildren(v);
+
+        if (procedureName != null) {
+            procedureName = (TableName) procedureName.accept(v);
+        }
+    }
 }
