@@ -21,35 +21,25 @@
 
 package	org.apache.derby.impl.sql.compile;
 
+import java.lang.reflect.Modifier;
+import java.util.List;
 import org.apache.derby.iapi.error.StandardException;
-
+import org.apache.derby.iapi.reference.ClassName;
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.classfile.VMOpcode;
+import org.apache.derby.iapi.services.compiler.LocalField;
+import org.apache.derby.iapi.services.compiler.MethodBuilder;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.compile.CompilerContext;
 import org.apache.derby.iapi.sql.compile.CostEstimate;
 import org.apache.derby.iapi.sql.compile.Visitor;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
 import org.apache.derby.iapi.sql.conn.Authorizer;
-
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
-import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.reference.ClassName;
-
-import org.apache.derby.iapi.types.DataTypeDescriptor;
-
-import org.apache.derby.iapi.services.compiler.MethodBuilder;
-import org.apache.derby.iapi.services.compiler.LocalField;
-
-import org.apache.derby.iapi.services.sanity.SanityManager;
-
 import org.apache.derby.iapi.store.access.Qualifier;
-
-import java.lang.reflect.Modifier;
-
-import java.util.List;
-import org.apache.derby.impl.sql.execute.OnceResultSet;
-
+import org.apache.derby.iapi.types.DataTypeDescriptor;
 import org.apache.derby.iapi.util.JBitSet;
-import org.apache.derby.iapi.services.classfile.VMOpcode;
-
+import org.apache.derby.impl.sql.execute.OnceResultSet;
 
 /**
  * A SubqueryNode represents a subquery.  Subqueries return values to their
@@ -81,7 +71,7 @@ import org.apache.derby.iapi.services.classfile.VMOpcode;
  *
  */
 
-public class SubqueryNode extends ValueNode
+class SubqueryNode extends ValueNode
 {
 	/*
 	** This must be a single-column result set.  If the subquery is
@@ -176,29 +166,29 @@ public class SubqueryNode extends ValueNode
 	 * NotNode above the SubqueryNode in the tree.
 	 *
 	 */
-	public final static int NOTIMPLEMENTED_SUBQUERY		= -1;
-	public final static int FROM_SUBQUERY	    = 0;
-	public final static int IN_SUBQUERY			= 1;
-	public final static int NOT_IN_SUBQUERY		= 2;
-	public final static int EQ_ANY_SUBQUERY		= 3;
-	public final static int EQ_ALL_SUBQUERY		= 4;
-	public final static int NE_ANY_SUBQUERY		= 5;
-	public final static int NE_ALL_SUBQUERY		= 6;
-	public final static int GT_ANY_SUBQUERY		= 7;
-	public final static int GT_ALL_SUBQUERY		= 8;
-	public final static int GE_ANY_SUBQUERY		= 9;
-	public final static int GE_ALL_SUBQUERY		= 10;
-	public final static int LT_ANY_SUBQUERY		= 11;
-	public final static int LT_ALL_SUBQUERY		= 12;
-	public final static int LE_ANY_SUBQUERY		= 13;
-	public final static int LE_ALL_SUBQUERY		= 14;
-	public final static int EXISTS_SUBQUERY		= 15;
-	public final static int NOT_EXISTS_SUBQUERY	= 16;
-	public final static int EXPRESSION_SUBQUERY = 17;
+    final static int NOTIMPLEMENTED_SUBQUERY        = -1;
+    final static int FROM_SUBQUERY      = 0;
+    final static int IN_SUBQUERY            = 1;
+    final static int NOT_IN_SUBQUERY        = 2;
+    final static int EQ_ANY_SUBQUERY        = 3;
+    final static int EQ_ALL_SUBQUERY        = 4;
+    final static int NE_ANY_SUBQUERY        = 5;
+    final static int NE_ALL_SUBQUERY        = 6;
+    final static int GT_ANY_SUBQUERY        = 7;
+    final static int GT_ALL_SUBQUERY        = 8;
+    final static int GE_ANY_SUBQUERY        = 9;
+    final static int GE_ALL_SUBQUERY        = 10;
+    final static int LT_ANY_SUBQUERY        = 11;
+    final static int LT_ALL_SUBQUERY        = 12;
+    final static int LE_ANY_SUBQUERY        = 13;
+    final static int LE_ALL_SUBQUERY        = 14;
+    final static int EXISTS_SUBQUERY        = 15;
+    final static int NOT_EXISTS_SUBQUERY    = 16;
+    final static int EXPRESSION_SUBQUERY = 17;
 
 
 	/**
-	 * Initializer.
+     * Constructor.
 	 *
 	 * @param resultSet		The ResultSetNode for the subquery
 	 * @param subqueryType	The type of the subquery
@@ -207,23 +197,25 @@ public class SubqueryNode extends ValueNode
      * @param offset        OFFSET n ROWS
      * @param fetchFirst    FETCH FIRST n ROWS ONLY
 	 * @param hasJDBClimitClause True if the offset/fetchFirst clauses come from JDBC limit/offset escape syntax
+     * @param cm            Context Manager
 	 */
 
-	public void init(
-							Object resultSet,
-							Object subqueryType,
-							Object leftOperand,
-                            Object orderCols,
-                            Object offset,
-                            Object fetchFirst,
-                            Object hasJDBClimitClause)
+    SubqueryNode(ResultSetNode resultSet,
+                 int subqueryType,
+                 ValueNode leftOperand,
+                 OrderByList orderCols,
+                 ValueNode offset,
+                 ValueNode fetchFirst,
+                 boolean hasJDBClimitClause,
+                 ContextManager cm)
 	{
-		this.resultSet = (ResultSetNode) resultSet;
-		this.subqueryType = ((Integer) subqueryType).intValue();
-		this.orderByList = (OrderByList)orderCols;
-        this.offset = (ValueNode)offset;
-        this.fetchFirst = (ValueNode)fetchFirst;
-        this.hasJDBClimitClause = (hasJDBClimitClause == null) ? false : ((Boolean) hasJDBClimitClause).booleanValue();
+        super(cm);
+        this.resultSet = resultSet;
+        this.subqueryType = subqueryType;
+        this.orderByList = orderCols;
+        this.offset = offset;
+        this.fetchFirst = fetchFirst;
+        this.hasJDBClimitClause = hasJDBClimitClause;
 
 		/* Subqueries are presumed not to be under a top level AndNode by
 		 * default.  This is because expression normalization only recurses
@@ -231,7 +223,7 @@ public class SubqueryNode extends ValueNode
 		 * built-in functions, etc.
 		 */
 		underTopAndNode = false;
-		this.leftOperand = (ValueNode) leftOperand;
+        this.leftOperand = leftOperand;
 	}
 
 	/**
@@ -240,7 +232,7 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @return	This object as a String
 	 */
-
+    @Override
 	public String toString()
 	{
 		if (SanityManager.DEBUG)
@@ -265,8 +257,8 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @param depth		The depth of this node in the tree
 	 */
-
-	public void printSubNodes(int depth)
+    @Override
+    void printSubNodes(int depth)
 	{
 		if (SanityManager.DEBUG)
 		{
@@ -309,7 +301,7 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @return ResultSetNode underlying this SubqueryNode.
 	 */
-	public ResultSetNode getResultSet()
+    ResultSetNode getResultSet()
 	{
 		return resultSet;
 	}
@@ -319,7 +311,7 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @return int	Type of this subquery.
 	 */
-	public int getSubqueryType()
+    int getSubqueryType()
 	{
 		return subqueryType;
 	}
@@ -329,7 +321,7 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @param subqueryType of this subquery.
 	 */
-	public void setSubqueryType(int subqueryType)
+    void setSubqueryType(int subqueryType)
 	{
 		this.subqueryType = subqueryType;
 	}
@@ -341,7 +333,7 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @exception StandardException			Thrown on error
 	 */
-	public void setPointOfAttachment(int pointOfAttachment)
+    void setPointOfAttachment(int pointOfAttachment)
 		throws StandardException
 	{
 		/* Materialized subqueries always keep their point of
@@ -360,7 +352,7 @@ public class SubqueryNode extends ValueNode
 	 * @return boolean	Whether or not this subquery is immediately under a
 	 *					top level AndNode.
 	 */
-	public boolean getUnderTopAndNode()
+    boolean getUnderTopAndNode()
 	{
 		return underTopAndNode;
 	}
@@ -370,7 +362,7 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @return int		The ResultSet # for the point of attachment
 	 */
-	public int getPointOfAttachment()
+    int getPointOfAttachment()
 	{
 		if (SanityManager.DEBUG)
 		{
@@ -403,6 +395,7 @@ public class SubqueryNode extends ValueNode
   		parentComparisonOperator = parent;
   	}
 
+    @Override
     public boolean referencesSessionSchema() throws StandardException {
         return resultSet.referencesSessionSchema();
     }
@@ -415,7 +408,8 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @exception StandardException			Thrown on error
 	 */
-	public ValueNode remapColumnReferencesToExpressions()
+    @Override
+    ValueNode remapColumnReferencesToExpressions()
 		throws StandardException
 	{
 		/* We need to remap both the SELECT and Predicate lists 
@@ -447,14 +441,14 @@ public class SubqueryNode extends ValueNode
 	 *							NOTE: fromList will be null if the subquery appears
 	 *							in a VALUES clause.
 	 * @param subqueryList		The subquery list being built as we find SubqueryNodes
-	 * @param aggregateVector	The aggregate vector being built as we find AggregateNodes
+     * @param aggregates        The aggregate list being built as we find AggregateNodes
 	 *
 	 * @return	The new top of the expression tree.
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ValueNode bindExpression(FromList fromList, SubqueryList subqueryList,
-					List aggregateVector)
+    @Override
+    ValueNode bindExpression(FromList fromList, SubqueryList subqueryList, List<AggregateNode> aggregates)
 				throws StandardException
 	{
 		ResultColumnList	resultColumns;
@@ -549,7 +543,7 @@ public class SubqueryNode extends ValueNode
 		if (leftOperand != null)
 		{
 			leftOperand = leftOperand.bindExpression(fromList, subqueryList,
-									   aggregateVector);
+                                       aggregates);
 		}
 
 		if (orderByList != null) {
@@ -581,8 +575,7 @@ public class SubqueryNode extends ValueNode
 		 */
 		if (leftOperand != null && leftOperand.requiresTypeFromContext())
 		{
-			leftOperand.setType(
-				((ResultColumn) resultColumns.elementAt(0)).getTypeServices());
+            leftOperand.setType(resultColumns.elementAt(0).getTypeServices());
 		}
 
 		// Set the DataTypeServices
@@ -610,7 +603,8 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ValueNode preprocess(int numTables,
+    @Override
+    ValueNode preprocess(int numTables,
 								FromList outerFromList,
 								SubqueryList outerSubqueryList,
 								PredicateList outerPredicateList) 
@@ -628,7 +622,25 @@ public class SubqueryNode extends ValueNode
 		boolean		flattenable;
 		ValueNode	topNode = this;
 
-		resultSet = resultSet.preprocess(numTables, null, (FromList) null);
+        final boolean haveOrderBy; // need to remember for flattening decision
+
+        // Push the order by list down to the ResultSet
+        if (orderByList != null) {
+            haveOrderBy = true;
+            // If we have more than 1 ORDERBY columns, we may be able to
+            // remove duplicate columns, e.g., "ORDER BY 1, 1, 2".
+            if (orderByList.size() > 1)
+            {
+                orderByList.removeDupColumns();
+            }
+
+            resultSet.pushOrderByList(orderByList);
+            orderByList = null;
+        } else {
+            haveOrderBy = false;
+        }
+
+        resultSet = resultSet.preprocess(numTables, null, (FromList) null);
 
         if (leftOperand != null)
         {
@@ -690,7 +702,7 @@ public class SubqueryNode extends ValueNode
 		 */
 		flattenable = (resultSet instanceof RowResultSetNode) &&
 					  underTopAndNode && !havingSubquery &&
-                      orderByList == null &&
+                      !haveOrderBy &&
                       offset == null &&
                       fetchFirst == null &&
 					  !isWhereExistsAnyInWithWhereSubquery() &&
@@ -706,7 +718,7 @@ public class SubqueryNode extends ValueNode
 			leftOperand = parentComparisonOperator.getLeftOperand();
 			// Flatten the subquery
 			RowResultSetNode rrsn = (RowResultSetNode) resultSet;
-			FromList   fl = new FromList();
+            FromList fl = new FromList(getContextManager());
 
 			// Remove ourselves from the outer subquery list
 			outerSubqueryList.removeElement(this);
@@ -762,7 +774,7 @@ public class SubqueryNode extends ValueNode
 
 		flattenable = (resultSet instanceof SelectNode) &&
  			          !((SelectNode)resultSet).hasWindows() &&
-                      orderByList == null &&
+                      !haveOrderBy &&
                       offset == null &&
                       fetchFirst == null &&
 					  underTopAndNode && !havingSubquery &&
@@ -859,19 +871,7 @@ public class SubqueryNode extends ValueNode
 			}
 		}
 
-		// Push the order by list down to the ResultSet
-		if (orderByList != null) {
-			// If we have more than 1 ORDERBY columns, we may be able to
-			// remove duplicate columns, e.g., "ORDER BY 1, 1, 2".
-			if (orderByList.size() > 1)
-			{
-				orderByList.removeDupColumns();
-			}
-
-			resultSet.pushOrderByList(orderByList);
-			orderByList = null;
-		}
-
+        resultSet.pushQueryExpressionSuffix();
 
         resultSet.pushOffsetFetchFirst( offset, fetchFirst, hasJDBClimitClause );
 
@@ -1115,10 +1115,7 @@ public class SubqueryNode extends ValueNode
 		 */
 		if (leftOperand == null)
 		{
-			return (ValueNode) getNodeFactory().getNode(
-											C_NodeTypes.BOOLEAN_CONSTANT_NODE,
-											Boolean.TRUE,
-											getContextManager());
+           return new BooleanConstantNode(true, getContextManager());
 		}
 		else
 		{
@@ -1206,8 +1203,7 @@ public class SubqueryNode extends ValueNode
      * @return the right operand
      */
     private ValueNode getRightOperand() {
-        ResultColumn firstRC =
-                (ResultColumn) resultSet.getResultColumns().elementAt(0);
+        ResultColumn firstRC = resultSet.getResultColumns().elementAt(0);
         return firstRC.getExpression();
     }
 
@@ -1243,7 +1239,7 @@ public class SubqueryNode extends ValueNode
 	 *	references.
 	 * @exception StandardException		Thrown on error
 	 */
-	public boolean hasCorrelatedCRs() throws StandardException
+    boolean hasCorrelatedCRs() throws StandardException
 	{
 		if (doneCorrelationCheck)
 		{
@@ -1274,7 +1270,8 @@ public class SubqueryNode extends ValueNode
 			 */
 			if (oldRCL.size() > 1)
 			{
-				ResultColumnList newRCL = new ResultColumnList();
+                ResultColumnList
+                        newRCL = new ResultColumnList(getContextManager());
 				newRCL.addResultColumn(oldRCL.getResultColumn(1));
 				realSubquery.setResultColumns(newRCL);
 			}
@@ -1294,7 +1291,7 @@ public class SubqueryNode extends ValueNode
 				
 	/**
 	 * Transform:
-	 *		expresion QuantifiedOperator (select x from ...)
+     *      expression QuantifiedOperator (select x from ...)
 	 * into
 	 *		(select true from .. where expression <BinaryComparisonOperator> x ...)
 	 *		IS [NOT] NULL
@@ -1318,7 +1315,7 @@ public class SubqueryNode extends ValueNode
 	 *		o  The operator in the new predicate that is added to the subquery
 	 *		   will be a BinaryAllOperatorNode whose bcoNodeType corresponds to 
 	 *		   the negation of the operator that modifies the ALL.
-	 *		   (eg, <> for = ALL, with <> for NOT IN.)
+	 *		   (eg, &lt;&gt; for = ALL, with &lt;&gt; for NOT IN.)
 	 *
 	 * NOTE: This method is called after the underlying subquery has been
 	 * preprocessed, so we build a new Predicate, not just a new expression.
@@ -1335,14 +1332,13 @@ public class SubqueryNode extends ValueNode
 			throws StandardException
 	{
 		AndNode						andNode;
-		BinaryComparisonOperatorNode bcoNode = null;
 		JBitSet						tableMap;
 		Predicate					predicate;
 		ResultColumn				firstRC;
 		ResultColumnList			resultColumns;
 		UnaryComparisonOperatorNode	ucoNode = null;
-		ValueNode					oldWhereClause;
 		ValueNode					rightOperand;
+        ContextManager              cm = getContextManager();
 
 		/* We have to ensure that the resultSet immediately under us has
 		 * a PredicateList, otherwise we can't push the predicate down.
@@ -1361,8 +1357,7 @@ public class SubqueryNode extends ValueNode
 		*/
 		ResultColumnList newRCL = resultColumns.copyListAndObjects();
 		newRCL.genVirtualColumnNodes(resultSet, resultColumns);
-		resultSet = (ResultSetNode) getNodeFactory().getNode(
-										C_NodeTypes.PROJECT_RESTRICT_NODE,
+        resultSet = new ProjectRestrictNode(
 										resultSet,	// child
 										newRCL,			// result columns
 										null,			// restriction
@@ -1370,13 +1365,14 @@ public class SubqueryNode extends ValueNode
 										null,			// project subqueries
 										null,			// restrict subqueries	
 										null,
-										getContextManager());
+                                        cm);
 		resultColumns = newRCL;
 	
-		firstRC = (ResultColumn) resultColumns.elementAt(0);
+        firstRC = resultColumns.elementAt(0);
 		rightOperand = firstRC.getExpression();
 
-		bcoNode = getNewJoinCondition(leftOperand, rightOperand);
+        BinaryComparisonOperatorNode bcoNode =
+                getNewJoinCondition(leftOperand, rightOperand);
 
 		ValueNode andLeft = bcoNode;
 
@@ -1397,47 +1393,33 @@ public class SubqueryNode extends ValueNode
 			{
 				/* Create a normalized structure.
 				 */
-				BooleanConstantNode falseNode = (BooleanConstantNode) getNodeFactory().getNode(
-												C_NodeTypes.BOOLEAN_CONSTANT_NODE,
-												Boolean.FALSE,
-												getContextManager());
-				OrNode newOr = (OrNode) getNodeFactory().getNode(
-												C_NodeTypes.OR_NODE,
-												bcoNode,
-												falseNode,
-												getContextManager());
+                BooleanConstantNode
+                        falseNode = new BooleanConstantNode(false, cm);
+                OrNode newOr =
+                        new OrNode(bcoNode, falseNode, cm);
+
 				newOr.postBindFixup();
 				andLeft = newOr;
 
 				if (leftNullable)
 				{
-					UnaryComparisonOperatorNode leftIsNull = (UnaryComparisonOperatorNode)
-									getNodeFactory().getNode(
-														C_NodeTypes.IS_NULL_NODE,
-														leftOperand,
-														getContextManager());
+                    UnaryComparisonOperatorNode leftIsNull = new IsNullNode(
+                           leftOperand,
+                           false,
+                           cm);
 					leftIsNull.bindComparisonOperator();
-					newOr = (OrNode) getNodeFactory().getNode(
-													C_NodeTypes.OR_NODE,
-													leftIsNull,
-													andLeft,
-													getContextManager());
+                    newOr = new OrNode(leftIsNull, andLeft, cm);
 					newOr.postBindFixup();
 					andLeft = newOr;
 				}
 				if (rightNullable)
 				{
-					UnaryComparisonOperatorNode rightIsNull = (UnaryComparisonOperatorNode)
-									getNodeFactory().getNode(
-														C_NodeTypes.IS_NULL_NODE,
-														rightOperand,
-														getContextManager());
+                    UnaryComparisonOperatorNode rightIsNull = new IsNullNode(
+                           rightOperand,
+                           false,
+                           cm);
 					rightIsNull.bindComparisonOperator();
-					newOr = (OrNode) getNodeFactory().getNode(
-													C_NodeTypes.OR_NODE,
-													rightIsNull,
-													andLeft,
-													getContextManager());
+                    newOr = new OrNode(rightIsNull, andLeft, cm);
 					newOr.postBindFixup();
 					andLeft = newOr;
 				}
@@ -1445,22 +1427,14 @@ public class SubqueryNode extends ValueNode
 		}
 
 		/* Place an AndNode above the <BinaryComparisonOperator> */
-		andNode = (AndNode) getNodeFactory().getNode(
-													C_NodeTypes.AND_NODE,
-													andLeft,
-													getTrueNode(),
-													getContextManager());
+        andNode = new AndNode(andLeft, getTrueNode(), cm);
 
 		/* Build the referenced table map for the new predicate */
 		tableMap = new JBitSet(numTables);
 		andNode.postBindFixup();
 
 		/* Put the AndNode under a Predicate */
-		predicate = (Predicate) getNodeFactory().getNode(
-										C_NodeTypes.PREDICATE,
-										andNode,
-										tableMap,
-										getContextManager());
+        predicate = new Predicate(andNode, tableMap, cm);
 		predicate.categorize();
 
 		/* Push the new Predicate to the subquery's list */
@@ -1481,11 +1455,7 @@ public class SubqueryNode extends ValueNode
 			case LT_ANY_SUBQUERY:
 			case GE_ANY_SUBQUERY:
 			case GT_ANY_SUBQUERY:
-				ucoNode = (UnaryComparisonOperatorNode) 
-									getNodeFactory().getNode(
-												C_NodeTypes.IS_NOT_NULL_NODE,
-												this,
-												getContextManager());
+                ucoNode = new IsNullNode(this, true, cm);
 				break;
 
 			case NOT_IN_SUBQUERY:
@@ -1495,12 +1465,12 @@ public class SubqueryNode extends ValueNode
 			case LT_ALL_SUBQUERY:
 			case GE_ALL_SUBQUERY:
 			case GT_ALL_SUBQUERY:
-				ucoNode = (UnaryComparisonOperatorNode) 
-									getNodeFactory().getNode(
-													C_NodeTypes.IS_NULL_NODE,
-													this,
-													getContextManager());
+                ucoNode = new IsNullNode(this, false, cm);
 				break;
+            default:
+                if (SanityManager.DEBUG) {
+                    SanityManager.NOTREACHED();
+                }
 		}
 		ucoNode.bindComparisonOperator();
 		return ucoNode;
@@ -1521,8 +1491,6 @@ public class SubqueryNode extends ValueNode
 				ValueNode rightOperand)
 		throws StandardException
 	{
-		BinaryComparisonOperatorNode bcoNode = null;
-
 		/* NOTE: If we are an expression subquery that's getting
 		 * flattened then our subqueryType is EXPRESSION_SUBQUERY.
 		 * However, we can get the comparison type from the 
@@ -1573,7 +1541,7 @@ public class SubqueryNode extends ValueNode
 		}
 
 		int bcoType = 0;
-		int nodeType = 0;
+        int kind = -1;
 
 		/* Build the <BinaryComparisonOperator> */
 		switch (operatorType)
@@ -1582,32 +1550,32 @@ public class SubqueryNode extends ValueNode
 			case EQ_ANY_SUBQUERY:
 			case NOT_IN_SUBQUERY:
 			case NE_ALL_SUBQUERY:
-				nodeType = C_NodeTypes.BINARY_EQUALS_OPERATOR_NODE;
+                kind = BinaryRelationalOperatorNode.K_EQUALS;
 				break;
 
 			case NE_ANY_SUBQUERY:
 			case EQ_ALL_SUBQUERY:
-				nodeType = C_NodeTypes.BINARY_NOT_EQUALS_OPERATOR_NODE;
+                kind = BinaryRelationalOperatorNode.K_NOT_EQUALS;
 				break;
 
 			case LE_ANY_SUBQUERY:
 			case GT_ALL_SUBQUERY:
-				nodeType = C_NodeTypes.BINARY_LESS_EQUALS_OPERATOR_NODE;
+                kind = BinaryRelationalOperatorNode.K_LESS_EQUALS;
 				break;
 
 			case LT_ANY_SUBQUERY:
 			case GE_ALL_SUBQUERY:
-				nodeType = C_NodeTypes.BINARY_LESS_THAN_OPERATOR_NODE;
+                kind = BinaryRelationalOperatorNode.K_LESS_THAN;
 				break;
 
 			case GE_ANY_SUBQUERY:
 			case LT_ALL_SUBQUERY:
-				nodeType = C_NodeTypes.BINARY_GREATER_EQUALS_OPERATOR_NODE;
+                kind = BinaryRelationalOperatorNode.K_GREATER_EQUALS;
 				break;
 
 			case GT_ANY_SUBQUERY:
 			case LE_ALL_SUBQUERY:
-				nodeType = C_NodeTypes.BINARY_GREATER_THAN_OPERATOR_NODE;
+                kind = BinaryRelationalOperatorNode.K_GREATER_THAN;
 				break;
 
 			default:
@@ -1616,13 +1584,12 @@ public class SubqueryNode extends ValueNode
 					"subqueryType (" + subqueryType + ") is an unexpected type");
 		}
 
-		bcoNode =  (BinaryComparisonOperatorNode) 
-						getNodeFactory().getNode(
-							nodeType,
-							leftOperand,
-							rightOperand,
-							Boolean.FALSE,
-							getContextManager());
+        BinaryComparisonOperatorNode bcoNode = new BinaryRelationalOperatorNode(
+                kind,
+                leftOperand,
+                rightOperand,
+                false,
+                getContextManager());
 
 		bcoNode.bindComparisonOperator();
 		return bcoNode;
@@ -1645,6 +1612,7 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	ValueNode eliminateNots(boolean underNotNode) 
 					throws StandardException
 	{
@@ -1760,7 +1728,8 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ValueNode changeToCNF(boolean underTopAndNode) 
+    @Override
+    ValueNode changeToCNF(boolean underTopAndNode)
 					throws StandardException
 	{
 		/* Remember whether or not we are immediately under a top leve
@@ -1795,7 +1764,8 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public boolean categorize(JBitSet referencedTabs, boolean simplePredsOnly)
+    @Override
+    boolean categorize(JBitSet referencedTabs, boolean simplePredsOnly)
 		throws StandardException
 	{
 		/* We stop here when only considering simple predicates
@@ -1827,7 +1797,7 @@ public class SubqueryNode extends ValueNode
 	** it is an expression subquery that
 	** has no correlations and is invariant.
 	*/
-	boolean isMaterializable() throws StandardException
+    public boolean isMaterializable() throws StandardException
 	{
 		boolean retval = (subqueryType == EXPRESSION_SUBQUERY) && 
 						  !hasCorrelatedCRs() && 
@@ -1859,7 +1829,7 @@ public class SubqueryNode extends ValueNode
 	 * @exception StandardException		Thrown on error
 	 */
 
-	public void optimize(DataDictionary dataDictionary, double outerRows) 
+    void optimize(DataDictionary dataDictionary, double outerRows)
 					throws StandardException
 	{
 		/* RESOLVE - is there anything else that we need to do for this
@@ -1875,7 +1845,7 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public void modifyAccessPaths() throws StandardException
+    void modifyAccessPaths() throws StandardException
 	{
 		resultSet = resultSet.modifyAccessPaths();
 	}
@@ -1894,6 +1864,7 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
+    @Override
 	protected int getOrderableVariantType() throws StandardException
 	{
 		/* 
@@ -1930,10 +1901,9 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public void generateExpression(
-									ExpressionClassBuilder expressionBuilder,
-									MethodBuilder mbex)
+    @Override
+    void generateExpression(
+            ExpressionClassBuilder expressionBuilder, MethodBuilder mbex)
 								throws StandardException
 	{
 		CompilerContext	cc = getCompilerContext();
@@ -2008,10 +1978,11 @@ public class SubqueryNode extends ValueNode
 				mb.getField(subRS);
 				mb.conditionalIfNull();
 
-				ResultSetNode materialSubNode = new MaterializeSubqueryNode(subRS);
+                ResultSetNode materialSubNode =
+                        new MaterializeSubqueryNode(subRS, getContextManager());
 
 				// Propagate the resultSet's cost estimate to the new node.
-				materialSubNode.costEstimate = resultSet.getFinalCostEstimate();
+				materialSubNode.setCostEstimate( resultSet.getFinalCostEstimate() );
 
 				((ProjectRestrictNode) resultSet).setChildResult(materialSubNode);
 
@@ -2109,8 +2080,8 @@ public class SubqueryNode extends ValueNode
 		mb.push(subqResultSetNumber);
 		mb.push(subqueryNumber);
 		mb.push(pointOfAttachment);
-		mb.push(costEstimate.rowCount());
-		mb.push(costEstimate.getEstimatedCost());
+		mb.push( costEstimate.rowCount() );
+		mb.push( costEstimate.getEstimatedCost() );
 
 		mb.callMethod(VMOpcode.INVOKEINTERFACE, (String) null, resultSetString, ClassName.NoPutResultSet, nargs);
 
@@ -2274,10 +2245,7 @@ public class SubqueryNode extends ValueNode
 	{
 		if (trueNode == null)
 		{
-			trueNode = (BooleanConstantNode) getNodeFactory().getNode(
-											C_NodeTypes.BOOLEAN_CONSTANT_NODE,
-											Boolean.TRUE,
-											getContextManager());
+           trueNode = new BooleanConstantNode(true, getContextManager());
 		}
 		return trueNode;
 	}
@@ -2289,6 +2257,7 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @exception StandardException on error
 	 */
+    @Override
 	void acceptChildren(Visitor v)
 		throws StandardException
 	{
@@ -2376,71 +2345,48 @@ public class SubqueryNode extends ValueNode
 	private void changeToCorrespondingExpressionType()
 		throws StandardException
 	{
-  		BinaryOperatorNode bcon = null;
+        int nodeType = -1;
 
-  		switch (subqueryType)
+        switch (subqueryType)
   		{
   			case EQ_ANY_SUBQUERY:
   			case IN_SUBQUERY:
-  				bcon = (BinaryOperatorNode) getNodeFactory().getNode(
-  									C_NodeTypes.BINARY_EQUALS_OPERATOR_NODE,
-  									leftOperand,
-  									this,
-  									Boolean.FALSE,
-  									getContextManager());
+                nodeType = BinaryRelationalOperatorNode.K_EQUALS;
   				break;
 
   			case NE_ANY_SUBQUERY:
-  				bcon = (BinaryOperatorNode) getNodeFactory().getNode(
-  								C_NodeTypes.BINARY_NOT_EQUALS_OPERATOR_NODE,
-  								leftOperand,
-  								this,
-  								Boolean.FALSE,
-  								getContextManager());
+                nodeType = BinaryRelationalOperatorNode.K_NOT_EQUALS;
   				break;
 
   			case LE_ANY_SUBQUERY:
-  				bcon = (BinaryOperatorNode) getNodeFactory().getNode(
-  								C_NodeTypes.BINARY_LESS_EQUALS_OPERATOR_NODE,
-  								leftOperand,
-  								this,
-  								Boolean.FALSE,
-  								getContextManager());
+                nodeType = BinaryRelationalOperatorNode.K_LESS_EQUALS;
   				break;
 
   			case LT_ANY_SUBQUERY:
-  				bcon = (BinaryOperatorNode) getNodeFactory().getNode(
-  							C_NodeTypes.BINARY_LESS_THAN_OPERATOR_NODE,
-  							leftOperand,
-  							this,
-  							Boolean.FALSE,
-  							getContextManager());
+                nodeType = BinaryRelationalOperatorNode.K_LESS_THAN;
   				break;
 
   			case GE_ANY_SUBQUERY:
-  				bcon = (BinaryOperatorNode) getNodeFactory().getNode(
-  							C_NodeTypes.BINARY_GREATER_EQUALS_OPERATOR_NODE,
-  							leftOperand,
-  							this,
-  							Boolean.FALSE,
-  							getContextManager());
+                nodeType = BinaryRelationalOperatorNode.K_GREATER_EQUALS;
   				break;
 
   			case GT_ANY_SUBQUERY:
-  				bcon = (BinaryOperatorNode) getNodeFactory().getNode(
-  								C_NodeTypes.BINARY_GREATER_THAN_OPERATOR_NODE,
-  								leftOperand,
-  								this,
-  								Boolean.FALSE,
-  								getContextManager());
+                nodeType = BinaryRelationalOperatorNode.K_GREATER_THAN;
   				break;
+            default:
+                if (SanityManager.DEBUG) {
+                    SanityManager.NOTREACHED();
+                }
   		}
 
+        BinaryRelationalOperatorNode bcon = new BinaryRelationalOperatorNode(
+                nodeType, leftOperand, this, false, getContextManager());
+
   		// clean up the state of the tree to reflect a bound expression subquery
   		subqueryType = EXPRESSION_SUBQUERY;
   		setDataTypeServices(resultSet.getResultColumns());
 
-  		parentComparisonOperator = (BinaryComparisonOperatorNode) bcon;
+        parentComparisonOperator = bcon;
   		/* Set type info for the operator node */
   		parentComparisonOperator.bindComparisonOperator();
   		leftOperand = null;
@@ -2461,7 +2407,7 @@ public class SubqueryNode extends ValueNode
 		 */
 		if (subqueryType == EXPRESSION_SUBQUERY)
 		{
-			dts = ((ResultColumn) resultColumns.elementAt(0)).getTypeServices();
+            dts = resultColumns.elementAt(0).getTypeServices();
 		}
 		else
 		{
@@ -2474,7 +2420,7 @@ public class SubqueryNode extends ValueNode
     /**
      * {@inheritDoc}
      */    
-    protected boolean isEquivalent(ValueNode o)
+    boolean isEquivalent(ValueNode o)
     {
     	return false;
     }
@@ -2502,7 +2448,7 @@ public class SubqueryNode extends ValueNode
 	 *
 	 * @return true if it is part of a where clause, otherwise false
 	 */
-	public boolean isWhereSubquery() {
+    boolean isWhereSubquery() {
 		return whereSubquery;
 	}
 
@@ -2510,7 +2456,7 @@ public class SubqueryNode extends ValueNode
 	 * Mark this subquery as being part of a where clause.
 	 * @param whereSubquery
 	 */
-	public void setWhereSubquery(boolean whereSubquery) {
+    void setWhereSubquery(boolean whereSubquery) {
 		this.whereSubquery = whereSubquery;
 	}
 
@@ -2526,7 +2472,7 @@ public class SubqueryNode extends ValueNode
 	 * @return true if this subquery is a WHERE EXISTS | ANY | IN subquery with 
 	 *              a subquery in its own WHERE clause
 	 */
-	public boolean isWhereExistsAnyInWithWhereSubquery() 
+    boolean isWhereExistsAnyInWithWhereSubquery()
 			throws StandardException
 	{
 		if ( isWhereSubquery() && (isEXISTS() || isANY() || isIN()) ) {
