@@ -21,42 +21,53 @@
 
 package	org.apache.derby.impl.sql.compile;
 
-import org.apache.derby.iapi.services.compiler.MethodBuilder;
-import org.apache.derby.iapi.services.compiler.LocalField;
-import org.apache.derby.iapi.services.io.StoredFormatIds;
-import org.apache.derby.iapi.services.sanity.SanityManager;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
-import org.apache.derby.iapi.sql.compile.Visitor;
-import org.apache.derby.iapi.error.StandardException;
-
-import org.apache.derby.iapi.sql.compile.TypeCompiler;
-import org.apache.derby.iapi.types.TypeId;
-import org.apache.derby.iapi.types.DataTypeDescriptor;
-
-import org.apache.derby.iapi.reference.SQLState;
-import org.apache.derby.iapi.reference.ClassName;
-import org.apache.derby.iapi.services.classfile.VMOpcode;
-
-import org.apache.derby.iapi.util.JBitSet;
-import org.apache.derby.iapi.util.ReuseFactory;
-
 import java.lang.reflect.Modifier;
-
 import java.sql.Types;
 import java.util.List;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.reference.ClassName;
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.classfile.VMOpcode;
+import org.apache.derby.iapi.services.compiler.LocalField;
+import org.apache.derby.iapi.services.compiler.MethodBuilder;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.services.io.StoredFormatIds;
+import org.apache.derby.shared.common.sanity.SanityManager;
+import org.apache.derby.iapi.sql.compile.TypeCompiler;
+import org.apache.derby.iapi.sql.compile.Visitor;
+import org.apache.derby.iapi.types.DataTypeDescriptor;
+import org.apache.derby.iapi.types.TypeId;
+import org.apache.derby.iapi.util.JBitSet;
+
 /**
  * A TernaryOperatorNode represents a built-in ternary operators.
- * This covers  built-in functions like substr().
+ * This covers  built-in functions like {@code substr()}.
  * Java operators are not represented here: the JSQL language allows Java
  * methods to be called from expressions, but not Java operators.
  *
  */
 
-public class TernaryOperatorNode extends OperatorNode
+class TernaryOperatorNode extends OperatorNode
 {
+
+    // Allowed kinds
+    final static int K_TRIM = 0;
+    final static int K_LOCATE = 1;
+    final static int K_SUBSTRING = 2;
+    final static int K_LIKE = 3;
+    final static int K_TIMESTAMPADD = 4;
+    final static int K_TIMESTAMPDIFF = 5;
+
+    /**
+     * This class is used to hold logically different objects for
+     * space efficiency. {@code kind} represents the logical object
+     * type. See also {@link ValueNode#isSameNodeKind}.
+     */
+    final int kind;
+
+
 	String		operator;
 	String		methodName;
-	int			operatorType;
 	ValueNode	receiver; 
 
 	ValueNode	leftOperand;
@@ -68,12 +79,6 @@ public class TernaryOperatorNode extends OperatorNode
 	String		rightInterfaceType;
 	int			trimType;
 
-	public static final int TRIM = 0;
-	public static final int LOCATE = 1;
-	public static final int SUBSTRING = 2;
-	public static final int LIKE = 3;
-	public static final int TIMESTAMPADD = 4;
-	public static final int TIMESTAMPDIFF = 5;
 	static final String[] TernaryOperators = {"trim", "LOCATE", "substring", "like", "TIMESTAMPADD", "TIMESTAMPDIFF"};
 	static final String[] TernaryMethodNames = {"ansiTrim", "locate", "substring", "like", "timestampAdd", "timestampDiff"};
 	static final String[] TernaryResultType = {ClassName.StringDataValue, 
@@ -92,34 +97,70 @@ public class TernaryOperatorNode extends OperatorNode
 	};
 
 	/**
-	 * Initializer for a TernaryOperatorNode
+     * Constructor for a TernaryOperatorNode
 	 *
-	 * @param receiver		The receiver (eg, string being operated on in substr())
+     * @param receiver      The receiver (e.g., string being operated on in
+     *                      {@code substr()})
 	 * @param leftOperand	The left operand of the node
 	 * @param rightOperand	The right operand of the node
-	 * @param operatorType	The type of the operand
-	 */
+     * @param kind          The kind of the operand
+     * @param cm            The context manager
+     */
+    TernaryOperatorNode(
+                    ValueNode receiver,
+                    ValueNode leftOperand,
+                    ValueNode rightOperand,
+                    int kind,
+                    ContextManager cm)
+    {
+        super(cm);
+        this.kind = kind;
+        constructorMinion(
+                receiver, leftOperand, rightOperand, -1);
+    }
 
-	public void init(
-					Object receiver,
-					Object leftOperand,
-					Object rightOperand,
-					Object operatorType,
-					Object trimType)
-	{
-		this.receiver = (ValueNode) receiver;
-		this.leftOperand = (ValueNode) leftOperand;
-		this.rightOperand = (ValueNode) rightOperand;
-		this.operatorType = ((Integer) operatorType).intValue();
-		this.operator = (String) TernaryOperators[this.operatorType];
-		this.methodName = (String) TernaryMethodNames[this.operatorType];
-		this.resultInterfaceType = (String) TernaryResultType[this.operatorType];
-		this.receiverInterfaceType = (String) TernaryArgType[this.operatorType][0];
-		this.leftInterfaceType = (String) TernaryArgType[this.operatorType][1];
-		this.rightInterfaceType = (String) TernaryArgType[this.operatorType][2];
-		if (trimType != null)
-				this.trimType = ((Integer) trimType).intValue();
-	}
+    /**
+     * Constructor for a TernaryOperatorNode
+     *
+     * @param receiver      The receiver (e.g., string being operated on in
+     *                      {@code substr()})
+     * @param leftOperand   The left operand of the node
+     * @param rightOperand  The right operand of the node
+     * @param kind          The kind of the operand
+     * @param trimType      The trim type
+     * @param cm            The context manager
+     */
+    TernaryOperatorNode(ValueNode receiver,
+                        ValueNode leftOperand,
+                        ValueNode rightOperand,
+                        int kind,
+                        int trimType,
+                        ContextManager cm)
+    {
+        super(cm);
+        this.kind = kind;
+        constructorMinion(
+                receiver, leftOperand, rightOperand, trimType);
+    }
+
+    private void constructorMinion(ValueNode receiver,
+                                   ValueNode leftOperand,
+                                   ValueNode rightOperand,
+                                   int trimType) {
+        this.receiver = receiver;
+        this.leftOperand = leftOperand;
+        this.rightOperand = rightOperand;
+        this.operator = TernaryOperators[this.kind];
+        this.methodName = TernaryMethodNames[this.kind];
+        this.resultInterfaceType = TernaryResultType[this.kind];
+        this.receiverInterfaceType = TernaryArgType[this.kind][0];
+        this.leftInterfaceType = TernaryArgType[this.kind][1];
+        this.rightInterfaceType = TernaryArgType[this.kind][2];
+
+        if (trimType != -1) {
+            this.trimType = trimType;
+        }
+    }
 
 	/**
 	 * Convert this object to a String.  See comments in QueryTreeNode.java
@@ -128,6 +169,7 @@ public class TernaryOperatorNode extends OperatorNode
 	 * @return	This object as a String
 	 */
 
+    @Override
 	public String toString()
 	{
 		if (SanityManager.DEBUG)
@@ -152,8 +194,8 @@ public class TernaryOperatorNode extends OperatorNode
 	 *
 	 * @param depth		The depth of this node in the tree
 	 */
-
-	public void printSubNodes(int depth)
+    @Override
+    void printSubNodes(int depth)
 	{
 		if (SanityManager.DEBUG)
 		{
@@ -186,38 +228,39 @@ public class TernaryOperatorNode extends OperatorNode
 	 * @param fromList		The FROM list for the query this
 	 *				expression is in, for binding columns.
 	 * @param subqueryList		The subquery list being built as we find SubqueryNodes
-	 * @param aggregateVector	The aggregate vector being built as we find AggregateNodes
+     * @param aggregates        The aggregate list being built as we find AggregateNodes
 	 *
 	 * @return	The new top of the expression tree.
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public ValueNode bindExpression(FromList fromList, SubqueryList subqueryList,
-		List aggregateVector)
+    @Override
+    ValueNode bindExpression(FromList fromList, SubqueryList subqueryList, List<AggregateNode> aggregates)
 			throws StandardException
 	{
 		receiver = receiver.bindExpression(fromList, subqueryList, 
-			aggregateVector);
+            aggregates);
 
 		leftOperand = leftOperand.bindExpression(fromList, subqueryList,
-			    aggregateVector);
+                aggregates);
 
 		if (rightOperand != null)
 		{
 			rightOperand = rightOperand.bindExpression(fromList, subqueryList, 
-				aggregateVector);
+                aggregates);
 		}
-		if (operatorType == TRIM)
+
+        if (kind == K_TRIM) {
 			trimBind();
-		else if (operatorType == LOCATE)
+        } else if (kind == K_LOCATE) {
 			locateBind();
-		else if (operatorType == SUBSTRING)
+        } else if (kind == K_SUBSTRING) {
 			substrBind();
-		else if (operatorType == TIMESTAMPADD)
+        } else if (kind == K_TIMESTAMPADD) {
             timestampAddBind();
-		else if (operatorType == TIMESTAMPDIFF)
+        } else if (kind == K_TIMESTAMPDIFF) {
             timestampDiffBind();
+        }
 
 		return this;
 	}
@@ -237,7 +280,8 @@ public class TernaryOperatorNode extends OperatorNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ValueNode preprocess(int numTables,
+    @Override
+    ValueNode preprocess(int numTables,
 								FromList outerFromList,
 								SubqueryList outerSubqueryList,
 								PredicateList outerPredicateList) 
@@ -267,9 +311,8 @@ public class TernaryOperatorNode extends OperatorNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	public void generateExpression(ExpressionClassBuilder acb,
-											MethodBuilder mb)
+    @Override
+    void generateExpression(ExpressionClassBuilder acb, MethodBuilder mb)
 		throws StandardException
 	{
 		int nargs = 0;
@@ -279,7 +322,7 @@ public class TernaryOperatorNode extends OperatorNode
 		LocalField field = acb.newFieldDeclaration(Modifier.PRIVATE, resultInterfaceType);
 
 		receiver.generateExpression(acb, mb);
-		if (operatorType == TRIM)
+        if (kind == K_TRIM)
 		{
 			mb.push(trimType);
 			leftOperand.generateExpression(acb, mb);
@@ -289,7 +332,7 @@ public class TernaryOperatorNode extends OperatorNode
 			nargs = 3;
 			receiverType = receiverInterfaceType;
 		}
-		else if (operatorType == LOCATE)
+        else if (kind == K_LOCATE)
 		{
 			leftOperand.generateExpression(acb, mb); 
 			mb.upCast(leftInterfaceType);
@@ -299,7 +342,7 @@ public class TernaryOperatorNode extends OperatorNode
 			nargs = 3;
 		
 		}
-		else if (operatorType == SUBSTRING)
+        else if (kind == K_SUBSTRING)
 		{
 			leftOperand.generateExpression(acb, mb); 
 			mb.upCast(leftInterfaceType);
@@ -318,7 +361,7 @@ public class TernaryOperatorNode extends OperatorNode
 			nargs = 4;
 			receiverType = receiverInterfaceType;
 		}
-		else if (operatorType == TIMESTAMPADD || operatorType == TIMESTAMPDIFF)
+        else if (kind == K_TIMESTAMPADD || kind == K_TIMESTAMPDIFF)
         {
             Object intervalType = leftOperand.getConstantValueAsObject();
             if( SanityManager.DEBUG)
@@ -326,7 +369,7 @@ public class TernaryOperatorNode extends OperatorNode
                                       "Invalid interval type used for " + operator);
             mb.push( ((Integer) intervalType).intValue());
             rightOperand.generateExpression( acb, mb);
-            mb.upCast( TernaryArgType[ operatorType][2]);
+            mb.upCast(TernaryArgType[kind][2]);
             acb.getCurrentDateExpression( mb);
 			mb.getField(field);
 			nargs = 4;
@@ -347,7 +390,7 @@ public class TernaryOperatorNode extends OperatorNode
 	 *
 	 * @param newLeftOperand	The new leftOperand
 	 */
-	public void setLeftOperand(ValueNode newLeftOperand)
+    void setLeftOperand(ValueNode newLeftOperand)
 	{
 		leftOperand = newLeftOperand;
 	}
@@ -357,7 +400,7 @@ public class TernaryOperatorNode extends OperatorNode
 	 *
 	 * @return The current leftOperand.
 	 */
-	public ValueNode getLeftOperand()
+    ValueNode getLeftOperand()
 	{
 		return leftOperand;
 	}
@@ -367,7 +410,7 @@ public class TernaryOperatorNode extends OperatorNode
 	 *
 	 * @param newRightOperand	The new rightOperand
 	 */
-	public void setRightOperand(ValueNode newRightOperand)
+    void setRightOperand(ValueNode newRightOperand)
 	{
 		rightOperand = newRightOperand;
 	}
@@ -377,7 +420,7 @@ public class TernaryOperatorNode extends OperatorNode
 	 *
 	 * @return The current rightOperand.
 	 */
-	public ValueNode getRightOperand()
+    ValueNode getRightOperand()
 	{
 		return rightOperand;
 	}
@@ -407,7 +450,8 @@ public class TernaryOperatorNode extends OperatorNode
 	 *						or a VirtualColumnNode.
 	 * @exception StandardException			Thrown on error
 	 */
-	public boolean categorize(JBitSet referencedTabs, boolean simplePredsOnly)
+    @Override
+    boolean categorize(JBitSet referencedTabs, boolean simplePredsOnly)
 		throws StandardException
 	{
 		boolean pushable;
@@ -428,7 +472,8 @@ public class TernaryOperatorNode extends OperatorNode
 	 *
 	 * @exception StandardException			Thrown on error
 	 */
-	public ValueNode remapColumnReferencesToExpressions()
+    @Override
+    ValueNode remapColumnReferencesToExpressions()
 		throws StandardException
 	{
 		receiver = receiver.remapColumnReferencesToExpressions();
@@ -445,7 +490,8 @@ public class TernaryOperatorNode extends OperatorNode
 	 *
 	 * @return	Whether or not this expression tree represents a constant expression.
 	 */
-	public boolean isConstantExpression()
+    @Override
+    boolean isConstantExpression()
 	{
 		return (receiver.isConstantExpression() &&
 				leftOperand.isConstantExpression() &&
@@ -453,7 +499,8 @@ public class TernaryOperatorNode extends OperatorNode
 	}
 
 	/** @see ValueNode#constantExpression */
-	public boolean constantExpression(PredicateList whereClause)
+    @Override
+    boolean constantExpression(PredicateList whereClause)
 	{
 		return (receiver.constantExpression(whereClause) &&
 				leftOperand.constantExpression(whereClause) &&
@@ -468,6 +515,7 @@ public class TernaryOperatorNode extends OperatorNode
 	 *
 	 * @exception StandardException on error
 	 */
+    @Override
 	void acceptChildren(Visitor v)
 		throws StandardException
 	{
@@ -606,7 +654,7 @@ public class TernaryOperatorNode extends OperatorNode
 	 * @exception StandardException		Thrown on error
 	 */
 
-	public ValueNode locateBind() throws StandardException
+    ValueNode locateBind() throws StandardException
 	{
 		TypeId	firstOperandType, secondOperandType, offsetType;
 
@@ -708,12 +756,7 @@ public class TernaryOperatorNode extends OperatorNode
 	                vnTC.getCastToCharWidth(
 		                    vn.getTypeServices()));
 
-			ValueNode newNode = (ValueNode)
-						getNodeFactory().getNode(
-							C_NodeTypes.CAST_NODE,
-							vn,
-							dtd,
-							getContextManager());
+            ValueNode newNode = new CastNode(vn, dtd, getContextManager());
             
             // DERBY-2910 - Match current schema collation for implicit cast as we do for
             // explicit casts per SQL Spec 6.12 (10)                                                
@@ -733,7 +776,7 @@ public class TernaryOperatorNode extends OperatorNode
 	 * @exception StandardException		Thrown on error
 	 */
 
- 	public ValueNode substrBind() 
+    ValueNode substrBind()
 			throws StandardException
 	{
 		TypeId	receiverType;
@@ -848,8 +891,7 @@ public class TernaryOperatorNode extends OperatorNode
             if( jdbcType != Types.TINYINT && jdbcType != Types.SMALLINT &&
                 jdbcType != Types.INTEGER && jdbcType != Types.BIGINT)
                 throw StandardException.newException(SQLState.LANG_INVALID_FUNCTION_ARG_TYPE,
-                                                     rightOperand.getTypeId().getSQLTypeName(),
-                                                     ReuseFactory.getInteger( 2),
+                                                     rightOperand.getTypeId().getSQLTypeName(), 2,
                                                      operator);
         }
         bindDateTimeArg( receiver, 3);
@@ -880,8 +922,7 @@ public class TernaryOperatorNode extends OperatorNode
         {
             if( ! arg.getTypeId().isDateTimeTimeStampTypeId())
                 throw StandardException.newException(SQLState.LANG_INVALID_FUNCTION_ARG_TYPE,
-                                                     arg.getTypeId().getSQLTypeName(),
-                                                     ReuseFactory.getInteger( argNumber),
+                                                     arg.getTypeId().getSQLTypeName(), argNumber,
                                                      operator);
         }
     } // end of bindDateTimeArg
@@ -910,7 +951,7 @@ public class TernaryOperatorNode extends OperatorNode
         return false;
     } // end of bindParameter
 
-	public ValueNode getReceiver()
+    ValueNode getReceiver()
 	{
 		return receiver;
 	}
@@ -960,11 +1001,16 @@ public class TernaryOperatorNode extends OperatorNode
 		return new DataTypeDescriptor(TypeId.getBuiltInTypeId(Types.VARCHAR), true);
 	}
         
-    protected boolean isEquivalent(ValueNode o) throws StandardException
+    boolean isSameNodeKind(ValueNode o) {
+        return super.isSameNodeKind(o) &&
+                ((TernaryOperatorNode)o).kind == this.kind;
+    }
+
+
+    boolean isEquivalent(ValueNode o) throws StandardException
     {
-    	if (isSameNodeType(o)) 
-	{
-		TernaryOperatorNode other = (TernaryOperatorNode)o;
+        if (isSameNodeKind(o)) {
+            TernaryOperatorNode other = (TernaryOperatorNode)o;
 		
 			/*
 			 * SUBSTR function can either have 2 or 3 arguments.  In the 
