@@ -22,13 +22,11 @@
 package	org.apache.derby.impl.sql.compile;
 
 import java.util.List;
-
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.compiler.MethodBuilder;
-import org.apache.derby.iapi.services.sanity.SanityManager;
-import org.apache.derby.iapi.sql.compile.C_NodeTypes;
-import org.apache.derby.iapi.sql.compile.NodeFactory;
+import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.compile.Optimizable;
 import org.apache.derby.iapi.sql.compile.TypeCompiler;
 import org.apache.derby.iapi.store.access.Qualifier;
@@ -61,8 +59,8 @@ public abstract class ValueNode extends QueryTreeNode
 	** Binding will replace all untyped ValueNodes with typed ValueNodes
 	** when it figures out what their types should be.
 	*/
-	public ValueNode()
-	{
+    ValueNode(ContextManager cm) {
+        super(cm);
 	}
     
     /**
@@ -74,13 +72,7 @@ public abstract class ValueNode extends QueryTreeNode
        throws StandardException
        
        {
-        setType(
-                new DataTypeDescriptor(
-                            (TypeId) typeId,
-                            isNullable,
-                            maximumWidth
-                        )
-                    );           
+        setType(new DataTypeDescriptor(typeId, isNullable, maximumWidth));
        }
 
     /**
@@ -94,7 +86,7 @@ public abstract class ValueNode extends QueryTreeNode
     {
         setType(
                 new DataTypeDescriptor(
-                            (TypeId) typeId,
+                            typeId,
                             precision,
                             scale,
                             isNullable,
@@ -103,67 +95,6 @@ public abstract class ValueNode extends QueryTreeNode
                     );   
     }
 
-	/**
-	 * Initializer for numeric types.
-	 * 
-	 *
-	 * @param typeId	The TypeID of this new node
-	 * @param precision	The precision of this new node
-	 * @param scale		The scale of this new node
-	 * @param isNullable	The nullability of this new node
-	 * @param maximumWidth	The maximum width of this new node
-	 *
-	 * @exception StandardException
-	 */
-
-	public void init(
-			Object typeId,
-			Object precision,
-			Object scale,
-			Object isNullable,
-			Object maximumWidth)
-		throws StandardException
-	{
-		setType(
-			new DataTypeDescriptor(
-						(TypeId) typeId,
-						((Integer) precision).intValue(),
-						((Integer) scale).intValue(),
-						((Boolean) isNullable).booleanValue(),
-						((Integer) maximumWidth).intValue()
-					)
-				);
-	}
-
-	/**
-	 * Initializer for non-numeric types.
-	 * 
-	 *
-	 * @param tcf		The factory to get the
-	 *					DataTypeServicesFactory from
-	 * @param typeId	The TypeID of this new node
-	 * @param isNullable	The nullability of this new node
-	 * @param maximumWidth	The maximum width of this new node
-	 *
-	 * @exception StandardException
-	 */
-
-	ValueNode(
-			Object tcf,
-			Object typeId,
-			Object isNullable,
-			Object maximumWidth)
-		throws StandardException
-	{
-		setType(new DataTypeDescriptor(
-						(TypeId) typeId,
-						((Boolean) isNullable).booleanValue(),
-						((Integer) maximumWidth).intValue()
-						)
-				);
-	}
-
-			
 	/**
 	 * Convert this object to a String.  See comments in QueryTreeNode.java
 	 * for how this should be done for tree printing.
@@ -171,6 +102,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 * @return	This object as a String
 	 */
 
+    @Override
 	public String toString()
 	{
 		if (SanityManager.DEBUG)
@@ -192,7 +124,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 * @return	The DataTypeServices from this ValueNode.  This
 	 *		may be null if the node isn't bound yet.
 	 */
-	public DataTypeDescriptor getTypeServices()
+    DataTypeDescriptor getTypeServices()
 	{
 		return dataTypeServices;
 	}
@@ -246,7 +178,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 * @return	The TypeId from this ValueNode.  This
 	 *		may be null if the node isn't bound yet.
 	 */
-	public TypeId getTypeId() throws StandardException
+    TypeId getTypeId() throws StandardException
 	{
         DataTypeDescriptor dtd = getTypeServices();
         if (dtd != null)
@@ -269,7 +201,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 * @return	This ValueNode's TypeCompiler
 	 *
 	 */
-	public final TypeCompiler getTypeCompiler() throws StandardException
+    final TypeCompiler getTypeCompiler() throws StandardException
 	{
 		return getTypeCompiler(getTypeId());
 	}
@@ -282,7 +214,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *				ValueNode
 	 */
 
-	public void setType(DataTypeDescriptor dataTypeServices) throws StandardException
+    void setType(DataTypeDescriptor dataTypeServices) throws StandardException
 	{
         // bind the type in case it is a user defined type. this will create a dependency on the udt.
         if ( dataTypeServices != null )
@@ -338,7 +270,7 @@ public abstract class ValueNode extends QueryTreeNode
      * is not sourced by a column.
 	 */
 
-	public ResultColumn getSourceResultColumn()
+    ResultColumn getSourceResultColumn()
 	{
 		return null;
 	}
@@ -365,11 +297,11 @@ public abstract class ValueNode extends QueryTreeNode
 	}
 
 	
-	public ValueNode bindExpression(
-            FromList fromList, SubqueryList subqueryList, List aggregateVector)
+    ValueNode bindExpression(
+            FromList fromList, SubqueryList subqueryList, List<AggregateNode> aggregates)
 		throws StandardException
 	{
-		return bindExpression(fromList, subqueryList, aggregateVector,false);
+        return bindExpression(fromList, subqueryList, aggregates, false);
 	}
 	
 
@@ -380,17 +312,17 @@ public abstract class ValueNode extends QueryTreeNode
 	 * @param fromList			The FROM list to use for binding
 	 * @param subqueryList		The SubqueryList we are building as we hit
 	 *							SubqueryNodes.
-	 * @param aggregateVector	The aggregate vector being built as we find AggregateNodes
+     * @param aggregates        The aggregate list being built as we find AggregateNodes
 	 *
 	 * @return	The new top of the expression tree.
 	 *
 	 * @exception StandardException	Thrown on error
 	 */
 
-	public ValueNode bindExpression(
-			FromList fromList, SubqueryList subqueryList,
-			List aggregateVector, boolean forQueryRewrite)
-				throws StandardException
+    ValueNode bindExpression(FromList fromList,
+                             SubqueryList subqueryList,
+                             List<AggregateNode> aggregates,
+                             boolean forQueryRewrite) throws StandardException
 	{
 		/* There are a bizillion classes which extend ValueNode.  Here is info
 		 * on some of the classes that bindExpression() should not be called on
@@ -410,7 +342,7 @@ public abstract class ValueNode extends QueryTreeNode
 	}
 
 	/**
-	 * Generate a SQL->Java->SQL conversion tree above the current node
+	 * Generate a SQL-&gt;Java-&gt;SQL conversion tree above the current node
 	 * and bind the new nodes individually.
 	 * This is useful when doing comparisons, built-in functions, etc. on
 	 * java types which have a direct mapping to system built-in types.
@@ -419,7 +351,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @exception StandardException	Thrown on error
 	 */
-	public ValueNode genSQLJavaSQLTree()
+    ValueNode genSQLJavaSQLTree()
 		throws StandardException
 	{
 		if (SanityManager.DEBUG)
@@ -430,15 +362,10 @@ public abstract class ValueNode extends QueryTreeNode
 				"genSQLJavaSQLTree() only expected to be called on user types");
 		}
 
-		JavaValueNode stjvn = (JavaValueNode) getNodeFactory().getNode(
-									C_NodeTypes.SQL_TO_JAVA_VALUE_NODE,
-									this,
-									getContextManager());
+        final ContextManager cm = getContextManager();
+        JavaValueNode stjvn = new SQLToJavaValueNode(this, cm);
 
-		ValueNode jtsvn = (ValueNode) getNodeFactory().getNode(
-									C_NodeTypes.JAVA_TO_SQL_VALUE_NODE,
-									stjvn,
-									getContextManager());
+        ValueNode jtsvn = new JavaToSQLValueNode(stjvn, cm);
 
         DataTypeDescriptor  resultType;
         if ( (getTypeServices() != null) && getTypeId().userType() ) { resultType = getTypeServices(); }
@@ -464,7 +391,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ValueNode preprocess(int numTables,
+    ValueNode preprocess(int numTables,
 								FromList outerFromList,
 								SubqueryList outerSubqueryList,
 								PredicateList outerPredicateList) 
@@ -533,25 +460,20 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ValueNode genEqualsFalseTree()
+    ValueNode genEqualsFalseTree()
 			throws StandardException
 	{
 		BinaryRelationalOperatorNode equalsNode;
 		BooleanConstantNode		 falseNode;
 		boolean 				nullableResult;
-		NodeFactory				nodeFactory = getNodeFactory();
 
-		falseNode = (BooleanConstantNode) nodeFactory.getNode(
-									C_NodeTypes.BOOLEAN_CONSTANT_NODE,
-									Boolean.FALSE,
-									getContextManager());
-		equalsNode = (BinaryRelationalOperatorNode)
-							nodeFactory.getNode(
-								C_NodeTypes.BINARY_EQUALS_OPERATOR_NODE,
-								this,
-								falseNode,
-								Boolean.FALSE,
-								getContextManager());
+        falseNode = new BooleanConstantNode(false, getContextManager());
+        equalsNode = new BinaryRelationalOperatorNode(
+                BinaryRelationalOperatorNode.K_EQUALS,
+                this,
+                falseNode,
+                false,
+                getContextManager());
 		nullableResult = getTypeServices().isNullable();
 		equalsNode.setType(new DataTypeDescriptor(
 									TypeId.BOOLEAN_ID,
@@ -574,11 +496,7 @@ public abstract class ValueNode extends QueryTreeNode
 	{
 		IsNullNode isNullNode;
 
-		isNullNode = (IsNullNode)
-							getNodeFactory().getNode(
-													notNull ? C_NodeTypes.IS_NOT_NULL_NODE : C_NodeTypes.IS_NULL_NODE,
-													this,
-													getContextManager());
+        isNullNode = new IsNullNode(this, notNull, getContextManager());
 		isNullNode.setType(new DataTypeDescriptor(
 									TypeId.BOOLEAN_ID,
 									false)
@@ -614,20 +532,12 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ValueNode putAndsOnTop() 
+    ValueNode putAndsOnTop()
 					throws StandardException
 	{
-		NodeFactory		nodeFactory = getNodeFactory();
-
-        QueryTreeNode trueNode = (QueryTreeNode) nodeFactory.getNode(
-										C_NodeTypes.BOOLEAN_CONSTANT_NODE,
-										Boolean.TRUE,
-										getContextManager());
-		AndNode andNode = (AndNode) nodeFactory.getNode(
-										C_NodeTypes.AND_NODE,
-										this,
-										trueNode,
-										getContextManager());
+        BooleanConstantNode trueNode =
+                new BooleanConstantNode(true, getContextManager());
+        AndNode andNode = new AndNode(this, trueNode, getContextManager());
 		andNode.postBindFixup();
 		return andNode;
 	}
@@ -638,7 +548,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @return		Boolean which reflects validity of the tree.
 	 */
-	public boolean verifyPutAndsOnTop()
+    boolean verifyPutAndsOnTop()
 	{
 		return true;
 	}
@@ -671,7 +581,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-	public ValueNode changeToCNF(boolean underTopAndNode) 
+    ValueNode changeToCNF(boolean underTopAndNode)
 					throws StandardException
 	{
 		return this;
@@ -686,7 +596,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @return		Boolean which reflects validity of the tree.
 	 */
-	public boolean verifyChangeToCNF()
+    boolean verifyChangeToCNF()
 	{
 		return true;
 	}
@@ -717,7 +627,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @exception StandardException			Thrown on error
 	 */
-	public boolean categorize(JBitSet referencedTabs, boolean simplePredsOnly)
+    boolean categorize(JBitSet referencedTabs, boolean simplePredsOnly)
 		throws StandardException
 	{
 		return true;
@@ -738,7 +648,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @return the default schema name for an expression -- null
 	 */
-	public String getSchemaName() throws StandardException
+    String getSchemaName() throws StandardException
 	{
 		return null;
 	}
@@ -758,7 +668,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @return the default table name for an expression -- null
 	 */
-	public String getTableName()
+    String getTableName()
 	{
 		return null;
 	}
@@ -777,7 +687,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @return the default column name for an expression -- null.
 	 */
-	public String getColumnName()
+    String getColumnName()
 	{
 		return null;
 	}
@@ -802,7 +712,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @return boolean	Whether or not this expression tree is cloneable.
 	 */
-	public boolean isCloneable()
+    boolean isCloneable()
 	{
 		return false;
 	}
@@ -814,7 +724,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @exception StandardException			Thrown on error
 	 */
-	public ValueNode getClone() throws StandardException
+    ValueNode getClone() throws StandardException
 	{
 		if (SanityManager.DEBUG)
 		{
@@ -831,7 +741,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 * @param oldVN		The ValueNode to copy from.
 	 *
 	 */
-	public void copyFields(ValueNode oldVN) throws StandardException
+    void copyFields(ValueNode oldVN) throws StandardException
 	{
 		dataTypeServices = oldVN.getTypeServices();
 	}
@@ -844,7 +754,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @exception StandardException			Thrown on error
 	 */
-	public ValueNode remapColumnReferencesToExpressions() throws StandardException
+    ValueNode remapColumnReferencesToExpressions() throws StandardException
 	{
 		return this;
 	}
@@ -854,7 +764,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @return	Whether or not this expression tree represents a constant expression.
 	 */
-	public boolean isConstantExpression()
+    boolean isConstantExpression()
 	{
 		return false;
 	}
@@ -867,7 +777,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @return	True means this expression tree represents a constant value.
 	 */
-	public boolean constantExpression(PredicateList whereClause)
+    boolean constantExpression(PredicateList whereClause)
 	{
 		return false;
 	}
@@ -973,32 +883,13 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @exception StandardException		Thrown on error
 	 */
-
-	protected final	void generate(ActivationClassBuilder acb,
-										MethodBuilder mb)
+    @Override
+    final void generate(ActivationClassBuilder acb, MethodBuilder mb)
 								throws StandardException
 	{
 		generateExpression( acb, mb );
 	}
 
-	/**
-	 * The only reason this routine exists is so that I don't have to change
-	 * the protection on generateExpression() and rototill all of QueryTree.
-	 *
-	 * @param ecb	The ExpressionClassBuilder for the class being built
-	 * @param mb	The method the expression will go into
-	 *
-	 *
-	 * @exception StandardException		Thrown on error
-	 */
-	public	void generateFilter(ExpressionClassBuilder ecb,
-										MethodBuilder mb)
-		throws StandardException
-	{
-		generateExpression( ecb, mb );
-	}
-	
-
 	/**
 	 * The default selectivity for value nodes is 50%.  This is overridden
 	 * in specific cases, such as the RelationalOperators.
@@ -1119,8 +1010,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 * @exception StandardException		Thrown on error
 	 */
 
-	public void generateExpression(ExpressionClassBuilder acb,
-											MethodBuilder mb)
+    void generateExpression(ExpressionClassBuilder acb, MethodBuilder mb)
 						throws StandardException
 	{
 		if (SanityManager.DEBUG)
@@ -1274,14 +1164,14 @@ public abstract class ValueNode extends QueryTreeNode
 
 	/**
 	 * Returns true if this ValueNode is a relational operator. Relational
-	 * Operators are <, <=, =, >, >=, <> as well as IS NULL and IS NOT
+	 * Operators are &lt;, &lt;=, =, &gt;, &gt;=, &lt;&gt; as well as IS NULL and IS NOT
 	 * NULL. This is the preferred way of figuring out if a ValueNode is
 	 * relational or not. 
 	 * @see RelationalOperator
 	 * @see BinaryRelationalOperatorNode
 	 * @see IsNullNode
 	*/
-	public boolean isRelationalOperator()
+    boolean isRelationalOperator()
 	{
 		return false;
 	}
@@ -1291,7 +1181,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @see ValueNode#isRelationalOperator
 	 */
-	public boolean isBinaryEqualsOperatorNode()
+    boolean isBinaryEqualsOperatorNode()
 	{
 		return false;
 	}
@@ -1304,7 +1194,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 * an equals operator of the form "col = ?" that we generated
 	 * during preprocessing to allow index multi-probing.
 	 */
-	public boolean isInListProbeNode()
+    boolean isInListProbeNode()
 	{
 		return false;
 	}
@@ -1328,7 +1218,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 * @param isNullOkay if set to true we also consider IS NULL predicates;
 	 * otherwise consider only = predicates.
 	 */
-	public boolean optimizableEqualityNode(Optimizable optTable, 
+    boolean optimizableEqualityNode(Optimizable optTable,
 										   int columnNumber, 
 										   boolean isNullOkay)
 		throws StandardException
@@ -1344,7 +1234,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 *
 	 * @return Whether this node's type will be determined from the context
 	 */
-	public boolean requiresTypeFromContext()
+    boolean requiresTypeFromContext()
 	{
 		return false;
 	}
@@ -1354,7 +1244,7 @@ public abstract class ValueNode extends QueryTreeNode
 	 * with Parameter Nodes.
 	 *
 	 */
-	public boolean isParameterNode()
+    boolean isParameterNode()
 	{
 		return false;
 	}
@@ -1408,23 +1298,25 @@ public abstract class ValueNode extends QueryTreeNode
 	 * 
 	 * @throws StandardException 
 	 */
-	protected abstract boolean isEquivalent(ValueNode other)
+    abstract boolean isEquivalent(ValueNode other)
 		throws StandardException;
 
-	/**
-	 * Tests if this node is of the same type as the specified node as
-	 * reported by {@link QueryTreeNode#getNodeType()}.
-	 * 
-	 * @param other the node to compare this value node against. 
-	 * 
-	 * @return <code>true</code> if the two nodes are of the same type.  
-	 */
-	protected final boolean isSameNodeType(ValueNode other)
-	{
-		if (other != null) {
-			return other.getNodeType() == getNodeType();
-		}
-		return false;
-	}
-	
+    /**
+     * Some node classes represent several logical node types (to reduce
+     * footprint), which we call <em>kinds</em>.
+     * This means that implementations of {@link #isEquivalent}
+     * cannot always just use {@code instanceof} to check if the other node
+     * represents the same kind. Hence this method needs to be
+     * overridden by all node classes that represent several kinds.
+     * This default implementation does not look at kinds.
+     * It is only called from implementations of {@code isEquivalent}.
+     *
+     * @param other The other value node whose kind we want to compare with.
+     * @return {@code true} if {@code this} and {@code o} represent the same
+     *         logical node type, i.e. kind.
+     */
+    boolean isSameNodeKind(ValueNode other) {
+        // Default implementation does not look at kinds.
+        return other != null && other.getClass().equals(this.getClass());
+    }
 }
