@@ -23,7 +23,7 @@ package org.apache.derby.impl.sql.conn;
 
 import org.apache.derby.iapi.services.context.Context;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.services.monitor.Monitor;
 
@@ -64,7 +64,7 @@ final class GenericStatementContext
 	private boolean		setSavePoint;
 	private String		internalSavePointName;
 	private ResultSet	topResultSet;
-	private ArrayList		dependencies;
+	private ArrayList<Dependency>		dependencies;
 	private NoPutResultSet[] subqueryTrackingArray;
 	private NoPutResultSet[] materializedSubqueries;
 	private	final LanguageConnectionContext lcc;
@@ -133,7 +133,7 @@ final class GenericStatementContext
      * returned to the application).
      *
      * When the StatementContext object is assigned with setInUse(),
-     * a CancelQueryTask is scheduled if a timeout > 0 has been set.
+     * a CancelQueryTask is scheduled if a timeout &gt; 0 has been set.
      */
     private static class CancelQueryTask
         extends
@@ -188,7 +188,7 @@ final class GenericStatementContext
     }
 
     private static TimerFactory getTimerFactory() {
-        return Monitor.getMonitor().getTimerFactory();
+        return GenericLanguageConnectionFactory.getMonitor().getTimerFactory();
     }
 
 	// StatementContext Interface
@@ -353,7 +353,7 @@ final class GenericStatementContext
 				}
 				for (int index = 0; index < subqueryTrackingArray.length; index++)
 				{
-					if (this.subqueryTrackingArray[index] != null)
+                    if (this.materializedSubqueries[index] != null)
 					{
 						subqueryTrackingArray[index] = this.materializedSubqueries[index];
 					}
@@ -463,7 +463,7 @@ final class GenericStatementContext
 		
 		if (dependencies == null)
 		{
-			dependencies = new ArrayList();
+			dependencies = new ArrayList<Dependency>();
 		}
 		dependencies.add(dy);
 	}
@@ -502,6 +502,7 @@ final class GenericStatementContext
 			}
 		}
 
+		try {
 		/*
 		** If it isn't a StandardException, then assume
 		** session severity.  It is probably an unexpected
@@ -563,9 +564,9 @@ final class GenericStatementContext
 		{
 			DependencyManager dmgr = lcc.getDataDictionary().getDependencyManager();
 
-			for (Iterator iterator = dependencies.iterator(); iterator.hasNext(); ) 
+			for (Iterator<Dependency> iterator = dependencies.iterator(); iterator.hasNext(); ) 
 			{
-				Dependency dy = (Dependency) iterator.next();
+				Dependency dy = iterator.next();
 				dmgr.clearInMemoryDependency(dy);
 			}
 
@@ -605,6 +606,13 @@ final class GenericStatementContext
 
 		/* Pop the context */
 		lcc.popStatementContext(this, error);
+		} catch(Exception ex) {
+			//DERBY-6722(GenericStatementContext.cleanupOnError()  
+			//needs protection from later errors during statement 
+			//cleanup
+			ex.initCause(error);
+			throw StandardException.unexpectedUserException(ex);
+		}
 	}
 
 	/**
