@@ -21,14 +21,22 @@
 
 package org.apache.derby.impl.sql.execute;
 
+import java.lang.reflect.Array;
+import java.lang.reflect.Field;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.sql.SQLWarning;
 import java.sql.Timestamp;
+import java.util.Arrays;
+import java.util.ArrayList;
+import java.util.Comparator;
+import org.w3c.dom.Element;
 
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.i18n.MessageService;
 import org.apache.derby.iapi.services.io.FormatableBitSet;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.Activation;
 import org.apache.derby.iapi.sql.ResultDescription;
 import org.apache.derby.iapi.sql.ResultSet;
@@ -1045,4 +1053,130 @@ implements NoPutResultSet
 		warnings = null;
 		return w;
 	}
+
+    public Element toXML( Element parentNode, String tag ) throws Exception
+    {
+        return childrenToXML( toXML( parentNode, tag, this ), this );
+    }
+
+    /**
+     * <p>
+     * Pretty-print a ResultSet as an xml child of a parent node.
+     * Return the node representing the ResultSet child.
+     * </p>
+     */
+    public  static  Element    toXML( Element parentNode, String childTag, ResultSet rs ) throws Exception
+    {
+        Element result = parentNode.getOwnerDocument().createElement( childTag );
+        result.setAttribute( "type", stripPackage( rs.getClass().getName() ) );
+
+        parentNode.appendChild( result );
+        return result;
+    }
+
+    /** Strip the package location from a class name */
+    private static  String  stripPackage( String className )
+    {
+        return className.substring( className.lastIndexOf( "." ) + 1 );
+    }
+
+    /**
+     * <p>
+     * Pretty-print the inner ResultSet fields inside an outer ResultSet.
+     * Returns the outerNode.
+     * </p>
+     */
+    public  static  Element    childrenToXML
+        ( Element outerNode, ResultSet outerRS ) throws Exception
+    {
+        ArrayList<Field>    fieldList = new ArrayList<Field>();
+        findResultSetFields( fieldList, outerRS.getClass() );
+
+        //
+        // Get a array of all the ResultSet fields inside the outerRS,
+        // ordered by field name.
+        //
+        Field[] fields = new Field[ fieldList.size() ];
+        fieldList.toArray( fields );
+        Arrays.sort( fields, new FieldComparator() );
+
+        // Add those fields as children of the outer element.
+        for ( Field field : fields )
+        {
+            Object  fieldContents = field.get( outerRS );
+
+            if ( fieldContents != null )
+            {
+                if ( field.getType().isArray() )
+                {
+                    Element arrayNode = outerNode.getOwnerDocument().createElement( "array" );
+                    arrayNode.setAttribute( "arrayName", field.getName() );
+                    String  typeName = stripPackage( field.getType().getComponentType().getName() ) + "[]";
+                    arrayNode.setAttribute( "type", typeName );
+                    outerNode.appendChild( arrayNode );
+
+                    int arrayLength = Array.getLength( fieldContents );
+                    for ( int i = 0; i < arrayLength; i++ )
+                    {
+                        ResultSet   cellRS = (ResultSet) Array.get( fieldContents, i );
+
+                        if ( cellRS != null )
+                        {
+                            Element cellNode = cellRS.toXML( arrayNode, "cell" );
+                            cellNode.setAttribute( "cellNumber", Integer.toString( i ) );
+                        }
+                    }
+                }
+                else
+                {
+                    ResultSet   innerRS = (ResultSet) fieldContents;
+
+                    innerRS.toXML( outerNode, field.getName() );
+                }
+            }   // end if fieldContents is not null
+        }   // end loop through fields
+
+        return outerNode;
+    }
+
+    /**
+     * <p>
+     * Find all fields of type ResultSet.
+     * </p>
+     */
+    private static  void    findResultSetFields( ArrayList<Field> fieldList, final Class<?> klass )
+        throws Exception
+    {
+        if ( klass == null ) { return; }
+        
+        Field[] fields = AccessController.doPrivileged
+            (
+             new PrivilegedAction<Field[]>()
+             {
+                 public Field[] run()
+                 {
+                     return klass.getDeclaredFields();
+                 }
+             }
+             );
+
+        for ( Field field : fields )
+        {
+            if ( ResultSet.class.isAssignableFrom( field.getType() ) ) { fieldList.add( field ); }
+            else if ( field.getType().isArray() )
+            {
+                if ( ResultSet.class.isAssignableFrom( field.getType().getComponentType() ) )
+                {
+                    fieldList.add( field );
+                }
+            }
+        }
+
+        findResultSetFields( fieldList, klass.getSuperclass() );
+    }
+
+    public  static  final   class   FieldComparator implements Comparator<Field>
+    {
+        public  int compare( Field left, Field right ) { return left.getName().compareTo( right.getName() ); }
+    }
 }
