@@ -30,7 +30,7 @@ import org.apache.derby.iapi.services.i18n.MessageService;
 import org.apache.derby.iapi.services.io.FormatableArrayHolder;
 import org.apache.derby.iapi.services.io.FormatableIntHolder;
 import org.apache.derby.iapi.services.loader.GeneratedMethod;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.Activation;
 import org.apache.derby.iapi.sql.execute.CursorResultSet;
 import org.apache.derby.iapi.sql.execute.ExecIndexRow;
@@ -82,9 +82,7 @@ public class HashScanResultSet extends ScanResultSet
 	private int initialCapacity;
 	private float loadFactor;
 	private int maxCapacity;
-	public String tableName;
 	public String userSuppliedOptimizerOverrides;
-	public String indexName;
 	public boolean forUpdate;
 	private boolean runTimeStatisticsOn;
 	public int[] keyColumns;
@@ -178,7 +176,8 @@ public class HashScanResultSet extends ScanResultSet
 		FormatableArrayHolder fah = (FormatableArrayHolder)
 										(activation.getPreparedStatement().
 											getSavedObject(hashKeyItem));
-		FormatableIntHolder[] fihArray = (FormatableIntHolder[]) fah.getArray(FormatableIntHolder.class);
+        FormatableIntHolder[] fihArray =
+                fah.getArray(FormatableIntHolder[].class);
 		keyColumns = new int[fihArray.length];
 		for (int index = 0; index < fihArray.length; index++)
 		{
@@ -188,6 +187,9 @@ public class HashScanResultSet extends ScanResultSet
 		runTimeStatisticsOn = 
             getLanguageConnectionContext().getRunTimeStatisticsMode();
 
+        // determine whether we should fetch row locations
+        setRowLocationsState();
+        
 		compactRow =
 				getCompactRow(candidate, accessedCols, false);
 		recordConstructorTime();
@@ -274,8 +276,8 @@ public class HashScanResultSet extends ScanResultSet
                     loadFactor,         // in memory Hashtable load factor
                     runTimeStatisticsOn,
 					skipNullKeyColumns,
-					keepAfterCommit);
-
+					keepAfterCommit,
+					fetchRowLocations);
 
 			if (runTimeStatisticsOn)
 			{
@@ -425,26 +427,23 @@ public class HashScanResultSet extends ScanResultSet
 					{
 						entryVector = (List) hashEntry;
 						entryVectorSize = entryVector.size();
-						columns = 
-                            (DataValueDescriptor[]) entryVector.get(0);
+						columns = unpackHashValue( entryVector.get( 0 ) );
 					}
 					else
 					{
 						entryVector = null;
 						entryVectorSize = 0;
-						columns = (DataValueDescriptor[]) hashEntry;
+						columns = unpackHashValue( hashEntry );
 					}
 				}
 				else if (numFetchedOnNext < entryVectorSize)
 				{
 					// We are walking a list and there are more rows left.
-					columns = (DataValueDescriptor[]) 
-                        entryVector.get(numFetchedOnNext);
+					columns = unpackHashValue( entryVector.get( numFetchedOnNext ) );
 				}
 
 				if (columns != null)
 				{
-
 					// See if the entry satisfies all of the other qualifiers
 
 					/* We've already "evaluated" the 1st keyColumns qualifiers 
