@@ -23,7 +23,7 @@ package org.apache.derby.impl.sql.execute;
 
 import org.apache.derby.iapi.services.loader.GeneratedMethod;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.services.io.Storable;
 
@@ -133,7 +133,7 @@ class HashTableResultSet extends NoPutResultSetImpl
         projection = p;
 		projectMapping = ((ReferencedColumnsDescriptorImpl) a.getPreparedStatement().getSavedObject(mapRefItem)).getReferencedColumnPositions();
 		FormatableArrayHolder fah = (FormatableArrayHolder) a.getPreparedStatement().getSavedObject(keyColItem);
-		FormatableIntHolder[] fihArray = (FormatableIntHolder[]) fah.getArray(FormatableIntHolder.class);
+        FormatableIntHolder[] fihArray = fah.getArray(FormatableIntHolder[].class);
 		keyColumns = new int[fihArray.length];
 		for (int index = 0; index < fihArray.length; index++)
 		{
@@ -370,7 +370,7 @@ class HashTableResultSet extends NoPutResultSetImpl
 						// Columns is really a Storable[]
 						for (int i = 0; i < columns.length; i++)
 						{
-							if (! (columns[0] instanceof Storable))
+                           if (! (columns[i] instanceof Storable))
 							{
 								SanityManager.THROWASSERT(
 								"columns[" + i + "] expected to be Storable, not " +
