@@ -26,7 +26,7 @@ import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.io.FormatableBitSet;
 import org.apache.derby.iapi.services.loader.GeneratedMethod;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.Activation;
 import org.apache.derby.iapi.sql.execute.CursorResultSet;
 import org.apache.derby.iapi.sql.execute.ExecRow;
@@ -79,6 +79,14 @@ class IndexRowToBaseRowResultSet extends NoPutResultSetImpl
 	/* Run time statistics variables */
 	public long restrictionTime;
 
+    //
+    // For managing row locations which are returned for the left join
+    // which drives the execution of the MERGE statement.
+    //
+    private int _baseColumnCount;
+    private boolean _includeRowLocation;
+	private FormatableBitSet _heapColsWithoutRowLocation;
+
     //
     // class interface
     //
@@ -97,21 +105,22 @@ class IndexRowToBaseRowResultSet extends NoPutResultSetImpl
 					GeneratedMethod restriction,
 					boolean forUpdate,
 					double optimizerEstimatedRowCount,
-					double optimizerEstimatedCost) 
+					double optimizerEstimatedCost,
+					int baseColumnCount) 
 		throws StandardException
 	{
 		super(a, resultSetNumber, optimizerEstimatedRowCount, optimizerEstimatedCost);
 		final GenericPreparedStatement gp =
 			(GenericPreparedStatement)a.getPreparedStatement();
-		final Object[] saved = gp.getSavedObjects();
 
-		scoci = (StaticCompiledOpenConglomInfo)saved[scociItem];
+        scoci = (StaticCompiledOpenConglomInfo) gp.getSavedObject(scociItem);
 		TransactionController tc = activation.getTransactionController();
 		dcoci = tc.getDynamicCompiledConglomInfo(conglomId);
         this.source = source;
 		this.indexName = indexName;
 		this.forUpdate = forUpdate;
 		this.restriction = restriction;
+        _baseColumnCount = baseColumnCount;
 
 		/* RESOLVE - once we push Qualifiers into the store we
 		 * need to clear their Orderable cache on each open/reopen.
@@ -120,20 +129,24 @@ class IndexRowToBaseRowResultSet extends NoPutResultSetImpl
 		// retrieve the valid column list from
 		// the saved objects, if it exists
 		if (heapColRefItem != -1) {
-			this.accessedHeapCols = (FormatableBitSet)saved[heapColRefItem];
+            this.accessedHeapCols =
+                    (FormatableBitSet) gp.getSavedObject(heapColRefItem);
 		}
 		if (allColRefItem != -1) {
-			this.accessedAllCols = (FormatableBitSet)saved[allColRefItem];
+            this.accessedAllCols =
+                    (FormatableBitSet) gp.getSavedObject(allColRefItem);
 		}
 
 		// retrieve the array of columns coming from the index
 		indexCols = 
 			((ReferencedColumnsDescriptorImpl)
-			 saved[indexColMapItem]).getReferencedColumnPositions();
+                gp.getSavedObject(indexColMapItem))
+                .getReferencedColumnPositions();
 
 		/* Get the result row template */
-        ExecRow resultRow = ((ExecRowBuilder) saved[resultRowAllocator])
-                                .build(a.getExecutionFactory());
+        ExecRow resultRow =
+                ((ExecRowBuilder) gp.getSavedObject(resultRowAllocator))
+                    .build(a.getExecutionFactory());
 
 		// Note that getCompactRow will assign its return value to the
 		// variable compactRow which can be accessed through
@@ -154,7 +167,7 @@ class IndexRowToBaseRowResultSet extends NoPutResultSetImpl
 			final DataValueDescriptor[] resultRowArray =
 				resultRow.getRowArray();
 			final FormatableBitSet heapOnly =
-				(FormatableBitSet)saved[heapOnlyColRefItem];
+                (FormatableBitSet) gp.getSavedObject(heapOnlyColRefItem);
 			final int heapOnlyLen = heapOnly.getLength();
 
 			// Need a separate DataValueDescriptor array in this case
@@ -169,6 +182,18 @@ class IndexRowToBaseRowResultSet extends NoPutResultSetImpl
 				}
 			}
 		}
+
+        //
+        // The row location column may be needed by the left join which drives
+        // the execution of the MERGE statement.
+        //
+        _includeRowLocation = (_baseColumnCount < accessedHeapCols.getLength());
+        if ( _includeRowLocation )
+        {
+            _heapColsWithoutRowLocation = (FormatableBitSet) accessedHeapCols.clone();
+            _heapColsWithoutRowLocation.clear( accessedHeapCols.getLength() - 1 );
+        }
+
 		recordConstructorTime();
 	}
 
@@ -335,8 +360,12 @@ class IndexRowToBaseRowResultSet extends NoPutResultSetImpl
 
 				// Fetch the columns coming from the heap
 				boolean row_exists = 
-                    baseCC.fetch(
-                        baseRowLocation, rowArray, accessedHeapCols);
+                    baseCC.fetch
+                    (
+                     baseRowLocation,
+                     rowArray,
+                     _includeRowLocation ? _heapColsWithoutRowLocation : accessedHeapCols
+                     );
 
                 if (row_exists)
                 {
@@ -391,6 +420,11 @@ class IndexRowToBaseRowResultSet extends NoPutResultSetImpl
 					currentRow = compactRow;
 				}
 
+                if ( _includeRowLocation )
+                {
+                    currentRow.setColumn( currentRow.nColumns(), baseRowLocation );
+                }
+
 				/* Update the run time statistics */
 				rowsSeen++;
 
