@@ -26,7 +26,7 @@ import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.services.i18n.MessageService;
 import org.apache.derby.iapi.services.io.FormatableBitSet;
 import org.apache.derby.iapi.services.monitor.Monitor;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.stream.HeaderPrintWriter;
 import org.apache.derby.iapi.sql.Activation;
 import org.apache.derby.iapi.sql.ResultDescription;
@@ -44,6 +44,7 @@ import org.apache.derby.iapi.store.access.Qualifier;
 import org.apache.derby.iapi.store.access.RowLocationRetRowSource;
 import org.apache.derby.iapi.store.access.RowSource;
 import org.apache.derby.iapi.types.DataValueDescriptor;
+import org.apache.derby.iapi.types.LocatedRow;
 import org.apache.derby.iapi.types.Orderable;
 import org.apache.derby.iapi.types.RowLocation;
 
@@ -70,7 +71,8 @@ extends BasicNoPutResultSetImpl
 
 	// fields used when being called as a RowSource
 	private boolean needsRowLocation;
-	protected ExecRow clonedExecRow;
+    private boolean needsRowLocationForDeferredCheckConstraints;
+    protected ExecRow clonedExecRow;
 	protected TargetResultSet	targetResultSet;
 
 	/* beetle 4464. compact flags into array of key column positions that we do check/skip nulls,
@@ -213,6 +215,10 @@ extends BasicNoPutResultSetImpl
 		this.needsRowLocation = needsRowLocation;
 	}
 
+    public void setHasDeferrableChecks() {
+        this.needsRowLocationForDeferredCheckConstraints = true;
+    }
+
 	// RowSource interface
 	
 	/** 
@@ -275,7 +281,12 @@ extends BasicNoPutResultSetImpl
 		return needsRowLocation;
 	}
 
-	/**
+    public boolean needsRowLocationForDeferredCheckConstraints()
+    {
+        return needsRowLocationForDeferredCheckConstraints;
+    }
+
+    /**
 	 * @see RowLocationRetRowSource#rowLocation
 	 * @exception StandardException on error
 	 */
@@ -285,7 +296,11 @@ extends BasicNoPutResultSetImpl
 		targetResultSet.changedRow(clonedExecRow, rl);
 	}
 
+    public void offendingRowLocation(
+            RowLocation rl, long containdId) throws StandardException {
 
+        targetResultSet.offendingRowLocation(rl, containdId);
+    }
 	// class implementation
 
 	/**
@@ -555,5 +570,14 @@ extends BasicNoPutResultSetImpl
 		// is a no-op
 	}
 
+    /**
+     * Get all of the columns out of a value stored in a BackingStoreHashtable.
+     */
+    protected DataValueDescriptor[]   unpackHashValue( Object hashValue )
+    {
+        if ( hashValue == null ) { return null; }
+        else if ( hashValue instanceof DataValueDescriptor[] ) { return (DataValueDescriptor[]) hashValue; }
+        else { return ((LocatedRow) hashValue).flatten(); }
+    }
 
 }
