@@ -23,7 +23,7 @@ package org.apache.derby.impl.sql.execute;
 
 import org.apache.derby.iapi.services.io.FormatableBitSet;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.error.StandardException;
 
@@ -42,7 +42,7 @@ import org.apache.derby.iapi.store.access.ConglomerateController;
 import org.apache.derby.iapi.store.access.DynamicCompiledOpenConglomInfo;
 import org.apache.derby.iapi.store.access.StaticCompiledOpenConglomInfo;
 import org.apache.derby.iapi.store.access.TransactionController;
-
+import org.apache.derby.iapi.transaction.TransactionControl;
 import org.apache.derby.iapi.types.DataValueDescriptor;
 
 import org.apache.derby.iapi.types.RowLocation;
@@ -108,7 +108,7 @@ class RowChangerImpl	implements	RowChanger
 		to be updated.  Only used for updates
 	  @param tc the transaction controller
 	  @param baseRowReadList bit set of columns read from base row. 1 based.
-	  @param baseRowReadMap BaseRowReadMap[heapColId]->ReadRowColumnId. (0 based)
+	  @param baseRowReadMap BaseRowReadMap[heapColId]-&gt;ReadRowColumnId. (0 based)
 	  @exception StandardException		Thrown on error
 	  */
 	public RowChangerImpl(
@@ -272,7 +272,7 @@ class RowChangerImpl	implements	RowChanger
 	  Open this RowChanger to avoid fixing indexes that do not change
 	  during update operations. 
 
-	  @param fixOnUpdate fixOnUpdat[ix] == true ==> fix index 'ix' on
+	  @param fixOnUpdate fixOnUpdat[ix] == true ==&gt; fix index 'ix' on
 	  an update operation.
 	  @param lockMode	The lock mode to use
 							(row or table, see TransactionController)
@@ -301,7 +301,7 @@ class RowChangerImpl	implements	RowChanger
 		int isolationLevel;
 		if (lcc == null)
 		{
-			isolationLevel = ExecutionContext.READ_COMMITTED_ISOLATION_LEVEL;
+			isolationLevel = TransactionControl.READ_COMMITTED_ISOLATION_LEVEL;
 		}
 		else
 		{
@@ -313,22 +313,22 @@ class RowChangerImpl	implements	RowChanger
 		{
 			// Even though we preserve the isolation level at READ UNCOMMITTED,
 			// Store will overwrite it to READ COMMITTED for update.
-			case ExecutionContext.READ_UNCOMMITTED_ISOLATION_LEVEL:
+			case TransactionControl.READ_UNCOMMITTED_ISOLATION_LEVEL:
 				isolationLevel = 
                     TransactionController.ISOLATION_READ_UNCOMMITTED;
 				break;
 
-			case ExecutionContext.READ_COMMITTED_ISOLATION_LEVEL:
+			case TransactionControl.READ_COMMITTED_ISOLATION_LEVEL:
 				isolationLevel = 
                     TransactionController.ISOLATION_READ_COMMITTED;
 				break;
 
-			case ExecutionContext.REPEATABLE_READ_ISOLATION_LEVEL:
+			case TransactionControl.REPEATABLE_READ_ISOLATION_LEVEL:
 				isolationLevel = 
                     TransactionController.ISOLATION_REPEATABLE_READ;
 				break;
 
-			case ExecutionContext.SERIALIZABLE_ISOLATION_LEVEL:
+			case TransactionControl.SERIALIZABLE_ISOLATION_LEVEL:
 				isolationLevel = 
                     TransactionController.ISOLATION_SERIALIZABLE;
 				break;
@@ -434,7 +434,7 @@ class RowChangerImpl	implements	RowChanger
 	  @param baseRow the row.
 	  @exception StandardException		Thrown on error
 	  */
-	public void insertRow(ExecRow baseRow)
+    public RowLocation insertRow(ExecRow baseRow, boolean getRL)
 		 throws StandardException
 	{
 		if (SanityManager.DEBUG)
@@ -447,16 +447,24 @@ class RowChangerImpl	implements	RowChanger
 		}
 		else
 		{
-			if (isc != null)
+            if (isc != null || getRL)
 			{
+                if (baseRowLocation == null) {
+                    baseRowLocation = baseCC.newRowLocationTemplate();
+                }
 				baseCC.insertAndFetchLocation(baseRow.getRowArray(), baseRowLocation);
-				isc.insert(baseRow, baseRowLocation);
+
+                if (isc != null) {
+                    isc.insert(baseRow, baseRowLocation);
+                }
 			}
 			else
 			{
-				baseCC.insert(baseRow.getRowArray());
+                baseCC.insert(baseRow.getRowArray());
 			}
 		}
+
+        return getRL ? baseRowLocation : null;
 	}
 
 		
