@@ -21,7 +21,7 @@
 
 package org.apache.derby.impl.sql.execute;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.sql.execute.CursorResultSet;
 import org.apache.derby.iapi.sql.execute.ExecPreparedStatement;
@@ -180,9 +180,8 @@ class SortResultSet extends NoPutResultSetImpl
 		sortTemplateRow = ((ExecRowBuilder) ps.getSavedObject(ra))
                                 .build(a.getExecutionFactory());
 
-		order = (ColumnOrdering[])
-					((FormatableArrayHolder) ps.getSavedObject(orderingItem))
-					.getArray(ColumnOrdering.class);
+        order = ((FormatableArrayHolder) ps.getSavedObject(orderingItem))
+                .getArray(ColumnOrdering[].class);
 
 		/* NOTE: We need to save order to another variable
 		 * in the constructor and reset it on every open.
