@@ -23,10 +23,11 @@ package org.apache.derby.impl.sql.execute;
 
 import java.sql.SQLWarning;
 import java.sql.Timestamp;
+import org.w3c.dom.Element;
 
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.services.io.FormatableBitSet;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.sql.Activation;
 import org.apache.derby.iapi.sql.ResultDescription;
 import org.apache.derby.iapi.sql.ResultSet;
@@ -674,7 +675,7 @@ class TemporaryRowHolderResultSet implements CursorResultSet, NoPutResultSet, Cl
 	 */
 	public int resultSetNumber()
 	{
-		return 0;
+		return TEMPORARY_RESULT_SET_NUMBER;
 	}
 
 	/**
@@ -1192,7 +1193,7 @@ class TemporaryRowHolderResultSet implements CursorResultSet, NoPutResultSet, Cl
 	  not in the partial row if validColumns.get(N) returns false.  Column N is
 	  in the partial row if validColumns.get(N) returns true.  If column N is
 	  in the partial row then it maps to DataValueDescriptor[M] where M is the
-      count of calls to validColumns.get(i) that return true where i < N.  If
+      count of calls to validColumns.get(i) that return true where i &lt; N.  If
 	  DataValueDescriptor.length is greater than the number of columns 
       indicated by validColumns the extra entries are ignored.  
 	*/
@@ -1230,6 +1231,18 @@ class TemporaryRowHolderResultSet implements CursorResultSet, NoPutResultSet, Cl
 		return false;
 	}
 
+    public void setHasDeferrableChecks() {
+        if (SanityManager.DEBUG) {
+            SanityManager.NOTREACHED();
+        }
+    }
+
+    public boolean needsRowLocationForDeferredCheckConstraints()
+    {
+        return false;
+    }
+
+
 	/**
 		rowLocation is a callback for the drainer of the row source to return
 		the rowLocation of the current row, i.e, the row that is being returned
@@ -1269,6 +1282,13 @@ class TemporaryRowHolderResultSet implements CursorResultSet, NoPutResultSet, Cl
 	public void rowLocation(RowLocation rl) throws StandardException
 	{ }
 
+    public void offendingRowLocation(
+            RowLocation rl, long containdId) throws StandardException {
+        if (SanityManager.DEBUG) {
+            SanityManager.NOTREACHED();
+        }
+    }
+
 	/**
 	 * @see NoPutResultSet#positionScanAtRowLocation
 	 *
@@ -1349,4 +1369,10 @@ class TemporaryRowHolderResultSet implements CursorResultSet, NoPutResultSet, Cl
 	public final Activation getActivation() {
 		return holder.activation;
 	}
+    
+    public Element toXML( Element parentNode, String tag ) throws Exception
+    {
+        return BasicNoPutResultSetImpl.childrenToXML( BasicNoPutResultSetImpl.toXML( parentNode, tag, this ), this );
+    }
+
 }
