@@ -23,7 +23,7 @@ package org.apache.derby.impl.sql.execute;
 
 import org.apache.derby.catalog.TypeDescriptor;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.sql.execute.CursorResultSet;
 import org.apache.derby.iapi.sql.execute.ExecPreparedStatement;
@@ -44,7 +44,7 @@ import org.apache.derby.iapi.store.access.Qualifier;
 import org.apache.derby.iapi.error.StandardException;
 
 import org.apache.derby.iapi.services.loader.GeneratedMethod;
-
+import org.apache.derby.iapi.transaction.TransactionControl;
 import org.apache.derby.iapi.types.RowLocation;
 import org.apache.derby.iapi.reference.SQLState;
 
@@ -54,15 +54,17 @@ import org.apache.derby.iapi.services.io.FormatableHashtable;
 import org.apache.derby.vti.DeferModification;
 import org.apache.derby.vti.IFastPath;
 import org.apache.derby.vti.VTIEnvironment;
+import org.apache.derby.vti.AwareVTI;
 import org.apache.derby.vti.RestrictedVTI;
 import org.apache.derby.vti.Restriction;
+import org.apache.derby.vti.VTIContext;
 
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.SQLWarning;
 import java.sql.ResultSetMetaData;
-
+import org.w3c.dom.Element;
 
 /**
  */
@@ -81,8 +83,7 @@ class VTIResultSet extends NoPutResultSetImpl
 	private boolean version2;
 	private boolean reuseablePs;
 	private boolean isTarget;
-	private FormatableHashtable compileTimeConstants;
-	private int ctcNumber;
+    private final FormatableHashtable compileTimeConstants;
 
 	private boolean pushedProjection;
 	private IFastPath	fastPath;
@@ -100,11 +101,14 @@ class VTIResultSet extends NoPutResultSetImpl
     private String[] vtiProjection;
     private Restriction vtiRestriction;
 
+    private String  vtiSchema;
+    private String  vtiName;
+
 	/**
 		Specified isolation level of SELECT (scan). If not set or
-		not application, it will be set to ExecutionContext.UNSPECIFIED_ISOLATION_LEVEL
+		not application, it will be set to TransactionControl.UNSPECIFIED_ISOLATION_LEVEL
 	*/
-	private int scanIsolationLevel = ExecutionContext.UNSPECIFIED_ISOLATION_LEVEL;
+	private int scanIsolationLevel = TransactionControl.UNSPECIFIED_ISOLATION_LEVEL;
 
     //
     // class interface
@@ -123,7 +127,9 @@ class VTIResultSet extends NoPutResultSetImpl
 				 boolean isDerbyStyleTableFunction,
                  int returnTypeNumber,
                  int vtiProjectionNumber,
-                 int vtiRestrictionNumber
+                 int vtiRestrictionNumber,
+                 String vtiSchema,
+                 String vtiName
                  ) 
 		throws StandardException
 	{
@@ -137,6 +143,8 @@ class VTIResultSet extends NoPutResultSetImpl
 		this.pushedQualifiers = pushedQualifiers;
 		this.scanIsolationLevel = scanIsolationLevel;
 		this.isDerbyStyleTableFunction = isDerbyStyleTableFunction;
+        this.vtiSchema = vtiSchema;
+        this.vtiName = vtiName;
 
         ExecPreparedStatement ps = activation.getPreparedStatement();
 
@@ -161,10 +169,17 @@ class VTIResultSet extends NoPutResultSetImpl
 								getSavedObject(erdNumber));
 		}
 
-		this.ctcNumber = ctcNumber;
 		compileTimeConstants = (FormatableHashtable) (activation.getPreparedStatement().
 								getSavedObject(ctcNumber));
 
+        // compileTimeConstants cannot be null, even if there are no
+        // constants, since VTIResultSet.setSharedState() may want to
+        // add constants to it during execution.
+        if (SanityManager.DEBUG) {
+            SanityManager.ASSERT(compileTimeConstants != null,
+                                 "compileTimeConstants is null");
+        }
+
 		recordConstructorTime();
     }
 
@@ -241,6 +256,21 @@ class VTIResultSet extends NoPutResultSetImpl
 
                     restrictedVTI.initScan( vtiProjection, cloneRestriction( activation ) );
                 }
+
+                if ( userVTI instanceof AwareVTI )
+                {
+                    AwareVTI awareVTI = (AwareVTI) userVTI;
+
+                    awareVTI.setContext
+                        (
+                         new VTIContext
+                         (
+                          vtiSchema,
+                          vtiName,
+                          activation.getLanguageConnectionContext().getStatementContext().getStatementText()
+                          )
+                         );
+                }
 			}
 
 			// Set up the nullablity of the runtime columns, may be delayed
@@ -701,7 +731,7 @@ class VTIResultSet extends NoPutResultSetImpl
 	}
 
 	public final int getStatementIsolationLevel() {
-		return ExecutionContext.CS_TO_JDBC_ISOLATION_LEVEL_MAP[getScanIsolationLevel()];
+		return TransactionControl.jdbcIsolationLevel( getScanIsolationLevel() );
 	}
 
 
@@ -709,20 +739,6 @@ class VTIResultSet extends NoPutResultSetImpl
 		if (key == null)
 			return;
 
-		if (compileTimeConstants == null) {
-
-			Object[] savedObjects = activation.getPreparedStatement().getSavedObjects();
-
-			synchronized (savedObjects) {
-
-				compileTimeConstants = (FormatableHashtable) savedObjects[ctcNumber];
-				if (compileTimeConstants == null) {
-					compileTimeConstants = new FormatableHashtable();
-					savedObjects[ctcNumber] = compileTimeConstants;
-				}
-			}
-		}
-
 		if (value == null)
 			compileTimeConstants.remove(key);
 		else
@@ -844,4 +860,11 @@ class VTIResultSet extends NoPutResultSetImpl
         vsdv.setWidth( dtd.getPrecision(), dtd.getScale(), false );
     }
     
+    public Element toXML( Element parentNode, String tag ) throws Exception
+    {
+        Element myNode = super.toXML( parentNode, tag );
+        myNode.setAttribute( "javaClassName", javaClassName );
+        
+        return myNode;
+    }
 }
