@@ -21,21 +21,26 @@
 
 package org.apache.derby.impl.store.access;
 
+import org.apache.derby.iapi.security.Securable;
+import org.apache.derby.iapi.security.SecurityUtil;
+
 import org.apache.derby.iapi.services.cache.Cacheable;
 import org.apache.derby.iapi.services.cache.CacheableFactory;
 import org.apache.derby.iapi.services.cache.CacheFactory;
 import org.apache.derby.iapi.services.cache.CacheManager;
 
 import org.apache.derby.iapi.services.context.ContextManager;
+import org.apache.derby.iapi.services.context.Context;
 import org.apache.derby.iapi.services.context.ContextService;
 import org.apache.derby.iapi.services.daemon.Serviceable;
 import org.apache.derby.iapi.services.locks.LockFactory;
 import org.apache.derby.iapi.services.monitor.ModuleControl;
+import org.apache.derby.iapi.services.monitor.ModuleFactory;
 import org.apache.derby.iapi.services.monitor.Monitor;
 import org.apache.derby.iapi.services.monitor.PersistentService;
 import org.apache.derby.iapi.services.property.PropertySetCallback;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.error.StandardException;
 
@@ -63,6 +68,10 @@ import org.apache.derby.catalog.UUID;
 import org.apache.derby.iapi.reference.SQLState;
 import org.apache.derby.iapi.reference.Attribute;
 
+import java.security.PrivilegedAction;
+import java.security.PrivilegedActionException;
+import java.security.PrivilegedExceptionAction;
+import java.security.AccessController;
 import java.util.Dictionary;
 import java.util.Enumeration;
 import java.util.Hashtable;
@@ -90,12 +99,12 @@ public abstract class RAMAccessManager
     /**
     Hash table on primary implementation type.
     **/
-    private Hashtable implhash;
+    private Hashtable<String,MethodFactory> implhash;
 
     /**
     Hash table on primary format.
     **/
-    private Hashtable formathash;
+    private Hashtable<UUID,MethodFactory> formathash;
 
 	/**
 	Service properties.  These are supplied from ModuleControl.boot(),
@@ -159,8 +168,8 @@ public abstract class RAMAccessManager
     {
         // Intialize the hash tables that hold the access methods that
         // this access manager knows about.
-        implhash   = new Hashtable();
-        formathash = new Hashtable();
+        implhash   = new Hashtable<String,MethodFactory>();
+        formathash = new Hashtable<UUID,MethodFactory>();
     }
 
     /**************************************************************************
@@ -383,7 +392,7 @@ public abstract class RAMAccessManager
             // just in case language passes in a bad factory id.
 			throw StandardException.newException(
                 SQLState.STORE_CONGLOMERATE_DOES_NOT_EXIST, 
-                new Long(conglom_id));
+                conglom_id);
         }
     }
 
@@ -430,7 +439,7 @@ public abstract class RAMAccessManager
     {
         // Get a cache factory to create the conglomerate cache.
 		CacheFactory cf = 
-            (CacheFactory) Monitor.startSystemModule(
+            (CacheFactory) startSystemModule(
                  org.apache.derby.iapi.reference.Module.CacheFactory);
 
         // Now create the conglomerate cache.
@@ -455,13 +464,11 @@ public abstract class RAMAccessManager
      *
 	 * @exception  StandardException  Standard exception policy.
      **/
-    /* package */ Conglomerate conglomCacheFind(
-    TransactionManager  xact_mgr,
-    long                conglomid)
+    /* package */ Conglomerate conglomCacheFind(long conglomid)
         throws StandardException
     {
         Conglomerate conglom       = null;
-        Long         conglomid_obj = new Long(conglomid);
+        Long         conglomid_obj = conglomid;
 
         CacheableConglomerate cache_entry =
             (CacheableConglomerate) conglom_cache.find(conglomid_obj);
@@ -506,7 +513,7 @@ public abstract class RAMAccessManager
     {
         // Insert the new entry.
         CacheableConglomerate conglom_entry = (CacheableConglomerate)
-            conglom_cache.create(new Long(conglomid), conglom);
+            conglom_cache.create(conglomid, conglom);
         conglom_cache.release(conglom_entry);
     }
 
@@ -522,7 +529,7 @@ public abstract class RAMAccessManager
         throws StandardException
     {
         CacheableConglomerate conglom_entry = (CacheableConglomerate)
-            conglom_cache.findCached(new Long(conglomid));
+            conglom_cache.findCached(conglomid);
 
         if (conglom_entry != null) {
             conglom_cache.remove(conglom_entry);
@@ -545,16 +552,16 @@ public abstract class RAMAccessManager
      */
     RAMTransactionContext getCurrentTransactionContext() {
         RAMTransactionContext rtc =
-            (RAMTransactionContext) ContextService.getContext(
+            (RAMTransactionContext) getContext(
                 AccessFactoryGlobals.RAMXACT_INTERNAL_CONTEXT_ID);
 
         if (rtc == null) {
-            rtc = (RAMTransactionContext) ContextService.getContext(
+            rtc = (RAMTransactionContext) getContext(
                     AccessFactoryGlobals.RAMXACT_CHILD_CONTEXT_ID);
         }
 
         if (rtc == null) {
-            rtc = (RAMTransactionContext) ContextService.getContext(
+            rtc = (RAMTransactionContext) getContext(
                     AccessFactoryGlobals.RAMXACT_CONTEXT_ID);
         }
 
@@ -585,16 +592,16 @@ public abstract class RAMAccessManager
         
         // See if there's an access method that supports the desired
         // format type as its primary format type.
-        factory = (MethodFactory) formathash.get(format);
+        factory = formathash.get(format);
         if (factory != null)
             return factory;
 
         // No primary format.  See if one of the access methods
         // supports it as a secondary format.
-        Enumeration e = formathash.elements();
+        Enumeration<MethodFactory> e = formathash.elements();
         while (e.hasMoreElements())
         {
-            factory = (MethodFactory) e.nextElement();
+            factory = e.nextElement();
             if (factory.supportsFormat(format))
                 return factory;
         }
@@ -612,16 +619,16 @@ public abstract class RAMAccessManager
     {
         // See if there's an access method that supports the desired
         // implementation type as its primary implementation type.
-        MethodFactory factory = (MethodFactory) implhash.get(impltype);
+        MethodFactory factory = implhash.get(impltype);
         if (factory != null)
 				return factory;
 
         // No primary implementation.  See if one of the access methods
         // supports the implementation type as a secondary.
-        Enumeration e = implhash.elements();
+        Enumeration<MethodFactory> e = implhash.elements();
         while (e.hasMoreElements())
         {
-            factory = (MethodFactory) e.nextElement();
+            factory = e.nextElement();
             if (factory.supportsImplementation(impltype))
                 return factory;
         }
@@ -636,7 +643,7 @@ public abstract class RAMAccessManager
 
 		try {
 			factory = 
-               (MethodFactory) Monitor.bootServiceModule(
+               (MethodFactory) bootServiceModule(
                     false, this, MethodFactory.MODULE, 
                     impltype, conglomProperties);
 		} catch (StandardException se) {
@@ -860,12 +867,12 @@ public abstract class RAMAccessManager
         rawstore.createDataWarningFile();
 
         //creating readme in log directory
-        LogFactory logFactory =(LogFactory) Monitor.findServiceModule(this, rawstore.getLogFactoryModule());
+        LogFactory logFactory =(LogFactory) findServiceModule(this, rawstore.getLogFactoryModule());
         logFactory.createDataWarningFile();
 
         //creating readme in root database directory
-        DataFactory dataFactory =(DataFactory) Monitor.findServiceModule(this, rawstore.getDataFactoryModule());
-        PersistentService ps = Monitor.getMonitor().getServiceType(rawstore);
+        DataFactory dataFactory =(DataFactory) findServiceModule(this, rawstore.getDataFactoryModule());
+        PersistentService ps = getMonitor().getServiceType(rawstore);
         ps.createDataWarningFile(dataFactory.getStorageFactory());
     }
 	
@@ -876,7 +883,7 @@ public abstract class RAMAccessManager
 		// set up the initial values by calling the validate and apply methods.
 		// the map methods are not called as they will have been called
 		// at runtime when the user set the property.
-		Dictionary d = new Hashtable();
+        Dictionary<Object, Object> d = new Hashtable<Object, Object>();
 		try {
 			xactProperties.getProperties(tc,d,false/*!stringsOnly*/,false/*!defaultsOnly*/);
 		} catch (StandardException se) {
@@ -929,11 +936,19 @@ public abstract class RAMAccessManager
 
 	public void freeze() throws StandardException
 	{
+        // make sure that application code doesn't bypass security checks
+        // by calling this public entry point
+        SecurityUtil.authorize( Securable.FREEZE_DATABASE );
+            
 		rawstore.freeze();
 	}
 
 	public void unfreeze() throws StandardException
 	{
+        // make sure that application code doesn't bypass security checks
+        // by calling this public entry point
+        SecurityUtil.authorize( Securable.UNFREEZE_DATABASE );
+            
 		rawstore.unfreeze();
 	}
 
@@ -942,6 +957,14 @@ public abstract class RAMAccessManager
     boolean wait) 
         throws StandardException
 	{
+        // make sure that application code doesn't bypass security checks
+        // by calling this public entry point
+        SecurityUtil.authorize
+            (
+             wait ?
+             Securable.BACKUP_DATABASE :
+             Securable.BACKUP_DATABASE_NOWAIT
+             );
 		rawstore.backup(backupDir, wait);
 	}
 
@@ -952,6 +975,14 @@ public abstract class RAMAccessManager
     boolean wait)
 		throws StandardException 
 	{
+        // make sure that application code doesn't bypass security checks
+        // by calling this public entry point
+        SecurityUtil.authorize
+            (
+             wait ?
+             Securable.BACKUP_DATABASE_AND_ENABLE_LOG_ARCHIVE_MODE :
+             Securable.BACKUP_DATABASE_AND_ENABLE_LOG_ARCHIVE_MODE_NOWAIT
+             );
 		rawstore.backupAndEnableLogArchiveMode(backupDir, 
                                                deleteOnlineArchivedLogFiles, 
                                                wait);
@@ -960,6 +991,10 @@ public abstract class RAMAccessManager
 	public void disableLogArchiveMode(boolean deleteOnlineArchivedLogFiles)
 		throws StandardException
 	{
+        // make sure that application code doesn't bypass security checks
+        // by calling this public entry point
+        SecurityUtil.authorize( Securable.DISABLE_LOG_ARCHIVE_MODE );
+            
 		rawstore.disableLogArchiveMode(deleteOnlineArchivedLogFiles);
 	}
 
@@ -967,6 +1002,10 @@ public abstract class RAMAccessManager
 
 	public void checkpoint() throws StandardException
 	{
+        // make sure that application code doesn't bypass security checks
+        // by calling this public entry point
+        SecurityUtil.authorize( Securable.CHECKPOINT_DATABASE );
+            
 		rawstore.checkpoint();
 	}
 
@@ -997,16 +1036,21 @@ public abstract class RAMAccessManager
 
         // Access depends on a Raw Store implementations.  Load it.
         //
-        rawstore = (RawStoreFactory) Monitor.bootServiceModule(
+        rawstore = (RawStoreFactory) bootServiceModule(
             create, this, RawStoreFactory.MODULE, serviceProperties);
 
+        // initialize handler with raw store to be called in the event of
+        // aborted inserts.  Store will use the call back to reclaim space
+        // when these events happen.  See DERBY-4057.
+        rawstore.setUndoInsertEventHandler(new RAMAccessUndoHandler(this));
+
 		// Note: we also boot this module here since we may start Derby
 		// system from store access layer, as some of the unit test case,
 		// not from JDBC layer.(See
 		// /protocol/Database/Storage/Access/Interface/T_AccessFactory.java)
 		// If this module has already been booted by the JDBC layer, this will 
 		// have no effect at all.
-		Monitor.bootServiceModule(
+		bootServiceModule(
             create, this, org.apache.derby.iapi.reference.Module.PropertyFactory, 
             startParams);
 
@@ -1018,7 +1062,7 @@ public abstract class RAMAccessManager
         // Create the conglom conglom from within a separate system xact
         RAMTransaction tc =
             (RAMTransaction) getAndNameTransaction(
-                ContextService.getFactory().getCurrentContextManager(),
+                getContextService().getCurrentContextManager(),
                 AccessFactoryGlobals.USER_TRANS_NAME);
 
         // looking up lock_mode is dependant on access booting, but
@@ -1122,7 +1166,7 @@ public abstract class RAMAccessManager
 
         // set up the property validation
         pf = (PropertyFactory) 
-            Monitor.findServiceModule(
+            findServiceModule(
                 this, org.apache.derby.iapi.reference.Module.PropertyFactory);
 
         // set up the transaction properties.  On J9, over NFS, runing on a
@@ -1237,4 +1281,178 @@ public abstract class RAMAccessManager
 		return new CacheableConglomerate(this);
 	}
 
+    // ///////////////////////////////////////////////////////////////
+
+    /**
+     * Privileged lookup of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private static  ContextService    getContextService()
+    {
+        if ( System.getSecurityManager() == null )
+        {
+            return ContextService.getFactory();
+        }
+        else
+        {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedAction<ContextService>()
+                 {
+                     public ContextService run()
+                     {
+                         return ContextService.getFactory();
+                     }
+                 }
+                 );
+        }
+    }
+
+    /**
+     * Privileged lookup of a Context. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Context    getContext( final String contextID )
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<Context>()
+             {
+                 public Context run()
+                 {
+                     return ContextService.getContext( contextID );
+                 }
+             }
+             );
+    }
+
+    
+    /**
+     * Privileged Monitor lookup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ModuleFactory  getMonitor()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ModuleFactory>()
+             {
+                 public ModuleFactory run()
+                 {
+                     return Monitor.getMonitor();
+                 }
+             }
+             );
+    }
+
+    
+    /**
+     * Privileged startup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Object  startSystemModule( final String factoryInterface )
+        throws StandardException
+    {
+        try {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedExceptionAction<Object>()
+                 {
+                     public Object run()
+                         throws StandardException
+                     {
+                         return Monitor.startSystemModule( factoryInterface );
+                     }
+                 }
+                 );
+        } catch (PrivilegedActionException pae)
+        {
+            throw StandardException.plainWrapException( pae );
+        }
+    }
+
+    /**
+     * Privileged startup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Object bootServiceModule
+        (
+         final boolean create, final Object serviceModule,
+         final String factoryInterface, final Properties properties
+         )
+        throws StandardException
+    {
+        try {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedExceptionAction<Object>()
+                 {
+                     public Object run()
+                         throws StandardException
+                     {
+                         return Monitor.bootServiceModule( create, serviceModule, factoryInterface, properties );
+                     }
+                 }
+                 );
+        } catch (PrivilegedActionException pae)
+        {
+            throw StandardException.plainWrapException( pae );
+        }
+    }
+
+
+    /**
+     * Privileged startup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Object bootServiceModule
+        (
+         final boolean create, final Object serviceModule,
+         final String factoryInterface, final String identifier, final Properties properties
+         )
+        throws StandardException
+    {
+        try {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedExceptionAction<Object>()
+                 {
+                     public Object run()
+                         throws StandardException
+                     {
+                         return Monitor.bootServiceModule( create, serviceModule, factoryInterface, identifier, properties );
+                     }
+                 }
+                 );
+        } catch (PrivilegedActionException pae)
+        {
+            throw StandardException.plainWrapException( pae );
+        }
+    }
+
+    /**
+     * Privileged startup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Object findServiceModule( final Object serviceModule, final String factoryInterface)
+        throws StandardException
+    {
+        try {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedExceptionAction<Object>()
+                 {
+                     public Object run()
+                         throws StandardException
+                     {
+                         return Monitor.findServiceModule( serviceModule, factoryInterface );
+                     }
+                 }
+                 );
+        } catch (PrivilegedActionException pae)
+        {
+            throw StandardException.plainWrapException( pae );
+        }
+    }
+
 }
