@@ -21,33 +21,26 @@
 
 package org.apache.derby.impl.store.access.sort;
 
-import org.apache.derby.iapi.reference.SQLState;
-
-import org.apache.derby.iapi.services.io.FormatableBitSet;
-
-import org.apache.derby.iapi.services.sanity.SanityManager;
-import org.apache.derby.iapi.error.StandardException;
-import org.apache.derby.iapi.store.access.conglomerate.ScanControllerRowSource;
-import org.apache.derby.iapi.store.access.conglomerate.Sort;
-import org.apache.derby.iapi.store.access.conglomerate.TransactionManager;
-import org.apache.derby.iapi.store.access.ColumnOrdering;
-import org.apache.derby.iapi.store.access.RowUtil;
-import org.apache.derby.iapi.store.access.ScanController;
-import org.apache.derby.iapi.store.access.SortObserver;
-import org.apache.derby.iapi.store.access.SortController;
-import org.apache.derby.iapi.store.access.TransactionController;
-
-import org.apache.derby.iapi.store.raw.StreamContainerHandle;
-import org.apache.derby.iapi.store.raw.RawStoreFactory;
-import org.apache.derby.iapi.store.raw.Transaction;
-
-import org.apache.derby.iapi.types.DataValueDescriptor;
-
-import org.apache.derby.iapi.types.Orderable;
-
 import java.util.Enumeration;
 import java.util.Properties;
 import java.util.Vector;
+import org.apache.derby.iapi.error.StandardException;
+import org.apache.derby.iapi.reference.SQLState;
+import org.apache.derby.iapi.services.io.FormatableBitSet;
+import org.apache.derby.iapi.store.access.ColumnOrdering;
+import org.apache.derby.iapi.store.access.RowUtil;
+import org.apache.derby.iapi.store.access.SortController;
+import org.apache.derby.iapi.store.access.SortObserver;
+import org.apache.derby.iapi.store.access.TransactionController;
+import org.apache.derby.iapi.store.access.conglomerate.ScanControllerRowSource;
+import org.apache.derby.iapi.store.access.conglomerate.ScanManager;
+import org.apache.derby.iapi.store.access.conglomerate.Sort;
+import org.apache.derby.iapi.store.access.conglomerate.TransactionManager;
+import org.apache.derby.iapi.store.raw.RawStoreFactory;
+import org.apache.derby.iapi.store.raw.StreamContainerHandle;
+import org.apache.derby.iapi.store.raw.Transaction;
+import org.apache.derby.iapi.types.DataValueDescriptor;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 /**
 
@@ -153,7 +146,7 @@ class MergeSort implements Sort
 	Might be null if no merge runs were produced.
 	It is a vector of container ids.
 	**/
-	private Vector mergeRuns = null;
+	private Vector<Long> mergeRuns = null;
 
 	/**
 	An ordered set of the leftover rows that didn't go
@@ -223,7 +216,7 @@ class MergeSort implements Sort
 	@see Sort#openSortScan
 	**/
 
-	public ScanController openSortScan(
+	public ScanManager openSortScan(
     TransactionManager tran,
     boolean            hold)
 			throws StandardException
@@ -244,7 +237,7 @@ class MergeSort implements Sort
 		{
 			// Dump the rows in the sort buffer to a merge run.
 			long containerId = createMergeRun(tran, sortBuffer);
-			mergeRuns.addElement(new Long(containerId));
+			mergeRuns.addElement(containerId);
 
 			// If there are more merge runs than we can sort
 			// at once with our sort buffer, we have to reduce
@@ -303,7 +296,7 @@ class MergeSort implements Sort
 		{
 			// Dump the rows in the sort buffer to a merge run.
 			long containerId = createMergeRun(tran, sortBuffer);
-			mergeRuns.addElement(new Long(containerId));
+			mergeRuns.addElement(containerId);
 
 			// If there are more merge runs than we can sort
 			// at once with our sort buffer, we have to reduce
@@ -486,6 +479,19 @@ class MergeSort implements Sort
 		// ordering array.
         for (int i = 0; i < colsToCompare; i++)
         {
+            if (i == colsToCompare - 1 && sortObserver.deferrable()) {
+                if (sortObserver.deferred()) {
+                    // Last column, which is RowLocation. We compared equal
+                    // this far so duplicate: remember till end of
+                    // transaction, but continue sorting on RowLocation, since
+                    // the index needs to be sorted on the column, too, for
+                    // the Btree to load.
+                    sortObserver.rememberDuplicate(r1);
+                } else {
+                    // report the duplicate
+                    break;
+                }
+            }
 			// Get columns to compare.
             int colid = this.columnOrderingMap[i];
             boolean nullsLow = this.columnOrderingNullsLowMap[i];
@@ -571,7 +577,7 @@ class MergeSort implements Sort
 	An inserter is closing.
 	**/
 	void doneInserting(MergeInserter inserter,
-		SortBuffer sortBuffer, Vector mergeRuns)
+		SortBuffer sortBuffer, Vector<Long> mergeRuns)
 	{
         if (SanityManager.DEBUG)
         {
@@ -603,7 +609,7 @@ class MergeSort implements Sort
 	}
 
 	void doneScanning(Scan scan, SortBuffer sortBuffer,
-		Vector mergeRuns)
+		Vector<Long> mergeRuns)
 	{
 		this.mergeRuns = mergeRuns;
 
@@ -620,7 +626,7 @@ class MergeSort implements Sort
 	{
 		if (mergeRuns != null)
 		{
-			Enumeration e = mergeRuns.elements();
+			Enumeration<Long> e = mergeRuns.elements();
 
 			try 
 			{
@@ -629,7 +635,7 @@ class MergeSort implements Sort
 
 				while (e.hasMoreElements())
 				{
-					long containerId = ((Long) e.nextElement()).longValue();
+					long containerId = (e.nextElement()).longValue();
 					rawTran.dropStreamContainer(segmentId, containerId);
 				}
 			}
@@ -667,26 +673,26 @@ class MergeSort implements Sort
 	private void multiStageMerge(TransactionManager tran)
 		throws StandardException
 	{
-		Enumeration e;
+		Enumeration<Long> e;
 		//int iterations = 0; // DEBUG (nat)
 		int maxMergeRuns = sortBuffer.capacity();
 
 		if (maxMergeRuns > ExternalSortFactory.DEFAULT_MAX_MERGE_RUN)
 			maxMergeRuns = ExternalSortFactory.DEFAULT_MAX_MERGE_RUN;
 
-		Vector subset;
-		Vector leftovers;
+		Vector<Long> subset;
+		Vector<Long> leftovers;
 
 		while (mergeRuns.size() > maxMergeRuns)
 		{
 			// Move maxMergeRuns elements from the merge runs
 			// vector into a subset, leaving the rest.
-			subset = new Vector(maxMergeRuns);
-			leftovers = new Vector(mergeRuns.size() - maxMergeRuns);
+			subset = new Vector<Long>(maxMergeRuns);
+			leftovers = new Vector<Long>(mergeRuns.size() - maxMergeRuns);
 			e = mergeRuns.elements();
 			while (e.hasMoreElements())
 			{
-				Long containerId = (Long) e.nextElement();
+				Long containerId = e.nextElement();
 				if (subset.size() < maxMergeRuns)
 					subset.addElement(containerId);
 				else
@@ -726,7 +732,7 @@ class MergeSort implements Sort
 			long id = rawTran.addAndLoadStreamContainer(segmentId,
 				properties, msRowSource);
 
-			mergeRuns.addElement(new Long(id));
+			mergeRuns.addElement(id);
 
 			// Drop the conglomerates in the merge subset
 			e = subset.elements();
