@@ -23,12 +23,11 @@ package org.apache.derby.impl.store.raw.data;
 
 import org.apache.derby.iapi.reference.SQLState;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.io.FormatIdUtil;
 
 import org.apache.derby.iapi.util.InterruptStatus;
 import org.apache.derby.iapi.util.InterruptDetectedException;
-import org.apache.derby.iapi.util.ReuseFactory;
 
 import org.apache.derby.iapi.error.StandardException;
 
@@ -54,7 +53,7 @@ import java.security.PrivilegedActionException;
 	for FileContainers which are implemented on java.io.RandomAccessFile.
 */
 
-class RAFContainer extends FileContainer implements PrivilegedExceptionAction
+class RAFContainer extends FileContainer implements PrivilegedExceptionAction<Object>
 {
 
 	/*
@@ -785,7 +784,17 @@ class RAFContainer extends FileContainer implements PrivilegedExceptionAction
                             }
                         }
 
-                        directory.limitAccessToOwner();
+                        try {
+                            directory.limitAccessToOwner();
+                        } catch (IOException ioe) {
+                            if (errorOK) {
+                                return null;
+                            } else {
+                                throw StandardException.newException(
+                                        SQLState.FILE_CANNOT_CREATE_SEGMENT,
+                                        ioe, directory);
+                            }
+                        }
                     }
                 }
             }
@@ -823,15 +832,10 @@ class RAFContainer extends FileContainer implements PrivilegedExceptionAction
      */
     private void copyFile(final StorageFile from, final File to)
             throws StandardException {
-        Boolean success = (Boolean) AccessController.doPrivileged(
-                new PrivilegedAction() {
-                    public Object run() {
-                        return ReuseFactory.getBoolean(FileUtil.copyFile(
-                                dataFactory.getStorageFactory(), from, to));
-                    }
-                });
-
-        if (!success.booleanValue()) {
+        PrivilegedAction<Boolean> pa = () ->
+                FileUtil.copyFile(dataFactory.getStorageFactory(), from, to);
+        boolean success = AccessController.doPrivileged(pa);
+        if (!success) {
             throw StandardException.newException(
                     SQLState.RAWSTORE_ERROR_COPYING_FILE,
                     from, to);
@@ -844,15 +848,9 @@ class RAFContainer extends FileContainer implements PrivilegedExceptionAction
      * @throws StandardException if the file could not be removed
      */
     private void removeFile(final File file) throws StandardException {
-        Boolean success = (Boolean) AccessController.doPrivileged(
-            new PrivilegedAction() {
-                public Object run() {
-                    return ReuseFactory.getBoolean(
-                            !file.exists() || file.delete());
-                }
-        });
-
-        if (!success.booleanValue()) {
+        PrivilegedAction<Boolean> pa = () -> !file.exists() || file.delete();
+        boolean success = AccessController.doPrivileged(pa);
+        if (!success) {
             throw StandardException.newException(
                     SQLState.UNABLE_TO_DELETE_FILE, file);
         }
@@ -1311,13 +1309,14 @@ class RAFContainer extends FileContainer implements PrivilegedExceptionAction
      * @return a RandomAccessFile
      * @throws FileNotFoundException if {@code file} cannot be opened in
      * read-write mode
+     * @throws IOException if some other I/O error happens
      */
     private RandomAccessFile getRandomAccessFile(final File file)
-            throws FileNotFoundException {
+            throws IOException {
         try {
-            return (RandomAccessFile) AccessController.doPrivileged(
-                new PrivilegedExceptionAction() {
-                    public Object run() throws FileNotFoundException {
+            return AccessController.doPrivileged(
+                new PrivilegedExceptionAction<RandomAccessFile>() {
+                    public RandomAccessFile run() throws IOException {
                         boolean preExisting = file.exists();
                         RandomAccessFile raf = new RandomAccessFile(file, "rw");
                         if (!preExisting) {
@@ -1327,7 +1326,7 @@ class RAFContainer extends FileContainer implements PrivilegedExceptionAction
                     }
                 });
         } catch (PrivilegedActionException pae) {
-            throw (FileNotFoundException) pae.getCause();
+            throw (IOException) pae.getCause();
         }
     }
 
@@ -1702,10 +1701,10 @@ class RAFContainer extends FileContainer implements PrivilegedExceptionAction
 
                  return result;
              }
-             catch (FileNotFoundException fnfe)
+             catch (IOException ioe)
              {
                  throw StandardException.newException(
-                     SQLState.FILE_CREATE, fnfe, actionFile.getPath());
+                     SQLState.FILE_CREATE, ioe, actionFile.getPath());
              }
 		 } // end of case BACKUP_CONTAINER_ACTION
 
