@@ -21,6 +21,10 @@
 
 package org.apache.derby.jdbc;
 
+import java.security.PrivilegedExceptionAction;
+import java.security.PrivilegedActionException;
+import java.security.PrivilegedAction;
+import java.security.AccessController;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 
@@ -30,6 +34,7 @@ import javax.transaction.xa.Xid;
 import javax.transaction.xa.XAException;
 
 import org.apache.derby.iapi.error.ExceptionSeverity;
+import org.apache.derby.iapi.error.ExceptionUtil;
 import org.apache.derby.iapi.error.StandardException;
 import org.apache.derby.iapi.jdbc.BrokeredConnection;
 import org.apache.derby.iapi.jdbc.ResourceAdapter;
@@ -43,7 +48,8 @@ import org.apache.derby.iapi.store.access.xa.XAResourceManager;
 import org.apache.derby.iapi.store.access.xa.XAXactId;
 import org.apache.derby.impl.jdbc.EmbedConnection;
 import org.apache.derby.impl.jdbc.TransactionResourceImpl;
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.impl.jdbc.Util;
+import org.apache.derby.shared.common.sanity.SanityManager;
 import org.apache.derby.iapi.services.property.PropertyUtil;
 import org.apache.derby.iapi.reference.Property;
 
@@ -57,6 +63,8 @@ class EmbedXAResource implements XAResource {
     private XAXactId currentXid;    
     /** The value of the transaction timeout on this resource. */
     private int timeoutSeconds;
+
+    private LanguageConnectionContext   lcc;
     
     EmbedXAResource (EmbedPooledConnection con, ResourceAdapter ra) {
         this.con = con;
@@ -93,7 +101,7 @@ class EmbedXAResource implements XAResource {
             // RM also does not know about this xid.
             if (inDoubtCM == null)
                 throw new XAException(XAException.XAER_NOTA);
-            ContextService csf = ContextService.getFactory();
+            ContextService csf = getContextService();
             csf.setCurrentContextManager(inDoubtCM);
             try {
                 rm.commit(inDoubtCM, xid_im, onePhase);
@@ -304,7 +312,15 @@ class EmbedXAResource implements XAResource {
                     return XAResource.XA_RDONLY;
                 }
             } catch (SQLException sqle) {
-                throw wrapInXAException(sqle);
+                XAException xe = wrapInXAException(sqle);
+
+                if (ExceptionUtil.
+                        isDeferredConstraintViolation(sqle.getSQLState())) {
+                    // We are rolling back
+                    returnConnectionToResource(tranState, xid_im);
+                }
+
+                throw xe;
             }
         }
         
@@ -397,7 +413,7 @@ class EmbedXAResource implements XAResource {
             if (inDoubtCM == null)
                 throw new XAException(XAException.XAER_NOTA);
             
-            ContextService csf = ContextService.getFactory();
+            ContextService csf = getContextService();
             
             csf.setCurrentContextManager(inDoubtCM);
             try {
@@ -447,7 +463,7 @@ class EmbedXAResource implements XAResource {
             if (inDoubtCM == null)
                 throw new XAException(XAException.XAER_NOTA);
             
-            ContextService csf = ContextService.getFactory();
+            ContextService csf = getContextService();
             
             csf.setCurrentContextManager(inDoubtCM);
             try {
@@ -534,7 +550,7 @@ class EmbedXAResource implements XAResource {
      */
     private long getDefaultXATransactionTimeout() throws XAException {
         try {
-            LanguageConnectionContext lcc = con.getLanguageConnection();
+            LanguageConnectionContext lcc = getLanguageConnectionContext( con );
             TransactionController tc = lcc.getTransactionExecute();
 
             long timeoutMillis = 1000 * (long) PropertyUtil.getServiceInt(
@@ -647,7 +663,7 @@ class EmbedXAResource implements XAResource {
                     con.realConnection.setHoldability(
                             ResultSet.CLOSE_CURSORS_AT_COMMIT);
                     
-                    con.realConnection.getLanguageConnection().
+                    getLanguageConnectionContext( con.realConnection ).
                             getTransactionExecute().
                             createXATransactionFromLocalTransaction(
                                                 xid_im.getFormatId(),
@@ -661,9 +677,11 @@ class EmbedXAResource implements XAResource {
                     throw wrapInXAException(sqle);
                 }
                 
-                tranState = new XATransactionState(
-                    con.realConnection.getContextManager(),
-                    con.realConnection, this, xid_im);
+                tranState = new XATransactionState
+                    (
+                     getContextManager( con.realConnection ),
+                     con.realConnection, this, xid_im
+                     );
                 if (!ra.addConnection(xid_im, tranState))
                     throw new XAException(XAException.XAER_DUPID);
                 
@@ -820,9 +838,12 @@ class EmbedXAResource implements XAResource {
         else if (sqlstate.equals(SQLState.LOCK_TIMEOUT))
             xaErrorCode = XAException.XA_RBTIMEOUT;
         else if (seErrorCode >=  ExceptionSeverity.SESSION_SEVERITY)
-            xaErrorCode = XAException.XAER_RMFAIL;            
-        else
+            xaErrorCode = XAException.XAER_RMFAIL;
+        else if (ExceptionUtil.isDeferredConstraintViolation(sqlstate)) {
+            xaErrorCode = XAException.XA_RBINTEGRITY;
+        } else {
             xaErrorCode = XAException.XAER_RMERR;
+        }
         
         xae = new XAException(message);
         xae.errorCode = xaErrorCode;
@@ -917,4 +938,88 @@ class EmbedXAResource implements XAResource {
         currentXid = aCurrentXid;
     }
 
+    
+    /**
+     * Privileged lookup of the ContextService. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ContextService    getContextService()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ContextService>()
+             {
+                 public ContextService run()
+                 {
+                     return ContextService.getFactory();
+                 }
+             }
+             );
+    }
+
+    /**
+     * Privileged lookup of the ContextManager. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  ContextManager    getContextManager( final EmbedConnection conn )
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ContextManager>()
+             {
+                 public ContextManager run()
+                 {
+                     return conn.getContextManager();
+                 }
+             }
+             );
+    }
+
+	/**
+	  *	Gets the LanguageConnectionContext for this connection.
+	  */
+	private	LanguageConnectionContext	getLanguageConnectionContext( final EmbedConnection conn )
+	{
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<LanguageConnectionContext>()
+             {
+                 public LanguageConnectionContext run()
+                 {
+                     return conn.getLanguageConnection();
+                 }
+             }
+             );
+	}
+
+    /**
+     * Privileged LCC lookup. Must be private so that user code
+     * can't call this entry point.
+     */
+	private	LanguageConnectionContext	getLanguageConnectionContext( final EmbedPooledConnection conn )
+        throws SQLException
+    {
+        if ( lcc == null )
+        {
+            try {
+                lcc = AccessController.doPrivileged
+                    (
+                     new PrivilegedExceptionAction<LanguageConnectionContext>()
+                     {
+                         public LanguageConnectionContext run()
+                             throws SQLException
+                         {
+                             return conn.getLanguageConnection();
+                         }
+                     }
+                     );
+            } catch (PrivilegedActionException pae)
+            {
+                throw Util.javaException( pae );
+            }
+        }
+
+        return lcc;
+    }
+
 }
