@@ -38,6 +38,7 @@ import org.apache.derby.iapi.sql.compile.Visitable;
 import org.apache.derby.iapi.sql.conn.LanguageConnectionContext;
 import org.apache.derby.iapi.sql.depend.DependencyManager;
 import org.apache.derby.iapi.sql.dictionary.AliasDescriptor;
+import org.apache.derby.iapi.sql.dictionary.BulkInsertCounter;
 import org.apache.derby.iapi.sql.dictionary.ColPermsDescriptor;
 import org.apache.derby.iapi.sql.dictionary.ColumnDescriptor;
 import org.apache.derby.iapi.sql.dictionary.ConglomerateDescriptor;
@@ -47,7 +48,7 @@ import org.apache.derby.iapi.sql.dictionary.DataDescriptorGenerator;
 import org.apache.derby.iapi.sql.dictionary.DataDictionary;
 import org.apache.derby.iapi.sql.dictionary.DependencyDescriptor;
 import org.apache.derby.iapi.sql.dictionary.FileInfoDescriptor;
-import org.apache.derby.iapi.sql.dictionary.GenericDescriptorList;
+import org.apache.derby.iapi.sql.dictionary.TriggerDescriptorList;
 import org.apache.derby.iapi.sql.dictionary.PasswordHasher;
 import org.apache.derby.iapi.sql.dictionary.PermissionsDescriptor;
 import org.apache.derby.iapi.sql.dictionary.RoutinePermsDescriptor;
@@ -55,6 +56,7 @@ import org.apache.derby.iapi.sql.dictionary.SPSDescriptor;
 import org.apache.derby.iapi.sql.dictionary.SchemaDescriptor;
 import org.apache.derby.iapi.sql.dictionary.RoleGrantDescriptor;
 import org.apache.derby.iapi.sql.dictionary.SequenceDescriptor;
+import org.apache.derby.iapi.sql.dictionary.StatisticsDescriptor;
 import org.apache.derby.iapi.sql.dictionary.PermDescriptor;
 import org.apache.derby.iapi.sql.dictionary.RoleClosureIterator;
 import org.apache.derby.iapi.sql.dictionary.SubKeyConstraintDescriptor;
@@ -460,7 +462,7 @@ public class EmptyDictionary implements DataDictionary, ModuleSupportable {
 		return null;
 	}
 
-	public List getAllSPSDescriptors() throws StandardException {
+    public List<SPSDescriptor> getAllSPSDescriptors() throws StandardException {
 		// Auto-generated method stub
 		return null;
 	}
@@ -506,6 +508,23 @@ public class EmptyDictionary implements DataDictionary, ModuleSupportable {
 
 	}
 
+	public int[] examineTriggerNodeAndCols(
+			Visitable actionStmt,
+			String oldReferencingName,
+			String newReferencingName,
+			String triggerDefinition,
+			int[] referencedCols,
+			int[] referencedColsInTriggerAction,
+			int actionOffset,
+			TableDescriptor triggerTableDescriptor,
+			int triggerEventMask,
+            boolean createTriggerTime,
+            List<int[]> replacements
+			) throws StandardException
+	{
+		return null;
+	}
+
 	public String getTriggerActionString(
 			Visitable actionStmt,
 			String oldReferencingName,
@@ -516,7 +535,9 @@ public class EmptyDictionary implements DataDictionary, ModuleSupportable {
 			int actionOffset,
 			TableDescriptor td,
 			int triggerEventMask,
-			boolean createTriggerTime)
+            boolean createTriggerTime,
+            List<int[]> replacements,
+            int[] cols)
 	throws StandardException {
 		// Auto-generated method stub
 		return null;
@@ -534,7 +555,7 @@ public class EmptyDictionary implements DataDictionary, ModuleSupportable {
 		return null;
 	}
 
-	public GenericDescriptorList getTriggerDescriptors(TableDescriptor td)
+    public TriggerDescriptorList getTriggerDescriptors(TableDescriptor td)
 			throws StandardException {
 		// Auto-generated method stub
 		return null;
@@ -553,14 +574,18 @@ public class EmptyDictionary implements DataDictionary, ModuleSupportable {
 
 	}
 
-	public Hashtable hashAllConglomerateDescriptorsByNumber(
-			TransactionController tc) throws StandardException {
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+    public Hashtable<Long, ConglomerateDescriptor>
+        hashAllConglomerateDescriptorsByNumber(TransactionController tc)
+            throws StandardException {
 		// Auto-generated method stub
 		return null;
 	}
 
-	public Hashtable hashAllTableDescriptorsByTableId(TransactionController tc)
-			throws StandardException {
+    @SuppressWarnings("UseOfObsoleteCollectionType")
+    public Hashtable<UUID, TableDescriptor>
+        hashAllTableDescriptorsByTableId(TransactionController tc)
+            throws StandardException {
 		// Auto-generated method stub
 		return null;
 	}
@@ -621,19 +646,20 @@ public class EmptyDictionary implements DataDictionary, ModuleSupportable {
 
 	}
 
-	public List getDependentsDescriptorList(String dependentID)
+	public List<DependencyDescriptor> getDependentsDescriptorList(String dependentID)
 			throws StandardException {
 		// Auto-generated method stub
 		return null;
 	}
 
-	public List getProvidersDescriptorList(String providerID)
+	public List<DependencyDescriptor> getProvidersDescriptorList(String providerID)
 			throws StandardException {
 		// Auto-generated method stub
 		return null;
 	}
 
-	public List getAllDependencyDescriptorsList() throws StandardException {
+    public List<TupleDescriptor> getAllDependencyDescriptorsList()
+            throws StandardException {
 		// Auto-generated method stub
 		return null;
 	}
@@ -673,8 +699,10 @@ public class EmptyDictionary implements DataDictionary, ModuleSupportable {
 		return null;
 	}
 
-	public List getRoutineList(String schemaID, String routineName,
-			char nameSpace) throws StandardException {
+    public List<AliasDescriptor> getRoutineList(String schemaID,
+                                                String routineName,
+                                                char nameSpace)
+            throws StandardException {
 		// Auto-generated method stub
 		return null;
 	}
@@ -743,6 +771,21 @@ public class EmptyDictionary implements DataDictionary, ModuleSupportable {
 		return null;
 	}
 
+	public void computeSequenceRowLocation
+        ( TransactionController tc, String sequenceIDstring, RowLocation[] rowLocation, SequenceDescriptor[] sequenceDescriptor )
+		throws StandardException
+    {
+		// Auto-generated method stub
+    }
+
+    public  boolean updateCurrentSequenceValue
+        ( TransactionController tc, RowLocation rowLocation, boolean wait, Long oldValue, Long newValue )
+        throws StandardException
+    {
+		// Auto-generated method stub
+        return true;
+    }
+               
     public void getCurrentValueAndAdvance
         ( String sequenceUUIDstring, NumberDataValue returnValue )
         throws StandardException
@@ -750,6 +793,12 @@ public class EmptyDictionary implements DataDictionary, ModuleSupportable {
 		// Auto-generated method stub
     }
     
+    public Long peekAtIdentity( String schemaName, String tableName )
+        throws StandardException
+    {
+		return null;
+    }
+    
     public Long peekAtSequence( String schemaName, String sequenceName )
         throws StandardException
     {
@@ -776,7 +825,7 @@ public class EmptyDictionary implements DataDictionary, ModuleSupportable {
 
 	}
 
-	public List getStatisticsDescriptors(TableDescriptor td)
+	public List<StatisticsDescriptor> getStatisticsDescriptors(TableDescriptor td)
 			throws StandardException {
 		// Auto-generated method stub
 		return null;
@@ -960,4 +1009,13 @@ public class EmptyDictionary implements DataDictionary, ModuleSupportable {
             int formatId, byte[] columnBitMap) {
         return null;
     }
+
+    public  BulkInsertCounter   getBulkInsertCounter
+        ( String sequenceUUIDString, boolean restart )
+    { return null; }
+    
+    public  void   flushBulkInsertCounter
+        ( String sequenceUUIDString, BulkInsertCounter bic )
+        throws StandardException
+    {}
 }
