@@ -350,7 +350,7 @@ public class RunList
         // Build command string for RunTest()
         StringBuffer sb = new StringBuffer();
 	    jvm = jvm.getJvm(jvmName);
-	    Vector jvmProps = new Vector();
+	    Vector<String> jvmProps = new Vector<String>();
 	    if ((javaCmd.length()>0) )
 	    {
 	        jvm.setJavaCmd(javaCmd);
@@ -490,7 +490,7 @@ public class RunList
             jvm.setClasspath(classpath);
 
         jvm.setD(jvmProps);
-        Vector v = jvm.getCommandLine();
+        Vector<String> v = jvm.getCommandLine();
         v.addElement("org.apache.derbyTesting.functionTests.harness.RunTest");
 
         String str = "";
@@ -1113,7 +1113,7 @@ public class RunList
 	if (runwithibmjvm != null) 
 	{ 
 	    if (runwithibmjvm.equals("")) { needIBMjvm = null; }
-	    else { needIBMjvm = new Boolean(runwithibmjvm); }
+	    else { needIBMjvm = Boolean.valueOf(runwithibmjvm); }
 	}
 	if (runwithibmjvm == null) { needIBMjvm = null; }
 	if (needIBMjvm != null)
@@ -1212,7 +1212,7 @@ public class RunList
 
 	if (excludeJCC != null)
 	{
-	    Class c = null;
+	    Class<?> c = null;
 	    Method m = null;
 	    Object o = null;
 	    Integer i = null;
@@ -1221,7 +1221,7 @@ public class RunList
 	    try	
 	    {
 		c = Class.forName("com.ibm.db2.jcc.DB2Driver");
-		o = c.newInstance();
+		o = c.getConstructor().newInstance();
 		m = c.getMethod("getMajorVersion", null);
 		i = (Integer)m.invoke(o, null);
 		jccMajor = i.intValue();
