@@ -34,19 +34,15 @@ import java.sql.Time;
 import java.sql.Timestamp;
 import java.sql.Types;
 import java.util.HashMap;
-import java.util.List;
 import java.util.regex.Pattern;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
-import org.apache.derbyTesting.junit.JDBC;
-import org.apache.derbyTesting.junit.TestConfiguration;
-
 import org.apache.derbyTesting.functionTests.tests.compatibility.helpers.DummyBlob;
 import org.apache.derbyTesting.functionTests.tests.compatibility.helpers.DummyClob;
 import org.apache.derbyTesting.junit.BaseJDBCTestSetup;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.DerbyVersion;
+import org.apache.derbyTesting.junit.JDBC;
+import org.apache.derbyTesting.junit.TestConfiguration;
 
 /**
  * A set of client driver compatibility tests.
@@ -140,23 +136,23 @@ public class JDBCDriverTest
     {
         // 10.0 columns
 
-        new Long( 1L ),
+        1L,
         new DummyBlob( SAMPLE_BYTES ),
         SAMPLE_STRING,
         SAMPLE_BYTES,
         new DummyClob( SAMPLE_STRING ),
         new java.sql.Date( 1L ),
         new BigDecimal( 1.0 ),
-        new Double( 1.0 ),
-        new Double( 1.0 ),
-        new Float( (float) 1.0 ),
-        new Double( 1.0 ),
-        new Integer( 1 ),
+        1.0,
+        1.0,
+        (float) 1.0,
+        1.0,
+        1,
         SAMPLE_STRING,
         SAMPLE_BYTES,
         new BigDecimal( 1.0 ),
-        new Float( (float) 1.0 ),
-        new Short( (short) 1 ),
+        (float) 1.0,
+        (short) 1,
         new Time( 1L ),
         new Timestamp( 1L ),
         SAMPLE_STRING,
@@ -333,7 +329,7 @@ public class JDBCDriverTest
         }
         else
         {
-            objectValue = expectedValue ? new Integer( 1 ) : new Integer( 0 );
+            objectValue = expectedValue ? 1 : 0;
         }
         String stringValue = objectValue.toString();
 
@@ -1073,7 +1069,7 @@ public class JDBCDriverTest
     }
     private    void    putCoercionIndex( int index )
     {
-        _coercionIndex.put( new Integer( COERCIONS[ index ].getJdbcType() ), new Integer( index ) );
+        _coercionIndex.put( COERCIONS[ index ].getJdbcType(), index );
     }
 
     //
@@ -1083,7 +1079,7 @@ public class JDBCDriverTest
     {
         if ( _coercionIndex.isEmpty() ) { buildCoercionMap(); }
 
-        return ((Integer) _coercionIndex.get( new Integer( jdbcType ) )).intValue();
+        return ((Integer) _coercionIndex.get( jdbcType )).intValue();
     }
 
     /////////////////////////////////////////////////////////////
@@ -1350,7 +1346,7 @@ public class JDBCDriverTest
      * whose names start with the string "test" in the named classes.
      */
     public static Test suite() {
-        TestSuite testSuite = new TestSuite("JDBCDriverTest suite");
+        BaseTestSuite testSuite = new BaseTestSuite("JDBCDriverTest suite");
         testSuite.addTestSuite(JDBCDriverTest.class);
         return TestConfiguration.defaultExistingServerDecorator(
                 new BaseJDBCTestSetup(testSuite) {
