@@ -27,12 +27,10 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.sql.Types;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
 import org.apache.derbyTesting.functionTests.util.SQLStateConstants;
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.CleanDatabaseTestSetup;
 import org.apache.derbyTesting.junit.JDBC;
 import org.apache.derbyTesting.junit.TestConfiguration;
@@ -42,7 +40,7 @@ import org.apache.derbyTesting.junit.TestConfiguration;
  */
 public class TestDbMetaData extends BaseJDBCTestCase {
 
-    private static  final   Integer FUNCTION_NO_TABLE_VALUE = new Integer( DatabaseMetaData.functionNoTable );
+    private static  final   Integer FUNCTION_NO_TABLE_VALUE = DatabaseMetaData.functionNoTable;
     
     private DatabaseMetaData meta;
 
@@ -81,7 +79,7 @@ public class TestDbMetaData extends BaseJDBCTestCase {
     }
 
     private static Test baseSuite(String name) {
-        TestSuite testSuite = new TestSuite(name);
+        BaseTestSuite testSuite = new BaseTestSuite(name);
         testSuite.addTestSuite(TestDbMetaData.class);
         return new CleanDatabaseTestSetup(testSuite) {
                 protected void decorateSQL(Statement s) throws SQLException {
@@ -91,7 +89,7 @@ public class TestDbMetaData extends BaseJDBCTestCase {
     }
 
     public static Test suite() {
-        TestSuite suite = new TestSuite("TestDbMetaData suite");
+        BaseTestSuite suite = new BaseTestSuite("TestDbMetaData suite");
         suite.addTest(baseSuite("TestDbMetaData:embedded"));
         suite.addTest(TestConfiguration.clientServerDecorator(
             baseSuite("TestDbMetaData:client")));
@@ -113,7 +111,7 @@ public class TestDbMetaData extends BaseJDBCTestCase {
 
     public void testGetClientInfoProperties() throws SQLException {
         ResultSet rs = meta.getClientInfoProperties();
-        JDBC.assertColumnNames(rs, new String[] {
+        JDBC.assertDatabaseMetaDataColumns(rs, null, new String[] {
             "NAME", "MAX_LEN", "DEFAULT_VALUE", "DESCRIPTION" });
         JDBC.assertEmpty(rs);
     }
@@ -132,12 +130,12 @@ public class TestDbMetaData extends BaseJDBCTestCase {
 
     /** Check that the column names are as expected from getFunctions(). */
     private void assertGetFunctionsRs(ResultSet rs) throws SQLException {
-        JDBC.assertColumnNames(rs, new String[] {
+        int[] ColTypes = new int[] {
+                Types.VARCHAR, Types.VARCHAR, Types.VARCHAR, Types.VARCHAR,
+                Types.SMALLINT, Types.VARCHAR };
+        JDBC.assertDatabaseMetaDataColumns(rs, ColTypes, new String[] {
             "FUNCTION_CAT", "FUNCTION_SCHEM", "FUNCTION_NAME", "REMARKS",
             "FUNCTION_TYPE", "SPECIFIC_NAME" });
-        JDBC.assertColumnTypes(rs, new int[] {
-            Types.VARCHAR, Types.VARCHAR, Types.VARCHAR, Types.VARCHAR,
-            Types.SMALLINT, Types.VARCHAR });
     }
     
     private static final JDBC.GeneratedId GENERIC_NAME = new JDBC.GeneratedId();
@@ -152,6 +150,9 @@ public class TestDbMetaData extends BaseJDBCTestCase {
         { null, "SYSCS_UTIL", "SYSCS_CHECK_TABLE",
           "org.apache.derby.catalog.SystemProcedures.SYSCS_CHECK_TABLE",
           FUNCTION_NO_TABLE_VALUE, GENERIC_NAME },
+        { null, "SYSCS_UTIL", "SYSCS_GET_DATABASE_NAME",
+          "org.apache.derby.catalog.SystemProcedures." +
+          "SYSCS_GET_DATABASE_NAME", FUNCTION_NO_TABLE_VALUE, GENERIC_NAME },
         { null, "SYSCS_UTIL", "SYSCS_GET_DATABASE_PROPERTY",
           "org.apache.derby.catalog.SystemProcedures." +
           "SYSCS_GET_DATABASE_PROPERTY", FUNCTION_NO_TABLE_VALUE, GENERIC_NAME },
@@ -167,6 +168,9 @@ public class TestDbMetaData extends BaseJDBCTestCase {
         { null, "SYSCS_UTIL", "SYSCS_GET_XPLAIN_SCHEMA",
           "org.apache.derby.catalog.SystemProcedures." +
           "SYSCS_GET_XPLAIN_SCHEMA", FUNCTION_NO_TABLE_VALUE, GENERIC_NAME },
+        { null, "SYSCS_UTIL", "SYSCS_PEEK_AT_IDENTITY",
+          "org.apache.derby.catalog.SystemProcedures." +
+          "SYSCS_PEEK_AT_IDENTITY", FUNCTION_NO_TABLE_VALUE, GENERIC_NAME },
         { null, "SYSCS_UTIL", "SYSCS_PEEK_AT_SEQUENCE",
           "org.apache.derby.catalog.SystemProcedures." +
           "SYSCS_PEEK_AT_SEQUENCE", FUNCTION_NO_TABLE_VALUE, GENERIC_NAME },
@@ -300,7 +304,7 @@ public class TestDbMetaData extends BaseJDBCTestCase {
     /** Check that the column names are as expected from
      * getFunctionColumns(). */
     private void assertGetFunctionColumnsRs(ResultSet rs) throws SQLException {
-        JDBC.assertColumnNames(rs, new String[] {
+        JDBC.assertDatabaseMetaDataColumns(rs, null, new String[] {
             "FUNCTION_CAT", "FUNCTION_SCHEM", "FUNCTION_NAME", "COLUMN_NAME",
             "COLUMN_TYPE", "DATA_TYPE", "TYPE_NAME", "PRECISION", "LENGTH",
             "SCALE", "RADIX", "NULLABLE", "REMARKS", "CHAR_OCTET_LENGTH",
@@ -315,59 +319,59 @@ public class TestDbMetaData extends BaseJDBCTestCase {
         ResultSet rs = meta.getFunctionColumns(null, null, "DUMMY%", null);
         assertGetFunctionColumnsRs(rs);
         Object[][] expectedRows = {
-            { null, "APP", "DUMMY1", "", new Integer(4), new Integer(5),
-              "SMALLINT", new Integer(5), new Integer(2), new Integer(0),
-              new Integer(10), new Integer(1), null, null, new Integer(0),
-              "YES", GENERIC_NAME, new Integer(1), new Integer(-1) },
-            { null, "APP", "DUMMY1", "X", new Integer(1), new Integer(5),
-              "SMALLINT", new Integer(5), new Integer(2), new Integer(0),
-              new Integer(10), new Integer(1), null, null, new Integer(1),
-              "YES", GENERIC_NAME, new Integer(1), new Integer(0) },
-            { null, "APP", "DUMMY2", "", new Integer(4), new Integer(4),
-              "INTEGER", new Integer(10), new Integer(4), new Integer(0),
-              new Integer(10), new Integer(1), null, null, new Integer(0),
-              "YES", GENERIC_NAME, new Integer(2), new Integer(-1) },
-            { null, "APP", "DUMMY2", "X", new Integer(1), new Integer(4),
-              "INTEGER", new Integer(10), new Integer(4), new Integer(0),
-              new Integer(10), new Integer(1), null, null, new Integer(1),
-              "YES", GENERIC_NAME, new Integer(2), new Integer(0) },
-            { null, "APP", "DUMMY2", "Y", new Integer(1), new Integer(5),
-              "SMALLINT", new Integer(5), new Integer(2), new Integer(0),
-              new Integer(10), new Integer(1), null, null, new Integer(2),
-              "YES", GENERIC_NAME, new Integer(2), new Integer(1) },
-            { null, "APP", "DUMMY3", "", new Integer(4), new Integer(12),
-              "VARCHAR", new Integer(16), new Integer(32), null, null,
-              new Integer(1), null, 32, new Integer(0), "YES",
-              GENERIC_NAME, new Integer(2), new Integer(-1) },
-            { null, "APP", "DUMMY3", "X", new Integer(1), new Integer(12),
-              "VARCHAR", new Integer(16), new Integer(32), null, null,
-              new Integer(1), null, 32, new Integer(1), "YES",
-              GENERIC_NAME, new Integer(2), new Integer(0) },
-            { null, "APP", "DUMMY3", "Y", new Integer(1), new Integer(4),
-              "INTEGER", new Integer(10), new Integer(4), new Integer(0),
-              new Integer(10), new Integer(1), null, null, new Integer(2),
-              "YES", GENERIC_NAME, new Integer(2), new Integer(1) },
-            { null, "APP", "DUMMY4", "", new Integer(4), new Integer(4),
-              "INTEGER", new Integer(10), new Integer(4), new Integer(0),
-              new Integer(10), new Integer(1), null, null, new Integer(0),
-              "YES", GENERIC_NAME, new Integer(2), new Integer(-1) },
-            { null, "APP", "DUMMY4", "X", new Integer(1), new Integer(12),
-              "VARCHAR", new Integer(128), new Integer(256),
-              null, null, new Integer(1), null, 256, new Integer(1), "YES",
+            { null, "APP", "DUMMY1", "", 4, 5,
+              "SMALLINT", 5, 2, 0,
+              10, 1, null, null, 0,
+              "YES", GENERIC_NAME, 1, -1 },
+            { null, "APP", "DUMMY1", "X", 1, 5,
+              "SMALLINT", 5, 2, 0,
+              10, 1, null, null, 1,
+              "YES", GENERIC_NAME, 1, 0 },
+            { null, "APP", "DUMMY2", "", 4, 4,
+              "INTEGER", 10, 4, 0,
+              10, 1, null, null, 0,
+              "YES", GENERIC_NAME, 2, -1 },
+            { null, "APP", "DUMMY2", "X", 1, 4,
+              "INTEGER", 10, 4, 0,
+              10, 1, null, null, 1,
+              "YES", GENERIC_NAME, 2, 0 },
+            { null, "APP", "DUMMY2", "Y", 1, 5,
+              "SMALLINT", 5, 2, 0,
+              10, 1, null, null, 2,
+              "YES", GENERIC_NAME, 2, 1 },
+            { null, "APP", "DUMMY3", "", 4, 12,
+              "VARCHAR", 16, 32, null, null,
+              1, null, 32, 0, "YES",
+              GENERIC_NAME, 2, -1 },
+            { null, "APP", "DUMMY3", "X", 1, 12,
+              "VARCHAR", 16, 32, null, null,
+              1, null, 32, 1, "YES",
+              GENERIC_NAME, 2, 0 },
+            { null, "APP", "DUMMY3", "Y", 1, 4,
+              "INTEGER", 10, 4, 0,
+              10, 1, null, null, 2,
+              "YES", GENERIC_NAME, 2, 1 },
+            { null, "APP", "DUMMY4", "", 4, 4,
+              "INTEGER", 10, 4, 0,
+              10, 1, null, null, 0,
+              "YES", GENERIC_NAME, 2, -1 },
+            { null, "APP", "DUMMY4", "X", 1, 12,
+              "VARCHAR", 128, 256,
+              null, null, 1, null, 256, 1, "YES",
               GENERIC_NAME,
-              new Integer(2), new Integer(0) },
-            { null, "APP", "DUMMY4", "Y", new Integer(1), new Integer(4),
-              "INTEGER", new Integer(10), new Integer(4), new Integer(0),
-              new Integer(10), new Integer(1), null, null, new Integer(2),
-              "YES", GENERIC_NAME, new Integer(2), new Integer(1) },
-            { null, "APP", "DUMMY5", "", new Integer(4), new Integer(16),
-              "BOOLEAN", new Integer(1), new Integer(1), null,
-              null, new Integer(1), null, null, new Integer(0),
-              "YES", GENERIC_NAME, new Integer(1), new Integer(-1) },
-            { null, "APP", "DUMMY5", "X", new Integer(1), new Integer(16),
-              "BOOLEAN", new Integer(1), new Integer(1), null,
-              null, new Integer(1), null, null, new Integer(1),
-              "YES", GENERIC_NAME, new Integer(1), new Integer(0) },
+              2, 0 },
+            { null, "APP", "DUMMY4", "Y", 1, 4,
+              "INTEGER", 10, 4, 0,
+              10, 1, null, null, 2,
+              "YES", GENERIC_NAME, 2, 1 },
+            { null, "APP", "DUMMY5", "", 4, 16,
+              "BOOLEAN", 1, 1, null,
+              null, 1, null, null, 0,
+              "YES", GENERIC_NAME, 1, -1 },
+            { null, "APP", "DUMMY5", "X", 1, 16,
+              "BOOLEAN", 1, 1, null,
+              null, 1, null, null, 1,
+              "YES", GENERIC_NAME, 1, 0 },
         };
         JDBC.assertFullResultSet(rs, expectedRows, false);
     }
@@ -377,34 +381,34 @@ public class TestDbMetaData extends BaseJDBCTestCase {
         ResultSet rs = meta.getFunctionColumns(null, null, "DUMMY%", "");
         assertGetFunctionColumnsRs(rs);
         Object[][] expectedRows = {
-            { null, "APP", "DUMMY1", "", new Integer(4), new Integer(5),
-              "SMALLINT", new Integer(5), new Integer(2), new Integer(0),
-              new Integer(10), new Integer(1), null, null, new Integer(0),
-              "YES", GENERIC_NAME, new Integer(1), new Integer(-1) },
-            { null, "APP", "DUMMY2", "", new Integer(4), new Integer(4),
-              "INTEGER", new Integer(10), new Integer(4), new Integer(0),
-              new Integer(10), new Integer(1), null, null, new Integer(0),
-              "YES", GENERIC_NAME, new Integer(2), new Integer(-1) },
-            { null, "APP", "DUMMY3", "", new Integer(4), new Integer(12),
-              "VARCHAR", new Integer(16), new Integer(32),
-              null, null, new Integer(1), null, 32, new Integer(0), "YES",
+            { null, "APP", "DUMMY1", "", 4, 5,
+              "SMALLINT", 5, 2, 0,
+              10, 1, null, null, 0,
+              "YES", GENERIC_NAME, 1, -1 },
+            { null, "APP", "DUMMY2", "", 4, 4,
+              "INTEGER", 10, 4, 0,
+              10, 1, null, null, 0,
+              "YES", GENERIC_NAME, 2, -1 },
+            { null, "APP", "DUMMY3", "", 4, 12,
+              "VARCHAR", 16, 32,
+              null, null, 1, null, 32, 0, "YES",
               GENERIC_NAME,
-              new Integer(2), new Integer(-1) },
-            { null, "APP", "DUMMY4", "", new Integer(4), new Integer(4),
-              "INTEGER", new Integer(10), new Integer(4), new Integer(0),
-              new Integer(10), new Integer(1), null, null, new Integer(0),
-              "YES", GENERIC_NAME, new Integer(2), new Integer(-1) },
-            { null, "APP", "DUMMY5", "", new Integer(4), new Integer(16),
-              "BOOLEAN", new Integer(1), new Integer(1), null,
-              null, new Integer(1), null, null, new Integer(0),
-              "YES", GENERIC_NAME, new Integer(1), new Integer(-1) },
+              2, -1 },
+            { null, "APP", "DUMMY4", "", 4, 4,
+              "INTEGER", 10, 4, 0,
+              10, 1, null, null, 0,
+              "YES", GENERIC_NAME, 2, -1 },
+            { null, "APP", "DUMMY5", "", 4, 16,
+              "BOOLEAN", 1, 1, null,
+              null, 1, null, null, 0,
+              "YES", GENERIC_NAME, 1, -1 },
         };
         JDBC.assertFullResultSet(rs, expectedRows, false);
     }
 
     /** Check that the column names are as expected from getSchemas(). */
     private void assertGetSchemasRs(ResultSet rs) throws SQLException {
-        JDBC.assertColumnNames(rs, new String[] {
+        JDBC.assertDatabaseMetaDataColumns(rs, null, new String[] {
             "TABLE_SCHEM", "TABLE_CATALOG" });
     }
 
