@@ -29,15 +29,12 @@ import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.Properties;
-
 import javax.naming.Reference;
 import javax.naming.Referenceable;
 import javax.naming.spi.ObjectFactory;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.J2EEDataSource;
 import org.apache.derbyTesting.junit.JDBC;
 import org.apache.derbyTesting.junit.JDBCDataSource;
@@ -145,7 +142,7 @@ public class DataSourceReferenceTest
        Test suite;
        if (JDBC.vmSupportsJSR169() || !JDBC.vmSupportsJNDI()) {
             // Referenceable is not supported with JSR169 or without JNDI
-            suite = new TestSuite(
+            suite = new BaseTestSuite(
                     "DatasourceReferenceTest cannot run with JSR169");
         } else {
             suite = TestConfiguration.defaultSuite(
@@ -223,7 +220,8 @@ public class DataSourceReferenceTest
                                         String className)
         throws Exception {
         // Instantiate a new data source object and get all its properties.
-        Object dsObj = Class.forName(className).newInstance();
+        Class<?> clazz = Class.forName(className);
+        Object dsObj = clazz.getConstructor().newInstance();
         String[] properties = getPropertyBeanList(dsObj);
         // Validate property set (existence and naming).
         assertDataSourceProperties(dsDesc, properties);
@@ -293,12 +291,14 @@ public class DataSourceReferenceTest
             throws Exception {
         println("Testing recreated empty data source.");
         // Create an empty data source.
-        Object ds = Class.forName(className).newInstance();
+        Class<?> clazz = Class.forName(className);
+        Object ds = clazz.getConstructor().newInstance();
         Referenceable refDs = (Referenceable)ds;
         Reference dsAsReference = refDs.getReference();
         String factoryClassName = dsAsReference.getFactoryClassName();
+        clazz = Class.forName(factoryClassName);
         ObjectFactory factory =
-            (ObjectFactory)Class.forName(factoryClassName).newInstance();
+            (ObjectFactory) clazz.getConstructor().newInstance();
         Object recreatedDs =
             factory.getObjectInstance(dsAsReference, null, null, null);
         // Empty, recreated data source should not be the same as the one we
@@ -336,7 +336,8 @@ public class DataSourceReferenceTest
                                                 String className)
             throws Exception {
         println("Testing recreated populated data source.");
-        Object ds = Class.forName(className).newInstance();
+        Class<?> clazz = Class.forName(className);
+        Object ds = clazz.getConstructor().newInstance();
         // Populate the data source.
         Iterator propIter = dsDesc.getPropertyIterator();
         while (propIter.hasNext()) {
@@ -364,8 +365,9 @@ public class DataSourceReferenceTest
         Referenceable refDs = (Referenceable)ds;
         Reference dsAsReference = refDs.getReference();
         String factoryClassName = dsAsReference.getFactoryClassName();
+        clazz = Class.forName(factoryClassName);
         ObjectFactory factory =
-            (ObjectFactory)Class.forName(factoryClassName).newInstance();
+            (ObjectFactory) clazz.getConstructor().newInstance();
         Object recreatedDs =
             factory.getObjectInstance(dsAsReference, null, null, null);
         // Recreated should not be same instance as original.
@@ -447,7 +449,7 @@ public class DataSourceReferenceTest
      */
     private static String[] getPropertyBeanList(Object ds) {
         Method[] allMethods = ds.getClass().getMethods();
-        ArrayList properties = new ArrayList();
+        ArrayList<String> properties = new ArrayList<String>();
 
         for (int i = 0; i < allMethods.length; i++) {
             Method method = allMethods[i];
@@ -477,7 +479,7 @@ public class DataSourceReferenceTest
             }
         }
 
-        return (String[])properties.toArray(new String[properties.size()]);
+        return properties.toArray(new String[properties.size()]);
     }
 
     /**
