@@ -32,8 +32,8 @@ import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 import javax.sql.DataSource;
 import junit.framework.Test;
-import junit.framework.TestSuite;
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.Derby;
 import org.apache.derbyTesting.junit.JDBC;
 import org.apache.derbyTesting.junit.SupportFilesSetup;
@@ -68,6 +68,8 @@ public class DataSourceSerializationTest
     private static final String VERSION_10_3_2_1 = "10_3_2_1";
     /** Constant for Derby version 10.10.1.0. */
     private static final String VERSION_10_10_1_0 = "10_10_1_0";
+    /** Constant for Derby version 10.11.1.0. */
+    private static final String VERSION_10_11_1_0 = "10_11_1_0";
     private final String _40Suffix = "40";
 
     public DataSourceSerializationTest(String name) {
@@ -233,13 +235,14 @@ public class DataSourceSerializationTest
         fname.append('-');
         fname.append(version);
         fname.append(".ser");
+        println( "Deserializing " + fname.toString() );
 
         // De-serialize the data source.
         InputStream is;
         try {
-            is = (FileInputStream)AccessController.doPrivileged(
-                  new PrivilegedExceptionAction() {
-                public Object run() throws FileNotFoundException {
+            is = AccessController.doPrivileged(
+                  new PrivilegedExceptionAction<InputStream>() {
+                public InputStream run() throws FileNotFoundException {
                     return new FileInputStream(
                             SupportFilesSetup.getReadOnly(fname.toString()));
                 }
@@ -303,8 +306,8 @@ public class DataSourceSerializationTest
                     "getFactoryClassName", null);
             String factoryClassName =
                     (String)getFactoryClassName.invoke(dsRef, null);
-            Object factory =
-                    Class.forName(factoryClassName).newInstance();
+            Class<?> clazz = Class.forName(factoryClassName);
+            Object factory = clazz.getConstructor().newInstance();
             Method getObjectInstance =
                     factory.getClass().getMethod("getObjectInstance",
                     new Class[] {
@@ -328,7 +331,9 @@ public class DataSourceSerializationTest
      * @return A test suite.
      */
     public static Test suite() {
-        TestSuite suite = new TestSuite("DataSourceSerializationTest");
+        BaseTestSuite suite =
+            new BaseTestSuite("DataSourceSerializationTest");
+
         String filePrefix = "functionTests/testData/serializedDataSources/";
         // De-serialize embedded data sources only if we have the engine code.
         if (Derby.hasEmbedded()) {
@@ -401,6 +406,28 @@ public class DataSourceSerializationTest
                 filePrefix +
                     "BasicClientConnectionPoolDataSource40-10_10_1_0.ser",
                 filePrefix + "BasicClientXADataSource40-10_10_1_0.ser",
+
+                // 10.11 resources
+                filePrefix + "EmbeddedDataSource-10_11_1_0.ser",
+                filePrefix + "EmbeddedDataSource40-10_11_1_0.ser",
+                filePrefix + "EmbeddedConnectionPoolDataSource-10_11_1_0.ser",
+                filePrefix + "EmbeddedConnectionPoolDataSource40-10_11_1_0.ser",
+                filePrefix + "EmbeddedXADataSource-10_11_1_0.ser",
+                filePrefix + "EmbeddedXADataSource40-10_11_1_0.ser",
+                filePrefix + "ClientDataSource-10_11_1_0.ser",
+                filePrefix + "ClientDataSource40-10_11_1_0.ser",
+                filePrefix + "ClientConnectionPoolDataSource-10_11_1_0.ser",
+                filePrefix + "ClientConnectionPoolDataSource40-10_11_1_0.ser",
+                filePrefix + "ClientXADataSource-10_11_1_0.ser",
+                filePrefix + "ClientXADataSource40-10_11_1_0.ser",
+                filePrefix + "BasicEmbeddedDataSource40-10_11_1_0.ser",
+                filePrefix +
+                    "BasicEmbeddedConnectionPoolDataSource40-10_11_1_0.ser",
+                filePrefix + "BasicEmbeddedXADataSource40-10_11_1_0.ser",
+                filePrefix + "BasicClientDataSource40-10_11_1_0.ser",
+                filePrefix +
+                    "BasicClientConnectionPoolDataSource40-10_11_1_0.ser",
+                filePrefix + "BasicClientXADataSource40-10_11_1_0.ser",
             });
     }
 }
