@@ -22,7 +22,6 @@
 package org.apache.derbyTesting.functionTests.tests.jdbcapi;
 
 import java.io.File;
-import java.security.AccessController;
 import java.sql.CallableStatement;
 import java.sql.Connection;
 import java.sql.ResultSet;
@@ -31,10 +30,10 @@ import java.sql.Statement;
 import java.util.Hashtable;
 import javax.sql.DataSource;
 import junit.framework.Test;
-import junit.framework.TestSuite;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derby.jdbc.ClientDataSourceInterface;
-import org.apache.derby.jdbc.EmbeddedSimpleDataSource;
 import org.apache.derby.jdbc.BasicEmbeddedDataSource40;
+import org.apache.derbyTesting.functionTests.util.PrivilegedFileOpsForTests;
 import org.apache.derbyTesting.functionTests.util.SecurityCheck;
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
 import org.apache.derbyTesting.junit.CleanDatabaseTestSetup;
@@ -66,7 +65,8 @@ public class DataSourceTest extends BaseJDBCTestCase {
      * the connections are closed when we are done, so they are stored
      * in this hashtable
      */
-    protected static Hashtable conns = new Hashtable();
+    protected static Hashtable<String, Connection> conns =
+            new Hashtable<String, Connection>();
     
     /** The expected format of a connection string. In English:
      * "<classname>@<hashcode> (XID=<xid>), (SESSION = <sessionid>),
@@ -96,7 +96,9 @@ public class DataSourceTest extends BaseJDBCTestCase {
      * @return A suite of tests to be run with client and/or embedded
      */
     private static Test baseSuite(String postfix) {
-        TestSuite suite = new TestSuite("ClientAndEmbedded" + postfix);
+        BaseTestSuite
+            suite = new BaseTestSuite("ClientAndEmbedded" + postfix);
+
         suite.addTest(new DataSourceTest("testBadConnectionAttributeSyntax"));
         suite.addTest(new DataSourceTest("testDescriptionProperty"));
         suite.addTest(new DataSourceTest("testAllDataSources"));
@@ -110,7 +112,7 @@ public class DataSourceTest extends BaseJDBCTestCase {
      * @return A suite of tests being run with client only
      */
     private static Test getClientSuite() {
-        TestSuite suite = new TestSuite("Client/Server");
+        BaseTestSuite suite = new BaseTestSuite("Client/Server");
         suite.addTest(new DataSourceTest("testClientDSConnectionAttributes"));
         suite.addTest(new DataSourceTest(
                 "testClientTraceFileDSConnectionAttribute"));
@@ -126,13 +128,13 @@ public class DataSourceTest extends BaseJDBCTestCase {
      * @return A suite of tests being run with embedded only
      */
     private static Test getEmbeddedSuite(String postfix) {
-        TestSuite suite = new TestSuite("Embedded" + postfix);
+        BaseTestSuite suite = new BaseTestSuite("Embedded" + postfix);
         suite.addTest(new DataSourceTest("testDSRequestAuthentication"));
         return suite;
     }
     
     public static Test suite() {
-        TestSuite suite = new TestSuite("DataSourceTest suite");
+        BaseTestSuite suite = new BaseTestSuite("DataSourceTest suite");
         // Add the tests that only run with embedded
         suite.addTest(getEmbeddedSuite("embedded"));
         // Add tests that will run with embedded
@@ -162,34 +164,27 @@ public class DataSourceTest extends BaseJDBCTestCase {
     
     public void tearDown() throws Exception {
         // attempt to get rid of any left-over trace files
-        AccessController.doPrivileged(new java.security.PrivilegedAction() {
-            public Object run() {
-                for (int i=0 ; i < 6 ; i++)
-                {   
-                    String traceFileName = "trace" + (i+1) + ".out";
-                    File traceFile = new File(traceFileName);
-                    if (traceFile.exists())
-                    {
-                        // if it exists, attempt to get rid of it
-                        traceFile.delete();
-                    }
-                } 
-                return null;
+        for (int i = 0; i < 6; i++) {
+            String traceFileName = "trace" + (i + 1) + ".out";
+            File traceFile = new File(traceFileName);
+            if (PrivilegedFileOpsForTests.exists(traceFile)) {
+                // if it exists, attempt to get rid of it
+                PrivilegedFileOpsForTests.delete(traceFile);
             }
-        });
+        }
         super.tearDown();
     }
 
     // the J2EEDataSourceTest version of this includes testing for
     // setTypeMap, which is supported for both EmbeddedDataSource and
-    // ClientDataSource, but not for JSR169/EmbeddedSimpleDataSource
+    // ClientDataSource
     public void testAllDataSources() throws SQLException, Exception
     {
         //Connection dmc = getConnection();
         
         Object[] expectedValues = {
-            new Integer(ResultSet.HOLD_CURSORS_OVER_COMMIT), "XJ010",
-            new Integer(2), new Boolean(true), new Boolean(false)};
+            ResultSet.HOLD_CURSORS_OVER_COMMIT, "XJ010",
+            2, true, false};
 
         if (usingEmbedded())
             assertTenConnectionsUnique();
@@ -201,19 +196,6 @@ public class DataSourceTest extends BaseJDBCTestCase {
         DataSource ds = dscs;
         assertConnectionOK(expectedValues, "DataSource", ds.getConnection());
         
-        DataSource dssimple = null;
-        // simple datasource is only supported with embedded
-        if (usingEmbedded())
-        {
-            EmbeddedSimpleDataSource realdssimple = 
-                new EmbeddedSimpleDataSource();
-            realdssimple.setDatabaseName(dbName);
-            ds = realdssimple;
-            dssimple = (DataSource)realdssimple;
-            assertConnectionOK(
-                expectedValues, "SimpleDataSource", ds.getConnection());
-        }
-
         if (JDBC.vmSupportsJDBC4()) {
             BasicEmbeddedDataSource40 nds = new BasicEmbeddedDataSource40();
             nds.setDatabaseName(dbName);
@@ -269,14 +251,15 @@ public class DataSourceTest extends BaseJDBCTestCase {
         
         ClientDataSourceInterface ds = null;
 
+        Class<?> clazz;
+
         if (JDBC.vmSupportsJNDI()) {
             // Use reflection to avoid class not found in non-JNDI context
-            ds = (ClientDataSourceInterface)Class.forName(
-                    "org.apache.derby.jdbc.ClientDataSource").newInstance();
+            clazz = Class.forName("org.apache.derby.jdbc.ClientDataSource");
+            ds = (ClientDataSourceInterface)clazz.getConstructor().newInstance();
         } else {
-            ds = (ClientDataSourceInterface)Class.forName(
-                    "org.apache.derby.jdbc.BasicClientDataSource40").
-                    newInstance();
+            clazz = Class.forName("org.apache.derby.jdbc.BasicClientDataSource40");
+            ds = (ClientDataSourceInterface) clazz.getConstructor().newInstance();
         }
 
         ds.setPortNumber(TestConfiguration.getCurrent().getPort());
@@ -321,8 +304,10 @@ public class DataSourceTest extends BaseJDBCTestCase {
         JDBCClient dsclient = getTestConfiguration().getJDBCClient();
         String dsName = dsclient.getDataSourceClassName();
         DataSource ds = null;
+        Class<?> clazz;
         try {
-            ds = (javax.sql.DataSource) Class.forName(dsName).newInstance();
+            clazz = Class.forName(dsName);
+            ds = (javax.sql.DataSource) clazz.getConstructor().newInstance();
         } catch (Exception e) {
             fail("unable to complete test because unable to create new instance of datasource");
         }
@@ -410,22 +395,11 @@ public class DataSourceTest extends BaseJDBCTestCase {
      */
     private static void assertTraceFilesExist() 
     {
-        AccessController.doPrivileged(new java.security.PrivilegedAction() {
-            public Object run() {
-                for (int i=0 ; i < 2 ; i++)
-                {   
-                    String traceFileName = "trace" + (i+1) + ".out";
-                    File traceFile = new File(traceFileName);
-                    if (i == 2)
-                        continue;
-                    else
-                    {
-                        assertTrue(traceFile.exists());
-                    }
-                } 
-                return null;
-            }
-        });
+        for (int i = 0; i < 2; i++) {
+            String traceFileName = "trace" + (i + 1) + ".out";
+            File traceFile = new File(traceFileName);
+            assertTrue(PrivilegedFileOpsForTests.exists(traceFile));
+        }
     }
 
     /**
@@ -452,13 +426,14 @@ public class DataSourceTest extends BaseJDBCTestCase {
         // DataSource - retrieveMessageTextProperty
         ClientDataSourceInterface ds = null;
 
+        Class<?> clazz;
         if (JDBC.vmSupportsJNDI()) {
             // Use reflection to avoid class not found in non-JNDI context
-            ds = (ClientDataSourceInterface)Class.forName(
-              "org.apache.derby.jdbc.ClientDataSource").newInstance();
+            clazz = Class.forName("org.apache.derby.jdbc.ClientDataSource");
+            ds = (ClientDataSourceInterface) clazz.getConstructor().newInstance();
         } else {
-            ds = (ClientDataSourceInterface)Class.forName(
-              "org.apache.derby.jdbc.BasicClientDataSource40").newInstance();
+            clazz = Class.forName("org.apache.derby.jdbc.BasicClientDataSource40");
+            ds = (ClientDataSourceInterface)clazz.getConstructor().newInstance();
         }
 
         ds.setPortNumber(TestConfiguration.getCurrent().getPort());
@@ -797,10 +772,7 @@ public class DataSourceTest extends BaseJDBCTestCase {
      */
     private static void clearConnections() throws SQLException
     {
-        java.util.Iterator it = conns.values().iterator();
-        while ( it.hasNext() )
-        {
-            Connection conn = (Connection)it.next();
+        for (Connection conn : conns.values()) {
             conn.close();
         }
         conns.clear();
