@@ -19,22 +19,15 @@
 
 package org.apache.derbyTesting.functionTests.tests.jdbcapi;
 
-import java.io.File;
 import java.lang.reflect.Method;
-import java.security.AccessController;
 import java.sql.SQLException;
-
-import javax.sql.DataSource;
 import javax.sql.ConnectionPoolDataSource;
 import javax.sql.XADataSource;
-
 import junit.extensions.TestSetup;
 import junit.framework.Assert;
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
-import org.apache.derbyTesting.junit.BaseTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.J2EEDataSource;
 import org.apache.derbyTesting.junit.JDBCDataSource;
 import org.apache.derbyTesting.junit.TestConfiguration;
@@ -64,7 +57,7 @@ public class PoolXADSCreateShutdownDBTest extends BaseJDBCTestCase {
 
     public static Test suite() 
     {
-        TestSuite suite = new TestSuite("PoolXADSCreateShutdownTest"); 
+        BaseTestSuite suite = new BaseTestSuite("PoolXADSCreateShutdownTest");
         Test test = TestConfiguration.defaultSuite(PoolXADSCreateShutdownDBTest.class);        
         //Test test = TestConfiguration.clientServerSuite(DSCreateShutdownDBTest.class);
         suite.addTest(test);
@@ -86,62 +79,11 @@ public class PoolXADSCreateShutdownDBTest extends BaseJDBCTestCase {
         // attempt to get rid of any databases. 
         // only 5 dbs (in addition to the defaultdb) should actually get
         // created, but just in case...
-        AccessController.doPrivileged(new java.security.PrivilegedAction() {
-            public Object run() {
-                for (int i=0 ; i < ADDITIONAL_DBS.length ; i++)
-                {   
-                    removeDatabase("emb" + ADDITIONAL_DBS[i]);
-                    removeDatabase("srv" + ADDITIONAL_DBS[i]);
-                } 
-                return null;
-            }
-            
-            void removeDatabase(String dbName)
-            {
-                //TestConfiguration config = TestConfiguration.getCurrent();
-                dbName = dbName.replace('/', File.separatorChar);
-                String dsh = BaseTestCase.getSystemProperty("derby.system.home");
-                if (dsh == null) {
-                    fail("not implemented");
-                } else {
-                    dbName = dsh + File.separator + dbName;
-                }
-                removeDirectory(dbName);
-            }
-
-            void removeDirectory(String path)
-            {
-                final File dir = new File(path);
-                removeDir(dir);
-            }
-
-            private void removeDir(File dir) {
-                
-                // Check if anything to do!
-                // Database may not have been created.
-                if (!dir.exists())
-                    return;
-
-                String[] list = dir.list();
-
-                // Some JVMs return null for File.list() when the
-                // directory is empty.
-                if (list != null) {
-                    for (int i = 0; i < list.length; i++) {
-                        File entry = new File(dir, list[i]);
-
-                        if (entry.isDirectory()) {
-                            removeDir(entry);
-                        } else {
-                            entry.delete();
-                            //assertTrue(entry.getPath(), entry.delete());
-                        }
-                    }
-                }
-                dir.delete();
-                //assertTrue(dir.getPath(), dir.delete());
-            }
-        });
+        TestConfiguration conf = TestConfiguration.getCurrent();
+        for (int i = 0; i < ADDITIONAL_DBS.length; i++) {
+            removeDirectory(conf.getDatabasePath("emb" + ADDITIONAL_DBS[i]));
+            removeDirectory(conf.getDatabasePath("srv" + ADDITIONAL_DBS[i]));
+        }
         super.tearDown();
     }
 
