@@ -36,18 +36,15 @@ import java.sql.Statement;
 import java.sql.Time;
 import java.sql.Timestamp;
 import java.sql.Types;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
 import org.apache.derby.iapi.types.HarmonySerialBlob;
 import org.apache.derby.iapi.types.HarmonySerialClob;
-
+import org.apache.derbyTesting.functionTests.tests.lang.Price;
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.CleanDatabaseTestSetup;
 import org.apache.derbyTesting.junit.JDBC;
 import org.apache.derbyTesting.junit.TestConfiguration;
-import org.apache.derbyTesting.functionTests.tests.lang.Price;
 
 /**
  * Tests of stored procedures.
@@ -858,7 +855,7 @@ public class ProcedureTest extends BaseJDBCTestCase {
      * @param i INOUT parameter that gets set to 10 by the procedure
      */
     public static void proc_2516(Integer[] i) {
-        i[0] = new Integer(10);
+        i[0] = 10;
     }
 
     /**
@@ -1032,7 +1029,7 @@ public class ProcedureTest extends BaseJDBCTestCase {
      * @return test suite
      */
     public static Test suite() {
-        TestSuite suite = new TestSuite("ProcedureTest");
+        BaseTestSuite suite = new BaseTestSuite("ProcedureTest");
 
         suite.addTest(baseSuite("ProcedureTest:embedded"));
 
@@ -1049,7 +1046,7 @@ public class ProcedureTest extends BaseJDBCTestCase {
      */
     private static Test baseSuite(String name)
     {
-        TestSuite suite = new TestSuite(name);
+        BaseTestSuite suite = new BaseTestSuite(name);
         
         // Need JDBC DriverManager to run these tests
         if (!JDBC.vmSupportsJDBC3())
@@ -1295,7 +1292,7 @@ public class ProcedureTest extends BaseJDBCTestCase {
     {
         return new AllTypesTuple
             (
-             new Long( 1L ),
+             1L,
              new HarmonySerialBlob( new byte[] { (byte) 1, (byte) 1, (byte) 1 } ),
              Boolean.TRUE,
              "firstt",
@@ -1303,12 +1300,12 @@ public class ProcedureTest extends BaseJDBCTestCase {
              new HarmonySerialClob( "firstt" ),
              new Date( 1L ),
              new BigDecimal( "1" ),
-             new Double( 1.0 ),
-             new Integer( 1 ),
+             1.0,
+             1 ,
              new String( "firstt" ),
              new byte[] { (byte) 1, (byte) 1, (byte) 1 },
-             new Float( 1.0F ),
-             new Integer( 1 ),
+             1.0F,
+             1,
              new Time( 1L ),
              new Timestamp( 1L ),
              new Price( "USD", new BigDecimal( "1" ), new Timestamp( 1 ) ),
@@ -1320,7 +1317,7 @@ public class ProcedureTest extends BaseJDBCTestCase {
     {
         return new AllTypesTuple
             (
-             new Long( 2L ),
+             2L,
              new HarmonySerialBlob( new byte[] { (byte) 2, (byte) 2, (byte) 2 } ),
              Boolean.FALSE,
              "second",
@@ -1328,12 +1325,12 @@ public class ProcedureTest extends BaseJDBCTestCase {
              new HarmonySerialClob( "second" ),
              new Date( 2L ),
              new BigDecimal( "2" ),
-             new Double( 2.0 ),
-             new Integer( 2 ),
+             2.0,
+             2,
              new String( "second" ),
              new byte[] { (byte) 2, (byte) 2, (byte) 2 },
-             new Float( 2.0F ),
-             new Integer( 2 ),
+             2.0F,
+             2,
              new Time( 2L ),
              new Timestamp( 2L ),
              new Price( "USD", new BigDecimal( "2" ), new Timestamp( 2 ) ),
