@@ -27,12 +27,10 @@ import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.sql.Statement;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
 import org.apache.derbyTesting.functionTests.util.SecurityCheck;
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.CleanDatabaseTestSetup;
 import org.apache.derbyTesting.junit.DatabasePropertyTestSetup;
 import org.apache.derbyTesting.junit.TestConfiguration;
@@ -272,11 +270,7 @@ public class ResultSetMiscTest extends BaseJDBCTestCase {
         assertFalse(met.isAutoIncrement(10));
         assertFalse(met.isCaseSensitive(10));
         assertTrue(met.isSearchable(10));
-        // DERBY-2423 Embedded and client differ on isCurrency() for
-        // DECIMAL and NUMERIC columns. Enable for embedded once the
-        // issue is fixed
-        if (usingDerbyNetClient())
-            assertFalse(met.isCurrency(10));
+        assertFalse(met.isCurrency(10));
         assertEquals(ResultSetMetaData.columnNullable, met.isNullable(10));
         assertTrue(met.isSigned(10));
         assertEquals(12, met.getColumnDisplaySize(10));
@@ -457,7 +451,7 @@ public class ResultSetMiscTest extends BaseJDBCTestCase {
     }
 
     /**
-     * Test fix for Bug4810 -Connection.commit() & rollback() do not
+     * Test fix for Bug4810 -Connection.commit() and rollback() do not
      * commit/rollback in auto-commit mode.
      */
     public void testBug4810() throws SQLException {
@@ -484,6 +478,31 @@ public class ResultSetMiscTest extends BaseJDBCTestCase {
         stmt.close();
     }
 
+     /**
+     * Test fix for DERBY-853 ResultSetMetaData.getScale 
+     *returns inconsistent values for DOUBLE type.
+     */
+    public void testDerby853() throws SQLException {
+        Connection con = getConnection();
+	
+        Statement stmt = con.createStatement();
+        stmt.executeUpdate("create table derby853(d1 decimal(10,2), d2 double)");
+      
+        ResultSet rs = stmt.executeQuery("select d1 - d2 from derby853");
+        ResultSetMetaData rsmd = rs.getMetaData();
+	assertEquals(0, rsmd.getScale(1));
+	assertEquals(15, rsmd.getPrecision(1));
+
+	rs = stmt.executeQuery("select d2 - d1 from derby853");
+	rsmd = rs.getMetaData();
+	assertEquals(0, rsmd.getScale(1));
+	assertEquals(15, rsmd.getPrecision(1));
+
+        stmt.execute("drop table derby853");
+       
+        stmt.close();
+    }
+
     /**
      * Setup up and run the auto-commit tests.
      * 
@@ -744,7 +763,7 @@ public class ResultSetMiscTest extends BaseJDBCTestCase {
      * @return test suite
      */
     public static Test suite() {
-        TestSuite suite = new TestSuite("ResultSetTest2");
+        BaseTestSuite suite = new BaseTestSuite("ResultSetTest2");
 
         suite.addTest(baseSuite("ResultSetTest2:embedded"));
 
@@ -754,7 +773,7 @@ public class ResultSetMiscTest extends BaseJDBCTestCase {
     }
 
     public static Test baseSuite(String name) {
-        TestSuite suite = new TestSuite(name);
+        BaseTestSuite suite = new BaseTestSuite(name);
         suite.addTestSuite(ResultSetMiscTest.class);
 
         // Some test cases expect lock timeouts, so reduce the timeout to
