@@ -28,15 +28,12 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.sql.Types;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
-import org.apache.derbyTesting.junit.TestConfiguration;
-import org.apache.derbyTesting.junit.JDBC;
-import org.apache.derbyTesting.junit.XML;
-
 import org.apache.derby.iapi.types.HarmonySerialBlob;
+import org.apache.derbyTesting.junit.BaseTestSuite;
+import org.apache.derbyTesting.junit.JDBC;
+import org.apache.derbyTesting.junit.TestConfiguration;
+import org.apache.derbyTesting.junit.XML;
 
 /**
  * <p>
@@ -99,7 +96,8 @@ public class BooleanValuesTest  extends GeneratedColumnsHelper
      */
     public static Test suite()
     {
-        Test result = (TestSuite) TestConfiguration.defaultSuite(BooleanValuesTest.class);
+        Test result = (BaseTestSuite)TestConfiguration.defaultSuite(
+            BooleanValuesTest.class);
 
         return result;
     }
@@ -1780,22 +1778,22 @@ public class BooleanValuesTest  extends GeneratedColumnsHelper
 
         ps.setInt( 1, 5 );
         ps.setString( 2, "setObject( Byte )" );
-        ps.setObject( 3, new Byte( (byte) value ) );
+        ps.setObject( 3, (byte) value );
         ps.execute();
 
         ps.setInt( 1, 6 );
         ps.setString( 2, "setObject( Short )" );
-        ps.setObject( 3, new Short( (short) value ) );
+        ps.setObject( 3, (short) value );
         ps.execute();
 
         ps.setInt( 1, 7 );
         ps.setString( 2, "setObject( Integer )" );
-        ps.setObject( 3, new Integer( value ) );
+        ps.setObject( 3, value );
         ps.execute();
 
         ps.setInt( 1, 8 );
         ps.setString( 2, "setObject( Long )" );
-        ps.setObject( 3, new Long( (long) value ) );
+        ps.setObject( 3, (long) value );
         ps.execute();
 
         String stringValue = Boolean.toString( (value != 0) );
