@@ -27,22 +27,18 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
-
 import java.text.Collator;
 import java.text.RuleBasedCollator;
-
 import java.util.Locale;
 import java.util.Properties; 
-
 import junit.framework.Assert;
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.Decorator;
 import org.apache.derbyTesting.junit.JDBC;
-import org.apache.derbyTesting.junit.SystemPropertyTestSetup;
 import org.apache.derbyTesting.junit.SupportFilesSetup;
+import org.apache.derbyTesting.junit.SystemPropertyTestSetup;
 import org.apache.derbyTesting.junit.TestConfiguration;
 
 /**
@@ -183,7 +179,7 @@ public class CollationTest2 extends BaseJDBCTestCase
      * <p>
      * Insert all data to tested against into LIKE_NAMES. A customer table
      * will be filled with this data.
-     * {p>
+     * <p>
      * Insert test cases for like string into the LIKE_TEST_CASES, results
      * are expected only to return a single row.
      * <p>
@@ -740,6 +736,42 @@ public class CollationTest2 extends BaseJDBCTestCase
         commit();
     }
 
+    private void addSomeMoreCustomers( int counter ) throws SQLException
+    { addSomeMoreCustomers( counter, true, true ); }
+
+    private void addSomeMoreCustomers( int counter, boolean useD1, boolean useD2 ) throws SQLException
+    {
+        PreparedStatement ps;
+	if( useD1 && useD2 )
+            ps = prepareStatement("INSERT INTO CUSTOMER VALUES(?,?,?,?,?,?,?)");
+	else if( useD2 )
+            ps = prepareStatement("INSERT INTO CUSTOMER VALUES(?,?,?,?,?,?)");
+	else
+            ps = prepareStatement("INSERT INTO CUSTOMER VALUES(?,?,?,?,?)");
+
+	int colNo = 1;
+        for (int i = 0; i < NAMES.length; i++)
+        {
+	    if( useD1 )
+                ps.setString(colNo++, "Another " + counter + NAMES[i]);
+	    if( useD2 )
+                ps.setString(colNo++, "Another " + counter + NAMES[i]);
+            ps.setString(colNo++, "Another " + counter + NAMES[i]);
+            ps.setInt( colNo++,   NAMES.length + counter + i);
+            ps.setInt( colNo++,   NAMES.length + counter + i);
+            ps.setString(colNo++, "Another " + counter + NAMES[i]);
+            ps.setString(colNo++, "Another " + counter + NAMES[i]);
+            ps.executeUpdate();
+	    colNo = 1;
+        }
+    }
+    private void dropExtraCustomers( int counter ) throws SQLException
+    {
+        PreparedStatement ps = prepareStatement("DELETE FROM CUSTOMER WHERE ID >= ?");
+        ps.setInt( 1, counter );
+        ps.executeUpdate();
+    }
+
     private void setUpLikeTable() throws SQLException 
     {
         Statement s = createStatement();
@@ -955,7 +987,7 @@ public class CollationTest2 extends BaseJDBCTestCase
     /**
      * Check simple boolean compare of string constant to column value.
      * <p>
-     * Check <, <=, =, >=, > of constant to column, ie. of the form
+     * Check &lt;, &lt;=, =, &gt;=, &gt; of constant to column, ie. of the form
      *     select * from table where col boolean constant
      *
      *
@@ -1093,7 +1125,7 @@ public class CollationTest2 extends BaseJDBCTestCase
     /**
      * Check simple boolean compare of string constant to column value.
      * <p>
-     * Check <, <=, =, >=, > of constant to column, ie. of the form
+     * Check &lt;, &glt;=, =, &gt;=, &gt; of constant to column, ie. of the form
      *     select * from table where col boolean constant
      *
      *
@@ -1436,6 +1468,8 @@ public class CollationTest2 extends BaseJDBCTestCase
 
         runQueries(db_index, null, null);
 
+        addSomeMoreCustomers( 100 );
+
         dropTable();
         commit();
     }
@@ -1451,21 +1485,24 @@ public class CollationTest2 extends BaseJDBCTestCase
      * T11: alter table drop column with indexes
      **/
     private void runAlterTableDropColumn(
-    Connection  conn,
     int         db_index)
         throws SQLException 
     {
-        Statement s = conn.createStatement();
+        Statement s = createStatement();
 
         setUpTable();
 
         s.execute("ALTER TABLE CUSTOMER DROP COLUMN D1");
         runQueries(db_index, null, null);
+        addSomeMoreCustomers( 100, false, true );
+	dropExtraCustomers( 100 );
 
         s.execute("CREATE INDEX IDX1 ON CUSTOMER (NAME)");
         s.execute("ALTER TABLE CUSTOMER DROP COLUMN D2");
         runQueries(db_index, null, null);
-        conn.rollback();
+        addSomeMoreCustomers( 100, false, false );
+
+        rollback();
 
         dropTable();
         commit();
@@ -1497,6 +1534,8 @@ public class CollationTest2 extends BaseJDBCTestCase
         s.execute("CREATE INDEX IDX1 ON CUSTOMER (NAME)");
         runQueries(db_index, null, null);
 
+        addSomeMoreCustomers( 100 );
+
         dropTable();
 
         commit();
@@ -1928,12 +1967,11 @@ public class CollationTest2 extends BaseJDBCTestCase
         runLikeTests(db_index);
 
         runDerby5530TruncateNoIndex();
+
         runDerby5530TruncateIndex();
 
-        /*
-        TODO -MIKEM, this test does not work yet.
-        runAlterTableDropColumn(conn, db_index);
-        */
+        dropTable();
+        runAlterTableDropColumn(db_index);
 
         commit();
     }
@@ -1997,7 +2035,7 @@ public class CollationTest2 extends BaseJDBCTestCase
     {
         // only test in embedded mode, all tests are server side actions.
         
-        TestSuite suite = new TestSuite("CollationTest2");
+        BaseTestSuite suite = new BaseTestSuite("CollationTest2");
         suite.addTest(new CollationTest2("testDefaultCollation"));
         suite.addTest(collatedTest("en", "testEnglishCollation"));
         suite.addTest(caseInsensitiveCollationSuite());
@@ -2050,7 +2088,9 @@ public class CollationTest2 extends BaseJDBCTestCase
      * @return A suite of tests.
      */
     private static Test caseInsensitiveCollationSuite() {
-        TestSuite suite = new TestSuite("Case insensitive specific tests");
+        BaseTestSuite suite =
+            new BaseTestSuite("Case insensitive specific tests");
+
         suite.addTest(new CollationTest2("ci_testDerby5367")); 
         return Decorator.territoryCollatedCaseInsensitiveDatabase(
                 suite, "en_US");
