@@ -658,10 +658,11 @@ public class IdentitySequenceTest extends GeneratedColumnsHelper
 		// Syntax error. 
         expectCompilationError( conn,"42X01" ,"create table t( a int generated always as identity(START WITH 47 ,))" );
 
-		// Does not support cycle option after altering the table. 
+		// Support cycle option after altering the table.
         goodStatement( conn, "create table t( a int generated always as identity(start with 2147483647 cycle), b int)" );
 	goodStatement( conn, "alter table t alter column a set increment by 4" );
-        expectExecutionError( conn,"2200H" ,"insert into t( b ) values ( 1 ), ( 2 )" );
+	goodStatement( conn ,"insert into t( b ) values ( 1 ), ( 2 )" );
+        
 
 	goodStatement( conn, "drop table t" );
 		// Alter table works fine without cycling with cycle option in the syntax.
@@ -684,10 +685,10 @@ public class IdentitySequenceTest extends GeneratedColumnsHelper
              );
 
 	goodStatement( conn, "drop table t" );
-		// Does not support cycle option after altering the table. 
+		// supports cycle option after altering the table.
         goodStatement( conn, "create table t( a int generated always as identity(increment by 2 cycle), b int)" );
 	goodStatement( conn, "alter table t alter column a restart with 2147483647" );
-        expectExecutionError( conn,"2200H" ,"insert into t( b ) values ( 1 ), ( 2 )" );
+    goodStatement( conn ,"insert into t( b ) values ( 1 ), ( 2 )" );
 
 	goodStatement( conn, "drop table t" );
 		// Alter table works fine without cycling with cycle option in the syntax.
@@ -710,30 +711,45 @@ public class IdentitySequenceTest extends GeneratedColumnsHelper
              );
 
 	goodStatement( conn, "drop table t" );
-	goodStatement( conn, "create table t( A_6852 int generated always as identity(start with 7 increment by 2 cycle))" );
+	goodStatement( conn, "create table t( A_6852 int generated always as identity(start with 7 increment by 2 cycle), b_6852 int)" );
 		//SELECT from sys.syscolumns with cycle option
+        // ADDING AUTOINCREMENTCYCLE Derby-6905
+
 	assertResults
             (
              conn,
-             "select AUTOINCREMENTVALUE, AUTOINCREMENTSTART, AUTOINCREMENTINC from sys.syscolumns where COLUMNNAME ='A_6852'",
+             "select AUTOINCREMENTVALUE, AUTOINCREMENTSTART, AUTOINCREMENTINC, AUTOINCREMENTCYCLE from sys.syscolumns where COLUMNNAME ='A_6852'",
              new String[][]
              {
-                 { "7", "7", "2" },
+                 { "7", "7", "2", "true" },
+             },
+             false
+             );
+
+	assertResults
+            (
+             conn,
+             "select AUTOINCREMENTVALUE, AUTOINCREMENTSTART, AUTOINCREMENTINC, AUTOINCREMENTCYCLE from sys.syscolumns where COLUMNNAME ='B_6852'",
+             new String[][]
+             {
+                 { null, null, null, "false" },
              },
              false
              );
 
 	goodStatement( conn, "drop table t" );
 		//SELECT from sys.syscolumns without cycle option
-	goodStatement( conn, "create table T_6852( A_6852 int generated always as identity(start with 7 increment by 2))" );
+        // ADDING AUTOINCREMENTCYCLE Derby-6905
+
+        goodStatement( conn, "create table T_6852( A_6852 int generated always as identity(start with 7 increment by 2))" );
 
 	assertResults
             (
              conn,
-             "select AUTOINCREMENTVALUE, AUTOINCREMENTSTART, AUTOINCREMENTINC from sys.syscolumns where COLUMNNAME ='A_6852'",
+             "select AUTOINCREMENTVALUE, AUTOINCREMENTSTART, AUTOINCREMENTINC,  AUTOINCREMENTCYCLE  from sys.syscolumns where COLUMNNAME ='A_6852'",
              new String[][]
              {
-                 { "7", "7", "2" },
+                 { "7", "7", "2", "false" },
              },
              false
              );
