@@ -24,30 +24,26 @@ package org.apache.derbyTesting.functionTests.tests.lang;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
-import java.sql.Statement;
 import java.sql.SQLException;
-
+import java.sql.Statement;
 import java.sql.Types;
 import java.util.ArrayList;
 import java.util.BitSet;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Random;
 import java.util.Set;
 import java.util.TreeMap;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
-import org.apache.derbyTesting.junit.JDBC;
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.CleanDatabaseTestSetup;
+import org.apache.derbyTesting.junit.JDBC;
+import org.apache.derbyTesting.junit.RuntimeStatisticsParser;
 import org.apache.derbyTesting.junit.SQLUtilities;
 import org.apache.derbyTesting.junit.TestConfiguration;
-import org.apache.derbyTesting.junit.RuntimeStatisticsParser;
 
 /**
  * Test to verify that Derby will perform "multi-probing" of an index
@@ -62,7 +58,7 @@ import org.apache.derbyTesting.junit.RuntimeStatisticsParser;
  *   3. The number of elements in the IN list is significantly less
  *      than the number of rows in the target table (in this test
  *      the data rows are not unique w.r.t the IN list values, so
- *      the size of the IN list should generally be <= 1/10th of the
+ *      the size of the IN list should generally be &lt;= 1/10th of the
  *      number rows in the table).
  *
  * If all three of these are true then we expect that Derby will perform
@@ -150,7 +146,7 @@ public class InListMultiProbeTest extends BaseJDBCTestCase {
      * key id, that means an IN list with "N" values in it will return
      * greater than N rows.
      */
-    protected TreeMap foreignIdToRowsMap;
+    protected TreeMap<String, List<DataRow>> foreignIdToRowsMap;
 
     /**
      * Public constructor required for running test as standalone JUnit.
@@ -176,7 +172,7 @@ public class InListMultiProbeTest extends BaseJDBCTestCase {
      */
     public static Test suite()
     {
-        TestSuite suite = new TestSuite("IN-list MultiProbe Suite");
+        BaseTestSuite suite = new BaseTestSuite("IN-list MultiProbe Suite");
 
         /* This is a language/optimization test so behavior will be the
          * same for embedded and client/server.  Therefore we only need
@@ -337,7 +333,7 @@ public class InListMultiProbeTest extends BaseJDBCTestCase {
          */
         readAllRows(createStatement());
 
-        List strategies = new ArrayList();
+        List<QueryStrategy> strategies = new ArrayList<QueryStrategy>();
         Random ran = new Random(2);
         Connection c = getConnection();
 
@@ -1367,6 +1363,36 @@ public class InListMultiProbeTest extends BaseJDBCTestCase {
         s.execute("drop table d4376");
     }
 
+    /**
+     * Test case that exercises an otherwise untested code path through
+     * {@code MultiProbeTableScanResultSet.getNextProbeValue()}, as suggested
+     * in DERBY-4378. The code path is taken if the highest value in the
+     * IN list is duplicated. Since duplicate literals are removed during
+     * compilation, the values must be non-literals for the path to be taken.
+     */
+    public void testDuplicateParameters() throws SQLException {
+        // Disable auto-commit to allow easy cleanup with rollback().
+        setAutoCommit(false);
+
+        // Create a test table.
+        Statement s = createStatement();
+        s.execute("create table d4378(x int primary key, y int)");
+        s.execute("insert into d4378 values (1,2),(3,4),(5,6),(7,8),(9,10)");
+        s.execute("insert into d4378 select y, x from d4378");
+
+        // Perform a query where all the elements in the probe list are
+        // parameters, and all parameters are set to the same value.
+        PreparedStatement ps =
+                prepareStatement("select * from d4378 where x in (?,?,?)");
+        ps.setInt(1, 1);
+        ps.setInt(2, 1);
+        ps.setInt(3, 1);
+        JDBC.assertFullResultSet(
+                ps.executeQuery(), new String[][] {{"1", "2"}});
+
+        rollback();
+    }
+
     /**
      * Insert the received number of rows into DATA_TABLE via
      * batch processing.
@@ -1425,16 +1451,15 @@ public class InListMultiProbeTest extends BaseJDBCTestCase {
      * @param strategies Different query strategies to execute
      * @param cnt Size of the IN list with which to query.
      */
-    private void testOneSize(List strategies, int cnt) throws SQLException
+    private void testOneSize(List<QueryStrategy> strategies, int cnt)
+            throws SQLException
     {
         if (cnt > allIds.length)
             return;
 
         String failedStrategy = null;
         Statement st = createStatement();
-        for (Iterator iter = strategies.iterator(); iter.hasNext();)
-        {
-            QueryStrategy strategy = (QueryStrategy) iter.next();
+        for (QueryStrategy strategy : strategies) {
             int numRows = strategy.testSize(cnt);
 
             ResultSet rs = st.executeQuery(GET_RUNTIME_STATS_QUERY);
@@ -1460,7 +1485,7 @@ public class InListMultiProbeTest extends BaseJDBCTestCase {
 
     /**
      * Select all rows from DATA_TABLE and store them into an in-memory
-     * map of "foreign_uuid -> rows".  So any given foreign_key_uuid can
+     * map of "foreign_uuid -&gt; rows".  So any given foreign_key_uuid can
      * be mapped to one or more rows from the table.
      *
      * We use the in-memory map to verify that all queries executed
@@ -1475,14 +1500,14 @@ public class InListMultiProbeTest extends BaseJDBCTestCase {
         throws SQLException
     {
         ResultSet rs = stmt.executeQuery(SELECT_ALL);
-        foreignIdToRowsMap = new TreeMap();
+        foreignIdToRowsMap = new TreeMap<String, List<DataRow>>();
         while (rs.next())
         {
             DataRow c = new DataRow(rs);
-            List list = (List) foreignIdToRowsMap.get(c.foreign_key_uuid);
+            List<DataRow> list = foreignIdToRowsMap.get(c.foreign_key_uuid);
             if (list == null)
             {
-                list = new ArrayList();
+                list = new ArrayList<DataRow>();
                 foreignIdToRowsMap.put(c.foreign_key_uuid, list);
             }
             list.add(c);
@@ -1711,22 +1736,17 @@ public class InListMultiProbeTest extends BaseJDBCTestCase {
          * as well.  This comparator object allows that sort to happen using
          * the JVM's own sort algorithm.
          */
-        Comparator rowComparator = new Comparator()
+        Comparator<String[]> rowComparator = new Comparator<String[]>()
         {
-            public int compare(Object o1, Object o2)
+            public int compare(String[] o1, String[] o2)
             {
                 /* "6" here is the index of the "id" field w.r.t the array
                  * returned from DataRow.getColumns().
                  */
-                long id1 = Long.valueOf(((String[])o1)[6]).longValue();
-                long id2 = Long.valueOf(((String[])o2)[6]).longValue();
+                Long id1 = Long.valueOf(o1[6]);
+                Long id2 = Long.valueOf(o2[6]);
 
-                if (id1 < id2)
-                    return -1;
-                else if (id1 == id2)
-                    return 0;
-                else
-                    return 1;
+                return id1.compareTo(id2);
             }
         };
 
@@ -1750,7 +1770,7 @@ public class InListMultiProbeTest extends BaseJDBCTestCase {
         public final int testSize(int size)
             throws SQLException
         {
-            Set s = new HashSet();
+            Set<String> s = new HashSet<String>();
 
             /* A Set contains no duplicate elements.  So if we, in our
              * randomness, try to insert a duplicate value, it will be
@@ -1799,14 +1819,15 @@ public class InListMultiProbeTest extends BaseJDBCTestCase {
             throws SQLException
         {
             // This will be a list of String arrays.
-            List expected = new ArrayList(foreignIdToRowsMap.size());
+            List<String[]> expected =
+                    new ArrayList<String[]>(foreignIdToRowsMap.size());
 
             // Search the in-memory map to find all expected rows.
             for (int i = 0; i < foreignIds.length; i++)
             {
-                List list = (List)foreignIdToRowsMap.get(foreignIds[i]);
+                List<DataRow> list = foreignIdToRowsMap.get(foreignIds[i]);
                 for (int j = list.size() - 1; j >= 0; j--)
-                    expected.add(((DataRow)list.get(j)).getColumns());
+                    expected.add(list.get(j).getColumns());
             }
 
             // Sort the rows.
