@@ -21,20 +21,17 @@
 
 package org.apache.derbyTesting.functionTests.tests.lang;
 
-import java.sql.ResultSet;
-import java.sql.SQLException;
 import java.sql.PreparedStatement;
-import java.sql.Statement;
+import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
-
+import java.sql.SQLException;
+import java.sql.Statement;
 import junit.framework.Test;
-import junit.framework.TestSuite;
-import junit.framework.Assert;
-
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
+import org.apache.derbyTesting.junit.CleanDatabaseTestSetup;
 import org.apache.derbyTesting.junit.JDBC;
 import org.apache.derbyTesting.junit.TestConfiguration;
-import org.apache.derbyTesting.junit.CleanDatabaseTestSetup;
 
 /**
  * OLAP functionality test.
@@ -54,7 +51,7 @@ public class OLAPTest extends BaseJDBCTestCase {
 
 	public static Test makeSuite() {
         Test clean = new CleanDatabaseTestSetup(
-            new TestSuite(OLAPTest.class)) {
+            new BaseTestSuite(OLAPTest.class)) {
                 protected void decorateSQL(Statement s)
                         throws SQLException
                 {
@@ -86,7 +83,7 @@ public class OLAPTest extends BaseJDBCTestCase {
 
 	public static Test suite()
     {
-		TestSuite suite = new TestSuite("OLAPTest");
+        BaseTestSuite suite = new BaseTestSuite("OLAPTest");
 		suite.addTest(makeSuite());
 		suite.addTest(TestConfiguration.clientServerDecorator(makeSuite()));
 		return suite;
@@ -590,6 +587,64 @@ public class OLAPTest extends BaseJDBCTestCase {
             s,
             "select * from t4 t_1 join t4 t_2 on " +
             "                     t_1.a = row_number() over () + t_2.a");
+
+        // DERBY-6565: NPE before
+        assertStatementError(
+                LANG_WINDOW_FUNCTION_CONTEXT_ERROR,
+                s,
+                "update t3 set y = y - row_number() over ()");
+
+        // DERBY-6688: subquery using SubqueryNode rather than FromSubquery
+        // had problems with presence of window function in order by.
+
+        JDBC.assertFullResultSet(s.executeQuery("select * from t3"),
+                new String[][]{{"4"},{"5"},{"6"},{"7"},{"8"}});
+
+        // failed prior to DERBY-6688
+        s.executeUpdate(
+            "update t3 set y = y - " +
+            "    (select y from t3 order by row_number() over () " +
+            "     fetch first 1 row only)");
+        JDBC.assertFullResultSet(s.executeQuery("select * from t3"),
+                new String[][]{{"0"},{"1"},{"2"},{"3"},{"4"}});
+
+        // Used to work before
+        JDBC.assertFullResultSet(s.executeQuery(
+            "select * from  " +
+            "    (select y from t3 order by row_number() over () " + 
+            "     fetch first 1 row only) tt"),
+            new String[][]{{"0"}});
+
+        // failed prior to DERBY-6688
+        JDBC.assertFullResultSet(s.executeQuery(
+            "select * from t3 where y = " +
+            "    (select y from t3 order by row_number() over () " + 
+            "     fetch first row only)"),
+            new String[][]{{"0"}});
+
+        // DERBY-6689: ArrayIndexOutOfBoundsException before
+        assertStatementError(LANG_WINDOW_FUNCTION_CONTEXT_ERROR,
+            s,
+            "merge into t2 using t3 on (t2.x=t3.y) when not matched then " + 
+            "    insert values (row_number() over ())");
+        
+        // DERBY-6691: NPE before
+        assertStatementError(LANG_WINDOW_FUNCTION_CONTEXT_ERROR,
+            s,
+            "call syscs_util.syscs_compress_table(" + 
+            "    'APP', 'T2', row_number() over ())");
+
+        // DERBY-6690: a window function in generated clause was not detected
+        // before
+        assertStatementError(LANG_WINDOW_FUNCTION_CONTEXT_ERROR,
+            s,
+            "create table t (x int generated always as " +
+            "    (row_number() over ()))");
+
+        assertStatementError(LANG_WINDOW_FUNCTION_CONTEXT_ERROR,
+            s,
+            "alter table t2 add column foo int generated always as " +
+            "    (row_number() over ())");
     }
 
 
