@@ -25,13 +25,11 @@ import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.Statement;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
-import org.apache.derbyTesting.junit.TestConfiguration;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.CleanDatabaseTestSetup;
 import org.apache.derbyTesting.junit.JDBC;
+import org.apache.derbyTesting.junit.TestConfiguration;
 
 /**
  * <p>
@@ -80,7 +78,9 @@ public class RestrictedVTITest  extends GeneratedColumnsHelper
      */
     public static Test suite()
     {
-        TestSuite suite = (TestSuite) TestConfiguration.embeddedSuite(RestrictedVTITest.class);
+        BaseTestSuite suite = (BaseTestSuite)TestConfiguration.embeddedSuite(
+            RestrictedVTITest.class);
+
         Test        result = new CleanDatabaseTestSetup( suite );
 
         return result;
@@ -1214,7 +1214,7 @@ public class RestrictedVTITest  extends GeneratedColumnsHelper
              }
              );
     }
-    private static Integer i( int intValue ) { return new Integer( intValue ); }
+    private static Integer i( int intValue ) { return intValue; }
 
     public static IntegerArrayVTI integerListSpecialColNames()
     {
