@@ -19,24 +19,22 @@
  */
 package org.apache.derbyTesting.functionTests.tests.lang;
 
-import junit.framework.Test;
-import junit.framework.TestSuite;
-
-import org.apache.derbyTesting.junit.JDBC;
-import org.apache.derbyTesting.junit.XML;
-import org.apache.derbyTesting.junit.BaseJDBCTestCase;
-import org.apache.derbyTesting.junit.BaseJDBCTestSetup;
-import org.apache.derbyTesting.junit.TestConfiguration;
-
+import java.sql.CallableStatement;
 import java.sql.Connection;
+import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
-import java.sql.Statement;
-import java.sql.CallableStatement;
-import java.sql.PreparedStatement;
 import java.sql.SQLException;
 import java.sql.SQLWarning;
+import java.sql.Statement;
 import java.sql.Types;
+import junit.framework.Test;
+import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseJDBCTestSetup;
+import org.apache.derbyTesting.junit.BaseTestSuite;
+import org.apache.derbyTesting.junit.JDBC;
+import org.apache.derbyTesting.junit.TestConfiguration;
+import org.apache.derbyTesting.junit.XML;
 
 /**
  * XMLTypeAndOpsTest this test is the JUnit equivalent to what used
@@ -79,7 +77,9 @@ public final class XMLTypeAndOpsTest extends BaseJDBCTestCase {
      */
     public static Test suite()
     {
-        TestSuite suite = new TestSuite("XML Type and Operators Suite\n");
+        BaseTestSuite suite =
+            new BaseTestSuite("XML Type and Operators Suite\n");
+
         if (!XML.classpathMeetsXMLReqs())
             return suite;
 
@@ -1481,6 +1481,27 @@ public final class XMLTypeAndOpsTest extends BaseJDBCTestCase {
 
         JDBC.assertFullResultSet(rs, expRS, true);
 
+        // If the query returns an atomic value (not a sequence), the XMLEXISTS
+        // operator should return TRUE.
+
+        expRS = new String[][] {
+            { "true" },
+            { "true" },
+            { "true" },
+            { "true" },
+            { "true" },
+        };
+
+        JDBC.assertFullResultSet(
+            st.executeQuery(
+                "select xmlexists('1+1' passing by ref x) from xqExists3"),
+            expRS);
+
+        JDBC.assertFullResultSet(
+            st.executeQuery(
+                "select xmlexists('1=2' passing by ref x) from xqExists3"),
+            expRS);
+
         // Cleanup.
 
         st.executeUpdate("drop table xqExists1");
@@ -2674,6 +2695,9 @@ public final class XMLTypeAndOpsTest extends BaseJDBCTestCase {
             { "0 div 0", "NaN" },
             { "3.14 div 0", "Infinity" },
             { "-3.14 div 0", "-Infinity" },
+            // Not strictly numeric, but let's test boolean too
+            { "1=1", "true" },
+            { "1=2", "false" },
         };
 
         Statement s = createStatement();
@@ -2702,7 +2726,7 @@ public final class XMLTypeAndOpsTest extends BaseJDBCTestCase {
      */
     private static class XMLTestSetup extends BaseJDBCTestSetup
     {
-        public XMLTestSetup(TestSuite tSuite) {
+        public XMLTestSetup(BaseTestSuite tSuite) {
             super(tSuite);
         }
 
