@@ -1,6 +1,6 @@
 /*
  
-Derby - Class org.apache.derbyTesting.functionTests.tests.replicationTests.ReplicationRun_Local_StateTest_part1_1
+Derby - Class org.apache.derbyTesting.functionTests.tests.replicationTests.ReplicationRun_Local_StateTest_part1_3
  
 Licensed to the Apache Software Foundation (ASF) under one or more
 contributor license agreements.  See the NOTICE file distributed with
@@ -20,6 +20,9 @@ limitations under the License.
  */
 package org.apache.derbyTesting.functionTests.tests.replicationTests;
 
+import java.sql.Connection;
+import java.sql.DriverManager;
+import java.sql.SQLException;
 import junit.framework.Test;
 import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.SecurityManagerSetup;
@@ -32,19 +35,15 @@ import org.apache.derbyTesting.junit.SecurityManagerSetup;
  * 
  */
 
-public class ReplicationRun_Local_StateTest_part1_1 extends ReplicationRun
+public class ReplicationRun_Local_StateTest_part1_3 extends ReplicationRun
 {
-    final static String REPLICATION_DB_NOT_BOOTED              = "XRE11";
-    final static String REPLICATION_NOT_IN_SLAVE_MODE          = "XRE40";
-    final static String SLAVE_OPERATION_DENIED_WHILE_CONNECTED = "XRE41";
-    final static String REPLICATION_SLAVE_SHUTDOWN_OK          = "XRE42";
-
+    
     /**
      * Creates a new instance of ReplicationRun_Local_StateTest_part1
      * 
      * @param testcaseName Identifying the test.
      */
-    public ReplicationRun_Local_StateTest_part1_1(String testcaseName)
+    public ReplicationRun_Local_StateTest_part1_3(String testcaseName)
     {
         super(testcaseName);
     }
@@ -52,9 +51,9 @@ public class ReplicationRun_Local_StateTest_part1_1 extends ReplicationRun
     public static Test suite()
     {
         BaseTestSuite suite =
-            new BaseTestSuite("ReplicationRun_Local_StateTest_part1_1 Suite");
+            new BaseTestSuite("ReplicationRun_Local_StateTest_part1_3 Suite");
         
-        suite.addTestSuite( ReplicationRun_Local_StateTest_part1_1.class );
+        suite.addTestSuite( ReplicationRun_Local_StateTest_part1_3.class );
         
         return SecurityManagerSetup.noSecurityManager(suite);
     }
@@ -70,14 +69,14 @@ public class ReplicationRun_Local_StateTest_part1_1 extends ReplicationRun
     ////
     //////////////////////////////////////////////////////////////
     
-    public void testReplication_Local_StateTest_part1_1()
+    public void testReplication_Local_StateTest_part1_3()
     throws Exception
     {
         cleanAllTestHosts();
         
         initEnvironment();
         
-       initMaster(masterServerHost,
+        initMaster(masterServerHost,
                 replicatedDb);
         
         startServer(masterJvmVersion, derbyMasterVersion,
@@ -85,7 +84,7 @@ public class ReplicationRun_Local_StateTest_part1_1 extends ReplicationRun
                 ALL_INTERFACES, // masterServerHost, // "0.0.0.0", // All. or use masterServerHost for interfacesToListenOn,
                 masterServerPort,
                 masterDbSubPath); // Distinguishing master/slave
-                
+        
         startServer(slaveJvmVersion, derbySlaveVersion,
                 slaveServerHost,
                 ALL_INTERFACES, // slaveServerHost, // "0.0.0.0", // All. or use slaveServerHost for interfacesToListenOn,
@@ -101,23 +100,18 @@ public class ReplicationRun_Local_StateTest_part1_1 extends ReplicationRun
                 masterServerPort, // master server interface accepting client requests
                 null // bootLoad, // The "test" to start when booting db.
                 );
-                
+        
         initSlave(slaveServerHost,
                 jvmVersion,
                 replicatedDb); // Trunk and Prototype V2: copy master db to db_slave.
-                
+        
         startSlave(jvmVersion, replicatedDb,
                 slaveServerHost, // slaveClientInterface // where the slave db runs
                 slaveServerPort,
                 slaveServerHost, // for slaveReplInterface
                 slaveReplPort,
                 testClientHost);
-                
-        // With master started above, next will fail! 
-        // Also seems failover will fail w/XRE21!
-        // Further testing: skipping next startMaster seems to 
-        // NOT remove failover failure!
-        /* TEMP: should be operational already - try skipping this. */
+        
         startMaster(jvmVersion, replicatedDb,
                 masterServerHost, // Where the startMaster command must be given
                 masterServerPort, // master server interface accepting client requests
@@ -125,11 +119,9 @@ public class ReplicationRun_Local_StateTest_part1_1 extends ReplicationRun
                 slaveServerPort, // Not used since slave don't allow clients.
                 slaveServerHost, // for slaveReplInterface
                 slaveReplPort);
-         /* */
         
-
-        _testPostStartedMasterAndSlave_StopSlave(); // Not in a state to continue.
-                
+        _testPostStartedMasterAndSlave_Failover(); // Not in a state to continue!
+        
         stopServer(jvmVersion, derbyVersion,
                 slaveServerHost, slaveServerPort);
         
@@ -138,65 +130,55 @@ public class ReplicationRun_Local_StateTest_part1_1 extends ReplicationRun
         
     }
 
-    private void _testPostStartedMasterAndSlave_StopSlave()
-            throws Exception
+    private void _testPostStartedMasterAndSlave_Failover()
     {
-        // 1. stopSlave to slave with connection to master should fail.
-        util.DEBUG("1. testPostStartedMasterAndSlave_StopSlave");
-
-        assertException(
-            stopSlave(slaveServerHost,
-                      slaveServerPort,
-                      slaveDatabasePath,
-                      replicatedDb,
-                      true),
-            SLAVE_OPERATION_DENIED_WHILE_CONNECTED);
-
+        Connection conn = null;
+        String db = slaveDatabasePath +FS+ReplicationRun.slaveDbSubPath +FS+ replicatedDb;
+        String connectionURL = "jdbc:derby:"  
+                + "//" + slaveServerHost + ":" + slaveServerPort + "/"
+                + db
+                + ";failover=true";
+        util.DEBUG("1. testPostStartedMasterAndSlave_Failover: " + connectionURL);
+        try
+        {
+            conn = DriverManager.getConnection(connectionURL);
+            util.DEBUG("Successfully connected as: " + connectionURL);
+            assertTrue("Successfully connected as: " + connectionURL, false);
+        }
+        catch (SQLException se)
+        {
+            int ec = se.getErrorCode();
+            String ss = se.getSQLState();
+            String msg = "As expected: Failover on slave should fail: " + ec + " " + ss + " " + se.getMessage();
+            util.DEBUG(msg);
+        }
         // Default replication test sequence still OK.
-
-        // 2. stopSlave to a master server should fail:
-        util.DEBUG("2. testPostStartedMasterAndSlave_StopSlave");
-
-        assertException(
-            stopSlave(masterServerHost,
-                      masterServerPort,
-                      masterDatabasePath,
-                      masterDbSubPath,
-                      replicatedDb,
-                      true),
-            REPLICATION_NOT_IN_SLAVE_MODE);
-
-        // Default replication test sequence still OK.
-
-        // Replication should still be up.
-
-        // Take down master - slave connection:
-        killMaster(masterServerHost, masterServerPort);
-
-        // 3.  stopSlave on slave should now result in an exception stating that
-        //     the slave database has been shutdown. A master shutdown results
-        //     in a behaviour that is similar to what happens when a stopMaster
-        //     is called.
-        util.DEBUG("3. testPostStartedMasterAndSlave_StopSlave");
-
-        stopSlave(slaveServerHost,
-                  slaveServerPort,
-                  slaveDatabasePath,
-                  replicatedDb,
-                  false); // master server is dead
-
-        // Default replication test sequence will NOT be OK after this point.
-
-        // 4. Try a normal connection:
-        String db = slaveDatabasePath + FS + slaveDbSubPath + FS + replicatedDb;
-
-        String connectionURL = "jdbc:derby:"
-            + "//" + slaveServerHost + ":" + slaveServerPort + "/"
-            + db;
-
-        util.DEBUG("4. testPostStartedMasterAndSlave_StopSlave: " +
-                   connectionURL);
-
-        waitForConnect(100L, 10, db, slaveServerHost, slaveServerPort);
+        
+        // Failover on master should succeed:
+        db = masterDatabasePath +FS+ReplicationRun.masterDbSubPath +FS+ replicatedDb;
+        connectionURL = "jdbc:derby:"  
+                + "//" + masterServerHost + ":" + masterServerPort + "/"
+                + db
+                + ";failover=true";
+        util.DEBUG("2. testPostStartedMasterAndSlave_Failover: " + connectionURL);
+        try
+        {
+            conn = DriverManager.getConnection(connectionURL);
+            util.DEBUG("Unexpectedly connected as: " + connectionURL);
+            assertTrue("Unexpectedly connected as: " + connectionURL, false);
+        }
+        catch (SQLException se)
+        {
+            int ec = se.getErrorCode();
+            String ss = se.getSQLState();
+            String msg = ec + " " + ss + " " + se.getMessage();
+            // Failover OK: SQLCODE: -1, SQLSTATE: XRE20
+            assertTrue("connectionURL " + " failed: " + msg, 
+                    // SQLState.REPLICATION_FAILOVER_SUCCESSFUL.equals(ss)); // "XRE20.D"
+                    "XRE20".equals(ss));
+            util.DEBUG("Failover on master succeeded: " + connectionURL + " " + msg);
+        }
+        // Not meaningful to continue default replication test sequence after this point!
     }
+    
 }
