@@ -23,11 +23,9 @@ package org.apache.derbyTesting.functionTests.tests.store;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
-
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.JDBC;
 import org.apache.derbyTesting.junit.TestConfiguration;
 
@@ -38,7 +36,7 @@ public class LongColumnTest extends BaseJDBCTestCase {
     }
     
     public static Test suite() {
-        TestSuite suite = new TestSuite("LongColumnTest");
+        BaseTestSuite suite = new BaseTestSuite("LongColumnTest");
         suite.addTest(TestConfiguration.defaultSuite(LongColumnTest.class));
 
         return suite;
@@ -659,4 +657,4 @@ public class LongColumnTest extends BaseJDBCTestCase {
         
         st.close();
     }
-}
\ No newline at end of file
+}
