@@ -28,17 +28,15 @@ import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.ArrayList;
 import javax.sql.DataSource;
-
+import junit.framework.Test;
 import org.apache.derby.catalog.SystemProcedures;
 import org.apache.derbyTesting.functionTests.util.streams.LoopingAlphabetStream;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 import org.apache.derbyTesting.junit.JDBC;
 import org.apache.derbyTesting.junit.JDBCDataSource;
 import org.apache.derbyTesting.junit.TestConfiguration;
 import org.apache.derbyTesting.junit.XML;
 
-import junit.framework.Test;
-import junit.framework.TestSuite;
-
 /**
  * Basic fixtures and setup for the upgrade test, not
  * tied to any specific release.
@@ -46,26 +44,9 @@ import junit.framework.TestSuite;
 public class BasicSetup extends UpgradeChange {
     
     public static Test suite() {
-        TestSuite suite = new TestSuite("Upgrade basic setup");
+        BaseTestSuite suite = new BaseTestSuite("Upgrade basic setup");
 
-        // Make the test cases run in a fixed order so they become
-        // more deterministic. Some orderings may make them run into
-        // problems in old versions. Since it's too late to fix bugs
-        // in old versions, we have to work around them like this
-        // instead.
-        //
-        // The specific problem that prompted this particular
-        // workaround, was a combination of DERBY-5947 and DERBY-4577.
-        // DERBY-5947 changed the size of the trigger plans stored in
-        // SYS.SYSSTATEMENTS. Although that change should be harmless
-        // in itself, it happened to make some of the triggers in this
-        // test have plans of the exact right size to hit DERBY-4577
-        // when downgrading to a version without the fix for
-        // DERBY-4577 in the post soft-upgrade phase. This only
-        // happened for some orderings, so always run the test with a
-        // known good ordering.
-
-        suite.addTest(TestConfiguration.orderedSuite(BasicSetup.class));
+        suite.addTest(new BaseTestSuite(BasicSetup.class));
 
         if (XML.classpathMeetsXMLReqs()) {
             // Only test XML operators if they are supported by the version
@@ -420,8 +401,21 @@ public class BasicSetup extends UpgradeChange {
         Statement s = createStatement();
         boolean modeDb2SqlOptional = oldAtLeast(10, 3);
 
-        dropTable("ATDC_BKUP1");
+        //DERBY-6684(Failure in testDERBY5120NumRowsInSydependsForTrigger when 
+        // upgrading from 10.11.1.0 to trunk)
+        //We ran into DERBY-6684 after the fix for DERBY-2041 went into 10.11
+        //The failure is the expected behavior since when dropping a table,
+        // we should detect any dependent triggers defined on other tables
+        // and drop table should fail if such dependencies exist. To fix the
+        // test failure, we just need to switch the order of drop table
+        //DERBY-2041(Trigger should register a dependency on tables and 
+        // columns used in its body)
+        //Drop ATDC_TAB1 before ATDC_BKUP1 because ATDC_TAB1 has a trigger
+        // on it which references ATDC_BKUP1. If we try dropping ATDC_BKUP1
+        // without first dropping ATDC_TAB1, there will be an error message
+        // that trigger ATDC_TAB1_TRG1 depends on ATDC_BKUP1
         dropTable("ATDC_TAB1");
+        dropTable("ATDC_BKUP1");
         s.execute("create table ATDC_TAB1(c11 int, c12 int)");
         s.execute("insert into ATDC_TAB1 values (1,11)");
         s.execute("create table ATDC_BKUP1(c111 int, c112 int)");
@@ -498,20 +492,20 @@ public class BasicSetup extends UpgradeChange {
         	//During the path back to original release, all the spses get
         	// marked invalid and hence they will be regenerated during 
         	// the next time they get fired. This regeneration will cause
-        	// the dependency between trigger action sps and trigger table
+        	// the dependency between trigger action sps and trigger table 
         	// be dropped except in 10.8.2.2 and higher releases where
         	// DERBY-5120 has already been fixed.
             assertStatementError("42802", s, " update ATDC_TAB1 set c11=2");
 
         	preapreFortDERBY5120();
-            s.execute("update ATDC_TAB1 set c12=11");
+            s.execute("update ATDC_TAB1 set c11=11");
             s.executeUpdate("alter table ATDC_TAB1 add column c113 int");
             //DERBY-5120 has been fixed in 10.8.2.2 and higher and hence we 
             // will not see the buggy behavior on those codelines
             if (oldLessThan(10,8,2,2)) 
-                s.execute("update ATDC_TAB1 set c12=11");
+                s.execute("update ATDC_TAB1 set c11=11");
             else
-                assertStatementError("42802", s, " update ATDC_TAB1 set c12=11");            
+                assertStatementError("42802", s, " update ATDC_TAB1 set c11=11");            
         	break;
 
         case PH_POST_HARD_UPGRADE:
@@ -680,8 +674,8 @@ public class BasicSetup extends UpgradeChange {
         Statement s = createStatement();
         boolean modeDb2SqlOptional = oldAtLeast(10, 3);
 
-        dropTable("BKUP1_5044_5120");
         dropTable("TAB1_5044_5120");
+        dropTable("BKUP1_5044_5120");
         s.execute("create table TAB1_5044_5120(c11 int, c12 int)");
         s.execute("insert into TAB1_5044_5120 values (1,11)");
         s.execute("create table BKUP1_5044_5120(c111 int, c112 int)");
@@ -1436,7 +1430,7 @@ public class BasicSetup extends UpgradeChange {
 
     private int[][] constructPowerSet( int count )
     {
-    	java.util.ArrayList list = new java.util.ArrayList();
+    	ArrayList<int[]> list = new ArrayList<int[]>();
         boolean[]           inclusions = new boolean[ count ];
 
         include( list, 0, inclusions );
@@ -1447,7 +1441,7 @@ public class BasicSetup extends UpgradeChange {
         return result;
     }
 
-    private void    include( ArrayList list, int idx, boolean[] inclusions )
+    private void include( ArrayList<int[]> list, int idx, boolean[] inclusions )
     {
         if ( idx >= inclusions.length )
         {
@@ -1477,7 +1471,8 @@ public class BasicSetup extends UpgradeChange {
         include( list, idx, inclusions, true );
     }
 
-    private void    include( ArrayList list, int idx, boolean[] inclusions, boolean currentCell )
+    private void include(ArrayList<int[]> list, int idx, boolean[] inclusions,
+                         boolean currentCell)
     {
         inclusions[ idx++ ] = currentCell;
 
@@ -1492,7 +1487,7 @@ public class BasicSetup extends UpgradeChange {
     ////////////////////////////////////////////////
     private int[][] permute( int[][] original )
     {
-        ArrayList list = new ArrayList();
+        ArrayList<int[]> list = new ArrayList<int[]>();
 
         for ( int i = 0; i < original.length; i++ )
         {
@@ -1505,7 +1500,7 @@ public class BasicSetup extends UpgradeChange {
         return result;
     }
 
-    private void   permute( ArrayList list, int[] start, int[] remainder )
+    private void permute( ArrayList<int[]> list, int[] start, int[] remainder )
     {
         int startLength = start.length;
         int remainderLength = remainder.length;
