@@ -197,6 +197,8 @@ class UpgradeRun extends UpgradeClassLoader
                     suite.addTest(Changes10_12.suite(phase));
                 if (oldMinor < 13)
                     suite.addTest(Changes10_13.suite(phase));
+                if (oldMinor < 14)
+                    suite.addTest(Changes10_14.suite(phase));
             }
             
             // Add DatabaseMetaData tests. Since metadata
