@@ -22,23 +22,19 @@ package org.apache.derbyTesting.functionTests.tests.upgradeTests;
 
 import java.lang.reflect.Method;
 import java.sql.Connection;
-import java.sql.DatabaseMetaData;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
-import java.sql.Statement;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Properties;
 import javax.sql.DataSource;
-
 import junit.extensions.TestSetup;
 import junit.framework.Test;
-import junit.framework.TestSuite;
-
-
 import org.apache.derbyTesting.junit.BaseJDBCTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
+import org.apache.derbyTesting.junit.ClassLoaderTestSetup;
 import org.apache.derbyTesting.junit.JDBCClient;
 import org.apache.derbyTesting.junit.JDBCClientSetup;
 import org.apache.derbyTesting.junit.JDBCDataSource;
@@ -209,9 +205,9 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
     private String             _trajectoryName;
     private boolean[]          _hardUpgradeRequests;
     
-    private HashMap     _unstableColumnValues = new HashMap();
+    private HashMap<String,String>     _unstableColumnValues = new HashMap<String,String>();
 
-    private static ThreadLocal _originalClassLoader = new ThreadLocal();
+    private static ThreadLocal<ClassLoader> _originalClassLoader = new ThreadLocal<ClassLoader>();
 
     // these are the system tables which must be read first in order to
     // prep the mapping of unstable identifiers
@@ -246,7 +242,7 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      */
     public static Test suite()
     {
-        TestSuite suite = new TestSuite("Upgrade trajectory test");
+        BaseTestSuite suite = new BaseTestSuite("Upgrade trajectory test");
 
         addTrajectories( suite );
         
@@ -260,7 +256,7 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      * Add all of the trajectories we intend to test.
      * </p>
      */
-    private static void addTrajectories( TestSuite suite )
+    private static void addTrajectories( BaseTestSuite suite )
     {
         Version.Trajectory[] trajectories = makeTrajectories();
         int                  count = trajectories.length;
@@ -314,7 +310,7 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      */
     private static boolean shouldBuildAllTrajectories()
     {
-        Boolean bool = new Boolean( getSystemProperty( ALL_TRAJECTORIES_PROPERTY ) );
+        Boolean bool = Boolean.valueOf(getSystemProperty( ALL_TRAJECTORIES_PROPERTY ));
 
         return bool.booleanValue();
     }
@@ -359,7 +355,8 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      * Add only the test case which hard-upgrades along all edges of the Trajectory.
      * </p>
      */
-    private static void addHardUpgradeOnlyTrajectory( TestSuite suite, Version.Trajectory trajectory )
+    private static void addHardUpgradeOnlyTrajectory(
+        BaseTestSuite suite, Version.Trajectory trajectory )
     {
         // a valid trajectory must have a start point and a different end point
         int       versionCount = trajectory.getVersionCount();
@@ -379,7 +376,8 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      * hard and softupgrade.
      * </p>
      */
-    private static void addTrajectory( TestSuite suite, Version.Trajectory trajectory )
+    private static void addTrajectory(
+        BaseTestSuite suite, Version.Trajectory trajectory )
     {
         // a valid trajectory must have a start point and a different end point
         int       versionCount = trajectory.getVersionCount();
@@ -393,7 +391,11 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
         addTrajectory( suite, trajectory, hardUpgradeRequests, 1 );
     }
 
-    private static void addTrajectory( TestSuite suite, Version.Trajectory trajectory, boolean[] hardUpgradeRequests, int idx )
+    private static void addTrajectory(
+        BaseTestSuite suite,
+        Version.Trajectory trajectory,
+        boolean[] hardUpgradeRequests,
+        int idx )
     {
         if ( idx >= trajectory.getVersionCount() )
         {
@@ -427,7 +429,10 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      * Add a single trajectory to the suite, with upgrade instructions.
      * </p>
      */
-    private static void addTrajectory( TestSuite suite, Version.Trajectory trajectory, boolean[] hardUpgradeRequests )
+    private static void addTrajectory(
+        BaseTestSuite suite,
+        Version.Trajectory trajectory,
+        boolean[] hardUpgradeRequests )
     {
         UpgradeTrajectoryTest utt = new UpgradeTrajectoryTest( trajectory, hardUpgradeRequests );
         TestSetup setup = TestConfiguration.additionalDatabaseDecorator( utt, UPGRADED_DATABASE );
@@ -518,7 +523,7 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
     private void compareDatabases( Version version, String leftDatabaseName, String rightDatabaseName )
         throws Exception
     {
-        UpgradeClassLoader.setThreadLoader( version.getClassLoader() );
+        ClassLoaderTestSetup.setThreadLoader( version.getClassLoader() );
 
         DataSource leftDS = makeDataSource( leftDatabaseName );
         DataSource rightDS = makeDataSource( rightDatabaseName );
@@ -576,11 +581,11 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      * list of initial tables.
      * </p>
      */
-    private ArrayList listSystemTables
+    private ArrayList<String> listSystemTables
         ( Connection conn )
         throws Exception
     {
-        ArrayList result = new ArrayList();
+        ArrayList<String> result = new ArrayList<String>();
 
         StringBuffer buffer = new StringBuffer();
 
@@ -621,7 +626,7 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
         String rightTableName = rightSchema + "." + tableName;
         StringBuffer columnList = new StringBuffer();
         StringBuffer insertList = new StringBuffer();
-        ArrayList columnNames = new ArrayList();
+        ArrayList<String> columnNames = new ArrayList<String>();
 
         PreparedStatement leftPS = chattyPrepare( leftConn, query );
         PreparedStatement rightPS = chattyPrepare( rightConn, query );
@@ -807,7 +812,7 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
     private void createDatabase( Version version, String logicalDatabaseName )
         throws Exception
     {
-        UpgradeClassLoader.setThreadLoader( version.getClassLoader() );
+        ClassLoaderTestSetup.setThreadLoader( version.getClassLoader() );
 
         DataSource ds = bootDatabase( logicalDatabaseName );
 
@@ -826,7 +831,7 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
     private void upgradeDatabase( Version softwareVersion, Version dataVersion, boolean hardUpgrade, String logicalDatabaseName )
         throws Exception
     {
-        UpgradeClassLoader.setThreadLoader( softwareVersion.getClassLoader() );
+        ClassLoaderTestSetup.setThreadLoader(softwareVersion.getClassLoader());
 
         DataSource ds = upgradeDatabase( logicalDatabaseName, hardUpgrade );
 
@@ -967,11 +972,14 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
     private void saveOriginalClassLoader()
     {
         // remember the original class loader so that we can reset
-        if ( _originalClassLoader.get() == null ) { _originalClassLoader.set( UpgradeClassLoader.getThreadLoader() ); }
+        if ( _originalClassLoader.get() == null ) { 
+            _originalClassLoader.set( ClassLoaderTestSetup.getThreadLoader() ); 
+        }
     }
     private void restoreOriginalClassLoader()
     {
-        UpgradeClassLoader.setThreadLoader( (ClassLoader) _originalClassLoader.get() );
+        ClassLoaderTestSetup.setThreadLoader(
+                (ClassLoader) _originalClassLoader.get() );
     }
 
     private String stringifyUpgradeRequests()
@@ -1215,7 +1223,7 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      */
     private static Version.Trajectory[] buildMinimalSet( Version[] supportedVersions )
     {
-        ArrayList   trajectoryList = new ArrayList();
+        ArrayList<Version.Trajectory>   trajectoryList = new ArrayList<Version.Trajectory>();
         int  versionCount = supportedVersions.length;
         boolean[]  include = new boolean[ versionCount ];
 
@@ -1238,7 +1246,7 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      */
     private static Version.Trajectory[] buildPowerSet( Version[] supportedVersions )
     {
-        ArrayList   trajectoryList = new ArrayList();
+        ArrayList<Version.Trajectory>   trajectoryList = new ArrayList<Version.Trajectory>();
         int            versionCount = supportedVersions.length;
         boolean[]  include = new boolean[ versionCount ];
 
@@ -1252,7 +1260,7 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      * Turn a list of trajectories into an array.
      * </p>
      */
-    private static Version.Trajectory[] squeezeArray( ArrayList trajectoryList )
+    private static Version.Trajectory[] squeezeArray( ArrayList<Version.Trajectory> trajectoryList )
     {
         Version.Trajectory[] result = new Version.Trajectory[ trajectoryList.size() ];
         trajectoryList.toArray( result );
@@ -1269,7 +1277,7 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      * </p>
      */
     private static void buildPowerSetMinion
-        ( Version[] supportedVersions, ArrayList result, boolean[] include, int idx, boolean removeSameBranchNeighbors )
+        ( Version[] supportedVersions, ArrayList<Version.Trajectory> result, boolean[] include, int idx, boolean removeSameBranchNeighbors )
     {
         int  versionCount = supportedVersions.length;
 
@@ -1292,11 +1300,11 @@ public class UpgradeTrajectoryTest extends BaseJDBCTestCase
      * </p>
      */
     private static void addSubset
-        ( Version[] supportedVersions, ArrayList result, boolean[] include, boolean removeSameBranchNeighbors )
+        ( Version[] supportedVersions, ArrayList<Version.Trajectory> result, boolean[] include, boolean removeSameBranchNeighbors )
     {
         int  versionCount = supportedVersions.length;
 
-        ArrayList seed = new ArrayList();
+        ArrayList<Version> seed = new ArrayList<Version>();
         Version   previousVersion = null;
         
         for ( int i = 0; i < versionCount; i++ )
