@@ -35,9 +35,10 @@ import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.security.PrivilegedExceptionAction;
 import java.security.PrivilegedActionException;
+import javax.sql.ConnectionPoolDataSource;
 import javax.sql.DataSource;
+import javax.sql.XADataSource;
 
-import org.apache.derby.iapi.services.info.JVMInfo;
 import org.apache.derbyTesting.functionTests.harness.JavaVersionHolder;
 import org.apache.derbyTesting.functionTests.harness.RunTest;
 
@@ -105,7 +106,6 @@ public class TestUtil {
 	private static String XA_DATASOURCE_STRING = "XA";
 	private static String CONNECTION_POOL_DATASOURCE_STRING = "ConnectionPool";
 	private static String REGULAR_DATASOURCE_STRING = "";
-	private static String JSR169_DATASOURCE_STRING = "Simple";
 	
 	// Methods for making framework dependent decisions in tests.
 
@@ -171,9 +171,9 @@ public class TestUtil {
 	{
 		if (framework != UNKNOWN_FRAMEWORK)
 			return framework;
-              String frameworkString = (String) AccessController.doPrivileged
-                  (new PrivilegedAction() {
-                          public Object run() {
+              String frameworkString = AccessController.doPrivileged
+                  (new PrivilegedAction<String>() {
+                          public String run() {
                               return System.getProperty("framework");
                           }
                       }
@@ -219,9 +219,9 @@ public class TestUtil {
     */
     public static String getHostName()
     {
-        String hostName = (String) AccessController.doPrivileged
-            (new PrivilegedAction() {
-                    public Object run() {
+        String hostName = AccessController.doPrivileged
+            (new PrivilegedAction<String>() {
+                    public String run() {
                         return System.getProperty("hostName");
                     }
                 }
@@ -291,9 +291,11 @@ public class TestUtil {
                                 
               try {
                   AccessController.doPrivileged
-                      (new PrivilegedExceptionAction() {
-                              public Object run() throws Exception {
-                                  return Class.forName(driverName).newInstance();
+                      (new PrivilegedExceptionAction<Void>() {
+                              public Void run() throws Exception {
+                                  Class<?> clazz = Class.forName(driverName);
+                                  clazz.getConstructor().newInstance();
+                                  return null;
                               }
                           }
                        );
@@ -311,26 +313,13 @@ public class TestUtil {
 	 *                will mean ds.setDatabaseName("wombat") will be called
 	 *  @return datasource for current framework
 	 */
-	public static javax.sql.DataSource getDataSource(Properties attrs)
+    public static DataSource getDataSource(Properties attrs)
 	{
-		String classname;
-		if(HAVE_DRIVER_CLASS)
-		{
-			classname = getDataSourcePrefix() + REGULAR_DATASOURCE_STRING + "DataSource";
-			classname = checkForJDBC40Implementation(classname);
-			return (javax.sql.DataSource) getDataSourceWithReflection(classname, attrs);
-		}
-		else
-			return getSimpleDataSource(attrs);
-		
+        String classname =
+            getDataSourcePrefix() + REGULAR_DATASOURCE_STRING + "DataSource40";
+        return (DataSource) getDataSourceWithReflection(classname, attrs);
 	}
 
-	public static DataSource getSimpleDataSource(Properties attrs)
-	{
-		String classname = getDataSourcePrefix() + JSR169_DATASOURCE_STRING + "DataSource";
-		return (javax.sql.DataSource) getDataSourceWithReflection(classname, attrs);
-	}
-	
 	/**
 	 * Get an xa  data source for the appropriate framework
 	 * @param attrs  A set of attribute values to set on the datasource.
@@ -339,12 +328,11 @@ public class TestUtil {
 	 *                will mean ds.setDatabaseName("wombat") will be called
 	 *  @return datasource for current framework
 	 */
-	public static javax.sql.XADataSource getXADataSource(Properties attrs)
+    public static XADataSource getXADataSource(Properties attrs)
 	{
-		
-		String classname = getDataSourcePrefix() + XA_DATASOURCE_STRING + "DataSource";
-		classname = checkForJDBC40Implementation(classname);
-		return (javax.sql.XADataSource) getDataSourceWithReflection(classname, attrs);
+        String classname =
+            getDataSourcePrefix() + XA_DATASOURCE_STRING + "DataSource40";
+        return (XADataSource) getDataSourceWithReflection(classname, attrs);
 	}
 
 	
@@ -356,35 +344,14 @@ public class TestUtil {
 	 *                will mean ds.setDatabaseName("wombat") will be called
 	 *  @return datasource for current framework
 	 */
-	public static javax.sql.ConnectionPoolDataSource getConnectionPoolDataSource(Properties attrs)
+    public static ConnectionPoolDataSource
+            getConnectionPoolDataSource(Properties attrs)
 	{
-		String classname = getDataSourcePrefix() + CONNECTION_POOL_DATASOURCE_STRING + "DataSource";
-                classname = checkForJDBC40Implementation(classname);
-		return (javax.sql.ConnectionPoolDataSource) getDataSourceWithReflection(classname, attrs);
+        String classname = getDataSourcePrefix() +
+                CONNECTION_POOL_DATASOURCE_STRING + "DataSource40";
+        return (ConnectionPoolDataSource)
+                getDataSourceWithReflection(classname, attrs);
 	}
-        
-        /**
-         * returns the class name for the JDBC40 implementation
-         * if present. otherwise returns the class name of the class
-         * written for the lower jdk versions
-         * @param classname String
-         * @return String containing the name of the appropriate 
-         *         implementation
-         */
-        public static String checkForJDBC40Implementation(String classname) {
-                String classname_ = classname;
-                // The JDBC 4.0 implementation of the  
-                // interface is suffixed with "40". Use it if it is available 
-                // and the JVM version is at least 1.6.
-                if (JVMInfo.JDK_ID >= JVMInfo.J2SE_16) {
-                        String classname40 = classname_ + "40";
-                        try {
-                                Class.forName(classname40);
-                                classname_ = classname40;
-                        } catch (ClassNotFoundException e) {}
-                }
-                return classname_;
-        }
 
 	public static String getDataSourcePrefix()
 		{
@@ -412,7 +379,7 @@ public class TestUtil {
 	static private Class[] INT_ARG_TYPE = {Integer.TYPE};
 	static private Class[] BOOLEAN_ARG_TYPE = { Boolean.TYPE };
 	// A hashtable of special non-string attributes.
-	private static Hashtable specialAttributes = null;
+	private static Hashtable<String, Class[]> specialAttributes = null;
 	
 
 	private static Object getDataSourceWithReflection(String classname, Properties attrs)
@@ -424,7 +391,7 @@ public class TestUtil {
 		
 		if (specialAttributes == null)
 		{
-			specialAttributes = new Hashtable();
+			specialAttributes = new Hashtable<String, Class[]>();
 			specialAttributes.put("portNumber",INT_ARG_TYPE);
 			specialAttributes.put("driverType",INT_ARG_TYPE);
 			specialAttributes.put("retrieveMessagesFromServerOnGetMessage",
@@ -434,7 +401,8 @@ public class TestUtil {
 		}
 		
 		try {
-		ds  = Class.forName(classname).newInstance();
+            Class<?> clazz = Class.forName(classname);
+            ds = clazz.getConstructor().newInstance();
 
 		// for remote server testing, check whether the hostName is set for the test
 		// if so, and serverName is not yet set explicitly for the datasource, set it now
@@ -453,11 +421,11 @@ public class TestUtil {
 			if (argType  == INT_ARG_TYPE)
 			{
 				args = new Integer[] 
-				{ new Integer(Integer.parseInt(value)) };	
+				{ Integer.valueOf(value) };	
 			}
 			else if (argType  == BOOLEAN_ARG_TYPE)
 			{
-				args = new Boolean[] { new Boolean(value) };	
+				args = new Boolean[] { Boolean.valueOf(value) };	
 			}
 			else if (argType == STRING_ARG_TYPE)
 			{
@@ -824,7 +792,8 @@ public class TestUtil {
                         break;
                 } 
                 // q: do we need a privileged action here, like in loadDriver?
-                Class.forName(driverName).newInstance();
+                Class<?> clazz = Class.forName(driverName);
+                clazz.getConstructor().newInstance();
 				
                 String url = getJdbcUrlPrefix() + databaseName;
                 if (connAttrs != null) url += ";" + connAttrs;
@@ -856,6 +825,14 @@ public class TestUtil {
     		System.out.println("FAILure: Not allowed to use class");
     		ille.printStackTrace();
     		return null;
+    	} catch (NoSuchMethodException ille) {
+    		System.out.println("FAILure: No such constructor");
+    		ille.printStackTrace();
+    		return null;
+    	} catch (java.lang.reflect.InvocationTargetException ille) {
+    		System.out.println("FAILure: Cannot execute constructor");
+    		ille.printStackTrace();
+    		return null;
     	}
     }
     
@@ -923,9 +900,9 @@ public class TestUtil {
     public static void dumpAllStackTracesIfSupported(PrintWriter log)
 	{
 		try {
-			String version =  (String) AccessController.doPrivileged
-				(new java.security.PrivilegedAction(){
-						public Object run(){
+			String version = AccessController.doPrivileged
+				(new PrivilegedAction<String>(){
+						public String run(){
 							return System.getProperty("java.version");
 						}
 					}
@@ -934,18 +911,18 @@ public class TestUtil {
 			JavaVersionHolder j=  new JavaVersionHolder(version); 
 			
 			if (j.atLeast(1,5)){
-				Class c = Class.forName("org.apache.derbyTesting.functionTests.util.ThreadDump");
+				Class<?> c = Class.forName("org.apache.derbyTesting.functionTests.util.ThreadDump");
 				final Method m = c.getMethod("getStackDumpString",new Class[] {});
 				
 				String dump;
 				try {
-					dump = (String) AccessController.doPrivileged
-						(new PrivilegedExceptionAction(){
-								public Object run() throws 
+					dump = AccessController.doPrivileged
+						(new PrivilegedExceptionAction<String>(){
+								public String run() throws
 									IllegalArgumentException, 
 									IllegalAccessException, 
 									InvocationTargetException{
-									return m.invoke(null, null);
+									return (String) m.invoke(null);
 								}
 							}
 						 );
