@@ -26,10 +26,12 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.BitSet;
 import java.util.Iterator;
+import java.util.List;
 import java.util.ListIterator;
 import java.util.Locale;
 
 import junit.framework.Assert;
+import junit.framework.AssertionFailedError;
 
 /**
  * JDBC utility methods for the JUnit tests.
@@ -112,6 +114,22 @@ public class JDBC {
     private static final boolean HAVE_SQLXML
                            = haveClass("java.sql.SQLXML");
 
+    /** Is the Lucene core jar file on the classpath */
+    public  static  final   boolean HAVE_LUCENE_CORE =
+        haveClass( "org.apache.lucene.analysis.Analyzer" );
+
+    /** Is the Lucene analyzer jar file on the classpath */
+    public  static  final   boolean HAVE_LUCENE_ANALYZERS =
+        haveClass( "org.apache.lucene.analysis.en.EnglishAnalyzer" );
+
+    /** Is the Lucene queryparser jar file on the classpath */
+    public  static  final   boolean HAVE_LUCENE_QUERYPARSER =
+        haveClass( "org.apache.lucene.queryparser.surround.parser.QueryParser" );
+
+    /** Is the json-simple core jar file on the classpath */
+    public  static  final   boolean HAVE_JSON_SIMPLE =
+        haveClass( "org.json.simple.JSONArray" );
+
     /**
      * Does java.sql.ResultSet implement java.lang.AutoCloseable?
      * Indicates JDBC 4.1.
@@ -120,7 +138,7 @@ public class JDBC {
     static {
         boolean autoCloseable;
         try {
-            Class acClass = Class.forName("java.lang.AutoCloseable");
+            Class<?> acClass = Class.forName("java.lang.AutoCloseable");
             autoCloseable = acClass.isAssignableFrom(ResultSet.class);
         } catch (Throwable t) {
             autoCloseable = false;
@@ -299,7 +317,20 @@ public class JDBC {
 		Connection conn = dmd.getConnection();
 		Assert.assertFalse(conn.getAutoCommit());
 		Statement s = dmd.getConnection().createStatement();
-        
+
+        // Triggers
+        PreparedStatement pstr = conn.prepareStatement(
+                "SELECT TRIGGERNAME FROM SYS.SYSSCHEMAS S, SYS.SYSTRIGGERS T "
+                + "WHERE S.SCHEMAID = T.SCHEMAID AND SCHEMANAME = ?");
+        pstr.setString(1, schema);
+        ResultSet trrs = pstr.executeQuery();
+        while (trrs.next()) {
+            String trigger = trrs.getString(1);
+            s.execute("DROP TRIGGER " + JDBC.escape(schema, trigger));
+        }
+        trrs.close();
+        pstr.close();
+
         // Functions - not supported by JDBC meta data until JDBC 4
         // Need to use the CHAR() function on A.ALIASTYPE
         // so that the compare will work in any schema.
@@ -458,7 +489,7 @@ public class JDBC {
 		String dropLeadIn = "DROP " + dropType + " ";
 		
         // First collect the set of DROP SQL statements.
-        ArrayList ddl = new ArrayList();
+        ArrayList<String> ddl = new ArrayList<String>();
 		while (rs.next())
 		{
             String objectName = rs.getString(mdColumn);
@@ -534,11 +565,11 @@ public class JDBC {
             do {
                 hadError = false;
                 didDrop = false;
-                for (ListIterator i = ddl.listIterator(); i.hasNext();) {
-                    Object sql = i.next();
+                for (ListIterator<String> i = ddl.listIterator(); i.hasNext();) {
+                    String sql = i.next();
                     if (sql != null) {
                         try {
-                            s.executeUpdate(sql.toString());
+                            s.executeUpdate(sql);
                             i.set(null);
                             didDrop = true;
                         } catch (SQLException e) {
@@ -735,22 +766,32 @@ public class JDBC {
 	    int expectedRows) throws SQLException
 	{
 		ResultSetMetaData rsmd = rs.getMetaData();
+        List<List<String>> seen = new ArrayList<List<String>>();
+        List<String> seenRow = new ArrayList<String>();
 		
 		int rows = 0;
 		while (rs.next()) {
 			for (int col = 1; col <= rsmd.getColumnCount(); col++)
 			{
 				String s = rs.getString(col);
+                seenRow.add(s);
 				Assert.assertEquals(s == null, rs.wasNull());
                 if (rs.wasNull())
-                    assertResultColumnNullable(rsmd, col);
+                    assertResultColumnNullable(rs, seen, seenRow, col);
 			}
 			rows++;
+            seen.add(new ArrayList<String>(seenRow));
+            seenRow.clear();
 		}
 		rs.close();
 
-		if (expectedRows >= 0)
-			Assert.assertEquals("Unexpected row count:", expectedRows, rows);
+        if (expectedRows >= 0) {
+            try {
+                Assert.assertEquals("Unexpected row count:", expectedRows, rows);
+            } catch (AssertionFailedError e) {
+                throw addRsToReport(e, rsmd, seen, seenRow, rs);
+            }
+        }
         
         return rows;
 	}
@@ -761,14 +802,26 @@ public class JDBC {
      * ResultSet sees a NULL value. If the value is NULL then
      * the column's definition in ResultSetMetaData must allow NULLs
      * (or not disallow NULLS).
-     * @param rsmd Metadata of the ResultSet
+     * @param rs the resultSet
+     * @param seen The set of entirely read rows so far
+     * @param seenRow The set of read columns in the current row so far
      * @param col Position of column just fetched that was NULL.
      * @throws SQLException Error accessing meta data
      */
-    private static void assertResultColumnNullable(ResultSetMetaData rsmd, int col)
+    private static void assertResultColumnNullable(
+            ResultSet rs,
+            List<List<String>> seen,
+            List<String> seenRow,
+            int col)
     throws SQLException
     {
-        Assert.assertFalse(rsmd.isNullable(col) == ResultSetMetaData.columnNoNulls); 
+        final ResultSetMetaData rsmd = rs.getMetaData();
+
+        try {
+            Assert.assertFalse(rsmd.isNullable(col) == ResultSetMetaData.columnNoNulls);
+        } catch (AssertionFailedError e) {
+            throw addRsToReport(e, rsmd, seen, seenRow, rs);
+        }
     }
 	
     /**
@@ -781,7 +834,7 @@ public class JDBC {
      * @param expectedColNames Array of expected column names.
      */
     public static void assertColumnNames(ResultSet rs,
-        String [] expectedColNames) throws SQLException
+        String... expectedColNames) throws SQLException
     {
         ResultSetMetaData rsmd = rs.getMetaData();
         int actualCols = rsmd.getColumnCount();
@@ -795,6 +848,122 @@ public class JDBC {
                 expectedColNames[i], rsmd.getColumnName(i+1));
         }
     }
+    /**
+     * Takes a result set and an array of expected colum names (as
+     * Strings)  and asserts that the column names in the result
+     * set metadata match the number, order, and names of those
+     * in the array.
+     * Does the same for column types
+     *      *
+     * @param rs ResultSet for which we're checking column names.
+     * @param expectedTypes Array of expected types for the columns.
+     * @param expectedColNames Array of expected column names.
+     */
+    public static void assertDatabaseMetaDataColumns(ResultSet rs,
+            int[] expectedTypes, 
+            String... expectedColNames) throws SQLException
+    {
+        assertDatabaseMetaDataColumns(rs, null, expectedTypes, expectedColNames);
+    }    
+    /**
+     * Takes a result set and an array of expected colum names (as
+     * Strings)  and asserts that the column names in the result
+     * set metadata match the number, order, and names of those
+     * in the array.
+     * Does the same for column types and nullability.
+     * 
+     * This is a variation of JDBC.assertColumnNames, here we only compare
+     * the expected columns and ignore additional ones, because columns 
+     * returned from DatabaseMetaData can be added to with newer JDBC versions.
+     * If the ResultSet can not change over time, JDBC.assertColumnNames 
+     * should be used. See DERBY-6180.
+     *
+     * @param rs ResultSet for which we're checking column names.
+     * @param expectedTypes Array of expected types for the columns.
+     * @param nullability Array of expected nullability values for the columns.
+     * @param expectedColNames Array of expected column names.
+     */
+    public static void assertDatabaseMetaDataColumns(ResultSet rs,
+            boolean[] nullability, int[] expectedTypes, 
+            String... expectedColNames) throws SQLException
+    {
+        ResultSetMetaData rsmd = rs.getMetaData();
+        int actualCols = rsmd.getColumnCount();
+        String actualColNames="[";
+        for (int c = 0; c < rsmd.getColumnCount(); c++) {
+            if (c==0)
+                actualColNames=actualColNames + rsmd.getColumnName(c+1);
+            else
+                actualColNames=actualColNames + ", " + rsmd.getColumnName(c+1);
+        }
+        actualColNames=actualColNames + "]";
+        
+        if (nullability != null)
+            Assert.assertEquals("Number of items in expected ColumnNames and " +
+                "expected nullability arrays don't match; fix up the test", 
+                expectedColNames.length, nullability.length);
+        if (expectedTypes != null)
+            Assert.assertEquals("Number of items in expected ColumnNames and " +
+                "expected ColumnTypes arrays don't match; fix up the test", 
+                expectedColNames.length, expectedTypes.length);
+        
+        if (expectedColNames.length == rsmd.getColumnCount()) {
+            // the count is the same, expect the names to be the same too
+            for (int i = 0; i < actualCols; i++)
+            {
+                Assert.assertEquals("Column names do not match:",
+                        expectedColNames[i], rsmd.getColumnName(i+1));
+                if (expectedTypes != null)
+                    Assert.assertEquals("Column types do not match for column " 
+                        + (i+1),
+                        expectedTypes[i], rsmd.getColumnType(i+1));
+                if (nullability != null) {
+                    int expected = nullability[i] ?
+                            ResultSetMetaData.columnNullable : 
+                                ResultSetMetaData.columnNoNulls;
+                    Assert.assertEquals(
+                            "Column nullability do not match for column " +
+                            (i+1), expected, rsmd.isNullable(i+1));
+                }
+            }
+        }
+        else
+        {
+            for (int i = 0; i < expectedColNames.length; i++)
+            {
+                String expectedColName = expectedColNames[i];
+                boolean found=false;
+                for (int j = 0 ; j < rsmd.getColumnCount(); j++)
+                {
+                    if (expectedColNames[i].equalsIgnoreCase(
+                            rsmd.getColumnName(j+1)))
+                    {
+                        found = true;
+                        if (expectedTypes != null)
+                            Assert.assertEquals("Column Type does not match for column " +
+                                expectedColNames[i] + "(" + (i) + ")",
+                                expectedTypes[i], rsmd.getColumnType(j+1));
+                        if (nullability != null) {
+                            int expected = nullability[i] ?
+                                ResultSetMetaData.columnNullable : 
+                                    ResultSetMetaData.columnNoNulls;
+                        Assert.assertEquals(
+                            "Column nullability does not match for " +
+                            "column " + (i), expected, rsmd.isNullable(j+1));
+                        }
+                        break;
+                    }
+                    else {
+                        continue;
+                    }
+                }
+                Assert.assertTrue("Missing an expected column: " + expectedColName + 
+                        "\n Expected: " + Arrays.toString(expectedColNames) +
+                        "\n Actual  : " + actualColNames,
+                        found);
+            }
+        }
+    }
     /**
      * Takes a result set and an array of expected column types
      * from java.sql.Types 
@@ -855,7 +1024,7 @@ public class JDBC {
 	            Assert.assertEquals
                     ("Types do not match for parameter " + (i+1),
                      expectedTypes[i],
-                     ((Integer) getParameterType.invoke( pmd, new Object[] { new Integer( i + 1 ) } )).intValue()
+                     ((Integer) getParameterType.invoke( pmd, new Object[] { i + 1 } )).intValue()
                      );
 	        }
 	    }
@@ -1002,7 +1171,7 @@ public class JDBC {
 
 
     /**
-     * assertFullResultSet() using trimmed string comparisions.
+     * assertFullResultSet() using trimmed string comparisons.
      * Equal to
      * <code>
      * assertFullResultSet(rs, expectedRows, true)
@@ -1038,12 +1207,19 @@ public class JDBC {
         int rows;
         ResultSetMetaData rsmd = rs.getMetaData();
 
+        List<List<String>> seen = new ArrayList<List<String>>();
+        List<String> seenRow = new ArrayList<String>();
+
         // Assert that we have the right number of columns. If we expect an
         // empty result set, the expected column count is unknown, so don't
         // check.
         if (expectedRows.length > 0) {
-            Assert.assertEquals("Unexpected column count:",
-                expectedRows[0].length, rsmd.getColumnCount());
+            try {
+                Assert.assertEquals("Unexpected column count:",
+                        expectedRows[0].length, rsmd.getColumnCount());
+            } catch (AssertionFailedError e) {
+                throw addRsToReport(e, rsmd, seen, seenRow, rs);
+            }
         }
 
         for (rows = 0; rs.next(); rows++)
@@ -1061,10 +1237,14 @@ public class JDBC {
                     wstr = w.getSQLState();
                 }
 
-                Assert.assertEquals(
-                    "Warning assertion error on row " + (rows+1),
-                    warnings[rows],
-                    wstr);
+                try {
+                    Assert.assertEquals(
+                            "Warning assertion error on row " + (rows+1),
+                            warnings[rows],
+                            wstr);
+                } catch (AssertionFailedError e) {
+                    throw addRsToReport(e, rsmd, seen, seenRow, rs);
+                }
             }
 
             /* If we have more actual rows than expected rows, don't
@@ -1073,15 +1253,23 @@ public class JDBC {
              */
             if (rows < expectedRows.length)
             {
-                assertRowInResultSet(rs, rows + 1,
+                // dumps itself in any assertion seem
+                assertRowInResultSet(rs, seen, seenRow, rows + 1,
                     expectedRows[rows], allAsTrimmedStrings);
             }
+
+            seen.add(new ArrayList<String>(seenRow));
+            seenRow.clear();
         }
 
         if ( closeResultSet ) { rs.close(); }
 
         // And finally, assert the row count.
-        Assert.assertEquals("Unexpected row count:", expectedRows.length, rows);
+        try {
+            Assert.assertEquals("Unexpected row count:", expectedRows.length, rows);
+        } catch (AssertionFailedError e) {
+            throw addRsToReport(e, rsmd, seen, seenRow, rs);
+        }
     }
 
 
@@ -1120,6 +1308,9 @@ public class JDBC {
         throws SQLException
     {
         int rows;
+        final List<List<String>> seen = new ArrayList<List<String>>();
+        final List<String> seenRow = new ArrayList<String>();
+        final ResultSetMetaData rsmd = rs.getMetaData();
 
         // Assert that we have the right number of columns. If we expect an
         // empty result set, the expected column count is unknown, so don't
@@ -1137,15 +1328,21 @@ public class JDBC {
              */
             if (rows < expectedRows.length)
             {
-                assertRowInResultSet(rs, rows + 1,
+                assertRowInResultSet(rs, seen, seenRow, rows + 1,
                     expectedRows[rows], true, colsToCheck);
             }
+            seen.add(new ArrayList<String>(seenRow));
+            seenRow.clear();
         }
 
         rs.close();
 
         // And finally, assert the row count.
-        Assert.assertEquals("Unexpected row count:", expectedRows.length, rows);
+        try {
+            Assert.assertEquals("Unexpected row count:", expectedRows.length, rows);
+        } catch (AssertionFailedError e) {
+            throw addRsToReport(e, rsmd, seen, seenRow, rs);
+        }
     }
 
     /**
@@ -1167,6 +1364,9 @@ public class JDBC {
      * not work if one of the acceptable values is <code>null</code>.)
      *
      * @param rs Result set whose current row we'll check.
+     * @param seen The set of entirely read rows so far (IN semantics)
+     * @param seenRow The set of read columns in the current row so far
+     *                (OUT semantics)
      * @param rowNum Row number (w.r.t expected rows) that we're
      *  checking.
      * @param expectedRow Array of objects representing the expected
@@ -1182,12 +1382,18 @@ public class JDBC {
      *  CHAR(8) column, asTrimmedStrings should be FALSE and the
      *  expected row should contain the expected padding, such as
      *  "FRED    ".
+     * @throws java.sql.SQLException
      */
-    private static void assertRowInResultSet(ResultSet rs, int rowNum,
-        Object [] expectedRow, boolean asTrimmedStrings) throws SQLException
+    private static void assertRowInResultSet(
+            ResultSet rs,
+            List<List<String>> seen,
+            List<String> seenRow,
+            int rowNum,
+            Object [] expectedRow,
+            boolean asTrimmedStrings) throws SQLException
     {
         assertRowInResultSet(
-            rs, rowNum, expectedRow, asTrimmedStrings, (BitSet)null);
+            rs, seen, seenRow, rowNum, expectedRow, asTrimmedStrings, (BitSet)null);
     }
 
     /**
@@ -1196,7 +1402,7 @@ public class JDBC {
      * @param colsToCheck If non-null then for every bit b
      *   that is set in colsToCheck, we'll compare the (b+1)-th column
      *   of the received result set's current row to the i-th column
-     *   of expectedRow, where 0 <= i < # bits set in colsToCheck.
+     *   of expectedRow, where 0 &lt;= i &lt; # bits set in colsToCheck.
      *   So if colsToCheck is { 0, 3 } then expectedRow should have
      *   two objects and we'll check that:
      *
@@ -1205,11 +1411,16 @@ public class JDBC {
      *
      *   If colsToCheck is null then the (i+1)-th column in the
      *   result set is compared to the i-th column in expectedRow,
-     *   where 0 <= i < expectedRow.length.
+     *   where 0 &lt;= i &lt; expectedRow.length.
      */
-    private static void assertRowInResultSet(ResultSet rs,
-        int rowNum, Object [] expectedRow, boolean asTrimmedStrings,
-        BitSet colsToCheck) throws SQLException
+    private static void assertRowInResultSet(
+            ResultSet rs,
+            List<List<String>> seen,
+            List<String> seenRow,
+            int rowNum,
+            Object [] expectedRow,
+            boolean asTrimmedStrings,
+            BitSet colsToCheck) throws SQLException
     {
         int cPos = 0;
         ResultSetMetaData rsmd = rs.getMetaData();
@@ -1243,7 +1454,6 @@ public class JDBC {
                         obj = (rs.getShort(cPos) == 0) ? "false" : "true";
                     else
                         obj = rs.getString(cPos);
-                        
                 }
                 else
                 {
@@ -1256,8 +1466,11 @@ public class JDBC {
                     obj = ((String)obj).trim();
 
             }
-            else
+            else {
                 obj = rs.getObject(cPos);
+            }
+
+            seenRow.add(obj == null ? "null" : obj.toString());
 
             boolean ok = (rs.wasNull() && (expectedRow[i] == null))
                 || (!rs.wasNull()
@@ -1274,15 +1487,20 @@ public class JDBC {
                     expected = bytesToString((byte[] )expectedRow[i]);
                     found = bytesToString((byte[] )obj);
                 }
-                Assert.fail("Column value mismatch @ column '" +
-                    rsmd.getColumnName(cPos) + "', row " + rowNum +
-                    ":\n    Expected: >" + expected +
-                    "<\n    Found:    >" + found + "<");
+
+                try {
+                    Assert.fail("Column value mismatch @ column '" +
+                            rsmd.getColumnName(cPos) + "', row " + rowNum +
+                            ":\n    Expected: >" + expected +
+                            "<\n    Found:    >" + found + "<");
+                } catch (AssertionFailedError e) {
+                    throw addRsToReport(e, rsmd, seen, seenRow, rs);
+                }
             }
             
-            if (rs.wasNull())
-                assertResultColumnNullable(rsmd, cPos);
-
+            if (rs.wasNull()) {
+                assertResultColumnNullable(rs, seen, seenRow, cPos);
+            }
         }
     }
     
@@ -1410,14 +1628,22 @@ public class JDBC {
         }
 
         ResultSetMetaData rsmd = rs.getMetaData();
-        Assert.assertEquals("Unexpected column count",
-                            expectedRows[0].length, rsmd.getColumnCount());
+        List<List<String>> seen = new ArrayList<List<String>>();
+        List<String> seenRow = new ArrayList<String>();
 
-        ArrayList expected = new ArrayList(expectedRows.length);
+        try {
+            Assert.assertEquals("Unexpected column count",
+                    expectedRows[0].length, rsmd.getColumnCount());
+        } catch (AssertionFailedError e) {
+            throw addRsToReport(e, rsmd, seen, seenRow, rs);
+        }
+
+        List<List<String>> expected =
+                new ArrayList<List<String>>(expectedRows.length);
         for (int i = 0; i < expectedRows.length; i++) {
             Assert.assertEquals("Different column count in expectedRows",
                                 expectedRows[0].length, expectedRows[i].length);
-            ArrayList row = new ArrayList(expectedRows[i].length);
+            List<String> row = new ArrayList<String>(expectedRows[i].length);
 
             for (int j = 0; j < expectedRows[i].length; j++) {
                 String val = (String) expectedRows[i][j];
@@ -1428,39 +1654,48 @@ public class JDBC {
             expected.add(row);
         }
 
-        ArrayList actual = new ArrayList(expectedRows.length);
+        List<List<String>> actual =
+                new ArrayList<List<String>>(expectedRows.length);
         while (rs.next()) {
-            ArrayList row = new ArrayList(expectedRows[0].length);
+            List<String> row = new ArrayList<String>(expectedRows[0].length);
             for (int i = 1; i <= expectedRows[0].length; i++) {
                 String s = rs.getString(i);
+                seenRow.add(s);
 
                 row.add(asTrimmedStrings ?
                         (s == null ? null : s.trim()) :
                         s);
 
-                if (rs.wasNull())
-                    assertResultColumnNullable(rsmd, i);
+                if (rs.wasNull()) {
+                    assertResultColumnNullable(rs, seen, seenRow, i);
+                }
             }
             actual.add(row);
+            seen.add(new ArrayList<String>(seenRow));
+            seenRow.clear();
         }
         rs.close();
 
-        if (rowCountsMustMatch) {
-            String message = "Unexpected row count, expected: " +
-                    expectedRows.length + ", actual: " + actual.size() + "\n" +
-                    "\t expected rows: \n\t\t" + expected + 
-                    "\n\t actual result: \n\t\t" + actual + "\n";
-            Assert.assertEquals(message,
-                                expectedRows.length, actual.size());
-        }
-        if ( !actual.containsAll(expected) )
-        {
-            expected.removeAll( actual );
-            BaseTestCase.println
-                ( "These expected rows don't appear in the actual result: " + expected );
-            String message = "Missing rows in ResultSet; \n\t expected rows: \n\t\t" 
-                + expected + "\n\t actual result: \n\t\t" + actual;
-            Assert.fail( message );
+        try {
+            if (rowCountsMustMatch) {
+                String message = "Unexpected row count, expected: " +
+                        expectedRows.length + ", actual: " + actual.size() + "\n" +
+                        "\t expected rows: \n\t\t" + expected +
+                        "\n\t actual result: \n\t\t" + actual + "\n";
+                Assert.assertEquals(message,
+                        expectedRows.length, actual.size());
+            }
+            if ( !actual.containsAll(expected) )
+            {
+                expected.removeAll( actual );
+                BaseTestCase.println
+                        ( "These expected rows don't appear in the actual result: " + expected );
+                String message = "Missing rows in ResultSet; \n\t expected rows: \n\t\t"
+                        + expected + "\n\t actual result: \n\t\t" + actual;
+                Assert.fail( message );
+            }
+        } catch (AssertionFailedError e) {
+            throw addRsToReport(e, rsmd, seen, seenRow, rs);
         }
     }
 
@@ -1674,4 +1909,102 @@ public class JDBC {
         rtsp.assertSequence(sequence);
     }
 
+    private static AssertionFailedError addRsToReport(
+            AssertionFailedError afe,
+            ResultSetMetaData rsmd,
+            List<List<String>> seen,
+            List<String> seenRow,
+            ResultSet rs) throws SQLException {
+
+        try {
+            if (rs == null) {
+                return BaseTestCase.newAssertionFailedError(
+                        afe.getMessage() + "\n<NULL>", afe);
+            }
+
+            final int c = rsmd.getColumnCount();
+            StringBuilder heading = new StringBuilder("    ");
+            StringBuilder underline = new StringBuilder("    ");
+
+            // Display column headings
+            for (int i=1; i<= c; i++) {
+                if (i > 1) {
+                    heading.append(",");
+                    underline.append(" ");
+                }
+
+                int len = heading.length();
+                heading.append(rsmd.getColumnLabel(i));
+                len = heading.length() - len;
+
+                for (int j = len; j > 0; j--) {
+                    underline.append("-");
+                }
+            }
+
+            heading.append("\n");
+            underline.append("\n");
+
+            StringBuilder rowImg = new StringBuilder();
+            rowImg.append(afe.getMessage()).
+                   append("\n\n").
+                   append(heading.toString()).
+                   append(underline.toString());
+
+            if (!rs.isClosed()) {
+                final int s = seenRow.size();
+
+                // Get any rest of columns of current row
+                for (int i=0; i < c - s; i++) {
+                    String column = null;
+
+                    try {
+                        column = rs.getString(s + i + 1);
+                    } catch (SQLException e) {
+                        // We may not yet have called next?
+                        if (e.getSQLState().equals("24000")) {
+                            if (rs.next()) {
+                                column = rs.getString(s + i + 1);
+                            } else {
+                                break;
+                            }
+                        }
+                    }
+                    seenRow.add(column);
+                }
+
+                if (seenRow.size() > 0) {
+                    seen.add(new ArrayList<String>(seenRow));
+                    seenRow.clear();
+                }
+
+                // Get any remaining rows
+                while (rs.next()) {
+                    for (int i = 0; i < c; i++) {
+                        seenRow.add(rs.getString(i + 1));
+                    }
+                    seen.add(new ArrayList<String>(seenRow));
+                    seenRow.clear();
+                }
+            }
+
+            // Display data
+            for (List<String> row : seen) {
+                rowImg.append("   ").
+                       append(row.toString()).
+                       append("\n");
+            }
+
+            return BaseTestCase.newAssertionFailedError(rowImg.toString(), afe);
+
+        } catch (Throwable t) {
+            // Something went wrong when adding the ResultSet to the error
+            // message. Don't return this error, as it would shadow the
+            // original error. Print it to help debugging.
+            BaseTestCase.printStackTrace(t);
+
+            // Return the original error.
+            return afe;
+        }
+    }
 }
