@@ -1,6 +1,6 @@
 /*
 
-   Derby - Class org.apache.derbyTesting.unitTests.junit.FormatableBitSet
+   Derby - Class org.apache.derbyTesting.unitTests.junit.FormatableBitSetTest
 
    Licensed to the Apache Software Foundation (ASF) under one or more
    contributor license agreements.  See the NOTICE file distributed with
@@ -21,20 +21,16 @@
 
 package org.apache.derbyTesting.unitTests.junit;
 
-import org.apache.derbyTesting.junit.BaseTestCase;
-import junit.framework.*;
-
-import org.apache.derby.iapi.services.io.FormatableBitSet;
-
-import org.apache.derby.shared.common.sanity.SanityManager;
-import org.apache.derby.shared.common.sanity.AssertFailure;
-
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.lang.reflect.Method;
+import junit.framework.Test;
+import org.apache.derby.iapi.services.io.FormatableBitSet;
+import org.apache.derbyTesting.junit.BaseTestCase;
+import org.apache.derbyTesting.junit.BaseTestSuite;
 
 
 /**
@@ -106,7 +102,7 @@ public class FormatableBitSetTest extends BaseTestCase {
      * @throws Exception
      */
     public static Test suite() {
-        TestSuite ts = new TestSuite(FormatableBitSetTest.class,
+        BaseTestSuite ts = new BaseTestSuite(FormatableBitSetTest.class,
                              "FormatableBitSetTest suite");
 
         if (bitCount != null) {
@@ -666,7 +662,7 @@ public class FormatableBitSetTest extends BaseTestCase {
 
     // count one-bits in a byte with Integer.bitCount()
     private static int bitsInByte(byte b) throws Exception {
-        Integer arg = new Integer(b & 0xff);
+        Integer arg = b & 0xff;
         Integer ret = (Integer) bitCount.invoke(null, new Object[] { arg });
         return ret.intValue();
     }
