@@ -36,12 +36,16 @@ import org.apache.derby.iapi.services.context.ContextManager;
 
 import org.apache.derby.iapi.services.monitor.Monitor;
 
-import org.apache.derby.iapi.services.sanity.SanityManager;
+import org.apache.derby.shared.common.sanity.SanityManager;
 
 import org.apache.derby.iapi.services.io.FormatIdUtil;
 
 import org.apache.derby.iapi.error.StandardException; 
 
+import java.security.PrivilegedActionException;
+import java.security.PrivilegedExceptionAction;
+import java.security.PrivilegedAction;
+import java.security.AccessController;
 import java.util.Properties; 
 
 import javax.transaction.xa.XAResource;
@@ -111,7 +115,7 @@ public class T_XA extends T_Generic
 		startParams = T_Util.setEncryptionParam(startParams);
 
 		try {
-			store = (AccessFactory) Monitor.createPersistentService(
+			store = (AccessFactory) createPersistentService(
 				getModuleToTestProtocolName(), testService, startParams);
 		} catch (StandardException mse) {
 			throw T_Fail.exceptionFail(mse);
@@ -179,7 +183,7 @@ public class T_XA extends T_Generic
         REPORT("(XATest_1) starting");
 
         ContextManager cm = 
-                ContextService.getFactory().getCurrentContextManager();
+                getContextService().getCurrentContextManager();
 
         // COMMIT AN IDLE TRANSACTION.
 
@@ -272,7 +276,7 @@ public class T_XA extends T_Generic
     {
         REPORT("(XATest_2) starting");
         ContextManager cm = 
-                ContextService.getFactory().getCurrentContextManager();
+                getContextService().getCurrentContextManager();
 
         // COMMIT AN IDLE TRANSACTION.
 
@@ -437,7 +441,7 @@ public class T_XA extends T_Generic
         REPORT("(XATest_3) starting");
 
         ContextManager cm = 
-                ContextService.getFactory().getCurrentContextManager();
+                getContextService().getCurrentContextManager();
 
         // ABORT AN IDLE TRANSACTION.
 
@@ -557,7 +561,7 @@ public class T_XA extends T_Generic
         REPORT("(XATest_4) starting");
 
         ContextManager cm = 
-                ContextService.getFactory().getCurrentContextManager();
+                getContextService().getCurrentContextManager();
 
         // ABORT AN IDLE TRANSACTION.
 
@@ -827,7 +831,7 @@ public class T_XA extends T_Generic
         }
 
         ContextManager cm = 
-                ContextService.getFactory().getCurrentContextManager();
+                getContextService().getCurrentContextManager();
 
         // COMMIT AN IDLE TRANSACTION.
 
@@ -1068,7 +1072,7 @@ public class T_XA extends T_Generic
         REPORT("(XATest_5) starting");
 
         ContextManager cm = 
-                ContextService.getFactory().getCurrentContextManager();
+                getContextService().getCurrentContextManager();
 
         TransactionController   tc = store.getTransaction(cm);
 
@@ -1180,6 +1184,49 @@ public class T_XA extends T_Generic
         REPORT("(XATest_6) finishing");
     }
 
+    
+    /**
+     * Privileged lookup of the ContextService. Package protected so that user code
+     * can't call this entry point.
+     */
+    static  ContextService    getContextService()
+    {
+        return AccessController.doPrivileged
+            (
+             new PrivilegedAction<ContextService>()
+             {
+                 public ContextService run()
+                 {
+                     return ContextService.getFactory();
+                 }
+             }
+             );
+    }
+
+    /**
+     * Privileged startup. Must be private so that user code
+     * can't call this entry point.
+     */
+    private  static  Object createPersistentService( final String factoryInterface, final String serviceName, final Properties properties ) 
+        throws StandardException
+    {
+        try {
+            return AccessController.doPrivileged
+                (
+                 new PrivilegedExceptionAction<Object>()
+                 {
+                     public Object run()
+                         throws StandardException
+                     {
+                         return Monitor.createPersistentService( factoryInterface, serviceName, properties );
+                     }
+                 }
+                 );
+        } catch (PrivilegedActionException pae)
+        {
+            throw StandardException.plainWrapException( pae );
+        }
+    }
 }
 
 class commit_method
@@ -1230,10 +1277,10 @@ class commit_method
 
                 SanityManager.ASSERT(
                     cm == 
-                    ContextService.getFactory().getCurrentContextManager(),
+                    T_XA.getContextService().getCurrentContextManager(),
                     "cm = " + cm +
                     "current = " + 
-                        ContextService.getFactory().getCurrentContextManager());
+                        T_XA.getContextService().getCurrentContextManager());
             }
 
             ((XAResourceManager) store.getXAResourceManager()).commit(
@@ -1275,14 +1322,15 @@ class commit_method
 
                 SanityManager.ASSERT(
                     cm == 
-                    ContextService.getFactory().getCurrentContextManager(),
+                    T_XA.getContextService().getCurrentContextManager(),
                     "cm = " + cm +
                     "current = " + 
-                        ContextService.getFactory().getCurrentContextManager());
+                        T_XA.getContextService().getCurrentContextManager());
             }
 
             ((XAResourceManager) store.getXAResourceManager()).rollback(
                 cm, xid);
         }
     }
+
 }
