@@ -30,7 +30,6 @@ import org.apache.derby.iapi.services.info.ProductGenusNames;
 import java.util.List;
 import java.util.Stack;
 import java.util.Hashtable;
-import java.util.Properties;
 
 import java.io.InputStream;
 import java.io.FileInputStream;
@@ -51,7 +50,7 @@ import java.sql.PreparedStatement;
 	single and dual connection ij runs.
 
  */
-public class utilMain implements java.security.PrivilegedAction {
+public class utilMain {
 
 	private StatementFinder[] commandGrabber;
 	UCode_CharStream charStream;
@@ -86,7 +85,7 @@ public class utilMain implements java.security.PrivilegedAction {
 	 * command can be redirected, so we stack up command
 	 * grabbers as needed.
 	 */
-	Stack oldGrabbers = new Stack();
+	Stack<StatementFinder> oldGrabbers = new Stack<StatementFinder>();
 
 	LocalizedResource langUtil = LocalizedResource.getInstance();
 	/**
@@ -139,7 +138,7 @@ public class utilMain implements java.security.PrivilegedAction {
 		ijTokMgr = new ijTokenManager(charStream);
 		ijParser = new ij(ijTokMgr, this);
 		this.out = out;
-		this.ignoreErrors = ignoreErrors;
+		if ( ignoreErrors != null ) { this.ignoreErrors = (Hashtable) ignoreErrors.clone(); }
 		
 		showErrorCode = 
 			Boolean.valueOf(
@@ -186,7 +185,13 @@ public class utilMain implements java.security.PrivilegedAction {
 				JDBCDisplayUtil.ShowException(out, i); // will continue past driver failure
 			} catch (IllegalAccessException ia) {
 				JDBCDisplayUtil.ShowException(out, ia); // will continue past driver failure
-			}
+			} catch (NoSuchMethodException ia) {
+				JDBCDisplayUtil.ShowException(out, ia); // will continue past driver failure
+			} catch (java.lang.reflect.InvocationTargetException ia) {
+				JDBCDisplayUtil.ShowException(out, ia); // will continue past driver failure
+            } catch (ijException ie) {
+                JDBCDisplayUtil.ShowException(out, ie); // will continue past driver failure
+            }
 		}
 	}
 
@@ -213,7 +218,8 @@ public class utilMain implements java.security.PrivilegedAction {
 		if (firstRun) {
 
 			// figure out which version this is
-			InputStream versionStream = (InputStream) java.security.AccessController.doPrivileged(this);
+            InputStream versionStream = util.getResourceAsStream(
+                    ProductGenusNames.TOOLS_INFO);
 
 			// figure out which version this is
 			ProductVersionHolder ijVersion = 
@@ -269,7 +275,7 @@ public class utilMain implements java.security.PrivilegedAction {
 	  //check if the property is set to not show select count and set the static variable
         //accordingly. 
         boolean showNoCountForSelect = Boolean.valueOf(util.getSystemProperty("ij.showNoCountForSelect")).booleanValue();
-        JDBCDisplayUtil.showSelectCount = !showNoCountForSelect;
+        JDBCDisplayUtil.setShowSelectCount( !showNoCountForSelect );
 
         //check if the property is set to not show initial connections and accordingly set the
         //static variable.
@@ -315,7 +321,7 @@ public class utilMain implements java.security.PrivilegedAction {
 				while (command == null && ! oldGrabbers.empty()) {
 					// close the old input file if not System.in
 					if (fileInput) commandGrabber[currCE].close();
-					commandGrabber[currCE] = (StatementFinder)oldGrabbers.pop();
+					commandGrabber[currCE] = oldGrabbers.pop();
 					if (oldGrabbers.empty())
 						fileInput = initialFileInput;
 					command = commandGrabber[currCE].nextStatement();
@@ -488,7 +494,7 @@ public class utilMain implements java.security.PrivilegedAction {
 				}
 				result.closeStatement();
             } else if (result.isMultipleResultSetResult()) {
-              List resultSets = result.getMultipleResultSets();
+              List<ResultSet> resultSets = result.getMultipleResultSets();
               try {
                 JDBCDisplayUtil.DisplayMultipleResults(out,resultSets,
                                      connEnv[currCE].getConnection(),
@@ -828,8 +834,4 @@ public class utilMain implements java.security.PrivilegedAction {
         checkScrollableCursor(rs, "GETCURRENTROWNUMBER");
 		return rs.getRow();
 	}
-
-	public final Object run() {
-		return  getClass().getResourceAsStream(ProductGenusNames.TOOLS_INFO);
-	}
 }
