@@ -21,12 +21,15 @@
 
 package org.apache.derby.impl.tools.planexporter;
 
+import java.lang.reflect.InvocationTargetException;
 import java.sql.Connection;
 import java.sql.DriverManager;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 
+import org.apache.derby.iapi.tools.ToolUtils;
+
 /**
  * This class will perform the database connection establishment,
  * querying the database, shut downing the database.
@@ -77,7 +80,7 @@ public class AccessDatabase {
      */
     public AccessDatabase(String dburl, String aSchema, String aQuery)
             throws InstantiationException, IllegalAccessException,
-            ClassNotFoundException, SQLException
+                   ClassNotFoundException, SQLException, NoSuchMethodException, InvocationTargetException
     {
         this(createConnection(dburl), aSchema, aQuery);
     }
@@ -112,14 +115,15 @@ public class AccessDatabase {
      */
     private static Connection createConnection(String dbURL)
             throws InstantiationException, IllegalAccessException,
-            ClassNotFoundException, SQLException
+                   ClassNotFoundException, SQLException, NoSuchMethodException,
+                   InvocationTargetException
     {
 
-        if(dbURL.indexOf("://") != -1)
-            Class.forName("org.apache.derby.jdbc.ClientDriver").newInstance();
-
-        else
-            Class.forName("org.apache.derby.jdbc.EmbeddedDriver").newInstance();
+        Class<?> clazz = (dbURL.indexOf("://") != -1) ?
+          Class.forName("org.apache.derby.jdbc.ClientDriver")
+          :
+          Class.forName("org.apache.derby.jdbc.EmbeddedDriver");
+        clazz.getConstructor().newInstance();
 
         //Get a connection
         return DriverManager.getConnection(dbURL);
@@ -586,6 +590,6 @@ public class AccessDatabase {
      * @return data array of TreeNode Objects
      */
     public TreeNode[] getData() {
-        return data;
+        return (TreeNode[]) ToolUtils.copy( data );
     }
 }
