@@ -27,10 +27,10 @@ import java.io.StringReader;
 import java.sql.DriverManager;
 import java.sql.ResultSet;
 import java.sql.Connection;
+import java.sql.Driver;
 import java.sql.Statement;
 import java.sql.PreparedStatement;
 import java.sql.SQLException;
-import java.sql.SQLWarning;
 import java.sql.Timestamp;
 
 import java.util.HashMap;
@@ -64,15 +64,15 @@ public final class dblook {
 
 	// Mappings from id to name for schemas and tables (for ease
 	// of reference).
-	private static HashMap schemaMap;
-	private static HashMap tableIdToNameMap;
+	private static HashMap<String,String> schemaMap;
+	private static HashMap<String,String> tableIdToNameMap;
 
 	// Command-line Parameters.
 	private static String sourceDBUrl;
 	private static String ddlFileName;
 	private static String stmtDelimiter;
 	private static boolean appendLogs;
-	private static ArrayList tableList;
+	private static ArrayList<String> tableList;
 	private static String schemaParam;
 	private static String targetSchema;
 	private static boolean skipViews;
@@ -132,11 +132,12 @@ public final class dblook {
 		if (!loadDriver()) {
 		// Failed when loading the driver.  We already logged
 		// the exception, so just return.
+            Logs.cleanup();    // Make sure the error log is flushed to disk.
 			return;
 		}
 
-		schemaMap = new HashMap();
-		tableIdToNameMap = new HashMap();
+		schemaMap = new HashMap<String,String>();
+		tableIdToNameMap = new HashMap<String,String>();
 
 		// Now run the utility.
 		go();
@@ -322,7 +323,15 @@ public final class dblook {
 	    }
 
 		try {
-			Class.forName(derbyDriver).newInstance();
+            Class<?> klass = Class.forName(derbyDriver);
+            if (Driver.class.isAssignableFrom(klass)) {
+                klass.getConstructor().newInstance();
+            } else {
+                Logs.debug(
+                        "TL_notInstanceOf",
+                        new String[] { derbyDriver, Driver.class.getName() });
+                return false;
+            }
 	    }
 		catch (Exception e)
 		{
@@ -432,7 +441,7 @@ public final class dblook {
 
 		int argIndex = start;
 		int count = 0;
-		tableList = new ArrayList();
+		tableList = new ArrayList<String>();
 		while (argIndex < args.length) {
 
 			if (((args[argIndex].length() > 0) && (args[argIndex].charAt(0) == '-')) ||
@@ -519,6 +528,7 @@ public final class dblook {
 
             boolean at10_6 = atVersion( conn, 10, 6 );
             boolean at10_9 = atVersion( conn, 10, 9 );
+            boolean at10_11 = atVersion(conn, 10, 11);
 
 			// Generate DDL.
 
@@ -544,7 +554,7 @@ public final class dblook {
 			if (!skipViews)
 				DB_View.doViews(this.conn);
 
-			DB_Trigger.doTriggers(this.conn);
+            DB_Trigger.doTriggers(this.conn, at10_11);
 
 			DB_Roles.doRoles(this.conn);
 			DB_GrantRevoke.doAuthorizations(this.conn, at10_6);
@@ -895,7 +905,7 @@ public final class dblook {
 		int strLen = str.length();
 		for (int i = 0; i < tableList.size(); i++) {
 
-			String tableName = (String)tableList.get(i);
+			String tableName = tableList.get(i);
 			tableName = expandDoubleQuotes(stripQuotes(tableName));
 			int nameLen = tableName.length();
 			String strCopy;
@@ -1055,7 +1065,7 @@ public final class dblook {
 
 	public static String lookupSchemaId(String schemaId) {
 
-		return (String)(schemaMap.get(schemaId));
+		return schemaMap.get(schemaId);
 
 	}
 
@@ -1069,7 +1079,7 @@ public final class dblook {
 
 	public static String lookupTableId(String tableId) {
 
-		return (String)(tableIdToNameMap.get(tableId));
+		return tableIdToNameMap.get(tableId);
 
 	}
 
