@@ -170,6 +170,8 @@ public class FilteredIssueLister {
         }
         reportDisqualifiedIssues =
                 Boolean.getBoolean(REPORT_DISQUALIFICATIONS_PROP);
+
+        printReleases();
     }
 
     /** Constructor for testing, where the Derby versions can be specified
@@ -293,6 +295,10 @@ public class FilteredIssueLister {
             if (dv.isReleased()) {
                 releases.add(dv);
             }
+            else
+            {
+                System.out.println(dv.toString() + " was NOT released.");
+            }
         }
         Collections.sort(releases, new Comparator() {
 
